'''
Created on Jul 25, 2013

@author: marten
'''
from basesoft.web.contrib.i18n import MONTHS_MAP
from django.utils.translation import ugettext, ugettext_lazy as _
from basesoft.web.core.config import DefaultConfig
from pvcycle.core.controllers import PvcTBSController
from pvcycle.core.models import PvcTbsLicense
from basesoft.web.contrib.ticketing.config import TicketingLicenseConfig
from basesoft.web.core.input.validation import plugins

class PvcTbsDeclarationLiceseConfigs(DefaultConfig):
    permissions = ('pvc_tbs_license_configs', )
    label = _('TBS declaration settings')

    def configs(self):
        return [{
            'name': 'pvc.tbs.declaration.period',
            'label': ugettext('Declaration period'),
            'options': {
                'options': {
                    PvcTbsLicense.DECLARATION_PERIOD_MONTHLY: ugettext('Monthly'),
                    PvcTbsLicense.DECLARATION_PERIOD_QUARTERLY: ugettext('Quarterly'),
                    PvcTbsLicense.DECLARATION_PERIOD_YEARLY: ugettext('Yearly'),
                }
            }
        }, {
            'name': 'pvc.tbs.declaration.closing.day',
            'label': ugettext('Declaration closing day')
        }, {
            'name': 'pvc.tbs.advance.invoicing',
            'label': ugettext('Use forecast for advance invoicing.')
        }, {
            'name': 'pvc.tbs.require.end.user.distinction',
            'label': ugettext('Member needs to provide a declaration line distinction between b2b and b2c.')
        }, {
            'name': 'pvc.tbs.contribution.automatic.end.user.distinction',
            'label': ugettext('The order is automatically split for B2B and B2C based on an order rule.')
        }, {
            'name': 'pvc.tbs.contribution.split.by.brand',
            'label': ugettext('The declaration need to be split "by brand".')
        }]

    def get_form_behaviour(self, validators, values):

        closing_day_integer_params = {'unsigned': True, 'allow_zero': False}
        if not values.has_key('pvc.tbs.declaration.period') or values['pvc.tbs.declaration.period'] == None:
            validators['pvc.tbs.declaration.closing.day'] = [(plugins.Integer, closing_day_integer_params, )]
            return validators

        p = int(values['pvc.tbs.declaration.period'])
        if p == PvcTbsLicense.DECLARATION_PERIOD_MONTHLY:
            closing_day_integer_params['max_value'] = 27 # february

        if p == PvcTbsLicense.DECLARATION_PERIOD_QUARTERLY:
            closing_day_integer_params['max_value'] = 89 # quarter 1

        if p == PvcTbsLicense.DECLARATION_PERIOD_YEARLY:
            closing_day_integer_params['max_value'] = 364

        validators['pvc.tbs.declaration.closing.day'] = [(plugins.Integer, closing_day_integer_params, )]

        return validators

    def get_save_callbacks(self, callbacks):
        callbacks['pvc.tbs.declaration.period'] = (self, 'save_tbs_periodicity_change')
        return callbacks

    def save_tbs_periodicity_change(self, original_value, value, tbs):
        # no change
        if int(original_value) == int(value):
            return None

        tc = PvcTBSController(self._controller.sa_session, account_id=self._controller.get_account().id)
        tc.update_periodicity_change(tbs.id, value, original_value)

class PvcTbsRegistrationLiceseConfigs(DefaultConfig):
    permissions = ('pvc_tbs_license_configs', )
    label = _('TBS registration settings')

    def configs(self):
        return [{
            'name': 'pvc.tbs.use.membership.instead.of.service.level',
            'label': ugettext('Use Membership levels instead of Service levels')
        }, {
            'name': 'pvc.tbs.membership.valid.for.years',
            'label': ugettext('Memberships are valid for how many years')
        }, {
            'name': 'pvc.tbs.auto.approve.member',
            'label': ugettext('Members applications are automatically approved')
        }, {
            'name': 'pvc.tbs.require.company.registration.number',
            'label': ugettext('Application of a member requires a Company Registration Number')
        }, {
            'name': 'pvc.tbs.require.accepting.of.terms.and.conditions',
            'label': ugettext('Members requires accepting of Terms and Conditions on application')
        }, {
            'name': 'pvc.tbs.require.member.in.own.country',
            'label': ugettext('Member company requires to be in the same country as the TBS')
        }, {
            'name': 'pvc.tbs.require.proof.of.financial.guarantee',
            'label': ugettext('Member needs to provide a proof of financial guarantee on registration.')
        }, {
            'name': 'pvc.tbs.guarantee.forecast',
            'label': ugettext('Members need to provide a Guarantee forecast')
        }, {
            'name': 'pvc.tbs.guarantee.forecast.trigger.month',
            'label': ugettext('Month in which next year forecast is asked.'),
            'options': {
                'options': MONTHS_MAP
            }

        }]


class PvcTbsOtherLiceseConfigs(DefaultConfig):
    permissions = ('pvc_tbs_license_configs', )
    label = _('TBS other settings')

    def configs(self):
        return [{
            'name': 'pvc.tbs.fee.day',
            'label': ugettext(u'Day in January on which the Administrative/Membership fee is triggered for this tbs')
        }, {
            'name': 'pvc.tbs.optional.uk.vat.no',
            'label': ugettext('VAT number for UK members is optional at this TBS')
        }, {
            'name': 'pvc.tbs.has.full.memberships', # members with voting rights
            'label': ugettext('Full membership available')
        }, {
            'name': 'pvc.tbs.was.active.in.2013',
            'label': ugettext('Membership was possible in 2013')
        }, {
            # Included from IQPortal order app
            'name': 'basesoft.web.contrib.order.minimum_invoiceable_amount',
            'label': ugettext('Minimum invoiceable amount'),
            'help': ugettext('If the total amount exluding VAT for the order is below the given amount, no invoice will be generated.')
        }]


license_configs = {
    'PvcTbsLicense' : (PvcTbsRegistrationLiceseConfigs(), PvcTbsOtherLiceseConfigs(), PvcTbsDeclarationLiceseConfigs(), ),
    'PvcOperatorLicense' : (TicketingLicenseConfig(), )
}
