# -*- coding: utf-8 -*-
# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.
from sqlalchemy import func, case, and_, or_
from sqlalchemy.orm.query import aliased

from django.utils.translation import ugettext as _

from basesoft.web.core.util.custom_query import CustomQueryField, CustomQueryModel, DATA_TYPE_BOOLEAN, DATA_TYPE_DECIMAL,CustomQuery,\
    DATA_TYPE_INT, SQLCustomQuery, SQLCustomQueryTable, SQLCustomQueryField, SQLCustomQueryUnion
from basesoft.web.contrib.license.models import Domain, LicenseConfig, Group, License
from basesoft.web.contrib.license.custom_query import LicenseCustomQuery, TempCompany
from basesoft.web.contrib.contact.models import Company, Person, Address, CompanyAddress
from basesoft.web.contrib.i18n.models import Country
from basesoft.util.virtual_date import today
from basesoft.web.contrib.account.models import Config, Account, GroupAccount
from basesoft.web.core.exceptions import PublicSystemException, PublicUserException

from pvcycle.core.models import PvcAccountantLicense, PvcTbsLicense, PvcAccountantTbs, PvcTbsServiceLevel,\
    PvcMemberLicense, PvcTbsMember, PvcTbsCertificate, PvcTbsMemberCertificate, PvcTbsMembership
from pvcycle.core.controllers import AbstractPvcController
from pvcycle.product.models import PvcTbsProduct
from pvcycle.pvmodule.models import PvcTbsMemberPVModule

Tbs = aliased(License)

class AbstractPvcCustomQuery(CustomQuery, AbstractPvcController):
    def export_master_data(self, split_by, params, applied_filters, static_filters, *args):

        self._set_cq_fields()

        # write excel

        asc = params['dir'] != 'desc'
        data = self.get_data(None, params['sort'], asc, None, None, applied_filters, static_filters, False)
        if len(data) == 0:
            raise PublicUserException(_('No data for the provided selection'))

        import xlwt
        import random
        from hashlib import sha256 as sha

        workbook = xlwt.Workbook(encoding="UTF-8")
        sheet = workbook.add_sheet('data')

        headers = []
        # add headers to the seeet
        for i, col in enumerate([c for c in self.cq_fields if c.in_select]):
            headers.append(col.key)
            sheet.write(0, i, unicode(col.label))

        # now data
        for row_index, row_data in enumerate(data):
            for column_index, column in enumerate(headers):
                sheet.write(row_index+1, column_index, row_data[column])

        import os
        from django.conf import settings

        _hash = sha(str(random.random())).hexdigest()[:8]
        workbook.save(os.path.join(settings.TMP_FILE_REPOSITORY, 'data_%s.xls' % _hash))

        return _hash

    def get_master_data_preview(self, split_by, params, applied_filters, static_filters, *args):

        self._set_cq_fields()
        asc = params['dir'] != 'desc'
        data = self.get_data(None, params['sort'], asc, None, 30, applied_filters, static_filters, False)
        if len(data) == 0:
            raise PublicUserException(_('No data for the provided selection'))

        header_keys = []
        header_labels = []
        # get the headers
        for i, col in enumerate([c for c in self.cq_fields if c.in_select]):
            header_keys.append(col.key)
            header_labels.append(unicode(col.label))

        return header_keys, header_labels, data

    def _set_cq_fields(self):
        aliases = {
            'member': aliased(License),
            'company': aliased(Company),
            'contact_person': aliased(Person),
            'invoice_person': aliased(Person),
            'group': aliased(Group),
            'group_account': aliased(GroupAccount),
            'account': aliased(Account),
            'account_person': aliased(Person),
            'address': aliased(Address),
            'company_address': aliased(CompanyAddress),
            'country': aliased(Country),
        }

        self.cq_fields = self._get_master_data_fields(aliases) + self.cq_fields
        self.cq_tables = self.cq_tables + self._get_master_data_tables(aliases)
        if self.group_by is not None:
            self.group_by = [gb for gb in self.group_by]
        else:
            self.group_by = []

        self.group_by.append(aliases['account'].id)
        self.group_by.append(aliases['member'].id)

        if self._get_member_column() is None:
            raise PublicSystemException(_('This overview does not support master data export'))

    def _get_master_data_fields(self, aliases):

        rv = [
            CustomQueryField(aliases['member'].name, 'Member name', key='md_member_name'),
        ]

        # use full cols here as it should include country specific fields.
        for column, label in Company.full_cols_i18n.iteritems():
            rv.append(CustomQueryField(getattr(aliases['company'], column), unicode(label), key='md_company_%s' % column))

        for column, label in Address.cols_i18n.iteritems():
            # somehow address has these fields listed in available cols?
            if column == 'id' or column == 'created' or column == 'last_updated':
                continue
            rv.append(CustomQueryField(getattr(aliases['address'], column), 'Address %s' % unicode(label), key='md_address_%s' % column))

        rv.append(CustomQueryField(aliases['country'].name, 'Country', key='md_country_name'))
        rv.append(CustomQueryField(aliases['account'].name, 'Account', key='md_account_name'))
        for column, label in Person.cols_i18n.iteritems():
            if column == 'preferred_locale':
                continue
            rv.append(CustomQueryField(getattr(aliases['account_person'], column), 'Account %s' % unicode(label), key='md_account_person_%s' % column))

        for column, label in Person.cols_i18n.iteritems():
            if column == 'preferred_locale':
                continue
            rv.append(CustomQueryField(getattr(aliases['contact_person'], column), 'Contact %s' % unicode(label), key='md_contact_person_%s' % column))

        for column, label in Person.cols_i18n.iteritems():
            if column == 'preferred_locale':
                continue
            rv.append(CustomQueryField(getattr(aliases['invoice_person'], column), 'Invoice %s' % unicode(label), key='md_invoice_person_%s' % column))

        return rv

    def _get_master_data_tables(self, aliases):

        return [

            CustomQueryModel(aliases['member'], onclause=aliases['member'].id==self._get_member_column()),
            CustomQueryModel(aliases['company'], onclause=aliases['company'].id==aliases['member'].company_id),

            CustomQueryModel(aliases['company_address'], onclause=aliases['company'].id==aliases['company_address'].company_id),
            CustomQueryModel(aliases['address'], onclause=aliases['address'].id==aliases['company_address'].address_id),
            CustomQueryModel(aliases['country'], onclause=aliases['country'].id==aliases['address'].country_id),

            CustomQueryModel(aliases['group'], onclause=aliases['group'].license_id==aliases['member'].id),
            CustomQueryModel(aliases['group_account'], onclause=aliases['group_account'].group_id==aliases['group'].id),
            CustomQueryModel(aliases['account'], onclause=aliases['account'].id==aliases['group_account'].account_id),
            CustomQueryModel(aliases['account_person'], onclause=aliases['account_person'].id==aliases['account'].person_id),

            CustomQueryModel(aliases['contact_person'], onclause=aliases['contact_person'].id==aliases['company'].contact_person_id, use_outerjoin=True),
            CustomQueryModel(aliases['invoice_person'], onclause=aliases['invoice_person'].id==aliases['company'].invoice_person_id, use_outerjoin=True),
        ]

    def _get_member_column(self):
        return None

class AccountantCustomQuery(LicenseCustomQuery):
    """
    We extend the License Custom Query for additional permission checks.
    """

    def validate_access(self):
        self.assert_any_permissions('pvc_accountant_license')
        LicenseCustomQuery.validate_access(self)

    def tables(self):
        return (
            CustomQueryModel(PvcAccountantLicense),
            CustomQueryModel(Domain, onclause=PvcAccountantLicense.domain_id==Domain.id),
            CustomQueryModel(PvcAccountantTbs, use_outerjoin=True),
            CustomQueryModel(Tbs, use_outerjoin=True, onclause=PvcAccountantTbs.tbs_id == Tbs.id),
        )

    def fields(self):
        return (
            CustomQueryField(PvcAccountantLicense.id, _('ID'), is_filter=True, key='license_id'),
            CustomQueryField(PvcAccountantLicense.name, _('Name'), is_filter=True, key='license_name'),
            CustomQueryField(PvcAccountantLicense.is_active, _('Active'), is_filter=True, key='is_active', type=DATA_TYPE_BOOLEAN),
            CustomQueryField(func.group_concat(Tbs.name), _('TBS'), key='tbs'),
        )

    group_by = (PvcAccountantLicense.id, )

class TbsCustomQuery(LicenseCustomQuery):
    """
    We extend the License Custom Query for additional permission checks.
    """

    def validate_access(self):
        self.assert_any_permissions('pvc_tbs_license')
        LicenseCustomQuery.validate_access(self)

    def tables(self):
        return (
            CustomQueryModel(PvcTbsLicense),
            CustomQueryModel(Domain, onclause=PvcTbsLicense.domain_id==Domain.id),
            CustomQueryModel(Country, onclause=PvcTbsLicense.country_id==Country.id, use_outerjoin=True),
            CustomQueryModel(Company, onclause=PvcTbsLicense.company_id==Company.id, use_outerjoin=True),
            CustomQueryModel(Person, onclause=Company.contact_person_id==Person.id),
            CustomQueryModel(TempCompany, onclause=PvcTbsLicense.temp_company_id==TempCompany.id, use_outerjoin=True),
            CustomQueryModel(Config, onclause=Config.name=='pvc.tbs.use.membership.instead.of.service.level', use_outerjoin=True),
            CustomQueryModel(LicenseConfig, onclause=and_(LicenseConfig.config_id==Config.id, LicenseConfig.license_id==PvcTbsLicense.id), use_outerjoin=True),
        )

    def fields(self):
        return (
            CustomQueryField(PvcTbsLicense.id, _('ID'), key='license_id', is_filter=True),
            CustomQueryField(PvcTbsLicense.name, _('Name'), is_filter=True),
            CustomQueryField(PvcTbsLicense.id, _('Settings'), key='settings'),
            CustomQueryField(PvcTbsLicense.is_active, _('Active'), is_filter=True, key='is_active', type=DATA_TYPE_BOOLEAN),
            CustomQueryField(PvcTbsLicense.id, _('Mandate'), key='mandate'),
            CustomQueryField(PvcTbsLicense.id, _('Terms & conditions'), key='terms_conditions'),
            CustomQueryField(PvcTbsLicense.id, _('Product fees'), key='product_fees'),
            CustomQueryField(func.ifnull(Country.name, 'Rest of Europe'), _('Country'), is_filter=True, key='country'),
            CustomQueryField(func.concat_ws(' ', Person.first_name, Person.middle_name, Person.last_name), _('Contact'), key='contact'),
            CustomQueryField(Company.name, _('Company')),
            CustomQueryField(TempCompany.is_temp, _('Company details change'), key='master_data', is_filter=True),
            CustomQueryField(LicenseConfig.config_id, _('Uses membership'), key='memberships'),
        )

    group_by = (PvcTbsLicense.id, )

class TbsServiceLevelManager(AbstractPvcCustomQuery):
    """
    """

    def validate_access(self):
        self.assert_any_permissions('pvc_tbs_license', 'pvc_tbs_service_level')

    def tables(self):
        return (
            CustomQueryModel(PvcTbsServiceLevel),
            CustomQueryModel(PvcTbsLicense, onclause=PvcTbsServiceLevel.tbs_id==PvcTbsLicense.id),
        )

    def fields(self):
        return (
            CustomQueryField(PvcTbsServiceLevel.id, _('ID'), is_filter=True, key='service_level_id'),
            CustomQueryField(PvcTbsLicense.name, _('TBS name'), is_filter=True, key='tbs_name'),
            CustomQueryField(PvcTbsServiceLevel.name, _('Service level name'), is_filter=True, key='service_level_name'),
            CustomQueryField(PvcTbsServiceLevel.level, _('Level'), is_filter=True, key='service_level_level'),
            CustomQueryField(PvcTbsServiceLevel.product_type, _('Product type'), is_filter=True, key='product_type',
                type=DATA_TYPE_INT,
                filter_options=[
                    {'value': PvcTbsProduct.TYPE_OTHER, 'label': _('Other')},
                    {'value': PvcTbsProduct.TYPE_PV_RELATED, 'label': _('PV related')},
                ]
            ),
            CustomQueryField(PvcTbsServiceLevel.status, _('Status'), is_filter=True, key='status',
                type=DATA_TYPE_INT,
                filter_options=[
                    {'value': PvcTbsServiceLevel.STATUS_ACTIVE, 'label': _('Active')},
                    {'value': PvcTbsServiceLevel.STATUS_DISABLED_FOR_NEW_REGISTRATIONS, 'label': _('Disabled for new registrations')},
                ]
            ),
            CustomQueryField(func.pvcGetCurrentAdministrationFee(PvcTbsServiceLevel.id, today().year), _('Current administration fee'), key='current_administration_fee', type=DATA_TYPE_DECIMAL),
            CustomQueryField(func.pvcGetCurrentContributionFee(PvcTbsServiceLevel.id, today()), _('Current contribution fee'), key='current_contribution_fee', type=DATA_TYPE_DECIMAL),
            CustomQueryField(PvcTbsServiceLevel.guarantee_forecast_min_value, _('Guarantee forecast minimal value'), is_filter=True, key='gf_min_value'),
        )

    def set_where(self):
        self.where = PvcTbsServiceLevel.tbs_id.in_( [x.id for x in self._get_take_back_schemes(permissions=['pvc_tbs_service_level'])] )

    group_by = (PvcTbsServiceLevel.id, )

class TbsMembershipManager(AbstractPvcCustomQuery):
    """
    """

    def validate_access(self):
        self.assert_any_permissions('pvc_tbs_license', 'pvc_tbs_membership_level')

    def tables(self):
        return (
            CustomQueryModel(PvcTbsMembership),
            CustomQueryModel(PvcTbsLicense, onclause=PvcTbsMembership.tbs_id==PvcTbsLicense.id),
        )

    def fields(self):
        return (
            CustomQueryField(PvcTbsMembership.id, _('ID'), is_filter=True, key='membership_id'),
            CustomQueryField(PvcTbsLicense.name, _('TBS name'), is_filter=True, key='tbs_name'),
            CustomQueryField(PvcTbsMembership.name, _('Membership level name'), is_filter=True, key='membership_name'),
            CustomQueryField(func.pvcGetCurrentMembershipFee(PvcTbsMembership.id, today().year), _('Current membership fee'), key='current_membership_fee', type=DATA_TYPE_DECIMAL),
        )

    def set_where(self):
        self.where = PvcTbsMembership.tbs_id.in_( [x.id for x in self._get_take_back_schemes(permissions=['pvc_tbs_membership_level'])] )

    group_by = (PvcTbsMembership.id, )

class TbsCertificateManager(AbstractPvcCustomQuery):
    """
    """

    def validate_access(self):
        self.assert_any_permissions('pvc_manage_certificates')

    def tables(self):
        return (
            CustomQueryModel(PvcTbsCertificate),
            CustomQueryModel(PvcTbsLicense, onclause=PvcTbsCertificate.tbs_id==PvcTbsLicense.id),
        )

    def fields(self):
        return (
            CustomQueryField(PvcTbsCertificate.id, _('ID'), is_filter=True, key='tbs_certificate_id'),
            CustomQueryField(PvcTbsLicense.name, _('TBS name'), is_filter=True, key='tbs_name'),
            CustomQueryField(PvcTbsCertificate.year, _('Year'), is_filter=True, key='year'),
            CustomQueryField(PvcTbsCertificate.id, _('Preview'), is_filter=True, key='preview'),
        )

    def set_where(self):
        self.where = PvcTbsCertificate.tbs_id.in_( [x.id for x in self._get_take_back_schemes(permissions=['pvc_tbs_service_level'])] )

    group_by = (PvcTbsCertificate.id, )

class MemberCertificateManager(AbstractPvcCustomQuery):
    """
    """

    def _get_member_column(self):
        return PvcTbsMember.member_id

    def validate_access(self):
        self.assert_any_permissions('pvc_member_license', 'pvc_manage_certificates')

    def tables(self):
        return (
            CustomQueryModel(PvcTbsMemberCertificate),
            CustomQueryModel(PvcTbsMember, onclause=PvcTbsMemberCertificate.tbs_member_id==PvcTbsMember.id),
            CustomQueryModel(Tbs, onclause=PvcTbsMember.tbs_id==Tbs.id),
            CustomQueryModel(PvcMemberLicense, onclause=PvcTbsMember.member_id==PvcMemberLicense.id),
        )

    def fields(self):
        return (
            CustomQueryField(PvcTbsMember.id, _('ID'), is_filter=True, key='tbs_member_id'),
            CustomQueryField(Tbs.name, _('TBS'), is_filter=True, key='tbs_name'),
            CustomQueryField(PvcMemberLicense.name, _('Member'), is_filter=True, key='member_name'),
            CustomQueryField(PvcTbsMemberCertificate.year, _('Year'), is_filter=True, key='year'),
            CustomQueryField(PvcTbsMemberCertificate.hash, _('Unique ID'), is_filter=True, key='unique_id'),
            CustomQueryField(PvcTbsMemberCertificate.created, _('Issue date'), key='issue_date'),
            CustomQueryField(PvcTbsMemberCertificate.status, _('Status'), key='status'),
            CustomQueryField(PvcTbsMember.id, _('Download'), is_filter=True, key='download'),
        )

    def set_where(self):
        self.where = PvcMemberLicense.id.in_([x.id for x in self._get_member_licenses()])

    #group_by = (PvcTbsCertificate.id, )



class MemberCustomQuery(AbstractPvcCustomQuery):
    """
    """
    permissions = ['pvc_tbs_member_manager']

    def _get_member_column(self):
        return PvcTbsMember.member_id

    def tables(self):
        return (
            CustomQueryModel(PvcMemberLicense),
            CustomQueryModel(PvcTbsMember, onclause=PvcMemberLicense.id==PvcTbsMember.member_id),
            CustomQueryModel(Tbs, onclause=PvcTbsMember.tbs_id==Tbs.id),
            CustomQueryModel(Company, onclause=PvcMemberLicense.company_id==Company.id, use_outerjoin=True),
            CustomQueryModel(Person, onclause=Company.contact_person_id==Person.id, use_outerjoin=True),
            CustomQueryModel(TempCompany, onclause=PvcMemberLicense.temp_company_id==TempCompany.id, use_outerjoin=True),
        )

    def fields(self):
        return (
            CustomQueryField(PvcTbsMember.id, _('TBS/Member ID'), key='id'),
            CustomQueryField(PvcTbsMember.member_id, _('Member ID'), key='member_id'),
            CustomQueryField(PvcMemberLicense.name, _('Member'), key='member', is_filter=True),
            CustomQueryField(Company.vat_no, _('VAT no.'), key='vat_no', is_filter=True),
            CustomQueryField(Tbs.name, _('TBS'), key='tbs', is_filter=True, is_group_by=True),
            CustomQueryField(PvcTbsMember.id, _('Details'), key='tbs_member_id_details'),
            CustomQueryField(PvcTbsMember.company_registration_number, _('(W)EEE-Register Number'), key='company_registration_number'),
            CustomQueryField(PvcMemberLicense.id, _('Accounts'), key='accounts'),
            CustomQueryField(func.pvcGetCurrentServiceLevel(PvcTbsMember.id, today()), _('Service level'), key='service_level'),
            CustomQueryField(func.pvcGetCurrentPvServiceLevel(PvcTbsMember.id, today()), _('PV service level'), key='pv_service_level'),
            CustomQueryField(func.pvcGetCurrentMembership(PvcTbsMember.id, today()), _('Membership'), key='membership'),
            CustomQueryField(PvcTbsMember.is_producer, _('Producer'), key='producer', is_filter=True, type=DATA_TYPE_BOOLEAN),
            CustomQueryField(PvcTbsMember.is_commissioner, _('Commissioner'), key='commissioner', is_filter=True, type=DATA_TYPE_BOOLEAN),
            CustomQueryField(PvcTbsMember.is_full_member, _('Is full member'), key='is_full_member', is_filter=True, type=DATA_TYPE_BOOLEAN),
            CustomQueryField(PvcTbsMember.has_voting_rights, _('Has voting rights'), key='has_voting_rights', is_filter=True, type=DATA_TYPE_BOOLEAN),
            CustomQueryField(PvcTbsMember.number_of_votes, _('Number of votes'), key='number_of_votes', is_filter=True, type=DATA_TYPE_INT),
            CustomQueryField(PvcTbsMember.was_member_in_2013, _('2013 Member'), key='was_member_in_2013', is_filter=True, type=DATA_TYPE_BOOLEAN),
            CustomQueryField(PvcTbsMember.selling_technique, _('Selling technique'), key='selling_technique'),
            CustomQueryField(PvcTbsMember.internet_sales_countries, _('Internet sales countries'), key='selling_technique_countries'),
            CustomQueryField(PvcTbsMember.status, _('Status'), key='status', is_filter=True, filter_options=[{
                'value': PvcTbsMember.STATUS_INCOMPLETE,
                'label': _("Incomplete"),
            },{
                'value': PvcTbsMember.STATUS_PENDING,
                'label': _("Pending"),
            },{
                'value': PvcTbsMember.STATUS_APPROVED,
                'label': _("Approved"),
            },{
                'value': PvcTbsMember.STATUS_REJECTED,
                'label': _("Rejected"),
            },{
                'value': PvcTbsMember.STATUS_SUSPENDED,
                'label': _("Suspended"),
            },{
                'value': PvcTbsMember.STATUS_RESIGNING,
                'label': _("Resigning"),
            },{
                'value': PvcTbsMember.STATUS_RESIGNED,
                'label': _("Resigned"),
            }]),
            CustomQueryField(PvcMemberLicense.is_active, _('Active'), key='is_active', is_filter=True, type=DATA_TYPE_BOOLEAN),
            CustomQueryField(func.concat_ws(' ', Person.first_name, Person.middle_name, Person.last_name), _('Contact'), key='contact'),
            CustomQueryField(TempCompany.is_temp, _('Company details change'), key='master_data', is_filter=True, type=DATA_TYPE_BOOLEAN),
            CustomQueryField(PvcTbsMember.created, _('Created'), key='created', is_filter=True),
            CustomQueryField(PvcTbsMember.last_updated, _('Last updated'), key='last_updated', is_filter=True),
            # get current service level
        )

    def set_value_processors(self):

        def proc(row, key):
            value = getattr(row, key)
            if value is None:
                return False
            rv = []
            if (value & PvcTbsMember.SELLING_TECHNIQUE_MANUFACTURE_AND_SELL) == PvcTbsMember.SELLING_TECHNIQUE_MANUFACTURE_AND_SELL:
                rv.append(_('Manufacturing and selling'))
            if (value & PvcTbsMember.SELLING_TECHNIQUE_DISTRIBUTE_OR_RESELL_REBRANDED) == PvcTbsMember.SELLING_TECHNIQUE_DISTRIBUTE_OR_RESELL_REBRANDED:
                rv.append(_('Distributing/reselling rebranded products'))
            if (value & PvcTbsMember.SELLING_TECHNIQUE_IMPORTING) == PvcTbsMember.SELLING_TECHNIQUE_IMPORTING:
                rv.append(_('Importing'))
            if (value & PvcTbsMember.SELLING_TECHNIQUE_INTERNET_SALES_OWN_COUNTRY) == PvcTbsMember.SELLING_TECHNIQUE_INTERNET_SALES_OWN_COUNTRY:
                rv.append(_('Internet selling in my country'))
            if (value & PvcTbsMember.SELLING_TECHNIQUE_INTERNET_SALES_OTHER_COUNTRIES) == PvcTbsMember.SELLING_TECHNIQUE_INTERNET_SALES_OTHER_COUNTRIES:
                rv.append(_('Internet selling in other EU countries'))

            return ', '.join(rv)

        self._value_processors['selling_technique'] = proc

    def set_where(self):
        self.where = PvcTbsMember.tbs_id.in_([x.id for x in self._get_take_back_schemes(permissions=['pvc_tbs_member_manager'])])
        self.where = and_(self.where, PvcTbsMember.archived == False)

    group_by = (PvcTbsMember.member_id, PvcTbsMember.tbs_id, )

ContactPerson = aliased(Person)

class MemberProgressCustomQuery(AbstractPvcCustomQuery):
    permissions = ['pvc_tbs_member_manager', 'pvc_member_license'] # OR

    def _get_member_column(self):
        return PvcTbsMember.member_id

    def validate_access(self):
        self.assert_any_permissions(*self.permissions)

    def tables(self):
        return (
            CustomQueryModel(PvcMemberLicense),
            CustomQueryModel(PvcTbsMember, onclause=PvcMemberLicense.id==PvcTbsMember.member_id, use_outerjoin=True),
            CustomQueryModel(Tbs, onclause=PvcTbsMember.tbs_id==Tbs.id, use_outerjoin=True),
            CustomQueryModel(Company, onclause=PvcMemberLicense.company_id==Company.id, use_outerjoin=True),
            CustomQueryModel(ContactPerson, onclause=Company.contact_person_id==ContactPerson.id, use_outerjoin=True),
            CustomQueryModel(PvcTbsMemberPVModule, onclause=PvcTbsMember.id==PvcTbsMemberPVModule.tbs_member_id, use_outerjoin=True),
        )

    def fields(self):
        return (
            CustomQueryField(PvcTbsMember.id, 'ID', key='id'),
            CustomQueryField(PvcMemberLicense.id, 'MemberID', key='member_id'),
            CustomQueryField(PvcMemberLicense.name, _('Member'), key='member', is_filter=True),
            CustomQueryField(Company.vat_no, _('VAT no.'), key='vat_no', is_filter=True),
            CustomQueryField(Tbs.name, _('TBS'), key='tbs', is_filter=True, is_group_by=True),
            CustomQueryField(PvcTbsMember.obligation_start_date, _('Obligation start date'), key='obligation_start_date'),
            CustomQueryField(PvcTbsMember.id, _('Service level'), key='service_level'),
            CustomQueryField(PvcTbsMember.status, _('Status'), key='status', is_filter=True, filter_options=[{
                'value': PvcTbsMember.STATUS_INCOMPLETE,
                'label': _("Incomplete"),
            },{
                'value': PvcTbsMember.STATUS_PENDING,
                'label': _("Pending"),
            },{
                'value': PvcTbsMember.STATUS_APPROVED,
                'label': _("Approved"),
            },{
                'value': PvcTbsMember.STATUS_REJECTED,
                'label': _("Rejected"),
            },{
                'value': PvcTbsMember.STATUS_SUSPENDED,
                'label': _("Suspended"),
            },{
                'value': PvcTbsMember.STATUS_RESIGNING,
                'label': _("Resigning"),
            },{
                'value': PvcTbsMember.STATUS_RESIGNED,
                'label': _("Resigned"),
            }]),
            CustomQueryField(PvcTbsMember.company_registration_number, _('(W)EEE-Register Number'), key='company_registration_number'),
            CustomQueryField(PvcTbsMember.selling_technique, _('Selling technique'), key='selling_technique'),
            CustomQueryField(Company.id, _('Company data'), key='company'),
            CustomQueryField(ContactPerson.id, _('Contact data'), key='contact'),
            CustomQueryField(Company.id, _('Invoice data'), key='invoice'),
            CustomQueryField(Tbs.id, _('Applied at TBS'), key='applied_at_tbs', is_filter=True, type=DATA_TYPE_BOOLEAN),
            CustomQueryField(PvcTbsMember.is_producer, _('Producer'), key='producer', is_filter=True, type=DATA_TYPE_BOOLEAN),
            CustomQueryField(PvcTbsMember.is_commissioner, _('Commissioner'), key='commissioner', is_filter=True, type=DATA_TYPE_BOOLEAN),
            CustomQueryField(func.count(PvcTbsMemberPVModule.id), _('PV Modules'), key='pv_modules_selected'),
            CustomQueryField(PvcTbsMember.id, _('Contract'), key='contract'),
        )

    def set_value_processors(self):

        def selling_technique_proc(row, key):
            value = getattr(row, key)
            if value is None:
                return False
            rv = []
            if (value & PvcTbsMember.SELLING_TECHNIQUE_MANUFACTURE_AND_SELL) == PvcTbsMember.SELLING_TECHNIQUE_MANUFACTURE_AND_SELL:
                rv.append(_('Manufacturing and selling'))
            if (value & PvcTbsMember.SELLING_TECHNIQUE_DISTRIBUTE_OR_RESELL_REBRANDED) == PvcTbsMember.SELLING_TECHNIQUE_DISTRIBUTE_OR_RESELL_REBRANDED:
                rv.append(_('Distributing/reselling rebranded products'))
            if (value & PvcTbsMember.SELLING_TECHNIQUE_IMPORTING) == PvcTbsMember.SELLING_TECHNIQUE_IMPORTING:
                rv.append(_('Importing'))
            if (value & PvcTbsMember.SELLING_TECHNIQUE_INTERNET_SALES_OWN_COUNTRY) == PvcTbsMember.SELLING_TECHNIQUE_INTERNET_SALES_OWN_COUNTRY:
                rv.append(_('Internet selling in my country'))
            if (value & PvcTbsMember.SELLING_TECHNIQUE_INTERNET_SALES_OTHER_COUNTRIES) == PvcTbsMember.SELLING_TECHNIQUE_INTERNET_SALES_OTHER_COUNTRIES:
                rv.append(_('Internet selling in other EU countries'))

            return ', '.join(rv)

        self._value_processors['selling_technique'] = selling_technique_proc

        def service_level_proc(row, key):
            sl_id = getattr(row, key)
            if sl_id is None:
                return None
            tbs_member_id = int(sl_id)
            tbs_member = self.sa_session.query(PvcTbsMember).filter(PvcTbsMember.id == tbs_member_id).one()

            sl = tbs_member.active_service_level
            if sl is None:
                return None

            rv = []

            if sl.service_level is not None:
                rv.append(sl.service_level.name)

            if sl.pv_service_level is not None:
                rv.append(sl.pv_service_level.name)

            return ' / '.join(rv)

        self._value_processors['service_level'] = service_level_proc

    def set_where(self):
        self.where = PvcMemberLicense.id.in_([x.id for x in self._get_member_licenses()])
        if not self.has_permissions('view_domain') and self.has_permissions('pvc_tbs_member_manager'):
            self.where = and_(self.where, PvcTbsMember.tbs_id.in_([x.id for x in self._get_take_back_schemes(permissions=['pvc_tbs_member_manager'])]))

    group_by = (PvcTbsMember.member_id, PvcTbsMember.tbs_id, )

class MemberAccountCustomQuery(AbstractPvcCustomQuery):
    permissions = ['pvc_tbs_member_manager']

    def _get_member_column(self):
        return PvcTbsMember.member_id

    def tables(self):
        return (
            CustomQueryModel(Account),
            CustomQueryModel(GroupAccount, onclause=Account.id==GroupAccount.account_id),
            CustomQueryModel(Group, onclause=GroupAccount.group_id==Group.id),
            CustomQueryModel(PvcMemberLicense),
            CustomQueryModel(PvcTbsMember, onclause=PvcMemberLicense.id==PvcTbsMember.member_id, use_outerjoin=True),
            CustomQueryModel(Tbs, onclause=PvcTbsMember.tbs_id==Tbs.id, use_outerjoin=True),
            CustomQueryModel(Person, onclause=Account.person_id==Person.id, use_outerjoin=True),
        )

    def fields(self):
        return (
            CustomQueryField(Tbs.name, _('TBS'), key='tbs', is_filter=True, is_group_by=True),
            CustomQueryField(PvcMemberLicense.name, _('Member'), key='member', is_filter=True),
            CustomQueryField(Account.name, _('Account name'), key='account_name', is_filter=True),
            CustomQueryField(func.concat(
                case([(Person.first_name!=None, Person.first_name)], else_=""),
                " ",
                case([(Person.middle_name!=None, Person.middle_name)], else_=""),
                " ",
                case([(Person.last_name!=None, Person.last_name)], else_=""),
                ), _('Name'), key='person_name'),
            CustomQueryField(Account.name, _('Switch to user'), key='switch_to_user'),
            CustomQueryField(Person.phone, _('Phone'), key="phone"),
            CustomQueryField(Person.email, _('Email'), key="email", is_filter=True),
            CustomQueryField(GroupAccount.active, _('Active'), key='active'),
            CustomQueryField(PvcTbsMember.status, _('Status'), key='status', is_filter=True, filter_options=[{
                'value': PvcTbsMember.STATUS_INCOMPLETE,
                'label': _("Incomplete"),
            },{
                'value': PvcTbsMember.STATUS_PENDING,
                'label': _("Pending"),
            },{
                'value': PvcTbsMember.STATUS_APPROVED,
                'label': _("Approved"),
            },{
                'value': PvcTbsMember.STATUS_REJECTED,
                'label': _("Rejected"),
            },{
                'value': PvcTbsMember.STATUS_SUSPENDED,
                'label': _("Suspended"),
            },{
                'value': PvcTbsMember.STATUS_RESIGNING,
                'label': _("Resigning"),
            },{
                'value': PvcTbsMember.STATUS_RESIGNED,
                'label': _("Resigned"),
            }]),
        )

    def set_where(self):
        self.where = and_(Account.archived==0, GroupAccount.active==1)
        self.where = and_(self.where, PvcMemberLicense.id.in_([x.id for x in self._get_member_licenses()]))
        if not self.has_permissions('view_domain') and self.has_permissions('pvc_tbs_member_manager'):
            self.where = and_(self.where, PvcTbsMember.tbs_id.in_([x.id for x in self._get_take_back_schemes(permissions=['pvc_tbs_member_manager'])]))


class CertificateDocumentCustomQuery(SQLCustomQuery, AbstractPvcController):
    def tables(self):
        return (
            SQLCustomQueryTable('tblPvcTbsMemberCertificate', 'cert'),
            SQLCustomQueryTable('tblPvcTbsMember', 'tbs_member', onclause=[('cert', 'tbs_member_id', 'tbs_member', 'id', )]),
            SQLCustomQueryTable('tblLicense', 'tbs', onclause=[('tbs_member', 'tbs_id', 'tbs', 'id', )]),
            SQLCustomQueryTable('tblLicense', 'member', onclause=[('tbs_member', 'member_id', 'member', 'id', )]),
        )

    def fields(self):
        return (
            SQLCustomQueryField('tbs', 'name', _('TBS'), key='tbs', is_filter=True),
            SQLCustomQueryField('member', 'name', _('Member'), key='member', is_filter=True),
            SQLCustomQueryField('cert', 'hash', 'object_id', key='object_id'),
            SQLCustomQueryField(None, '"certificate"', 'Type', key='type'),
            SQLCustomQueryField(None, 'CONCAT("%s ",`cert`.`year`)' % _('Certificate for year'), _('Description'), key='description'),
            SQLCustomQueryField(None, '"%s"' % _('Certificate'), _('Document type'), key='document_type'),
            SQLCustomQueryField(None, '"%s"' % _('Download'), _('Download'), key='download'),
        )

    def set_where(self):
        if self.has_permissions('pvc_member_license'):
            self.where.append('`member`.`id` IN (%s)' % ', '.join([str(x.id) for x in self._get_member_licenses()]))
        else:
            self.where.append('`tbs`.`id` IN (%s)' % ', '.join([str(x.id) for x in self._get_take_back_schemes(permissions=['pvc_tbs_member_manager'])]))


class ContractDocumentCustomQuery(SQLCustomQuery, AbstractPvcController):
    def tables(self):
        return (
            SQLCustomQueryTable('tblPvcTbsMemberContractFile', 'contract'),
            SQLCustomQueryTable('tblPvcTbsMember', 'tbs_member', onclause=[('contract', 'tbs_member_id', 'tbs_member', 'id', )]),
            SQLCustomQueryTable('tblLicense', 'tbs', onclause=[('tbs_member', 'tbs_id', 'tbs', 'id', )]),
            SQLCustomQueryTable('tblLicense', 'member', onclause=[('tbs_member', 'member_id', 'member', 'id', )]),
            SQLCustomQueryTable('tblFile', 'file', onclause=[('contract', 'file_id', 'file', 'id', )]),
        )

    def fields(self):
        return (
            SQLCustomQueryField('tbs', 'name', _('TBS'), key='tbs', is_filter=True),
            SQLCustomQueryField('member', 'name', _('Member'), key='member', is_filter=True),
            SQLCustomQueryField('file', 'file_hash', 'object_id', key='object_id'),
            SQLCustomQueryField(None, '"contract"', 'type', key='type'),
            SQLCustomQueryField(None, '"-"', _('Description'), key='description'),
            SQLCustomQueryField(None, '"%s"' % _('Contract'), _('Document type'), key='document_type'),
            SQLCustomQueryField(None, '"%s"' % _('Download'), _('Download'), key='download'),
        )

    def set_where(self):
        if self.has_permissions('pvc_member_license'):
            self.where.append('`member`.`id` IN (%s)' % ', '.join([str(x.id) for x in self._get_member_licenses()]))
        else:
            self.where.append('`tbs`.`id` IN (%s)' % ', '.join([str(x.id) for x in self._get_take_back_schemes(permissions=['pvc_tbs_member_manager'])]))


class SubrogationDocumentCustomQuery(SQLCustomQuery, AbstractPvcController):

    def tables(self):
        return (
            SQLCustomQueryTable('tblPvcSubrogation', 'sub'),
            SQLCustomQueryTable('tblPvcTbsMember', 'tbs_member', onclause=[('sub', 'subrogator_id', 'tbs_member', 'id', )]),
            SQLCustomQueryTable('tblLicense', 'tbs', onclause=[('tbs_member', 'tbs_id', 'tbs', 'id', )]),
            SQLCustomQueryTable('tblLicense', 'member', onclause=[('tbs_member', 'member_id', 'member', 'id', )]),
            SQLCustomQueryTable('tblPvcTbsMember', 'tbs_member_com', onclause=[('sub', 'commissioner_id', 'tbs_member_com', 'id', )]),
            SQLCustomQueryTable('tblLicense', 'commissioner', onclause=[('tbs_member_com', 'member_id', 'commissioner', 'id', )]),
            SQLCustomQueryTable('tblFile', 'file', onclause=[('sub', 'mandate_file_id', 'file', 'id', )]),
        )

    def fields(self):
        return (
            SQLCustomQueryField('tbs', 'name', _('TBS'), key='tbs', is_filter=True),
            SQLCustomQueryField('member', 'name', _('Member'), key='member', is_filter=True),
            SQLCustomQueryField('file', 'file_hash', 'object_id', key='object_id'),
            SQLCustomQueryField(None, '"mandate"', 'type', key='type'),
            SQLCustomQueryField(None, 'CONCAT("%s ",`commissioner`.`name`)' % _('Subrogation with commissioner'), _('Description'), key='description'),
            SQLCustomQueryField(None, '"%s"' % _('Mandate'), _('Document type'), key='document_type'),
            SQLCustomQueryField(None, '"%s"' % _('Download'), _('Download'), key='download'),
        )

    def set_where(self):
        if self.has_permissions('pvc_member_license'):
            self.where.append('`member`.`id` IN (%s)' % ', '.join([str(x.id) for x in self._get_member_licenses()]))
        else:
            self.where.append('`tbs`.`id` IN (%s)' % ', '.join([str(x.id) for x in self._get_take_back_schemes(permissions=['pvc_tbs_member_manager'])]))


class FinancialGuaranteeDocumentCustomQuery(SQLCustomQuery, AbstractPvcController):
    def tables(self):
        return (
            SQLCustomQueryTable('tblPvcTbsMember', 'tbs_member'),
            SQLCustomQueryTable('tblLicense', 'tbs', onclause=[('tbs_member', 'tbs_id', 'tbs', 'id', )]),
            SQLCustomQueryTable('tblLicense', 'member', onclause=[('tbs_member', 'member_id', 'member', 'id', )]),
            SQLCustomQueryTable('tblFile', 'file', onclause=[('tbs_member', 'guarantee_file_id', 'file', 'id', )]),
        )

    def fields(self):
        return (
            SQLCustomQueryField('tbs', 'name', _('TBS'), key='tbs', is_filter=True),
            SQLCustomQueryField('member', 'name', _('Member'), key='member', is_filter=True),
            SQLCustomQueryField('file', 'file_hash', 'object_id', key='object_id'),
            SQLCustomQueryField(None, '"contract"', 'type', key='type'),
            SQLCustomQueryField(None, '"-"', _('Description'), key='description'),
            SQLCustomQueryField(None, '"%s"' % _('Financial Guarantee'), _('Document type'), key='document_type'),
            SQLCustomQueryField(None, '"%s"' % _('Download'), _('Download'), key='download'),
        )

    def set_where(self):
        if self.has_permissions('pvc_member_license'):
            self.where.append('`member`.`id` IN (%s)' % ', '.join([str(x.id) for x in self._get_member_licenses()]))
        else:
            self.where.append('`tbs`.`id` IN (%s)' % ', '.join([str(x.id) for x in self._get_take_back_schemes(permissions=['pvc_tbs_member_manager'])]))


class DocumentCustomQuery(SQLCustomQueryUnion, AbstractPvcController):
    def custom_queries(self):
        return (
            CertificateDocumentCustomQuery,
            ContractDocumentCustomQuery,
            SubrogationDocumentCustomQuery,
            FinancialGuaranteeDocumentCustomQuery,
        )


custom_queries = (
    AccountantCustomQuery,
    TbsCustomQuery,
    TbsServiceLevelManager,
    TbsMembershipManager,
    TbsCertificateManager,
    MemberCertificateManager,
    MemberCustomQuery,
    MemberProgressCustomQuery,
    MemberAccountCustomQuery,
    DocumentCustomQuery,
)
