# Copyright (c) 2013 BaseSoft B.V.
# See LICENSE for details.

from basesoft.web.contrib.dashboard.controllers import DashboardWidgetCQStateController
from django.utils.translation import ugettext_lazy as _
from basesoft.util.ordered_dict import OrderedDict

class PVCAccountantOverviewWidget(DashboardWidgetCQStateController):
    """
    Accountant Overview Widget
    """
    label = _('Accountant manager')
    name = 'pvc_db_accountant_overview_widget'
    category = _('PVC Management')
    permissions = ['pvc_accountant_license']

class PVCTbsOverviewWidget(DashboardWidgetCQStateController):
    """
    TBS Overview Widget
    """
    label = _('TBS manager')
    name = 'pvc_db_tbs_overview_widget'
    category = _('PVC Management')
    permissions = ['pvc_tbs_license']

class PVCTbsServiceLevelOverviewWidget(DashboardWidgetCQStateController):
    label = _('TBS service level manager')
    name = 'pvc_db_tbs_service_level_widget'
    category = _('PVC Management')
    permissions = ['pvc_tbs_service_level']

class PVCMemberOverviewWidget(DashboardWidgetCQStateController):
    label = _('Member manager')
    name = 'pvc_db_member_overview_widget'
    category = _('PVC Management')
    permissions = ['pvc_tbs_member_manager']

class PVCMemberAccountManagerWidget(DashboardWidgetCQStateController):
    label = _('Member account manager')
    name = 'pvc_member_account_widget'
    category = _('PVC Management')
    permissions = ['pvc_tbs_member_manager']

class PVCMemberProgressWidget(DashboardWidgetCQStateController):
    """
    Widget with TODO's for new Members and their state.
    """
    label = _('Registration progress')
    name = 'pvc_db_member_progress_widget'
    category = _('PVC Member')
    permissions = ['pvc_member_license', 'pvc_tbs_member_manager'] # OR

    def has_permissions(self, controller):
        # Operator should be able to place this on dashboard for member
        return controller.has_any_permissions(*self.permissions)

class PVCTodayWidget(DashboardWidgetCQStateController):
    label = _('Today widget')
    name = 'pvc_db_today_widget'
    category = _('PVC Member')
    permissions = ['pvc_member_license', 'pvc_tbs_license'] # OR

    def has_permissions(self, controller):
        # Operator should be able to place this on dashboard for member
        return controller.has_any_permissions(*self.permissions)

class PVCMemberCertificateWidget(DashboardWidgetCQStateController):
    label = _('Certificate overview')
    name = 'pvc_db_member_certificate_widget'
    category = _('PVC Member')
    permissions = ['pvc_member_license', 'pvc_tbs_license'] # OR

    def has_permissions(self, controller):
        # Operator should be able to place this on dashboard for member
        return controller.has_any_permissions(*self.permissions)

class PVCTbsCertificateWidget(DashboardWidgetCQStateController):
    label = _('TBS certificate overview')
    name = 'pvc_db_tbs_certificate_widget'
    category = _('PVC Management')
    permissions = ['pvc_manage_certificates']


class DocumentWidget(DashboardWidgetCQStateController):
    label = _('Document overview')
    name = 'pvc_db_document_widget'
    category = _('PVC Management')
    permissions = ['pvc_member_license', 'pvc_tbs_license', 'pvc_tbs_member_manager']

    def has_permissions(self, controller):
        # Operator should be able to place this on dashboard for member
        return controller.has_any_permissions(*self.permissions)

widgets_mapper = OrderedDict([
    ('PVCAccountantOverviewWidget', PVCAccountantOverviewWidget, ),
    ('PVCTbsOverviewWidget', PVCTbsOverviewWidget, ),
    ('PVCMemberOverviewWidget', PVCMemberOverviewWidget, ),
    ('PVCTbsServiceLevelOverviewWidget', PVCTbsServiceLevelOverviewWidget, ),
    ('PVCMemberProgressWidget', PVCMemberProgressWidget, ),
    ('PVCTodayWidget', PVCTodayWidget, ),
    ('PVCMemberCertificateWidget', PVCMemberCertificateWidget, ),
    ('PVCTbsCertificateWidget', PVCTbsCertificateWidget, ),
    ('PVCMemberAccountManagerWidget', PVCMemberAccountManagerWidget, ),
    ('DocumentWidget', DocumentWidget, ),
])
