# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.
from django.conf import settings

from basesoft.web.contrib.branding.models import Site, Theme
from basesoft.web.contrib.license.models import Domain, Role
from sqlalchemy.orm.exc import NoResultFound
from basesoft.web.contrib.branding.db_sync import _sync_themes


def post_sync(sa_session, app_name):
    """
    when pvcycle is loaded, syncdb updates the default site to pvcycle
    """
    if app_name == 'basesoft.web.contrib.branding':

        # sync themes only after color_sets have been created
        themes = [{
            'name': u'PV CYCLE',
            'title': u'PV CYCLE',
            'module_name': u'pvcycle_theme_default',
            'css_class': u'pvcycle',
            'licenses': [],
            'default_graph_theme': u'Default',
        }]
        _sync_themes(sa_session, themes)

        #
        # setup existing default Site object with pvcycle domain/license settings
        #

        # get/create pvc domain
        try:
            domain = sa_session.query(Domain).filter(Domain.name == 'PV CYCLE').one()
        except NoResultFound:
            domain = Domain()
            domain.name = 'PV CYCLE'
            domain.settings = 0
        domain.settings |= Domain.SETTING_REQUIRE_PERSON_DETAILS_ON_ACCOUNT_ACTIVATION
        domain.settings |= Domain.SETTING_REQUIRE_EULA_ON_REGISTRATION
        # when a user registers for the domain he becomes the admin for the new member
        domain.registrant_role = sa_session.query(Role).filter(Role.name == 'PVC Simple Member').one()
        sa_session.add(domain)

        if settings.DEBUG:
            # set new public home app
            site = sa_session.query(Site).filter(Site.host_match=='').one()
        else:
            try:
                site = sa_session.query(Site).filter(Site.host_match=='.*pvcycle.*').one()
            except NoResultFound:
                site = Site()
                site.host_match = '.*pvcycle.*'
                site.order = 998
                site.trac_conn_str = 'http://'

        site.home_app = 'pvcycle'
        site.domain = domain
        site.theme = sa_session.query(Theme).filter(Theme.name=='PV CYCLE').one();
        sa_session.add(site)
        sa_session.flush()

def procedures():

    # Function pvcGetCurrentFee
    # Returns the current Fee for the given service_level

    return [

        """
        DROP FUNCTION IF EXISTS `pvcGetCurrentAdministrationFee`
        """,
        """
        CREATE FUNCTION `pvcGetCurrentAdministrationFee`(
            in_service_level_id INT,
            in_start_year YEAR(4) )
            RETURNS DECIMAL(10,5)
            READS SQL DATA
        BEGIN
            DECLARE fee DECIMAL(10,5);
            SET fee = NULL;

            SELECT f.fee INTO fee
            FROM tblPvcTbsServiceLevelAdministrationFee f
            WHERE
                f.`service_level_id` = in_service_level_id AND
                f.start_year <= in_start_year
            ORDER BY f.`start_year` DESC
            LIMIT 1;

            RETURN(fee);

        END
        """,

        """
        DROP FUNCTION IF EXISTS `pvcGetCurrentContributionFee`
        """,
        """
        CREATE FUNCTION `pvcGetCurrentContributionFee`(
            in_service_level_id INT,
            in_start_date DATE )
            RETURNS DECIMAL(10,5)
            READS SQL DATA
        BEGIN
            DECLARE fee DECIMAL(10,5);
            SET fee = NULL;

            SELECT f.fee INTO fee
            FROM tblPvcTbsServiceLevelContributionFee f
            WHERE
                f.`service_level_id` = in_service_level_id AND
                f.start_date <= in_start_date
            ORDER BY f.`start_date` DESC
            LIMIT 1;

            RETURN(fee);

        END
        """,

        """
        DROP FUNCTION IF EXISTS `pvcGetCurrentServiceLevelId`
        """,
        """
        CREATE FUNCTION `pvcGetCurrentServiceLevelId`(
            in_tbs_member_id INT,
            in_start_date DATE )
            RETURNS INT
            READS SQL DATA
        BEGIN
            DECLARE service_level_id INT;
            SET service_level_id = NULL;

            SELECT service_level_id INTO service_level_id
            FROM tblPvcTbsMemberServiceLevel sl
            WHERE
                sl.`tbs_member_id` = in_tbs_member_id AND
                sl.start_date <= in_start_date
            ORDER BY sl.`start_date` DESC
            LIMIT 1;

            RETURN(service_level_id);

        END
        """,
        """
        DROP FUNCTION IF EXISTS `pvcGetCurrentServiceLevel`
        """,
        """
        CREATE FUNCTION `pvcGetCurrentServiceLevel`(
            in_tbs_member_id INT,
            in_start_date DATE )
            RETURNS VARCHAR(255)
            READS SQL DATA
        BEGIN
            DECLARE service_level VARCHAR(255);
            SET service_level = NULL;

            SELECT s.name INTO service_level
            FROM tblPvcTbsServiceLevel s
            INNER JOIN tblPvcTbsMemberServiceLevel sl
                ON s.id = sl.service_level_id
            WHERE
                sl.`tbs_member_id` = in_tbs_member_id AND
                sl.start_date <= in_start_date
            ORDER BY sl.`start_date` DESC
            LIMIT 1;

            RETURN(service_level);

        END
        """,

        """
        DROP FUNCTION IF EXISTS `pvcGetCurrentPvServiceLevel`
        """,
        """
        CREATE FUNCTION `pvcGetCurrentPvServiceLevel`(
            in_tbs_member_id INT,
            in_start_date DATE )
            RETURNS VARCHAR(255)
            READS SQL DATA
        BEGIN
            DECLARE service_level VARCHAR(255);
            SET service_level = NULL;

            SELECT s.name INTO service_level
            FROM tblPvcTbsServiceLevel s
            INNER JOIN tblPvcTbsMemberServiceLevel sl
                ON s.id = sl.pv_service_level_id
            WHERE
                sl.`tbs_member_id` = in_tbs_member_id AND
                sl.start_date <= in_start_date
            ORDER BY sl.`start_date` DESC
            LIMIT 1;

            RETURN(service_level);

        END
        """,

        """
        DROP FUNCTION IF EXISTS `pvcGetCurrentMembershipFee`
        """,
        """
        CREATE FUNCTION `pvcGetCurrentMembershipFee`(
            in_membership_id INT,
            in_start_year YEAR(4) )
            RETURNS DECIMAL(10,5)
            READS SQL DATA
        BEGIN
            DECLARE fee DECIMAL(10,5);
            SET fee = NULL;

            SELECT f.fee INTO fee
            FROM tblPvcTbsMembershipFee f
            WHERE
                f.`membership_id` = in_membership_id AND
                f.start_year <= in_start_year
            ORDER BY f.`start_year` DESC
            LIMIT 1;

            RETURN(fee);

        END
        """,

        """
        DROP FUNCTION IF EXISTS `pvcGetCurrentMembership`
        """,
        """
        CREATE FUNCTION `pvcGetCurrentMembership`(
            in_tbs_member_id INT,
            in_start_date DATE )
            RETURNS VARCHAR(255)
            READS SQL DATA
        BEGIN
            DECLARE membership VARCHAR(255);
            SET membership = NULL;

            SELECT s.name INTO membership
            FROM tblPvcTbsMembership s
            INNER JOIN tblPvcTbsMemberMembership sl
                ON s.id = sl.tbs_membership_id
            WHERE
                sl.`tbs_member_id` = in_tbs_member_id AND
                sl.start_date <= in_start_date
            ORDER BY sl.`start_date` DESC
            LIMIT 1;

            RETURN(membership);

        END
        """,

        """
        DROP FUNCTION IF EXISTS `pvcGetCurrentMembershipStartYear`
        """,
        """
        CREATE FUNCTION `pvcGetCurrentMembershipStartYear`(
            in_membership_id INT,
            in_start_year YEAR(4) )
            RETURNS INT
            READS SQL DATA
        BEGIN
            DECLARE current_start_year INT;
            SET current_start_year = NULL;

            SELECT f.start_year INTO current_start_year
            FROM tblPvcTbsMembershipFee f
            WHERE
                f.`membership_id` = in_membership_id AND
                f.start_year <= in_start_year
            ORDER BY f.`start_year` DESC
            LIMIT 1;

            RETURN(current_start_year);

        END
        """,

    ]
