# Copyright (c) 2013 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext_lazy as _

from basesoft.web.contrib.account.decorators import assert_user_logged_in
from basesoft.web.contrib.iconbar.models import IconBarIcon
from basesoft.web.core.exceptions import SecurityException

from pvcycle.core.models import PvcTbsLicense, PvcAccountantLicense, PvcMemberLicense, PvcOperatorLicense

class AbstractPvcIcon(IconBarIcon):
    def _assert_pvc_license(self, controller, _types=None):
        _license = controller.get_default_group().license
        if _types is None:
            _types = [PvcOperatorLicense, PvcTbsLicense, PvcAccountantLicense, PvcMemberLicense]

        valid = False
        for t in _types:
            if isinstance(_license, t):
                valid = True
                break

        if not valid:
            raise SecurityException('Not a valid PV CYCLE License')

class PvcAccountantManagerIcon(AbstractPvcIcon):

    id = 'icon-pvc-accountant-manager'
    title = _('Accountants manager')
    component = 'pvc_accountant_manager'
    group = 'pvc'
    group_title = _('PVC Management')
    colorname = 'iconbar-denim'

    char_main = u'\uE813' # Suitcase icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('pvc_accountant_license')
        self._assert_pvc_license(controller)


class PvcTbsManagerIcon(AbstractPvcIcon):

    id = 'icon-pvc-tbs-manager'
    title = _('TBS manager')
    component = 'pvc_tbs_manager'
    group = 'pvc'
    group_title = _('PVC Management')
    colorname = 'iconbar-denim'

    char_main = u'\uE814' # Arrows recycle icon
    char_sub = u'\uE810' # File

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('pvc_tbs_license')
        self._assert_pvc_license(controller)


class PvcROECountryIcon(AbstractPvcIcon):
    id = 'icon-pvc-roe-manager'
    title = _('Rest of Europe countries')
    component = 'pvc_roe_country_details'
    group = 'pvc'
    group_title = _('PVC Management')
    colorname = 'iconbar-denim'

    char_main = u'\uE800'

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('pvc_tbs_license')
        self._assert_pvc_license(controller)


class PvcTbsServiceLevelIcon(AbstractPvcIcon):

    id = 'icon-pvc-tbs-service-level'
    title = _('TBS service level manager')
    icon_index = 75
    component = 'pvc_tbs_service_level_manager'
    group = 'pvc'
    group_title = _('PVC Management')
    colorname = 'iconbar-denim'

    char_main = u'\uE815' # Shield icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('pvc_tbs_service_level')
        self._assert_pvc_license(controller)

class PvcTbsMembershipLevelIcon(AbstractPvcIcon):

    id = 'icon-pvc-tbs-membership-level'
    title = _('TBS membership level manager')
    icon_index = 76
    component = 'pvc_tbs_membership_manager'
    group = 'pvc'
    group_title = _('PVC Management')
    colorname = 'iconbar-denim'

    char_main = u'\uE815' # Shield icon
    char_sub = u'\uE803' # User

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('pvc_tbs_membership_level')
        self._assert_pvc_license(controller)

class PvcTbsCertificateManager(AbstractPvcIcon):

    id = 'icon-pvc-tbs-certificate-manager'
    title = _('TBS certificate manager')
    icon_index = 75
    component = 'pvc_tbs_certificate_manager'
    group = 'pvc'
    group_title = _('PVC Management')
    colorname = 'iconbar-denim'

    char_main = u'\uE810' # Recycle icon
    char_sub = u'\uE83E' # Closed Certificate icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('pvc_manage_certificates')
        self._assert_pvc_license(controller)

class PvcMemberManagerIcon(AbstractPvcIcon):

    id = 'icon-pvc-member-manager'
    title = _('Member manager')
    component = 'pvc_member_manager'
    group = 'pvc'
    group_title = _('PVC Management')
    colorname = 'iconbar-denim'

    char_main = u'\uE803' # Users icon
    char_sub = u'\uE816' # + icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('pvc_tbs_member_manager')
        self._assert_pvc_license(controller)

class PvcMemberAccountManagerIcon(AbstractPvcIcon):

    id = 'icon-pvc-member-account-manager'
    title = _('Member account manager')
    component = 'pvc_member_account_manager'
    group = 'pvc'
    group_title = _('PVC Management')
    colorname = 'iconbar-denim'

    char_main = u'\uE803' # Users icon
    char_sub = u'\uE816' # + icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('pvc_tbs_member_manager')
        self._assert_pvc_license(controller)

class PvcMemberRegistrationIcon(AbstractPvcIcon):

    id = 'icon-pvc-member-registration-progress'
    title = _('Registration progress')
    component = 'pvc_member_progress_manager'
    group = 'pvc'
    group_title = _('PVC Management')
    colorname = 'iconbar-denim'

    char_main = u'\uE806' # Man icon
    char_sub = u'\uE832' # Numebred List icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('pvc_member_license')
        self._assert_pvc_license(controller)

class PvcMemberApplyAtTBS(AbstractPvcIcon):

    id = 'icon-pvc-member-apply-at-tbs'
    title = _('Register at new TBS')
    component = 'pvc_member_apply_at_tbs'
    group = 'pvc'
    group_title = _('PVC Management')
    colorname = 'iconbar-denim'
    icon_index = 76

    char_main = u'\uE814' # Arrows recycle icon
    char_sub = u'\uE806' # Man icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('pvc_member_license')
        self._assert_pvc_license(controller)

class PvcMemberRegisterANewCompany(AbstractPvcIcon):

    id = 'icon-pvc-member-register-a-new-company'
    title = _('Register a new company')
    component = 'pvc_member_wizard'
    group = 'pvc'
    group_title = _('PVC Management')
    colorname = 'iconbar-denim'
    icon_index = 76

    char_main = u'\uE865' # Corporate icon
    char_sub = u'\uE816' # Plus icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('pvc_member_license')
        self._assert_pvc_license(controller)


class PvcMemberCertificate(AbstractPvcIcon):

    id = 'icon-pvc-member-certificate'
    title = _('Certificate Overview')
    component = 'pvc_member_certificate_manager'
    group = 'pvc'
    group_title = _('PVC Management')
    colorname = 'iconbar-denim'
    icon_index = 76

    char_main = u'\uE823' # List icon
    char_sub = u'\uE83E' # Closed Certificate icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_any_permissions('pvc_member_license', 'pvc_manage_certificates')
        self._assert_pvc_license(controller)

def get_icons():
    """
    return the list of icons that the home app adds to the iconbar
    """
    return [
        PvcAccountantManagerIcon(),
        PvcTbsManagerIcon(),
        PvcTbsServiceLevelIcon(),
        PvcTbsMembershipLevelIcon(),
        PvcMemberManagerIcon(),
        PvcMemberAccountManagerIcon(),
        PvcMemberRegistrationIcon(),
        PvcMemberApplyAtTBS(),
        PvcTbsCertificateManager(),
        PvcMemberCertificate(),
        PvcMemberRegisterANewCompany(),
        PvcROECountryIcon(),
    ]
