'''
Created on Jul 16, 2013

@author: marten
'''
import datetime

from sqlalchemy.orm import mapper, relation
from sqlalchemy import UniqueConstraint, Column, Integer, ForeignKey, DateTime, Unicode, UnicodeText, Numeric, SmallInteger, Boolean, event, Date
from sqlalchemy.dialects.mysql import MEDIUMTEXT

from basesoft.sqlalchemy.table import IQTable
from basesoft.web.contrib.license import models as license_models
from basesoft.web.contrib.file.models import File

from pvcycle.core.models import PvcTbsLicense, PvcTbsMemberContractFile, PvcTbsMember,\
    PvcTbsMemberCertificate, PvcTbsCertificate, PvcROECountry, PvcTbsServiceLevel, PvcTbsMemberServiceLevel
from pvcycle.core.hooks import tbs_member_file_listener
from pvcycle.product.models import PvcTbsProduct

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    return tables

def do_tables(metadata):
    """
    This function is called by Django middleware when needed.
    """
    global __tables_defined
    if __tables_defined:
        return

    from pvcycle.core import models as core_models

    tblPvcOperatorLicense = IQTable('tblPvcOperatorLicense', metadata,
        Column('license_id', Integer, ForeignKey('tblLicense.id', ondelete='CASCADE'), primary_key=True)
    )

    tblPvcAccountantLicense = IQTable('tblPvcAccountantLicense', metadata,
        Column('license_id', Integer, ForeignKey('tblLicense.id', ondelete='CASCADE'), primary_key=True)
    )

    tblPvcTbsLicense = IQTable('tblPvcTbsLicense', metadata,
        Column('license_id', Integer, ForeignKey('tblLicense.id', ondelete='CASCADE'), primary_key=True),
        Column('tbs_type', Integer, nullable=False),
        Column('contract_text', MEDIUMTEXT),
        Column('mandate_text', UnicodeText),
        Column('terms_and_conditions_text', UnicodeText),
        Column('country_id', Integer, ForeignKey('tblCountry.id'), nullable=True),
    )

    tblPvcTbsServiceLevel = IQTable('tblPvcTbsServiceLevel', metadata,
        Column('id', Integer, primary_key=True),
        Column('tbs_id', Integer, ForeignKey('tblPvcTbsLicense.license_id', ondelete='CASCADE')),
        Column('name', Unicode(255)),
        Column('level', Integer, nullable=False),
        Column('contract_annex_text', UnicodeText),

        Column('guarantee_forecast_min_value', Integer, nullable=True),
        Column('product_type', SmallInteger, nullable=False, server_default=str(PvcTbsProduct.TYPE_OTHER) ),

        Column('status', SmallInteger, nullable=False, default=PvcTbsServiceLevel.STATUS_ACTIVE),
        Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
        Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        UniqueConstraint('tbs_id', 'name', name='tbs_name_index'),
        UniqueConstraint('tbs_id', 'level', 'product_type', name='tbs_level_index'),
    )

    # administration fee formally known as membership fee
    tblPvcTbsServiceLevelAdministrationFee  = IQTable('tblPvcTbsServiceLevelAdministrationFee', metadata,
        Column('service_level_id', Integer, ForeignKey('tblPvcTbsServiceLevel.id', ondelete='CASCADE'), primary_key=True),
        Column('start_year', SmallInteger, primary_key=True),
        Column('fee', Numeric(10, 5)),
        Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
        Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
    )

    # contribution fee, also known as declaration fee (per tbs declaration period)
    tblPvcTbsServiceLevelContributionFee  = IQTable('tblPvcTbsServiceLevelContributionFee', metadata,
        Column('service_level_id', Integer, ForeignKey('tblPvcTbsServiceLevel.id', ondelete='CASCADE'), primary_key=True),
        Column('start_date', Date(), primary_key=True),
        Column('fee', Numeric(10, 5)),
        Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
        Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
    )

    tblPvcTbsMembership = IQTable('tblPvcTbsMembership', metadata,
        Column('id', Integer, primary_key=True),
        Column('tbs_id', Integer, ForeignKey('tblPvcTbsLicense.license_id', ondelete='CASCADE')),
        Column('name', Unicode(255)),
        Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
        Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        UniqueConstraint('tbs_id', 'name', name='tbs_name_index'),
    )

    tblPvcTbsMembershipFee  = IQTable('tblPvcTbsMembershipFee', metadata,
        Column('membership_id', Integer, ForeignKey('tblPvcTbsMembership.id', ondelete='CASCADE'), primary_key=True),
        Column('start_year', SmallInteger, primary_key=True),
        Column('fee', Numeric(10, 5)),
        Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
        Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
    )

    tblPvcAccountantTbs = IQTable('tblPvcAccountantTbs', metadata,
        Column('accountant_id', Integer, ForeignKey('tblPvcAccountantLicense.license_id', ondelete='CASCADE'), primary_key=True),
        Column('tbs_id', Integer, ForeignKey('tblPvcTbsLicense.license_id', ondelete='CASCADE'), primary_key=True),
        Column('start_date', DateTime(), nullable=True),
        Column('end_date', DateTime(), nullable=True),
    )

    tblPvcMemberLicense = IQTable('tblPvcMemberLicense', metadata,
        Column('license_id', Integer, ForeignKey('tblLicense.id', ondelete='CASCADE'), primary_key=True),
        Column('is_legacy_member', Boolean, nullable=False, default=False),
    )

    tblPvcTbsMember = IQTable('tblPvcTbsMember', metadata,
        Column('id', Integer, autoincrement=True, primary_key=True),
        Column('tbs_id', Integer, ForeignKey('tblPvcTbsLicense.license_id', ondelete='CASCADE')),
        Column('member_id', Integer, ForeignKey('tblPvcMemberLicense.license_id', ondelete='CASCADE')),
        Column('is_producer', Boolean, nullable=False),
        Column('is_commissioner', Boolean, nullable=False),

        Column('is_full_member', Boolean, nullable=False, default=False),
        Column('has_voting_rights', Boolean, nullable=False, default=False),
        Column('number_of_votes', Integer, nullable=True),
        Column('is_representative', Boolean, nullable=False, default=False),
        Column('was_member_in_2013', Boolean, nullable=False, default=False),
        Column('company_registration_number', Unicode(255), nullable=True),
        Column('declaration_periodicity', SmallInteger, nullable=False, default=0),
        Column('upcoming_periodicity_change', SmallInteger, nullable=True, default=None),  # automatically set
        Column('declaration_closing_day', SmallInteger, nullable=True, default=None),
        Column('obligation_start_date', Date(), nullable=False, default=datetime.date(2014,1,1)),
        Column('is_free_rider', Boolean, nullable=False, default=False),
        Column('guarantee_file_id', Integer, ForeignKey('tblFile.id', ondelete='CASCADE')),
        Column('selling_technique', SmallInteger, nullable=True),
        Column('internet_sales_countries', UnicodeText, nullable=True),

        Column('status', SmallInteger, nullable=False, default=PvcTbsMember.STATUS_PENDING),
        Column('archived', Boolean, nullable=False, default=False),

        Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
        Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        UniqueConstraint('member_id', 'tbs_id', name='tbs_member_index'),
    )

    tblPvcTbsMemberServiceLevel = IQTable('tblPvcTbsMemberServiceLevel', metadata,
        Column('id', Integer, autoincrement=True, primary_key=True),
        Column('tbs_member_id', Integer, ForeignKey('tblPvcTbsMember.id', ondelete='CASCADE'), nullable=False),
        Column('service_level_id', Integer, ForeignKey('tblPvcTbsServiceLevel.id', ondelete='CASCADE'), nullable=True),
        Column('pv_service_level_id', Integer, ForeignKey('tblPvcTbsServiceLevel.id', ondelete='CASCADE'), nullable=True),
        Column('start_date', Date(), nullable=False),
        Column('status', SmallInteger, nullable=False, default=core_models.PvcTbsMemberServiceLevel.STATUS_PENDING),
        UniqueConstraint('tbs_member_id', 'start_date', name='tbs_member_service_level_index'),
    )

    tblPvcTbsMemberContractFile = IQTable('tblPvcTbsMemberContractFile', metadata,
        Column('file_id', Integer, ForeignKey('tblFile.id', ondelete='CASCADE'), primary_key=True),
        Column('tbs_member_id', Integer, ForeignKey('tblPvcTbsMember.id', ondelete='CASCADE'), nullable=False),
        Column('type', SmallInteger, nullable=False),
    )

    tblPvcTbsMemberMembership = IQTable('tblPvcTbsMemberMembership', metadata,
        Column('id', Integer, autoincrement=True, primary_key=True),
        Column('tbs_member_id', Integer, ForeignKey('tblPvcTbsMember.id', ondelete='CASCADE'), nullable=False),
        Column('tbs_membership_id', Integer, ForeignKey('tblPvcTbsMembership.id', ondelete='CASCADE'), nullable=False),
        Column('start_date', DateTime(), nullable=False),
        UniqueConstraint('tbs_member_id', 'start_date', name='tbs_member_membership_index'),
    )

    tblPvcTbsCertificate = IQTable('tblPvcTbsCertificate', metadata,
        Column('id', Integer, autoincrement=True, primary_key=True),
        Column('tbs_id', Integer, ForeignKey('tblPvcTbsLicense.license_id', ondelete='CASCADE')),
        Column('year', SmallInteger, nullable=False),
        Column('text', UnicodeText),
        UniqueConstraint('tbs_id', 'year', name='tbs_year_index'),
    )

    tblPvcTbsMemberCertificate = IQTable('tblPvcTbsMemberCertificate', metadata,
        Column('tbs_member_id', Integer, ForeignKey('tblPvcTbsMember.id', ondelete='CASCADE'), nullable=False, primary_key=True),
        Column('year', SmallInteger, nullable=False, primary_key=True),
        Column('status', SmallInteger, nullable=False),
        Column('hash', Unicode(8), nullable=False),
        Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
        Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
    )

    tblPvcROECountry = IQTable('tblPvcROECountry', metadata,
        Column('country_id', Integer, ForeignKey('tblCountry.id', ondelete='CASCADE'), nullable=False, primary_key=True),
        Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
        Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
    )

    tables['tblPvcOperatorLicense'] = tblPvcOperatorLicense
    tables['tblPvcAccountantLicense'] = tblPvcAccountantLicense
    tables['tblPvcTbsLicense'] = tblPvcTbsLicense

    tables['tblPvcTbsServiceLevel'] = tblPvcTbsServiceLevel
    tables['tblPvcTbsServiceLevelAdministrationFee'] = tblPvcTbsServiceLevelAdministrationFee
    tables['tblPvcTbsServiceLevelContributionFee'] = tblPvcTbsServiceLevelContributionFee

    tables['tblPvcTbsMembership'] = tblPvcTbsMembership
    tables['tblPvcTbsMembershipFee'] = tblPvcTbsMembershipFee

    tables['tblPvcAccountantTbs'] = tblPvcAccountantTbs
    tables['tblPvcMemberLicense'] = tblPvcMemberLicense
    tables['tblPvcTbsMember'] = tblPvcTbsMember
    tables['tblPvcTbsMemberServiceLevel'] = tblPvcTbsMemberServiceLevel
    tables['tblPvcTbsMemberContractFile'] = tblPvcTbsMemberContractFile
    tables['tblPvcTbsMemberMembership'] = tblPvcTbsMemberMembership

    tables['tblPvcTbsMemberCertificate'] = tblPvcTbsMemberCertificate
    tables['tblPvcTbsCertificate'] = tblPvcTbsCertificate

    tables['tblPvcROECountry'] = tblPvcROECountry

    __tables_defined = True

def do_mappings():
    """
    This function is called by Django middleware when needed.
    """
    global __mapping_defined
    if __mapping_defined:
        return

    from pvcycle.core import models as core_models

    from basesoft.web.contrib.i18n import models as i18n_models
    from basesoft.web.contrib.file.sa_datamapper import get_tables as file_tables
    from basesoft.web.contrib.file import models as file_models
    from basesoft.web.contrib.notification import models as notification_models

    mapper(core_models.PvcOperatorLicense, tables['tblPvcOperatorLicense'],
        polymorphic_identity='PvcOperatorLicense',
        inherits=license_models.License
    )

    mapper(core_models.PvcAccountantLicense, tables['tblPvcAccountantLicense'],
        properties = {
            'schemes': relation(core_models.PvcTbsLicense, secondary=tables['tblPvcAccountantTbs'])
        },
        polymorphic_identity='PvcAccountantLicense',
        inherits=license_models.License
    )

    mapper(core_models.PvcTbsLicense, tables['tblPvcTbsLicense'],
        properties = {
            'country': relation(i18n_models.Country)
        },
        polymorphic_identity='PvcTbsLicense',
        inherits=license_models.License
    )

    mapper(core_models.PvcAccountantTbs, tables['tblPvcAccountantTbs'],
        properties = {
            'accountant': relation(core_models.PvcAccountantLicense, backref='accountant_tbs'),
            'tbs': relation(core_models.PvcTbsLicense, backref='tbs_accountants'),
        }
    )

    mapper(core_models.PvcTbsServiceLevel, tables['tblPvcTbsServiceLevel'],
        properties = {
            'tbs': relation(core_models.PvcTbsLicense),
            'administration_fees': relation(core_models.PvcTbsServiceLevelAdministrationFee, order_by=tables['tblPvcTbsServiceLevelAdministrationFee'].c.start_year, passive_deletes=True),
            'contribution_fees': relation(core_models.PvcTbsServiceLevelContributionFee, order_by=tables['tblPvcTbsServiceLevelContributionFee'].c.start_date, passive_deletes=True)
        }
    )

    mapper(core_models.PvcTbsServiceLevelAdministrationFee, tables['tblPvcTbsServiceLevelAdministrationFee'],
        properties = {
            'service_level': relation(core_models.PvcTbsServiceLevel),
        }
    )

    mapper(core_models.PvcTbsServiceLevelContributionFee, tables['tblPvcTbsServiceLevelContributionFee'],
        properties = {
            'service_level': relation(core_models.PvcTbsServiceLevel),
        }
    )

    mapper(core_models.PvcTbsMembership, tables['tblPvcTbsMembership'],
        properties = {
            'tbs': relation(core_models.PvcTbsLicense),
            'membership_fees': relation(core_models.PvcTbsMembershipFee, order_by=tables['tblPvcTbsMembershipFee'].c.start_year, passive_deletes=True),
        }
    )

    mapper(core_models.PvcTbsMembershipFee, tables['tblPvcTbsMembershipFee'],
        properties = {
            'membership': relation(core_models.PvcTbsMembership),
        }
    )

    mapper(core_models.PvcMemberLicense, tables['tblPvcMemberLicense'],
        polymorphic_identity='PvcMemberLicense',
        inherits=license_models.License,

    )

    mapper(core_models.PvcTbsMember, tables['tblPvcTbsMember'],
        properties = {
            'tbs': relation(core_models.PvcTbsLicense),
            'member': relation(core_models.PvcMemberLicense),
            'service_levels': relation(core_models.PvcTbsMemberServiceLevel,
                order_by=tables['tblPvcTbsMemberServiceLevel'].c.start_date, passive_deletes=True),
            'guarantee_file': relation(file_models.File)
        }
    )

    mapper(core_models.PvcTbsMemberServiceLevel, tables['tblPvcTbsMemberServiceLevel'],
        properties = {
            'tbs_member': relation(core_models.PvcTbsMember),
            'service_level': relation(core_models.PvcTbsServiceLevel,
                primaryjoin=tables['tblPvcTbsMemberServiceLevel'].c.service_level_id==tables['tblPvcTbsServiceLevel'].c.id),
            'pv_service_level': relation(core_models.PvcTbsServiceLevel,
                primaryjoin=tables['tblPvcTbsMemberServiceLevel'].c.pv_service_level_id==tables['tblPvcTbsServiceLevel'].c.id),

        }
    )

    mapper(PvcTbsMemberContractFile, tables['tblPvcTbsMemberContractFile'],
        properties = {
            'tbs_member': relation(PvcTbsMember, backref='contracts')
        },
        polymorphic_identity='PvcTbsMemberContractFile',
        inherits=File,
        inherit_condition=file_tables()['tblFile'].c.id==tables['tblPvcTbsMemberContractFile'].c.file_id
    )
    event.listen(PvcTbsMemberContractFile, 'after_insert', tbs_member_file_listener)

    mapper(core_models.PvcTbsMemberMembership, tables['tblPvcTbsMemberMembership'],
        properties = {
            'tbs_member': relation(core_models.PvcTbsMember),
            'membership': relation(core_models.PvcTbsMembership),
        }
    )

    mapper(PvcTbsCertificate, tables['tblPvcTbsCertificate'],
        properties = {
            'tbs': relation(core_models.PvcTbsLicense),
        },
    )

    mapper(PvcTbsMemberCertificate, tables['tblPvcTbsMemberCertificate'],
        properties = {
            'tbs_member': relation(core_models.PvcTbsMember),
        },
    )

    mapper(PvcROECountry, tables['tblPvcROECountry'],
        properties = {
            'country': relation(i18n_models.Country),
        }
    )

    mapper(core_models.PvcRegistrationNotification,
           inherits=notification_models.Notification,
           polymorphic_identity='PvcRegistrationNotification'
    )

    mapper(core_models.PvcTbsMemberProofFinancialGuaranteeFile,
        polymorphic_identity='PvcTbsMemberProofFinancialGuaranteeFile',
        inherits=File
    )
    event.listen(core_models.PvcTbsMemberProofFinancialGuaranteeFile, 'after_insert', tbs_member_file_listener)

    __mapping_defined = True

def fill_data(conn):
    pass

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'PVC Core')

def app_permissions():
    from django.utils.translation import ugettext as _
    return [

        license_models.Permission(u'pvc_operator_license', u"""
            This gives the right to manage PVC Operator Licenses.
            """, _("Manage PVC Operator Licenses")),

        license_models.Permission(u'pvc_accountant_license', u"""
            This gives the right to manage PVC Accountant Licenses.
            """, _("Manage PVC Accountant Licenses")),

        license_models.Permission(u'pvc_tbs_license', u"""
            This gives the right to manage PVC TBS Licenses.
            """, _("Manage PVC TBS Licenses")),

        license_models.Permission(u'pvc_tbs_service_level', u"""
            This gives the right to manage PVC TBS Service Level.
            """, _("Manage PVC TBS Service Level")),

        license_models.Permission(u'pvc_tbs_membership_level', u"""
            This gives the right to manage PVC TBS Membership Level.
            """, _("Manage PVC TBS Membership Level")),

        license_models.Permission(u'pvc_tbs_member_manager', u"""
            This gives the right to manage PVC TBS Members (status, updates, etc.).
            """, _("Manage PVC TBS Members")),

        license_models.Permission(u'pvc_tbs_license_configs', u"""
            This gives the right to manage PVC TBS License Configurations/Settings.
            """, _("Manage PVC TBS License configuration and settings")),

        license_models.Permission(u'pvc_member_license', u"""
            This gives the rights for a member to manage their license
            """, _("This gives the rights for a member to manage their license")),

        license_models.Permission(u'pvc_view_tbs_master_data', u"""
            This gives the rights to view TBS Master Data (used for SOAP export).
            """, _("View PVC TBS master data")),
        license_models.Permission(u'pvc_view_member_master_data', u"""
            This gives the rights to view Member Master Data (used for SOAP export).
            """, _("View PVC Member master data")),

        license_models.Permission(u'pvc_manage_roles', u"""
            This gives the rights to manage PVC Roles.
            """, _("Manage PVC Roles")),

        license_models.Permission(u'pvc_manage_certificates', u"""
            This gives the rights to manage PVC Certificates
            """, _("Manage PVC Certificates")),

        license_models.Permission(u'pvc_version_1_access', u"""
            This gives the right to use PV CYCLE version 1.x functionality.
            """, _("Use PV CYCLE 1.x functionality")),

]

def app_configs():
    from basesoft.web.contrib.account import models as account_models
    configs = []

    # Can be monthly, yearly, (quarterly)
    config = account_models.Config()
    config.name = 'pvc.tbs.declaration.period'
    config.type = 'integer'
    config.default = PvcTbsLicense.DECLARATION_PERIOD_MONTHLY
    config.description = u'Periodicity when the Declaration happens.'
    configs.append(config)

    # Can be monthly, yearly, (quarterly), this can only be changed by the system
    config = account_models.Config()
    config.name = 'pvc.tbs.previous.declaration.period'
    config.type = 'integer'
    config.default = PvcTbsLicense.DECLARATION_PERIOD_MONTHLY
    config.description = u'Previous periodicity when the Declaration happens.'
    configs.append(config)

    # the number of days before the Declaration is automatically closed (or forfeited) after it becomes available.
    config = account_models.Config()
    config.name = 'pvc.tbs.declaration.closing.day'
    config.type = 'integer'
    config.default = 10
    config.description = u'Declaration closing day.'
    configs.append(config)

    # day in january when the admin/membership fee is triggered
    config = account_models.Config()
    config.name = 'pvc.tbs.fee.day'
    config.type = 'integer'
    config.default = 12
    config.description = u'Day in January on which the Administrative/Membership fee is triggered'
    configs.append(config)

    # this setting is used when the TBS uses WEIGHT to calculate the AMOUNTS
    config = account_models.Config()
    config.name = 'pvc.tbs.declaration.use.weight.for.amount.calculations'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'Amounts based on weight.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'pvc.tbs.has.full.memberships'
    config.type = 'boolean'
    config.default = 'True'
    config.description = u'Full membership available.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'pvc.tbs.was.active.in.2013'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'Membership was possible in 2013.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'pvc.tbs.minimum.year.for.historical.data'
    config.type = 'integer'
    config.default = 2010
    config.description = u'The minimal year possible for Historical Data entry.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'pvc.tbs.use.membership.instead.of.service.level'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'Whether or not this TBS uses Membership levels instead of Service Levels.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'pvc.tbs.membership.valid.for.years'
    config.type = 'integer'
    config.default = 3
    config.description = u'After how many years does a membership expire.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'pvc.tbs.auto.approve.member'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'Whether or not Members get auto-approved.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'pvc.tbs.require.company.registration.number'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'Whether or not the member has to supply a Company Registration Number.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'pvc.tbs.require.accepting.of.terms.and.conditions'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'Whether or not the member has to supply a Company Registration Number.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'pvc.tbs.require.member.in.own.country'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'Member company requires to be in the same country as the TBS.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'pvc.tbs.optional.uk.vat.no'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'VAT number for UK members is optional at this TBS.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'pvc.tbs.minimum.contribution.fee'
    config.type = 'integer'
    config.default = 0
    config.description = u'Minimum contribution fee for non-compliant members.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'pvc.tbs.advance.invoicing'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'Use forecast for advance invoicing.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'pvc.tbs.require.proof.of.financial.guarantee'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'Member needs to provide a proof of financial guarantee on registration.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'pvc.tbs.require.end.user.distinction'
    config.type = 'boolean'
    config.default = None
    config.description = u'Member needs to provide a declaration line distinction between B2B and B2C.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'pvc.tbs.contribution.automatic.end.user.distinction'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'The order is automatically split for B2B/B2C based on an order rule.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'pvc.tbs.contribution.split.by.brand'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'The declaration need to be split "by brand".'
    configs.append(config)

    config = account_models.Config()
    config.name = 'pvc.tbs.guarantee.forecast'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'TBS requires a guarantee forecast to determine service level'
    configs.append(config)

    config = account_models.Config()
    config.name = 'pvc.tbs.guarantee.forecast.trigger.month'
    config.type = 'integer'
    config.default = '11'
    config.description = u'Month in which the new year guarantee forecast is asked'
    configs.append(config)

    return configs

def app_roles():
    return [
        {
            'name': 'PVC Operator',
            'required_permission': 'manage_iqportal_roles',
            'permissions': [
                'view_dashboard','save_dashboard', 'distribute_group_permissions', 'distribute_license_permissions',
                'license_masterdata_change_approval','manage_license_configs',
                'create_file', 'view_file', 'update_license',
                'pvc_accountant_license', 'pvc_tbs_license', 'pvc_tbs_license_configs', 'pvc_tbs_service_level',
                'pvc_manage_pv_modules', 'pvc_approve_pv_modules', 'pvc_create_pv_modules', 'pvc_import_pv_modules',
                'view_domain', 'pvc_tbs_member_manager',
                'pvc_tbs_membership_level', 'switch_account',
                'create_mail','update_mail', 'view_mail', # for account activation and lost password mail.
                'create_account_mail', 'update_account_mail', 'view_account_mail', 'delete_account_mail',
                'create_notification', 'view_orders', 'pvc_view_orders', 'pvc_approve_declaration_resubmit',
                'manage_tax_classes', 'manage_country_tax', 'view_country_tax',
                'manage_gl_accounts', 'view_notification', 'archive_notification',
                'create_invoice_mail', 'update_invoice_mail', 'view_invoice_mail',
                'pvc_view_declarations','set_custom_query_preset',
                'create_account', 'update_account', 'account_activation_reset',
                'manage_payment_methods', 'payment_method_bank_transfer', 'payment_method_ogone',
                'manage_roles', 'pvc_manage_roles', 'apply_roles',
                'pvc_manage_certificates',
                'pvc_manage_subrogations', 'pvc_manage_subrogation_mail',
                'manage_announcements', 'receive_announcements',
                'pvc_manage_historical_data',
                'pvc_manage_declaration_mail',
                'view_datasource', 'view_datasource_widget',
                'view_weee_product_levels', 'manage_weee_product_levels', 'manage_pvc_product_levels',
                'view_order_rules', 'manage_order_rules', 'manage_order_simulations', 'manage_pvc_brands'
            ],
            'license_configs': [],
        },

        # accountant
        {
            'name': 'PVC Accountant',
            'required_permission': 'pvc_manage_roles',
            'permissions': [
                'view_dashboard','save_dashboard',
                'pvc_view_tbs_master_data', 'pvc_view_member_master_data',
                'soap_login', 'pvc_view_orders', 'pvc_view_declarations',
                'remove_order_invoice',
                'complete_orders', 'set_custom_query_preset',
            ],
            'license_configs': [],
        },

        # tbs
        {
            'name': 'PVC TBS Admin', # this is for the Admin group AND for the license.
            'required_permission': 'pvc_manage_roles',
            'permissions': [
                'view_dashboard', 'save_dashboard', 'manage_license_configs', 'pvc_tbs_license_configs',
                'license_masterdata_change_approval', 'pvc_tbs_member_manager', 'create_notification', 'switch_account',
                'view_orders', 'pvc_view_orders', 'view_file', 'create_file', 'pvc_view_declarations', 'pvc_approve_declaration_resubmit',
                'pvc_tbs_service_level', 'pvc_manage_pv_modules', 'set_custom_query_preset', 'pvc_approve_pv_modules',
                'pvc_manage_certificates', 'pvc_manage_subrogations',
                'receive_announcements', 'pvc_manage_historical_data',
                'view_datasource', 'view_datasource_widget',
            ],
            'license_configs': [],
        },
        {
            'name': 'PVC DTS TBS', # this is for the Admin group AND for the license.
            'required_permission': 'pvc_manage_roles',
            'permissions': [
                'view_dashboard', 'save_dashboard', 'manage_license_configs', 'pvc_tbs_license_configs',
                'license_masterdata_change_approval', 'pvc_tbs_member_manager', 'create_notification',
                'view_orders', 'pvc_view_orders', 'view_file', 'create_file',
                'pvc_tbs_membership_level', 'set_custom_query_preset',
                'pvc_manage_certificates',
                'receive_announcements',
            ],
            'license_configs': [],
        },


        # member
        {
            'name': 'PVC Simple Member', # This is used at the sign up of the member.
            'required_permission': 'pvc_manage_roles',
            'permissions': [
                'save_dashboard', 'view_dashboard', 'view_notification',
                'pvc_member_license', 'view_orders', 'pvc_view_orders', 'pay_orders',
                'update_license_masterdata',
                'create_file', 'view_file',
                'create_license', # this or a specific pvc create license perm?
                'receive_announcements',
            ],
            'license_configs': [
                ('basesoft.account.post_login.js_modules', 'pvc_member_post_login'),
            ],
            'account_configs': [
                ('basesoft.dashboard.compose.right_layout_unit.collapsed', 'True'),
                ('basesoft.dashboard.mode.value', 0),
            ]
        },
        {
            'name': 'PVC Member Admin',
            'required_permission': 'pvc_manage_roles',
            'permissions': [
                'save_dashboard', 'view_dashboard', # ADDED here so a Dashboard can be assigned to this specific role.
                'pvc_view_declarations', 'pvc_declare_declarations',
                'pvc_select_pv_modules', 'pvc_create_pv_modules',
                'pvc_request_for_subrogation', 'pvc_view_subrogation',
                'pvc_create_historical_data', 'pvc_update_historical_data',
            ],
            'license_configs': [
                ('basesoft.account.post_login.js_modules', 'pvc_member_post_login'),
            ],
            'account_configs': [
                ('basesoft.dashboard.compose.right_layout_unit.collapsed', 'True'),
                ('basesoft.dashboard.mode.value', 0),
                ('basesoft.account.use.iconbar', 'False'),
                ('basesoft.account.top.menu.show.vertical.submenu', 'True'),
            ]
        },
        {
            'name': 'PVC DTS Member',
            'required_permission': 'pvc_manage_roles',
            'permissions': [
                'save_dashboard', 'view_dashboard', # ADDED here so a Dashboard can be assigned to this specific role.
            ],
            'license_configs': [
                ('basesoft.account.post_login.js_modules', 'pvc_member_post_login'),
            ],
            'account_configs': [
                ('basesoft.dashboard.compose.right_layout_unit.collapsed', 'True'),
                ('basesoft.dashboard.mode.value', 0),
                ('basesoft.account.use.iconbar', 'False'),
                ('basesoft.account.top.menu.show.vertical.submenu', 'True'),
            ]
        },

    ]

