'''
Created on Oct 15, 2013

@author: marten
'''
from pyws.functions.register import register

from basesoft.web.contrib.soap import soap_assert_logged_in
from basesoft.web.core.exceptions import PublicUserException
from basesoft import logging
from basesoft.web.contrib.soap.args import _types

from pvcycle.core.controllers import PvcAccountantController

def register_soap_functions(server):

    @register(return_type=[_types.LabelValueDict], to=server, needs_context=True)
    def get_take_back_schemes(context):
        """
        Returns a list with the ID and the Name of each TBS linked to the current logged in Accountant.
        """

        soap_assert_logged_in(context=context)

        logger = logging.getLogger('pvcycle.core.soap', request=context)
        logger.info('get_take_back_schemes')
        ac = PvcAccountantController(context.Session(), account_id=context.session['user_id'])
        return [{'id': t.id, 'name': t.name} for t in ac.get_take_back_schemes()]

    @register(return_type=[_types.MasterDataDict], to=server, needs_context=True)
    def get_tbs_master_data(context):
        """
        Returns a list with TBS Master Data dictionaries.
        """
        soap_assert_logged_in(context=context)

        logger = logging.getLogger('pvcycle.core.soap', request=context)
        logger.info('get_tbs_master_data')
        ac = PvcAccountantController(context.Session(), account_id=context.session['user_id'])
        return ac.get_tbs_master_data()

    @register(args=((int, None),), return_type=[_types.MasterDataDict], to=server, needs_context=True)
    def get_member_master_data(tbs_id, context):
        """
        Returns a list with Member Master Data dictionaries for the provided TBS.
        """
        if tbs_id is None:
            raise PublicUserException('Missing required field') # TODO: how can we make this nillable=False in the wsdl?

        soap_assert_logged_in(context=context)

        logger = logging.getLogger('pvcycle.core.soap', request=context)
        logger.info('get_member_master_data')

        ac = PvcAccountantController(context.Session(), account_id=context.session['user_id'])
        return ac.get_member_master_data(tbs_id)









