# Copyright (c) 2013 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext as _
from django.shortcuts import render_to_response
from django.utils import simplejson
from django.template.context import RequestContext

from basesoft.web.contrib.wiki.parser import Parser
from basesoft.web.core.exceptions import PublicUserException

from pvcycle.core.controllers import PvcMemberController, ContractReplacementFieldsContext,\
    ServiceLevelReplacementFieldsContext, CertificateReplacementFieldsContext,\
    PvcTBSController
from basesoft.web.contrib.wiki.rpc import PreviewContext
from pvcycle.core.models import PvcTbsMember

class ServiceLevelContext(object):
    def __init__(self, text):
        self.text = text

def view_contract(request, params):

    account_id = request.session['user_id']

    contract_text = ''

    if params != '':
        params = simplejson.loads(params)

        controller = PvcMemberController(request.Session(), account_id=account_id, request=request)

        member = controller.get_licenses(params['member_id']).pop()
        if params['tbs_id'] is None: # Ticket #258
            raise PublicUserException(_('A Take Back Scheme selection is required in order to preview the contract.'))
        tbs = controller.get_take_back_schemes(id=params['tbs_id'])

        parser = Parser()

        service_level = None
        service_level_annex_text = ServiceLevelContext('')
        pv_service_level_annex_text = ServiceLevelContext('')
        if params['service_level_id'] is not None:
            service_level = controller.get_service_level(params['service_level_id'])
            service_level_annex_text = ServiceLevelContext(parser.render_to_html(service_level.contract_annex_text,
                ServiceLevelReplacementFieldsContext(member=member, service_level=service_level)))

        if params['pv_service_level_id'] is not None:
            pv_service_level = controller.get_service_level(params['pv_service_level_id'])
            pv_service_level_annex_text = ServiceLevelContext(parser.render_to_html(pv_service_level.contract_annex_text,
                ServiceLevelReplacementFieldsContext(member=member, service_level=pv_service_level)))

        contract_text = parser.render_to_html(tbs.contract_text,
                ContractReplacementFieldsContext(member=member, tbs=tbs,
                    service_level = service_level_annex_text,
                    pv_service_level = pv_service_level_annex_text))

    return render_to_response('pvcycle/core/view_contract.html', {
        'contract_text': contract_text,
        'params': params,
        'base_requires': ['pvc_contract_scss'],
        'theme_css': 'pvc-contract'
    }, context_instance=RequestContext(request))




def view_certificate(request, params):

    account_id = request.session['user_id']

    certificate_text = ''


    if params != '':
        params = simplejson.loads(params)

        parser = Parser()

        if params.has_key('hash'):
            controller = PvcMemberController(request.Session(), account_id=account_id, request=request)
            certificate = controller.validate_certificate(params['hash'])
            params['tbs_member_id'] = certificate.tbs_member.id
            params['year'] = certificate.year

        # this view can be called by the member to download his contract
        if params.has_key('tbs_member_id'):
            tbs_member_id = params['tbs_member_id']
            year = params['year']

            controller = PvcMemberController(request.Session(), account_id=account_id, request=request)
            tbs_member = controller.load_tbs_member(tbs_member_id)
            member = controller.get_licenses( tbs_member['member_id'] ).pop()

            cert, member_certificate = controller.get_certificate(tbs_member_id, year)

            certificate_text = parser.render_to_html(
                cert.text, CertificateReplacementFieldsContext(member=member, certificate=member_certificate)
            )

            if tbs_member['status'] not in (PvcTbsMember.STATUS_APPROVED, PvcTbsMember.STATUS_RESIGNING):
                certificate_text = _('The member is currently not approved')

        # or by the tbs to preview the contact
        elif params.has_key('tbs_certificate_id'):

            controller = PvcTBSController(request.Session(), account_id=account_id, request=request)
            cert = controller.get_tbs_certificate(params['tbs_certificate_id'])

            certificate_text = parser.render_to_html(
                cert.text, PreviewContext()
            )

        else:
            raise Exception()

    return render_to_response('pvcycle/core/view_certificate.html', {
        'certificate_text': certificate_text,
        'params': params,
        'base_requires': ['jquery_waitforimages', 'pvc_certificate_scss'],
        'theme_css': 'pvc-certificate'
    }, context_instance=RequestContext(request))
