# Copyright (c) 2015 BaseSoft B.V.
# See LICENSE for details.
from sqlalchemy import and_
from basesoft.web.core.util.custom_query import CustomQuery, CustomQueryModel, CustomQueryField
from django.utils.translation import ugettext_lazy as _
from pvcycle.core.models import PvcOperatorLicense
from pvcycle.logistics.models import PvcDeliveryRequest, PvcHolderLicense, PvcPickupRequest, PvcCarrierLicense, \
    PvcTreatmentCenter
from sqlalchemy.orm import aliased


class PvcPickupRequestCustomQuery(CustomQuery):

    def tables(self):
        return (
            CustomQueryModel(PvcPickupRequest),
            CustomQueryModel(PvcHolderLicense),
            CustomQueryModel(PvcDeliveryRequest, use_outerjoin=True),

        )

    def fields(self):
        return (
            CustomQueryField(PvcPickupRequest.id, _('ID'), key='id', is_filter=True),
            CustomQueryField(PvcHolderLicense.name, _('Holder'), key='holder_name', is_filter=True),
            CustomQueryField(PvcDeliveryRequest.id, _('Delivery request'), key='delivery_request_id', is_filter=True),
            CustomQueryField(PvcPickupRequest.number_modules, _('PV modules'), key='number_modules', is_filter=True),
            CustomQueryField(PvcPickupRequest.weight_holder, _('Weight'), key='weight_holder', is_filter=True),
            #CustomQueryField(PvcPickupRequest.weight_tc, _('Weight (treatment center)'), key='weight_tc', is_filter=True),
            CustomQueryField(PvcPickupRequest.status, _('Status'), key='status', is_filter=True, filter_options=[{
                'value': PvcPickupRequest.STATUS_DRAFT,
                'label': _("Draft"),
            },{
                'value': PvcPickupRequest.STATUS_PENDING,
                'label': _("Pending"),
            },{
                'value': PvcPickupRequest.STATUS_ACCEPTED,
                'label': _("Accepted"),
            },{
                'value': PvcPickupRequest.STATUS_REJECTED,
                'label': _("Rejected"),
            },{
                'value': PvcPickupRequest.STATUS_COMPLETED,
                'label': _("Completed"),
            }]),
            CustomQueryField(PvcPickupRequest.created, _('Created'), key='created', is_filter=True),
            CustomQueryField(PvcPickupRequest.last_updated, _('Last updated'), key='last_updated', is_filter=True),
        )

    def set_where(self):
        if isinstance(self.get_default_group().license, PvcOperatorLicense):
            return
        self.where = PvcPickupRequest.holder == self.get_default_group().license


holder_license = aliased(PvcHolderLicense)
carrier_license = aliased(PvcCarrierLicense)


class PvcDeliveryRequestCustomQuery(CustomQuery):

    def tables(self):
        return (
            CustomQueryModel(PvcDeliveryRequest),
            CustomQueryModel(carrier_license, onclause=PvcDeliveryRequest.carrier_id==carrier_license.license_id),
            CustomQueryModel(PvcTreatmentCenter, onclause=PvcDeliveryRequest.treatment_center_id==PvcTreatmentCenter.company_id),
            CustomQueryModel(PvcPickupRequest, onclause=PvcDeliveryRequest.id==PvcPickupRequest.delivery_request_id, use_outerjoin=True),
            CustomQueryModel(holder_license, onclause=PvcPickupRequest.holder_id==holder_license.license_id),
        )

    def fields(self):
        return (
            CustomQueryField(PvcDeliveryRequest.id, _('ID'), key='id', is_filter=True, is_group_by=True),
            CustomQueryField(carrier_license.name, _('Carrier'), key='carrier_name', is_filter=True, is_group_by=True),
            CustomQueryField(PvcTreatmentCenter.name, _('Treatment center'), key='treatment_center', is_filter=True, is_group_by=True),
            CustomQueryField(PvcDeliveryRequest.status, _('Status'), key='status', is_filter=True, filter_options=[{
                'value': PvcDeliveryRequest.STATUS_DRAFT,
                'label': _("Pending"),
            },{
                'value': PvcDeliveryRequest.STATUS_ASSIGNED,
                'label': _("Assigned"),
            },{
                'value': PvcDeliveryRequest.STATUS_ACCEPTED,
                'label': _("Accepted"),
            },{
                'value': PvcDeliveryRequest.STATUS_REJECTED,
                'label': _("Rejected"),
            },{
                'value': PvcDeliveryRequest.STATUS_COMPLETED,
                'label': _("Completed"),
            }]),

            CustomQueryField(PvcPickupRequest.id, _('Pickup ID'), key='pickup_request_id'),
            CustomQueryField(holder_license.name, _('Holder'), key='holder_name', is_filter=True, is_group_by=True),
            CustomQueryField(PvcPickupRequest.number_modules, _('PV modules'), key='number_modules', is_filter=True),
            CustomQueryField(PvcPickupRequest.weight_holder, _('Weight'), key='weight_holder', is_filter=True),
            #CustomQueryField(PvcPickupRequest.weight_tc, _('Weight (treatment center)'), key='weight_tc', is_filter=True),

            CustomQueryField(PvcDeliveryRequest.created, _('Created'), key='created', is_filter=True),
            CustomQueryField(PvcDeliveryRequest.last_updated, _('Last updated'), key='last_updated', is_filter=True),
        )

    def set_where(self):
        if isinstance(self.get_default_group().license, PvcOperatorLicense):
            return
        self.where = and_(
            PvcDeliveryRequest.carrier == self.get_default_group().license,
            PvcDeliveryRequest.status > PvcDeliveryRequest.STATUS_DRAFT
        )


custom_queries = (
    PvcPickupRequestCustomQuery,
    PvcDeliveryRequestCustomQuery,
)