# Copyright (c) 2015 BaseSoft B.V.
# See LICENSE for details.
from django.utils.translation import ugettext_lazy as _

from basesoft.web.contrib.account.decorators import assert_user_logged_in

from pvcycle.core.icons import AbstractPvcIcon


class PvcPickupRequestIcon(AbstractPvcIcon):
    id = 'icon-pvc-pickup-request-form'
    title = _('New pickup request')
    icon_index = 64
    component = 'pvc_pickup_request_details'
    group = 'logistics'
    group_title = _('Logistics')
    colorname = 'iconbar-cerulean'  # FIXME

    char_main = u'\uE805'  # FIXME
    char_sub = u'\uE806'  # FIXME

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('pvc_logistics')
        controller.assert_any_permissions('pvc_logistics_key_user', 'pvc_logistics_holder')

class PvcPickupRequestManagerIcon(AbstractPvcIcon):
    id = 'icon-pvc-pickup-request-manager'
    title = _('Pickup request manager')
    icon_index = 60
    component = 'pvc_pickup_request_manager'
    group = 'logistics'
    group_title = _('Logistics')
    colorname = 'iconbar-cerulean'  # FIXME

    char_main = u'\uE805'  # FIXME
    char_sub = u'\uE806'  # FIXME

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('pvc_logistics')
        controller.assert_any_permissions('pvc_logistics_key_user', 'pvc_logistics_holder')


class PvcDeliveryRequestIcon(AbstractPvcIcon):
    id = 'icon-pvc-delivery-request-form'
    title = _('New delivery request')
    icon_index = 65
    component = 'pvc_delivery_request_details'
    group = 'logistics'
    group_title = _('Logistics')
    colorname = 'iconbar-cerulean'  # FIXME

    char_main = u'\uE805'  # FIXME
    char_sub = u'\uE806'  # FIXME

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('pvc_logistics')
        controller.assert_any_permissions('pvc_logistics_key_user', 'pvc_logistics_carrier')


class PvcDeliveryRequestManagerIcon(AbstractPvcIcon):
    id = 'icon-pvc-delivery-request-manager'
    title = _('Delivery request manager')
    icon_index = 61
    component = 'pvc_delivery_request_manager'
    group = 'logistics'
    group_title = _('Logistics')
    colorname = 'iconbar-cerulean'  # FIXME

    char_main = u'\uE805'  # FIXME
    char_sub = u'\uE806'  # FIXME

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('pvc_logistics')
        controller.assert_any_permissions('pvc_logistics_key_user', 'pvc_logistics_carrier')

def get_icons():
    return [
        PvcPickupRequestIcon(),
        PvcPickupRequestManagerIcon(),
        PvcDeliveryRequestIcon(),
        PvcDeliveryRequestManagerIcon(),
    ]
