# Copyright (c) 2015 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext_lazy as _

from basesoft.web.contrib.contact.models import Company
from basesoft.web.contrib.license.models import License
from basesoft.web.contrib.notification.models import Notification

class PvcHolderLicense(License):
    TYPE_COLLECTION_POINT = 1
    TYPE_VOLUNTARY_COLLECTION_POINT = 2


class PvcCarrierLicense(License):
    pass


class PvcTreatmentCenter(Company):
    pass


class PvcPickupRequest(object):
    """
    Delivery Request is done by a collection point/holder
    """
    STATUS_DRAFT = 1 # holder is still editing
    STATUS_PENDING = 2 # waiting for assignment to a DeliveryRequest
    STATUS_ACCEPTED = 3 # linked to a DR, carrier on ze way
    STATUS_REJECTED = 4
    STATUS_INVALID = 5
    STATUS_COMPLETED = 6

    # Amorphous silicon (a-Si)
    # Cadmium telluride (CdTe)
    # Copper indium gallium selenide (CIS/CIGS)
    # Organic photovoltaic cells (OPC)
    TECH_ASI = 1
    TECH_CDTE = 2
    TECH_CIS = 3
    TECH_OPC = 4

    _technology_i18n_map = {
        TECH_ASI: _('Amorphous silicon (a-Si)'),
        TECH_CDTE: _('Cadmium telluride (CdTe)'),
        TECH_CIS: _('Copper indium gallium selenide (CIS/CIGS)'),
        TECH_OPC: _('Organic photovoltaic cells (OPC)'),
    }

    def _technology_i18n(self):
        return self._technology_i18n_map[self.technology]
    technology_i18n = property(_technology_i18n)

    CONDITION_NEW = 1
    CONDITION_USED = 2
    CONDITION_BURNED = 3

    _condition_i18n_map = {
        CONDITION_NEW: _('New'),
        CONDITION_USED: _('Used'),
        CONDITION_BURNED: _('Burned'),
    }

    def _condition_i18n(self):
        return self._condition_i18n_map[self.condition]
    condition_i18n = property(_condition_i18n)



class PvcDeliveryRequest(object):
    """
    Pickup Request is done by Key User (Operator)
    """
    STATUS_DRAFT = 1 # waiting for carrier to accept
    STATUS_ASSIGNED = 2 # assigned to courier, non editable
    STATUS_ACCEPTED = 3 # accepted by carrier.. on transport
    STATUS_REJECTED = 4 # rejected by carrier, implement later
    STATUS_COMPLETED = 5 # carrier completed his trip
    STATUS_APPROVED = 6 # key user approves (after completed)



class PvcDeliveryRequestNotification(Notification):
    """
    @DynamicAttrs
    """
    name_i18n = _('Delivery request')


class PvcPickupRequestNotification(Notification):
    """
    @DynamicAttrs
    """
    name_i18n = _('Pickup request')
