# Copyright (c) 2016 BaseSoft B.V.
# See LICENSE for details.

from iqportal.iqube.core.db_sync import AbstractConfigurableWidget
from django.utils.translation import ugettext as _

class ContributionResultWidget(AbstractConfigurableWidget):
    category = 'PVC Report'
    cube = 'PvcContributionDataCube'
    datasource = 'PvcOrderData'
    label = _('Declaration result')

    def config(self):
        return {
            "reporter": "pvcycle.order.PvcContributionReporter",
            'show_last_aggregation_update': True,
            "category": "Order",
            "has_quick_filter": True,
            "show_hide_empty": True,
            "transposable": True,
            "label": "Declaration result",
            "has_calculated_member": True,
            "show_excel_export": True,
            "cube": "PvcContributionDataCube",
            "permissions": ['pvc_view_declarations'],
            "has_graph": True,
            "table_object_name": "datatablecube",
            "table_module": "iqreport_graph_datatablecube",
            'filter_panel_module': 'pvc_contribution_report_item',
            "has_table": True,
            "has_hierarchy_names": True,
            "graph_config": {},

            "has_drill_position": False,
            "has_drill_replace": False,

            "has_drill_through": False,

            "dimensions": [{
                    "caption": "Invoice Type",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Invoice Type].[Invoice Type]",
                            "has_children_available": True,
                            "caption": "Invoice Type",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Invoice Type].[Invoice Type]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "TBS",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[TBS].[TBS]",
                            "has_children_available": True,
                            "caption": "TBS",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[TBS].[TBS]",
                            "method": "get_level_members"
                        }]
                },{
                    "caption": "Member",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Member].[Member]",
                            "has_children_available": True,
                            "caption": "Member",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Member].[Member]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "Producer",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Producer].[Producer]",
                            "has_children_available": True,
                            "caption": "Producer",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Producer].[Producer]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "Commissioner",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Commissioner].[Commissioner]",
                            "has_children_available": True,
                            "caption": "Producer",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Commissioner].[Commissioner]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "Brand",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Brand].[Brand]",
                            "has_children_available": True,
                            "caption": "Brand",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Brand].[Brand]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "Product",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Product].[WEEE Category]",
                            "has_children_available": True,
                            "caption": "WEEE Category",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Product].[WEEE Category]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "Local Product",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Local Product].[Product]",
                            "has_children_available": True,
                            "caption": "Product",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Local Product].[Product]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "Country",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "fancy_select_optionsRpcUrl": "pvcycle.report.ReportDimensionHandler",
                    "fancy_select_RpcMethod": "get_efta_countries",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Country].[Country]",
                            "has_children_available": True,
                            "caption": "Country",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Country].[Country]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "Declaration Year",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Declaration Year].[Year]",
                            "has_children_available": True,
                            "caption": "Year",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Declaration Year].[Year]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "Declaration Quarter",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Declaration Quarter].[Quarter]",
                            "has_children_available": True,
                            "caption": "Quarter",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Declarataon Quarter].[Quarter]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "Declaration Month",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Declaration Month].[Month]",
                            "has_children_available": True,
                            "caption": "Month",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Declaration Month].[Month]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "Submit Period",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "available_on_columns": True,
                    "show_calculated_members": False,
                    "show_full_caption":True,
                    "fancy_select_module": "level_filter_fancy_select",
                    "fancy_select_optionsRpcUrl": 'iqportal.iqube.core.IQubeRPCHandler',
                    "fancy_select_RpcMethod": 'get_time_children',

                    "children": [{
                        "unique_name": "[Submit Period.Quarter].[Year]",
                        "has_children_available": False,
                        "caption": "Year Quarter Month",
                        "has_all_member_available": False,
                        "show_members": True,
                        "method_argument": "[Submit Period.Quarter].[Year]",
                        "method": "get_level_members",
                        "has_drill_enabled": True,
                    }, {
                        "unique_name": "[Submit Period.WeekISO8601].[Year]",
                        "has_children_available": False,
                        "caption": "Year Week Day",
                        "has_all_member_available": False,
                        "show_members": True,
                        "method_argument": "[Submit Period.WeekISO8601].[Year]",
                        "method": "get_level_members",
                        "has_drill_enabled": True,
                    },]
                },{
                    "caption": "Payment Period",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "available_on_columns": True,
                    "show_calculated_members": False,

                    "fancy_select_module": "level_filter_fancy_select",
                    "fancy_select_optionsRpcUrl": 'iqportal.iqube.core.IQubeRPCHandler',
                    "fancy_select_RpcMethod": 'get_time_children',

                    "children": [{
                        "unique_name": "[Payment Period.Quarter].[Year]",
                        "has_children_available": False,
                        "caption": "Year Quarter Month",
                        "has_all_member_available": False,
                        "show_members": True,
                        "method_argument": "[Payment Period.Quarter].[Year]",
                        "method": "get_level_members",
                        "has_drill_enabled": True,
                    }, {
                        "unique_name": "[Payment Period.WeekISO8601].[Year]",
                        "has_children_available": False,
                        "caption": "Year Week Day",
                        "has_all_member_available": False,
                        "show_members": True,
                        "method_argument": "[Payment Period.WeekISO8601].[Year]",
                        "method": "get_level_members",
                        "has_drill_enabled": True,
                    },]
                },{
                    "caption": "Payment Status",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Payment Status].[Payment Status]",
                            "has_children_available": True,
                            "caption": "Payment Status",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Payment Status].[Payment Status]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "Payment Method",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Payment Method].[Payment Method]",
                            "has_children_available": True,
                            "caption": "Payment Method",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Payment Method].[Payment Method]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "GL Account",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[GL Account].[GL Account]",
                            "has_children_available": True,
                            "caption": "GL Account",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[GL Account].[GL Account]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "Is Full Member",
                    "is_simple_filter": True,
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Is Full Member].[Is Full Member]",
                            "has_children_available": False,
                            "caption": "Is Full Member",
                            "has_all_member_available": False,
                            "show_members": True,
                            "method_argument": "[Is Full Member].[Is Full Member]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "Has Voting Rights",
                    "is_simple_filter": True,
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Has Voting Rights].[Has Voting Rights]",
                            "has_children_available": False,
                            "caption": "Has Voting Rights",
                            "has_all_member_available": False,
                            "show_members": True,
                            "method_argument": "[Has Voting Rights].[Has Voting Rights]",
                            "method": "get_level_members"
                    }]
                },{
                    "caption": "Measures",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": False,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": True,
                    "children": [{
                            "unique_name": "[Measures].[MeasuresLevel]",
                            "has_children_available": True,
                            "caption": "Measures",
                            "has_all_member_available": False,
                            "show_members": True,
                            "method_argument": "[Measures].[MeasuresLevel]",
                            "method": "get_level_members"
                    }]
                }
            ],
        }

    def applied_filters(self):
        return {
            "dimensions": {
                "Invoice Type": "0",
                "TBS": "1",
                "Member": "2",
                "Producer": "3",
                "Commissioner": "4",
                "Brand": "5",
                "Product": "6",
                "Local Product": "7",
                "Country": "8",
                "Declaration Year": "9",
                "Declaration Quarter": "10",
                "Declaration Month": "11",
                "Submit Period": "12",
                "Payment Period": "13",
                "Payment Status": "14",
                "Payment Method": "15",
                "GL Account": "16",
                "Is Full Member": "17",
                "Has Voting Rights": "18",
                "Measures": "19"
            },

            "axes_0": "",
            "axes_1": "",
            "axes_2": "",
            "axes_3": "",
            "axes_4": "",
            "axes_5": "",
            "axes_6": "",
            "axes_7": "",
            "axes_8": "",
            "axes_9": "",
            "axes_10": "",
            "axes_11": "",
            "axes_12": "",
            "axes_13": "",
            "axes_14": "",
            "axes_15": "",
            "axes_16": "",
            "axes_17": "",
            "axes_18": "",
            "axes_19": "columns",


            "dimension_0": [],
            "dimension_1": [],
            "dimension_2": [],
            "dimension_3": [],
            "dimension_4": [],
            "dimension_5": [],
            "dimension_6": [],
            "dimension_7": [],
            "dimension_8": [],
            "dimension_9": [],
            "dimension_10": [],
            "dimension_11": [],
            "dimension_12": [],
            "dimension_13": [],
            "dimension_14": [],
            "dimension_15": [],
            "dimension_16": [],
            "dimension_17": [],
            "dimension_18": [],
            "dimension_19": ["[Measures].[Amount (excl. vat)]"],

            "column_order": [
                "Measures",
            ],
            "row_order": [
            ],
            "show_table": True,
            "show_slicers": True,
            "hide_empty_fields": True,

            "initialRefresh": False,

        }

configurable_widgets = (ContributionResultWidget,)