# Copyright (c) 2013 BaseSoft B.V.
# See LICENSE for details.

from basesoft.web.contrib.dashboard.controllers import DashboardWidgetCQStateController
from django.utils.translation import ugettext_lazy as _
from basesoft.util.ordered_dict import OrderedDict
from pvcycle.core.models import PvcMemberLicense


class PVCOrderOverviewWidget(DashboardWidgetCQStateController):
    """
    Order Overview Widget
    """
    label = _('Order manager')
    name = 'pvc_db_order_overview_widget'
    icon = 'icons/db_order_manager.png'
    category = _('PVC Financial')
    permissions = ['pvc_view_orders']

class PVCDeclarationOverviewWidget(DashboardWidgetCQStateController):
    """
    Declaration Overview Widget
    """
    label = _('Declaration manager (legacy)')
    name = 'pvc_db_declaration_overview_widget'
    icon = 'icons/db_declaration_manager.png'
    category = _('PVC Declaration')
    permissions = ['pvc_view_declarations']

    def has_permissions(self, controller):

        # Operator should be able to place this on dashboard for member
        _license = controller.get_default_group().license
        if isinstance(_license, PvcMemberLicense) and not _license.is_legacy_member:
            return False
        return controller.has_permissions(*self.permissions)

class PVCOutstandingDeclarationWidget(DashboardWidgetCQStateController):
    """
    Declaration Overview Widget
    """
    label = _('Outstanding declarations (legacy)')
    name = 'pvc_db_outstanding_declaration_widget'
    icon = 'icons/db_outstanding_declarations.png'
    category = _('PVC Declaration')
    permissions = ['pvc_view_declarations', 'pvc_version_1_access']


class ContributionOverviewWidget(DashboardWidgetCQStateController):
    """
    Declaration Overview Widget
    """
    label = _('Declaration manager')
    name = 'pvc_db_contribution_overview_widget'
    icon = 'icons/db_declaration_manager.png'
    category = _('PVC Declaration')
    permissions = ['pvc_view_declarations']


class OutstandingContributionWidget(DashboardWidgetCQStateController):
    """
    Declaration Overview Widget
    """
    label = _('Outstanding declarations')
    name = 'pvc_db_outstanding_contribution_widget'
    icon = 'icons/db_outstanding_declarations.png'
    category = _('PVC Declaration')
    permissions = ['pvc_view_declarations']



widgets_mapper = OrderedDict([
    ('PVCOrderOverviewWidget', PVCOrderOverviewWidget, ),
    ('PVCDeclarationOverviewWidget', PVCDeclarationOverviewWidget, ),
    ('ContributionOverviewWidget', ContributionOverviewWidget, ),
    ('OutstandingContributionWidget', OutstandingContributionWidget, ),
])
