'''
Created on Nov 5, 2013

@author: marten
'''
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext as _

from basesoft.web.core.forms.handler import FormValidator
from basesoft.web.core.input.validation import plugins
from basesoft.web.core.exceptions import PublicUserException

from pvcycle.order.controllers import PvcDeclarationOrderController, PvcContributionOrderController
from pvcycle.order.models import PvcDeclarationOrder, PvcDeclarationOrderLine, PvcContributionOrder, \
    PvcContributionOrderLine
from pvcycle.core.controllers import PvcMemberController, PvcTBSController
from pvcycle.product.controllers import PvcProductController
from pvcycle.pvmodule.models import PvcTbsMemberPVModule

class DeclarationHandler:

    def __init__(self, request, **kwargs):

        self._request = request
        self._sa_session = request.Session()
        self._account_id = request.session['user_id']
        self._controller = PvcDeclarationOrderController(self._sa_session, account_id=self._account_id, request=request)
        self._contrib_controller = PvcContributionOrderController(self._sa_session, account_id=self._account_id, request=request)
        self._product_controller = PvcProductController(self._sa_session, account_id=self._account_id, request=request)

    def get_open_declaration_count(self):
        rv = len(self._controller.get_open_orders(priv=PvcDeclarationOrder.PRIV_UPDATE))
        if rv > 0:
            return rv
        return None

    def get_open_contribution_count(self):
        rv = len(self._contrib_controller.get_open_orders(priv=PvcDeclarationOrder.PRIV_UPDATE))
        if rv > 0:
            return rv
        return None

    def close_declarations_before_specified_date(self, tbs_id, before):
        controller = PvcTBSController(self._sa_session, account_id=self._account_id, request=self._request)
        tbs = controller.get_take_back_schemes(tbs_id).pop()
        declarations = self._controller.close_declarations_created_before_specified_date(tbs, before)
        return len(declarations)

    def validate_additional_lines(self, args):
        v = {}
        for k in args.keys():
            if k[:19] == 'quantity_additional':
                v[k] = [(plugins.Integer, {'unsigned': True, 'allow_zero': True, 'max_value': PvcDeclarationOrderLine.MAX_LINE_QUANTITY}), (plugins.Required, {})]

        form_handler = FormValidator(v)
        return form_handler.validate(args)

    def validate_declaration(self, args):
        v = {}
        for k in args.keys():
            v[k] = [(plugins.Integer, {'unsigned': True, 'allow_zero': True, 'max_value': PvcDeclarationOrderLine.MAX_LINE_QUANTITY}), (plugins.Required, {})]

        form_handler = FormValidator(v)
        return form_handler.validate(args)

    def validate_contribution(self, args):
        v = {}
        for k in args.keys():
            if k[-8:] == 'quantity':
                v[k] = [(plugins.Integer, {'unsigned': True, 'allow_zero': True, 'max_value': PvcContributionOrderLine.MAX_LINE_QUANTITY}), (plugins.Required, {})]
            elif k[-6:] == 'is_b2b':
                continue
            elif k[-5:] == 'brand':
                v[k] = [(plugins.Required, {})]
            else:
                v[k] = [(plugins.Number, {'min_value': 0, 'inclusive': True, 'max_value': PvcContributionOrderLine.MAX_LINE_DECIMALS}), (plugins.Required, {})]

        form_handler = FormValidator(v)
        return form_handler.validate(args)

    def validate_manual_declaration(self, args):
        form_handler = FormValidator({
            'tbs_id': [(plugins.Required, {}), ]
        })
        return form_handler.validate(args)

    def load_contribution(self, _id, submit_dry_run=False):
        return self._contrib_controller.load_state(_id, True, submit_dry_run)

    def load_forecast(self, tbs_member_id):
        return self._contrib_controller.load_forecast_state(tbs_member_id)

    def save_forecast_order(self, tbs_member_id, products, is_b2b=None, brand=None):
        order = self._contrib_controller.add_forecast_order(tbs_member_id, products, is_b2b, brand)
        return order.id

    def save_guarantee_forecast_order(self, tbs_member_id, year, products, is_b2b=None, brand=None):
        order = self._contrib_controller.add_guarantee_forecast_order(tbs_member_id, year, products, is_b2b, brand)
        return order.id

    def get_available_tbs_products(self, _id, exclude):
        exclude = [int(i) for i in exclude]
        products = self._contrib_controller.get_available_tbs_products(_id, exclude)
        return [{
            'value': p.id,
            'label': '%s - %s' % (p.label, p.description)
        } for p in products]

    def get_available_tbs_member_products(self, tbs_member_id):
        products = self._contrib_controller.get_available_tbs_member_products(tbs_member_id)
        return [{
            'value': p.id,
            'label': '%s - %s' % (p.label, p.description)
        } for p in products]

    def get_available_and_selected_tbs_member_products(self, tbs_member_id):
        tbs_member = self._product_controller.get_tbs_member_licenses(tbs_member_id)[0]
        tp, tmp = self._product_controller.get_tbs_member_products(tbs_member.tbs_id, tbs_member.member_id, None)
        rv = {
            'selectedOptions': [{'value': p.id, 'label': '%s - %s' % (p.label, p.description)} for p in tmp],
            'defaultOptions': [{'value': p.id, 'label': '%s - %s' % (p.label, p.description)} for p in tp if p not in tmp],
        }
        return rv

    def load_declaration(self, _id):
        return self._controller.load_state(_id, True)

    def _save_declaration(self, _id, args):
        if not self._controller.has_permissions('pvc_version_1_access'):
            raise DeprecationWarning('This is no longer available (PV CYCLE 2.0)')
        subrogations = None
        additional = None
        if args.has_key('subrogation'):
            subrogations = {}
            for k, v in args['subrogation'].iteritems():
                if v == '':
                    continue
                subrogations[k[12:]] = int(v)
        if args.has_key('additional'):
            additional = args['additional']
        declaration = args
        if args.has_key('declaration'):
            declaration = args['declaration']
        return self._controller.update_declaration(_id, declaration, additional_lines=additional, subrogate=subrogations)

    def save_declaration(self, _id, args):
        order = self._save_declaration(_id, args)
        if type(order) != list:
            return order.id
        return _id

    def save_declaration_and_submit(self, _id, args):
        order = self._save_declaration(_id, args)
        if type(order) != list:
            self._controller.submit_order(order.id)
            return order.id
        [self._controller.submit_order(o.id) for o in order]
        return _id

    def save_contribution(self, _id, args):
        order = self._contrib_controller.save_contribution(_id, args)
        if type(order) != list:
            return order.id
        return _id

    def save_contribution_and_submit(self, _id, args):
        return self._contrib_controller.save_contribution_and_submit(_id, args)

    def checkout_declaration(self, _id):
        if not self._controller.has_permissions('pvc_version_1_access'):
            raise DeprecationWarning('This is no longer available (PV CYCLE 2.0)')
        order = self._controller.submit_order(_id)
        return order.id

    def get_new_lines_for_declaration(self, _id, module_id):
        if not self._controller.has_permissions('pvc_version_1_access'):
            raise DeprecationWarning('This is no longer available (PV CYCLE 2.0)')
        return self._controller.get_new_lines(_id, module_id)

    def get_new_lines_for_contribution(self, _id, products, is_b2b, countries):
        return self._contrib_controller.get_new_lines(_id, products, is_b2b, countries)

    def get_undeclared_contribution_periods(self, tbs_id, member_id=None, subrogation_id=None):
        mc = PvcMemberController(self._sa_session, account_id=self._account_id, request=self._request)
        tbs_member = [tm for tm in mc.get_active_for_tbs(member_id, include_resigning=True) if tm.tbs.id==int(tbs_id)].pop()
        return self._contrib_controller.get_undeclared_periods(tbs_member, subrogation_id)

    def get_undeclared_periods(self, tbs_id, member_id=None):
        if not self._controller.has_permissions('pvc_version_1_access'):
            raise DeprecationWarning('This is no longer available (PV CYCLE 2.0)')
        mc = PvcMemberController(self._sa_session, account_id=self._account_id, request=self._request)
        tbs_member = [tm for tm in mc.get_active_for_tbs(member_id) if tm.tbs.id==int(tbs_id)].pop()
        if len(tbs_member.modules) == 0:
            raise PublicUserException(_('No PV Modules selected at this TBS.'))
        return self._controller.get_undeclared_periods(tbs_member)

    def save_manual_declaration(self, bitbucket, params):
        if not self._controller.has_permissions('pvc_version_1_access'):
            raise DeprecationWarning('This is no longer available (PV CYCLE 2.0)')
        rv = self._manual_declaration(params, self._controller)
        if rv is None:
            raise PublicUserException(_('No declaration could be triggered for supplied period, are PV Modules selected?'))
        if isinstance(rv, PvcDeclarationOrder):
            return rv.id
        return [d.id for d in rv]

    def save_manual_contribution(self, bitbucket, params):
        rv = self._manual_declaration(params, self._contrib_controller)
        if rv is None:
            raise PublicUserException(_('No declaration could be triggered for supplied period, are there products selected?'))
        if isinstance(rv, PvcContributionOrder):
            return rv.id
        return [d.id for d in rv]

    def _manual_declaration(self, params, controller):
        member_id = params['tbs_member']['license_id']
        tbs_id = params['tbs_member']['tbs_id']
        del params['period_select']['warning']
        params['period_select']['month'] = None if params['period_select']['month'] in (None, '') else int(params['period_select']['month'])
        params['period_select']['quarter'] = None if params['period_select']['quarter'] in (None, '') else int(params['period_select']['quarter'])
        mc = PvcMemberController(self._sa_session, account_id=self._account_id, request=self._request)
        tbs_member = [tm for tm in mc.get_active_for_tbs(member_id, include_resigning=True) if tm.tbs.id==int(tbs_id)].pop()

        kwargs = params['period_select']
        subrogation_id = None if not params['tbs_member'].has_key('subrogation_id') else params['tbs_member']['subrogation_id']
        if subrogation_id is not None:
            kwargs['subrogation_id'] = subrogation_id

        rv = controller.add_open_declaration(tbs_member, **kwargs)
        return rv

    def remove_line(self, _id, line_id, remove_for_future=False):
        if not self._controller.has_permissions('pvc_version_1_access'):
            raise DeprecationWarning('This is no longer available (PV CYCLE 2.0)')
        remove_for_future = PvcTbsMemberPVModule.IS_INACTIVE if remove_for_future else None
        order = self._controller.remove_line(_id, line_id, remove_for_future)
        return order.id

    def remove_contribution_line(self, _id, line_id, deactivate=False):
        order = self._contrib_controller.remove_line(_id, line_id, deactivate)
        return order.id

    def send_declaration_reminder_email(self, declaration_ids):
        if not self._controller.has_permissions('pvc_version_1_access'):
            raise DeprecationWarning('This is no longer available (PV CYCLE 2.0)')
        self._controller.send_declaration_reminder_email(declaration_ids)

    def _order_type(self, order_id):
        try:
            order = self._controller.get(order_id)[0].type
        except IndexError:
            order = self._contrib_controller.get(order_id)[0].type

    def get_available_countries(self, order_id, bitbucket):
        order = self._contrib_controller.get(order_id).pop()
        return [{'label': x.name, 'value': x.id} for x in self._contrib_controller.get_tbs_member_countries(order.producer)]

    def get_available_countries_for_tbs_member(self, tbs_member_id):
        tbs_member = self._contrib_controller.get_tbs_member_licenses(tbs_member_id).pop()
        return [{'label': x.name, 'value': x.id} for x in self._contrib_controller.get_tbs_member_countries(tbs_member)]

    def request_for_resubmission(self, order_id):
        if self._order_type(order_id) == 'PvcDeclarationOrder':
            if not self._controller.has_permissions('pvc_version_1_access'):
                raise DeprecationWarning('This is no longer available (PV CYCLE 2.0)')
            self._controller.request_for_resubmission(order_id)
        else:
            self._contrib_controller.request_for_resubmission(order_id)

    def approve_resubmission(self, order_id):
        if self._order_type(order_id) == 'PvcDeclarationOrder':
            if not self._controller.has_permissions('pvc_version_1_access'):
                raise DeprecationWarning('This is no longer available (PV CYCLE 2.0)')
            self._controller.approve_resubmit_request(order_id)
        else:
            self._contrib_controller.approve_resubmit_request(order_id)

    def reject_resubmission(self, order_id, reason):
        if self._order_type(order_id) == 'PvcDeclarationOrder':
            if not self._controller.has_permissions('pvc_version_1_access'):
                raise DeprecationWarning('This is no longer available (PV CYCLE 2.0)')
            self._controller.reject_resubmit_request(order_id, reason)
        else:
            self._contrib_controller.reject_resubmit_request(order_id)

    def cancel_resubmission(self, order_id):
        if self._order_type(order_id) == 'PvcDeclarationOrder':
            self._controller.cancel_resubmit_request(order_id)
        else:
            self._contrib_controller.cancel_resubmit_request(order_id)

handlers = (DeclarationHandler,)
