'''
Created on Oct 8, 2013

@author: marten
'''
import datetime
from sqlalchemy import Integer, Column, String, ForeignKey, SmallInteger, DateTime, DECIMAL, event, Boolean, Numeric
from sqlalchemy.orm import mapper, relation

from basesoft.sqlalchemy.table import IQTable
from basesoft.web.contrib.license import models as license_models

from pvcycle.order import models
from pvcycle.order.hooks import pvc_order_listener
from pvcycle.order.models import PvcDeclarationReminderMail
from basesoft.web.contrib.license.hooks import PrivilegeObjectMapper

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    return tables

def do_tables(metadata):
    """
    This function is called by Django middleware when needed.
    """
    global __tables_defined
    if __tables_defined:
        return

    aggPvcDeclaration = IQTable('aggPvcDeclaration', metadata,
        Column('id', Integer, primary_key=True, autoincrement=True),
        Column('order_id', Integer, index=True, nullable=False),
        Column('order_line_id', Integer, index=True, nullable=False),
        Column('order_type', String(64), index=True, nullable=False),
        Column('order_status', SmallInteger, nullable=False),
        Column('invoice_type', String(32), index=True, nullable=False),
        Column('country_id', Integer, index=True, nullable=False),
        Column('year', SmallInteger, index=True, nullable=False),
        Column('quarter', SmallInteger, index=True, nullable=True),
        Column('month', SmallInteger, index=True, nullable=True),
        Column('tbs_id', Integer, index=True, nullable=False),
        Column('member_id', Integer, index=True, nullable=True),
        Column('producer_id', Integer, index=True, nullable=True),
        Column('pv_module_id', Integer, index=True, nullable=True),
        Column('supplier_id', Integer, index=True, nullable=True),
        Column('submitted_time_id', Integer, index=True, nullable=True),
        Column('payment_status', Integer, index=True, nullable=False),
        Column('payment_method', String(64), index=True, nullable=False),
        Column('payment_time_id', Integer, index=True, nullable=True),
        Column('gl_account', String(64), index=True, nullable=False),
        Column('quantity', Integer, index=True, nullable=True),
        Column('weight', DECIMAL, index=True, nullable=True),
        Column('power', DECIMAL, index=True, nullable=True),
        Column('line_amount_excluding_vat', DECIMAL(20,5), index=True, nullable=True),
        Column('line_amount_including_vat', DECIMAL(20,5), index=True, nullable=True),
        Column('price_per_unit', DECIMAL(20,5), index=True, nullable=True),
        Column('last_updated', DateTime(), index=True, nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
    )

    aggPvcContribution = IQTable('aggPvcContribution', metadata,
        Column('id', Integer, primary_key=True, autoincrement=True),
        Column('order_id', Integer, index=True, nullable=False),
        Column('order_line_id', Integer, index=True, nullable=False),
        Column('order_type', String(64), index=True, nullable=False),
        Column('order_status', SmallInteger, nullable=False),
        Column('invoice_type', String(32), index=True, nullable=False),
        Column('country_id', Integer, index=True, nullable=False),
        Column('year', SmallInteger, index=True, nullable=False),
        Column('quarter', SmallInteger, index=True, nullable=True),
        Column('month', SmallInteger, index=True, nullable=True),
        Column('tbs_id', Integer, index=True, nullable=False),
        Column('member_id', Integer, index=True, nullable=True),
        Column('producer_id', Integer, index=True, nullable=True),
        Column('commissioner_id', Integer, index=True, nullable=True),
        Column('submitted_time_id', Integer, index=True, nullable=True),
        Column('payment_status', Integer, index=True, nullable=False),
        Column('payment_method', String(64), index=True, nullable=False),
        Column('payment_time_id', Integer, index=True, nullable=True),
        Column('gl_account', String(64), index=True, nullable=False),
        Column('product_id', Integer, index=True, nullable=True),
        Column('local_product_id', Integer, index=True, nullable=True),
        Column('brand_id', Integer, index=True, nullable=True),
        Column('quantity', Integer, index=True, nullable=True),
        Column('weight', DECIMAL, index=True, nullable=True),
        Column('nominal_power', DECIMAL, index=True, nullable=True),
        Column('surface', DECIMAL, index=True, nullable=True),
        Column('line_amount_excluding_vat', DECIMAL(20,5), index=True, nullable=True),
        Column('line_amount_including_vat', DECIMAL(20,5), index=True, nullable=True),
        Column('price_per_unit', DECIMAL(20,5), index=True, nullable=True),
        Column('last_updated', DateTime(), index=True, nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
    )

    tblPvcOrder = IQTable('tblPvcOrder', metadata,
        Column('order_id', Integer, ForeignKey('tblOrder.id', ondelete='CASCADE'), primary_key=True),
        Column('tbs_member_id', Integer, ForeignKey('tblPvcTbsMember.id', ondelete='CASCADE'), nullable=False),
    )

    tblPvcOrderLineAdministrationFee = IQTable('tblPvcOrderLineAdministrationFee', metadata,
        Column('order_line_id', Integer, ForeignKey('tblOrderLine.id', ondelete='CASCADE'), primary_key=True),
        Column('tbs_member_service_level_id', Integer, ForeignKey('tblPvcTbsMemberServiceLevel.id'), nullable=False),
    )

    tblPvcOrderLineMembershipFee = IQTable('tblPvcOrderLineMembershipFee', metadata,
        Column('order_line_id', Integer, ForeignKey('tblOrderLine.id', ondelete='CASCADE'), primary_key=True),
        Column('tbs_member_membership_id', Integer, ForeignKey('tblPvcTbsMemberMembership.id'), nullable=False),
    )

    tblPvcDeclarationOrder = IQTable('tblPvcDeclarationOrder', metadata,
        Column('order_id', ForeignKey('tblOrder.id', ondelete='CASCADE'), primary_key=True),
        Column('tbs_member_id', Integer, ForeignKey('tblPvcTbsMember.id', ondelete='CASCADE'), nullable=False),
        Column('producer_id', Integer, ForeignKey('tblPvcTbsMember.id', ondelete='CASCADE'), nullable=False),

        Column('resubmit_request', SmallInteger, nullable=True),
        Column('is_compliant', SmallInteger, nullable=True, default=None),

        Column('year', SmallInteger, nullable=False),
        Column('month', SmallInteger, nullable=True),
        Column('quarter', SmallInteger, nullable=True),
    )

    tblPvcDeclarationOrderLine = IQTable('tblPvcDeclarationOrderLine', metadata,
        Column('order_line_id', Integer, ForeignKey('tblOrderLine.id', ondelete='CASCADE'), primary_key=True),
        Column('tbs_member_module_id', Integer, ForeignKey('tblPvcTbsMemberPVModule.id', ondelete='CASCADE'), nullable=False),
    )

    tblPvcDeclarationOrderLineMinimumCF = IQTable('tblPvcDeclarationOrderLineMinimumCF', metadata,
        Column('order_line_id', Integer, ForeignKey('tblOrderLine.id', ondelete='CASCADE'), primary_key=True),
    )

    tblPvcDeclarationReminderMail = IQTable('tblPvcDeclarationReminderMail', metadata,
        Column('mail_id', Integer, ForeignKey('tblMail.id', ondelete='CASCADE'), primary_key=True),
        Column('fallback_locale_id', Integer, ForeignKey('tblLocale.id'), nullable=False),
    )

    tblPvcContributionOrder = IQTable('tblPvcContributionOrder', metadata,
        Column('order_id', ForeignKey('tblOrder.id', ondelete='CASCADE'), primary_key=True),
        Column('tbs_member_id', Integer, ForeignKey('tblPvcTbsMember.id', ondelete='RESTRICT'), nullable=False),
        Column('producer_id', Integer, ForeignKey('tblPvcTbsMember.id', ondelete='RESTRICT'), nullable=False),

        Column('resubmit_request', SmallInteger, nullable=True),
        Column('is_compliant', SmallInteger, nullable=True, default=None),
        Column('is_forecast', Boolean, nullable=True, default=False),
        Column('is_guarantee_forecast', Boolean, nullable=True, default=False),
        Column('year', SmallInteger, nullable=False),
        Column('month', SmallInteger, nullable=True),
        Column('quarter', SmallInteger, nullable=True),
    )

    tblPvcContributionOrderLine = IQTable('tblPvcContributionOrderLine', metadata,
        Column('order_line_id', Integer, ForeignKey('tblOrderLine.id', ondelete='CASCADE'), primary_key=True),
        Column('tbs_member_product_id', Integer, ForeignKey('tblPvcTbsMemberProduct.id', ondelete='RESTRICT'), nullable=False),
        Column('brand_id', Integer, ForeignKey('tblPvcBrand.id', ondelete='RESTRICT'), nullable=True, default=None),
        Column('weight', Numeric(20, 5), nullable=True),
        Column('custom_unit', Numeric(20, 5), nullable=True),
        Column('is_b2b', Boolean), # null = Not supplied, 0 = b2c, 1 = b2b
    )

    tables['aggPvcDeclaration'] = aggPvcDeclaration
    tables['aggPvcContribution'] = aggPvcContribution

    tables['tblPvcOrder'] = tblPvcOrder
    tables['tblPvcOrderLineAdministrationFee'] = tblPvcOrderLineAdministrationFee # Registrations
    tables['tblPvcOrderLineMembershipFee'] = tblPvcOrderLineMembershipFee

    tables['tblPvcDeclarationOrder'] = tblPvcDeclarationOrder
    tables['tblPvcDeclarationOrderLine'] = tblPvcDeclarationOrderLine
    tables['tblPvcDeclarationOrderLineMinimumCF'] = tblPvcDeclarationOrderLineMinimumCF

    tables['tblPvcDeclarationReminderMail'] = tblPvcDeclarationReminderMail

    tables['tblPvcContributionOrder'] = tblPvcContributionOrder
    tables['tblPvcContributionOrderLine'] = tblPvcContributionOrderLine

    __tables_defined = True

def do_mappings():
    """
    This function is called by Django middleware when needed.
    """
    global __mapping_defined
    if __mapping_defined:
        return

    from basesoft.web.contrib.i18n import models as i18n_models
    from basesoft.web.contrib.mailing import models as mailing_models
    from basesoft.web.contrib.mailing.sa_datamapper import get_tables as mail_tables

    from basesoft.web.contrib.order import models as order_models
    from pvcycle.core import models as core_models
    from pvcycle.core.sa_datamapper import get_tables as core_tables
    from pvcycle.pvmodule import models as pvmodule_models
    from pvcycle.product import models as product_models
    from basesoft.web.contrib.notification import models as notification_models

    mapper(models.PvcOrder, tables['tblPvcOrder'],
        inherits=order_models.Order,
        properties = {
            'tbs_member': relation(core_models.PvcTbsMember)
        },
        polymorphic_identity='PvcOrder'
    )
    event.listen(models.PvcOrder, 'after_insert', pvc_order_listener)

    mapper(models.PvcOrderLineAdministrationFee, tables['tblPvcOrderLineAdministrationFee'],
        inherits=order_models.OrderLine,
        properties = {
            'tbs_member_service_level': relation(core_models.PvcTbsMemberServiceLevel)
        },
        polymorphic_identity='PvcOrderLineAdministrationFee'
    )

    mapper(models.PvcOrderLineMembershipFee, tables['tblPvcOrderLineMembershipFee'],
        inherits=order_models.OrderLine,
        properties = {
            'tbs_member_membership': relation(core_models.PvcTbsMemberMembership)
        },
        polymorphic_identity='PvcOrderLineMembershipFee'
    )

    mapper(models.PvcDeclarationOrder, tables['tblPvcDeclarationOrder'],
        inherits=order_models.Order,
        properties = {
            'tbs_member': relation(core_models.PvcTbsMember, primaryjoin=tables['tblPvcDeclarationOrder'].c.tbs_member_id==core_tables()['tblPvcTbsMember'].c.id),
            'producer': relation(core_models.PvcTbsMember, primaryjoin=tables['tblPvcDeclarationOrder'].c.producer_id==core_tables()['tblPvcTbsMember'].c.id),
        },
        polymorphic_identity='PvcDeclarationOrder'
    )

    mapper(models.PvcDeclarationOrderLine, tables['tblPvcDeclarationOrderLine'],
        inherits=order_models.OrderLine,
        properties = {
            'tbs_member_module': relation(pvmodule_models.PvcTbsMemberPVModule)
        },
        polymorphic_identity='PvcDeclarationOrderLine'
    )

    mapper(models.PvcDeclarationOrderLineMinimumCF, tables['tblPvcDeclarationOrderLineMinimumCF'],
        inherits=order_models.OrderLine,
        polymorphic_identity='PvcDeclarationOrderLineMinimumCF'
    )

    event.listen(models.PvcDeclarationOrder, 'after_insert', pvc_order_listener)

    mapper(PvcDeclarationReminderMail, tables['tblPvcDeclarationReminderMail'],
        extension = PrivilegeObjectMapper(),
        properties = {
            'fallback_locale': relation(i18n_models.Locale),
            'locales': relation(mailing_models.MailLocale, order_by=mail_tables()['tblMailLocale'].c.locale_id, passive_deletes=True)
        },
        polymorphic_identity='PvcDeclarationReminderMail',
        inherits=mailing_models.Mail
    )

    mapper(models.PvcDeclarationNotification,
           inherits=notification_models.Notification,
           polymorphic_identity='PvcDeclarationNotification'
    )

    mapper(models.PvcContributionOrder, tables['tblPvcContributionOrder'],
        inherits=order_models.Order,
        properties = {
            'tbs_member': relation(core_models.PvcTbsMember, primaryjoin=tables['tblPvcContributionOrder'].c.tbs_member_id==core_tables()['tblPvcTbsMember'].c.id),
            'producer': relation(core_models.PvcTbsMember, primaryjoin=tables['tblPvcContributionOrder'].c.producer_id==core_tables()['tblPvcTbsMember'].c.id),
        },
        polymorphic_identity='PvcContributionOrder'
    )

    event.listen(models.PvcContributionOrder, 'after_insert', pvc_order_listener)

    mapper(models.PvcContributionOrderLine, tables['tblPvcContributionOrderLine'],
        inherits=order_models.OrderLine,
        properties = {
            'tbs_member_product': relation(product_models.PvcTbsMemberProduct),
            'brand': relation(product_models.PvcBrand),
        },
        polymorphic_identity='PvcContributionOrderLine'
    )

    __mapping_defined = True

def fill_data(conn):
    pass

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'PVC Order')

def app_permissions():
    from django.utils.translation import ugettext as _
    return [

        license_models.Permission(u'pvc_view_orders', u"""
            This gives the right to view PVC Orders.
            """, _("View PVC orders")),

        license_models.Permission(u'pvc_view_declarations', u"""
            This gives the right to view PVC Declaration (Orders).
            """, _("View PVC Declaration orders")),

        license_models.Permission(u'pvc_declare_declarations', u"""
            This gives the right to declare PVC Declaration (Orders).
            """, _("Declare PVC Declaration orders")),

        license_models.Permission(u'pvc_manage_declaration_mail', u"""
            This gives the right to manage Declaration related mails
            """, _("Manage PVC declaration mails")),

        license_models.Permission(u'pvc_approve_declaration_resubmit', u"""
            This gives the right to approve Declaration resubmit request
            """, _("Approve PVC declaration resubmit requests")),

    ]





