'''
Created on Nov 4, 2013

@author: marten
'''
from basesoft.scheduler.task import ScheduleTask
from basesoft.scheduler.crontab import CronTab
from pvcycle.order.controllers import PvcDeclarationOrderController,\
    PvcDeclarationAggregator, PvcOrderController, PvcContributionOrderController, PvcContributionAggregator
from pvcycle.core.models import PvcTbsLicense
from iqportal.iqube.core.schedule import AbstractAggregateTask
from django.conf import settings

from basesoft.util.virtual_date import set_date, today


class NewDeclarationTask(ScheduleTask):
    """
    triggers new declarations
    @deprecated with PV CYCLE 2.0
    """

    # runs every 1st of the month at 00:05
    INTERVAL = CronTab('05 0 1 * *')

    def execute(self):

        pc = PvcDeclarationOrderController(self.sa_session)

        tbss = self.sa_session.query(PvcTbsLicense).filter(PvcTbsLicense.is_active==True).all()
        for tbs in tbss:
            pc.trigger_new_declarations(tbs)

class YearlyFeeTask(ScheduleTask):
    """
    triggers new declarations
    """

    # runs every day in january at 03:15
    INTERVAL = CronTab('15 3 * 1 *')

    def execute(self):

        tbss = self.sa_session.query(PvcTbsLicense).filter(PvcTbsLicense.is_active==True).all()
        for tbs in tbss:
            oc = PvcOrderController(self.sa_session)
            oc.trigger_yearly_administrative_fees(tbs)


class CloseDeclarationTask(ScheduleTask):
    """
    closes declarations
    @deprecated with PV CYCLE 2.0
    """

    # runs every day at 00:20
    INTERVAL = CronTab('20 0 * * *')

    def execute(self):

        pc = PvcDeclarationOrderController(self.sa_session)

        tbss = self.sa_session.query(PvcTbsLicense).filter(PvcTbsLicense.is_active==True).all()
        for tbs in tbss:
            pc.close_declarations(tbs)


class AggregatePvcDeclarationTask(AbstractAggregateTask):
    refresh_datasources = ('%s.PvcOrderData' % settings.SETTINGS_MODULE, )

    def populate(self):
        affected_rows = 0

        c = PvcDeclarationAggregator()
        self._aggregators.append(c)
        affected_rows = affected_rows + c.run(loop_validator=self.loop_validator)

        return affected_rows


class AggregatePvcContributionTask(AbstractAggregateTask):
    refresh_datasources = ('%s.PvcOrderData' % settings.SETTINGS_MODULE, )

    def populate(self):
        affected_rows = 0

        c = PvcContributionAggregator()
        self._aggregators.append(c)
        affected_rows = affected_rows + c.run(loop_validator=self.loop_validator)

        return affected_rows


class NewContributionOrderTask(ScheduleTask):
    # runs every 1st of the month at 00:05
    INTERVAL = CronTab('05 0 1 * *')

    def execute(self):
        pc = PvcContributionOrderController(self.sa_session)

        tbss = self.sa_session.query(PvcTbsLicense).filter(PvcTbsLicense.is_active==True).all()
        for tbs in tbss:
            pc.trigger_new_contributions(tbs)


class CloseContributionTask(ScheduleTask):
    """
    closes declarations
    @deprecated with PV CYCLE 2.0
    """

    # runs every day at 00:20
    INTERVAL = CronTab('20 0 * * *')

    def execute(self):

        pc = PvcContributionOrderController(self.sa_session)

        tbss = self.sa_session.query(PvcTbsLicense).filter(PvcTbsLicense.is_active==True).all()
        for tbs in tbss:
            pc.close_contributions(tbs)


tasks = (AggregatePvcContributionTask, YearlyFeeTask, NewContributionOrderTask, CloseContributionTask, )
