# Copyright (c) 2015 BaseSoft B.V.
# See LICENSE for details.

def procedures():

    # Function pvcGetCurrentFee
    # Returns the current Fee for the given service_level

    return [

        """
        DROP FUNCTION IF EXISTS `pvcGetCurrentProductContributionFee`
        """,
        """
        CREATE FUNCTION `pvcGetCurrentProductContributionFee`(
            in_product_id INT,
            in_service_level_id INT,
            in_start_date DATE,
            in_dimension INT )
            RETURNS DECIMAL(10,5)
            READS SQL DATA
        BEGIN
            DECLARE fee DECIMAL(10,5);
            SET fee = NULL;

            SELECT f.fee INTO fee
            FROM tblPvcTbsProductFee f
            WHERE
                f.`tbs_product_id` = in_product_id AND
                f.`service_level_id` = in_service_level_id AND
                f.`start_date` <= in_start_date AND
                f.`dimension` = in_dimension
            ORDER BY f.`start_date` DESC
            LIMIT 1;

            RETURN(fee);

        END
        """,

        """
        DROP FUNCTION IF EXISTS `pvcGetCurrentProductContributionFeeDate`
        """,
        """
        CREATE FUNCTION `pvcGetCurrentProductContributionFeeDate`(
            in_product_id INT,
            in_service_level_id INT,
            in_start_date DATE,
            in_dimension INT )
            RETURNS DATE
            READS SQL DATA
        BEGIN
            DECLARE rv_date DATE;
            DECLARE CONTINUE HANDLER FOR SQLSTATE '02000' BEGIN END;
            SET rv_date = NULL;

            SELECT f.start_date INTO rv_date
            FROM tblPvcTbsProductFee f
            WHERE
                f.`tbs_product_id` = in_product_id AND
                f.`service_level_id` = in_service_level_id AND
                f.`start_date` <= in_start_date AND
                f.`dimension` = in_dimension
            ORDER BY f.`start_date` DESC
            LIMIT 1;

            RETURN(rv_date);

        END
        """,

        """
        DROP FUNCTION IF EXISTS `pvcGetNextProductContributionFeeDate`
        """,
        """
        CREATE FUNCTION `pvcGetNextProductContributionFeeDate`(
            in_product_id INT,
            in_service_level_id INT,
            in_start_date DATE,
            in_dimension INT )
            RETURNS DATE
            READS SQL DATA
        BEGIN
            DECLARE next_date DATE;
            DECLARE CONTINUE HANDLER FOR SQLSTATE '02000' BEGIN END;
            SET next_date = NULL;

            SELECT f.start_date INTO next_date
            FROM tblPvcTbsProductFee f
            WHERE
                f.`tbs_product_id` = in_product_id AND
                f.`service_level_id` = in_service_level_id AND
                f.`start_date` > in_start_date AND
                f.`dimension` = in_dimension
            ORDER BY f.`start_date` ASC
            LIMIT 1;

            RETURN(next_date);

        END
        """
    ]