# Copyright (c) 2015 BaseSoft B.V.
# See LICENSE for details.
from django.utils.translation import ugettext_lazy as _

from basesoft.web.contrib.account.decorators import assert_user_logged_in
from pvcycle.core.icons import AbstractPvcIcon


class PvcTbsProductManagerIcon(AbstractPvcIcon):
    id = 'icon-pvc-tbs-product-manager'
    title = _('TBS product manager')
    icon_index = 100
    component = 'pvc_tbs_product_manager'
    group = 'product'
    group_title = _('PVC Product')
    colorname = 'iconbar-cerulean'  # FIXME

    char_main = u'\uE805'  # FIXME
    char_sub = u'\uE806'  # FIXME

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('manage_pvc_product_levels')
        self._assert_pvc_license(controller)


class PvcTbsProductSelectIcon(AbstractPvcIcon):
    id = 'icon-pvc-tbs-product-select'
    title = _('TBS product select')
    icon_index = 101
    component = 'pvc_tbs_product_select'
    group = 'product'
    group_title = _('PVC Product')
    colorname = 'iconbar-cerulean'  # FIXME

    char_main = u'\uE805'  # FIXME
    char_sub = u'\uE806'  # FIXME

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('manage_pvc_product_levels')
        self._assert_pvc_license(controller)


class PvcTbsMemberProductManagerIcon(AbstractPvcIcon):
    id = 'icon-pvc-tbs-member-product-manager'
    title = _('Member product manager')
    icon_index = 100
    component = 'pvc_tbs_member_product_manager'
    group = 'product'
    group_title = _('PVC Product')
    colorname = 'iconbar-cerulean'  # FIXME

    char_main = u'\uE805'  # FIXME
    char_sub = u'\uE806'  # FIXME

    @assert_user_logged_in
    def assert_permissions(self, controller):  # TODO: make available for TBS/Operator
        controller.assert_permissions('pvc_member_license')
        self._assert_pvc_license(controller)


class PvcBrandManagerIcon(AbstractPvcIcon):
    id = 'icon-brand-manager'
    title = _('Brand manager')
    icon_index = 102
    component = 'pvc_brand_manager'
    group = 'product'
    group_title = _('PVC Product')
    colorname = 'iconbar-cerulean'  # FIXME

    char_main = u'\uE805'  # FIXME
    char_sub = u'\uE806'  # FIXME

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('manage_pvc_brands')
        self._assert_pvc_license(controller)


def get_icons():
    return [
        PvcTbsProductManagerIcon(),
        PvcTbsProductSelectIcon(),
        PvcTbsMemberProductManagerIcon(),
        PvcBrandManagerIcon(),
    ]