# Copyright (c) 2015 BaseSoft B.V.
# See LICENSE for details.
from pvcycle.product.controllers import PvcProductController


class ProductHandler:
    def __init__(self, request, **kwargs):

        self._request = request
        self._sa_session = request.Session()
        self._account_id = request.session['user_id']
        self._controller = PvcProductController(self._sa_session, account_id=self._account_id, request=request)

    def get_available_products_for_tbs(self, tbs_id, parent_level_id=None, recursive=False):
        return self._controller.get_available_products_for_tbs(tbs_id, parent_level_id, recursive=recursive)

    def get_product_for_tbs(self, tbs_id, product_id):
        return self._controller.get_products_for_tbs(tbs_id, product_id)

    def save_tbs_product(self, _id, values):
        self._controller.save_or_update_tbs_product(_id, **values)
        return True

    def save_tbs_products(self, tbs_id, values):

        self._controller.save_products_for_tbs(tbs_id, values)

    def load_tbs_product(self, _id):
        p = self._controller.get_tbs_product(_id)
        if p is None:
            return {}
        return {
            'tbs_id': p.tbs.id,
            'parent_level_id': p.product.parent_level_id,
            'weee_label': p.product.label,
            'weee_description': p.product.description,
            'label': p.label,
            'description': p.description,
            'basis_for_invoicing': p.basis_for_invoicing
        }

    def merge_brands(self, brands, target_id):
        for brand_id in brands:
            self._controller.merge_brand_with(brand_id, target_id)
        return True

    def cancel_merge(self, brand_id):
        self._controller.cancel_merge(brand_id)
        return True

    def rename_brand(self, brand_id, name):
        self._controller.update_brand_name(brand_id, name)
        return True

    def get_brands(self, _like=''):
        brands = self._controller.get_brands_by_name(_like)
        return [b.name for b in brands]

    def load_product_fees(self, tbs_id, period, service_level_id):
        return self._controller.get_product_fees(tbs_id, service_level_id, period)

    def save_product_fees(self, tbs_id, period, service_level_id, values):
        self._controller.save_product_fees(tbs_id, service_level_id, period, values)
        return True

    def get_tbs_member_products(self, member_id, tbs_id, subrogation_id=None):
        tp, tmp = self._controller.get_tbs_member_products(tbs_id, member_id, subrogation_id, validate_types=True)
        rv = {
            'selectedOptions': [{'value': p.id, 'label': '%s - %s' % (p.label, p.description)} for p in tmp],
            'defaultOptions': [{'value': p.id, 'label': '%s - %s' % (p.label, p.description)} for p in tp if p not in tmp],
        }
        return rv

    def save_tbs_member_products(self, _id, values):
        self._controller.save_tbs_member_products(**values)
        return True

    def deactivate_member_brand(self, tbs_member_product_id, brand_id):
        self._controller.deactivate_product_brand(tbs_member_product_id, brand_id)
        return True

    def deactivate_tbs_member_product(self, tbs_member_product_id):
        self._controller.deactivate_tbs_member_product(tbs_member_product_id)
        return True

    def reactivate_tbs_member_product(self, tbs_member_product_id):
        self._controller.reactivate_tbs_member_product(tbs_member_product_id)
        return True

    def has_brands(self, tbs_id):
        tbs = self._controller._get_take_back_schemes(tbs_id).pop()
        return tbs.get_config('pvc.tbs.contribution.split.by.brand')

handlers = (ProductHandler, )
