# Copyright (c) 2015 BaseSoft B.V.
# See LICENSE for details.
from basesoft.scheduler.task import ScheduleTask
from pvcycle.order.controllers import PvcContributionOrderController
from pvcycle.product.controllers import PvcProductController
from pvcycle.product.models import PvcTbsProductFee


class MergeBrandTask(ScheduleTask):
    """
    Background action for merging brands.
    """
    INTERVAL = 7200  # once every two hours

    def execute(self):
        pc = PvcProductController(self.sa_session)
        brands = pc.mark_merge_brands_as_processing()
        if brands is None:
            return

        pc.merge_brands(brands)


class AlterFeeTask(ScheduleTask):
    INTERVAL = 1800  # every 30 minutes

    def execute(self):
        fees = self.sa_session.query(PvcTbsProductFee).filter(PvcTbsProductFee.process_status == PvcTbsProductFee.STATUS_PROCESS).all()
        for pf in fees:
            pf.process_status = PvcTbsProductFee.STATUS_PROCESSING
            self.sa_session.add(pf)
        self.sa_session.flush()

        cc = PvcContributionOrderController(self.sa_session)

        for pf in fees:
            cc.update_with_new_fee(pf)
            pf.process_status = PvcTbsProductFee.STATUS_DONE
            self.sa_session.add(pf)

        self.sa_session.flush()


tasks = (MergeBrandTask, AlterFeeTask, )
