'''
Created on Aug 26, 2013

@author: marten
'''

import datetime
from sqlalchemy.orm import mapper, relation
from sqlalchemy import Column, Integer, Unicode, DateTime, ForeignKey, SmallInteger, UniqueConstraint, Numeric

from basesoft.sqlalchemy.table import IQTable
from basesoft.web.contrib.license import models as license_models

from pvcycle.pvmodule import models as pvmodule_models

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    return tables

def do_tables(metadata):
    """
    This function is called by Django middleware when needed.
    """
    global __tables_defined
    if __tables_defined:
        return

    tblPvcPVModuleSupplier = IQTable('tblPvcPVModuleSupplier', metadata,
        Column('id', Integer, primary_key=True, autoincrement=True),
        Column('name', Unicode(255), nullable=False, unique=True),
        Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
        Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
    )

    tblPvcPVModuleType = IQTable('tblPvcPVModuleType', metadata,
        Column('id', Integer, primary_key=True, autoincrement=True),
        Column('name', Unicode(255), nullable=False, unique=True),
        Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
        Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
    )

    tblPvcPVModuleCellType = IQTable('tblPvcPVModuleCellType', metadata,
        Column('id', Integer, primary_key=True, autoincrement=True),
        Column('name', Unicode(255), nullable=False, unique=True),
        Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
        Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
    )

    tblPvcPVModule = IQTable('tblPvcPVModule', metadata,
        Column('id', Integer, primary_key=True, autoincrement=True),
        Column('photon_id', Integer, nullable=True, default=None),

        Column('supplier_id', Integer, ForeignKey('tblPvcPVModuleSupplier.id', ondelete='RESTRICT'), nullable=False),
        Column('type_id', Integer, ForeignKey('tblPvcPVModuleType.id', ondelete='RESTRICT'), nullable=False),
        Column('cell_type_id', Integer, ForeignKey('tblPvcPVModuleCellType.id', ondelete='RESTRICT'), nullable=False),

        Column('approval_status', SmallInteger, nullable=False, default=pvmodule_models.PvcPVModule.PV_MODULE_STATUS_PENDING),
        Column('first_approval_by_id', Integer, ForeignKey('tblAccount.id'), nullable=True),
        Column('second_approval_by_id', Integer, ForeignKey('tblAccount.id'), nullable=True),

        Column('nominal_power', Numeric(20, 5), nullable=False),
        Column('weight', Numeric(20, 5), nullable=False),

        Column('length', Numeric(20, 5), nullable=False),
        Column('width', Numeric(20, 5), nullable=False),

        Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
        Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        UniqueConstraint('supplier_id', 'type_id')
    )

    # Used for scheduling/creating new Declaration Orders
    tblPvcTbsMemberPVModule = IQTable('tblPvcTbsMemberPVModule', metadata,
        Column('id', Integer, primary_key=True, autoincrement=True), # used as backref for Declaration Line
        Column('tbs_member_id', Integer, ForeignKey('tblPvcTbsMember.id', ondelete='CASCADE'), nullable=False),
        Column('country_id', Integer, ForeignKey('tblCountry.id', ondelete='CASCADE'), nullable=False),
        Column('module_id', Integer, ForeignKey('tblPvcPVModule.id', ondelete='CASCADE'), nullable=False),
        Column('subrogation_id', Integer, ForeignKey('tblPvcSubrogation.id', ondelete='CASCADE'), nullable=True),
        Column('is_active', SmallInteger, nullable=False, default=pvmodule_models.PvcTbsMemberPVModule.IS_ACTIVE),

        Column('commissioner_approval_status', SmallInteger, nullable=True, default=None),

        Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
        Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),

        # IMPORTANT: Please note that Subrogation can be NULL, that combo should be checked in business logic
        UniqueConstraint('tbs_member_id', 'country_id', 'module_id', 'subrogation_id')
    )

    tables['tblPvcPVModuleSupplier'] = tblPvcPVModuleSupplier
    tables['tblPvcPVModuleType'] = tblPvcPVModuleType
    tables['tblPvcPVModuleCellType'] = tblPvcPVModuleCellType
    tables['tblPvcPVModule'] = tblPvcPVModule
    tables['tblPvcTbsMemberPVModule'] = tblPvcTbsMemberPVModule

    __tables_defined = True

def do_mappings():
    """
    This function is called by Django middleware when needed.
    """
    global __mapping_defined
    if __mapping_defined:
        return

    from pvcycle.core import models as core_models
    from pvcycle.subrogation import models as subrogation_models
    from basesoft.web.contrib.account import models as account_models
    from basesoft.web.contrib.account.sa_datamapper import get_tables as account_tables
    from basesoft.web.contrib.i18n import models as i18n_models

    mapper(pvmodule_models.PvcPVModuleSupplier, tables['tblPvcPVModuleSupplier'])
    mapper(pvmodule_models.PvcPVModuleType, tables['tblPvcPVModuleType'])
    mapper(pvmodule_models.PvcPVModuleCellType, tables['tblPvcPVModuleCellType'])

    mapper(pvmodule_models.PvcPVModule, tables['tblPvcPVModule'],
        properties = {
            'first_approval_by': relation(account_models.Account, primaryjoin=tables['tblPvcPVModule'].c.first_approval_by_id==account_tables()['tblAccount'].c.id),
            'second_approval_by': relation(account_models.Account, primaryjoin=tables['tblPvcPVModule'].c.second_approval_by_id==account_tables()['tblAccount'].c.id),
            'supplier': relation(pvmodule_models.PvcPVModuleSupplier),
            'type': relation(pvmodule_models.PvcPVModuleType),
            'cell_type': relation(pvmodule_models.PvcPVModuleCellType),
        }
    )

    mapper(pvmodule_models.PvcTbsMemberPVModule, tables['tblPvcTbsMemberPVModule'],
        properties = {
            'tbs_member': relation(core_models.PvcTbsMember, backref='modules'),
            'country': relation(i18n_models.Country),
            'module': relation(pvmodule_models.PvcPVModule),
            'subrogation': relation(subrogation_models.PvcSubrogation)
        }
    )

    __mapping_defined = True

def fill_data(conn):
    pass

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'PVC PV Modules')

def app_permissions():
    from django.utils.translation import ugettext as _
    return [

        license_models.Permission(u'pvc_manage_pv_modules', u"""
            This gives the right to manage PV Modules
            """, _("Manage PVC PV Modules")),

        license_models.Permission(u'pvc_approve_pv_modules', u"""
            This gives the right to approve PV Modules
            """, _('Approve PVC PV Modules')),

        license_models.Permission(u'pvc_create_pv_modules', u"""
            This gives the right to create PV Modules
            """, _("Create PVC PV Modules")),

        license_models.Permission(u'pvc_select_pv_modules', u"""
            This gives the right to select PV Modules for Declaration
            """, _("Select PVC PV Modules")),

        license_models.Permission(u"pvc_import_pv_modules", u"""
            This gives the right to import PV Modules from the Photon DB
            """, _("Import PVC PV Modules"))

    ]
