# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext as _

from iqportal.iqube.core.db_sync import AbstractConfigurableWidget


class HistoricalResultWidget(AbstractConfigurableWidget):
    category = 'PVC Report'
    cube = 'HistoricalDataCube'
    datasource = 'PVCHistoricalData'
    label = _('Historical data result')

    def config(self):
        return {
            "reporter": "iqportal.iqube.core.ConfigurableWidgetReporter",
            "category": "Report",

            "has_drill_through": False,
            "show_hide_empty": True,
            "transposable": True,
            "datasource_id": 3,
            "label": "Historical data result",
            "show_excel_export": True,
            "has_drill_position": False,
            "cube": "HistoricalDataCube",
            "has_graph": True,
            "id": 3,
            "table_object_name": "datatablecube",
            "table_module": "iqreport_graph_datatablecube",
            "has_table": True,
            "graph_config": {},
            "has_drill_replace": False,

            "dimensions": [{
                "caption": "TBS",
                "available_on_rows": True,
                "show_named_sets": False,
                "has_select_as_group": True,
                "available_as_split_by": False,
                "required": False,
                "available_as_slicer": True,
                "fancy_select_module": "iq_fancyselect",
                "available_on_columns": True,
                "show_calculated_members": False,
                "children": [{
                        "unique_name": "[TBS].[TBS]",
                        "has_children_available": True,
                        "caption": "TBS",
                        "has_all_member_available": True,
                        "show_members": True,
                        "method_argument": "[TBS].[TBS]",
                        "method": "get_level_members"
                }]

                },  {
                    "caption": "Member",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Member].[Member]",
                            "has_children_available": True,
                            "caption": "Member",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Member].[Member]",
                            "method": "get_level_members"
                    }]
                }, {
                    "caption": "Country",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "fancy_select_optionsRpcUrl": "pvcycle.report.ReportDimensionHandler",
                    "fancy_select_RpcMethod": "get_efta_countries",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Country].[Country]",
                            "has_children_available": True,
                            "caption": "Country",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Country].[Country]",
                            "method": "get_level_members"
                    }]
                }, {
                    "caption": "Year",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                        "children": [{
                            "unique_name": "[Year].[Year]",
                            "has_children_available": True,
                            "caption": "Year",
                            "has_all_member_available": True,
                            "show_members": True,
                            "method_argument": "[Year].[Year]",
                            "method": "get_level_members"
                    }]
                },
                {
                    "caption": "Measures",
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": False,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                    "children": [{
                            "unique_name": "[Measures].[MeasuresLevel]",
                            "has_children_available": False,
                            "caption": "MeasuresLevel",
                            "has_all_member_available": False,
                            "show_members": True,
                            "method_argument": "[Measures].[MeasuresLevel]",
                            "method": "get_level_members"
                    }]
                },
            ],
        }

    def applied_filters(self):
        return {
            "dimensions": {
                "TBS": "0",
                "Member": "1",
                "Country": "2",
                "Year": "3",
                "Measures": "4"
            },

            "axes_0": "",
            "axes_1": "",
            "axes_2": "",
            "axes_3": "",
            "axes_4": "columns",

            "dimension_0": [],
            "dimension_1": [],
            "dimension_2": [],
            "dimension_3": [],
            "dimension_4": ["[Measures].[Quantity]"],
            
            "column_order": [
                "Measures",
            ],
            "row_order": [
            ],
            "show_table": True,
            "show_slicers": True,
            "hide_empty_fields": True,

            "initialRefresh": False,

        }

configurable_widgets = (HistoricalResultWidget,)