# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.
from sqlalchemy.orm.util import aliased
from django.utils.translation import ugettext as _

from basesoft.web.core.util.custom_query import CustomQueryModel, CustomQueryField, DATA_TYPE_INT
from basesoft.web.contrib.i18n.models import Country

from pvcycle.report.models import PvcMemberYearlyAggregatedData
from pvcycle.core.custom_query import AbstractPvcCustomQuery
from pvcycle.core.models import PvcTbsMember, PvcTbsLicense, PvcMemberLicense

Tbs = aliased(PvcTbsLicense) # we need to alias this, cuz both TbsLicense and AccountantLicense inherit License.

class HistoricalDataCustomQuery(AbstractPvcCustomQuery):

    permissions = ['pvc_manage_historical_data', 'pvc_create_historical_data'] # either.

    def _get_member_column(self):
        return PvcTbsMember.member_id

    def validate_access(self):
        if self.permissions is not None:
            self.assert_any_permissions(*self.permissions)

    def tables(self):
        return (
            CustomQueryModel(PvcMemberYearlyAggregatedData),
            CustomQueryModel(PvcTbsMember),
            CustomQueryModel(PvcMemberLicense, onclause=PvcTbsMember.member_id==PvcMemberLicense.id),
            CustomQueryModel(Tbs, onclause=PvcTbsMember.tbs_id==Tbs.id),
            CustomQueryModel(Country, onclause=PvcMemberYearlyAggregatedData.country_id==Country.id),
        )

    def fields(self):
        return (
            CustomQueryField(PvcMemberLicense.name, _('Member'), key='member', is_filter=True),
            CustomQueryField(Tbs.name, _('TBS'), key='tbs', is_filter=True, filter_options=Tbs.name),
            CustomQueryField(Country.name, _('Country'), key='country', is_filter=True, filter_options=Country.name),
            CustomQueryField(PvcMemberYearlyAggregatedData.year, _('Year'), key='year', is_filter=True, type=DATA_TYPE_INT),
            CustomQueryField(PvcMemberYearlyAggregatedData.id, _('Details'), key='details'),
            CustomQueryField(PvcMemberYearlyAggregatedData.total_megawatt, _('Total MW'), key='megawatt', is_filter=True),
            CustomQueryField(PvcMemberYearlyAggregatedData.total_quantity, _('Total quantity'), key='quantity', is_filter=True),
            CustomQueryField(PvcMemberYearlyAggregatedData.total_weight, _('Total weight (kg)'), key='weight', is_filter=True),
            CustomQueryField(PvcMemberYearlyAggregatedData.distribution_b2b, _('B2B (%)'), key='b2b', is_filter=True),
            CustomQueryField(PvcMemberYearlyAggregatedData.distribution_b2c, _('B2C (%)'), key='b2c', is_filter=True),
        )

    def set_where(self):
        self.where = PvcMemberYearlyAggregatedData.tbs_member_id.in_( [x.id for x in self._get_tbs_member_licenses(permissions=['pvc_create_historical_data'])] )

custom_queries = (HistoricalDataCustomQuery, )
