# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.

import datetime
from sqlalchemy.orm import mapper, relation
from sqlalchemy import UniqueConstraint, Column, Integer, DateTime, ForeignKey, SmallInteger, Numeric, Boolean

from basesoft.sqlalchemy.table import IQTable
from basesoft.web.contrib.license import models as license_models

from pvcycle.report import models as report_models

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    return tables

def do_tables(metadata):
    """
    This function is called by Django middleware when needed.
    """
    global __tables_defined
    if __tables_defined:
        return

    tblPvcMemberYearlyAggregatedData = IQTable('tblPvcMemberYearlyAggregatedData', metadata,
        Column('id', Integer, autoincrement=True, primary_key=True),
        Column('type', SmallInteger, nullable=False, default=report_models.PvcMemberYearlyAggregatedData.TYPE_MANUAL),
        Column('is_stale', Boolean, nullable=False, default=False),

        Column('tbs_member_id', Integer, ForeignKey('tblPvcTbsMember.id', ondelete='CASCADE'), nullable=False),
        Column('year', SmallInteger, nullable=False),
        Column('country_id', Integer, ForeignKey('tblCountry.id', ondelete='CASCADE'), nullable=False),

        Column('total_megawatt', Numeric(20,2), nullable=False),
        Column('total_quantity', Integer, nullable=False),
        Column('total_weight', Numeric(20,2), nullable=False),

        Column('distribution_b2b', Numeric(5,2), nullable=False), # max 100,00%, Business to Business
        Column('distribution_b2c', Numeric(5,2), nullable=False), # max 100,00%, Business to Consumer

        Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
        Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),

        UniqueConstraint('tbs_member_id', 'country_id', 'year', name='tbs_member_country_year'),
    )

    tables['tblPvcMemberYearlyAggregatedData'] = tblPvcMemberYearlyAggregatedData

    __tables_defined = True

def do_mappings():
    """
    This function is called by Django middleware when needed.
    """
    global __mapping_defined
    if __mapping_defined:
        return

    from basesoft.web.contrib.i18n import models as i18n_models
    from pvcycle.core import models as core_models

    mapper(report_models.PvcMemberYearlyAggregatedData, tables['tblPvcMemberYearlyAggregatedData'],
        properties = {
            'tbs_member': relation(core_models.PvcTbsMember),
            'country': relation(i18n_models.Country),
        }
    )

    __mapping_defined = True

def fill_data(conn):
    pass

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'PVC Reporting')

def app_permissions():
    from django.utils.translation import ugettext as _
    return [
        license_models.Permission(u'pvc_manage_historical_data', u"""
            This gives the right to manage Historical Data
            """, _("Manage PVC Historical Data")),

        license_models.Permission(u'pvc_create_historical_data', u"""
            This gives the right to create Historical Data
            """, _("Create PVC Historical Data")),

        license_models.Permission(u'pvc_update_historical_data', u"""
            This gives the right to create Historical Data
            """, _("Update PVC Historical Data")),

        license_models.Permission(u'pvc_delete_historical_data', u"""
            This gives the right to delete Historical Data
            """, _("Delete PVC Historical Data")),
    ]
