# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.
from django.utils.translation import ugettext_lazy as _

from basesoft.web.contrib.account.decorators import assert_user_logged_in
from pvcycle.core.controllers import PvcMemberController

from pvcycle.core.icons import AbstractPvcIcon
from pvcycle.subrogation.controllers import PvcSubrogationController
from basesoft.web.core.exceptions import SecurityException

class PvcSubrogationManagerIcon(AbstractPvcIcon):

    id = 'icon-pvc-subrogation-manager'
    title = _('Subrogation manager')
    icon_index = 81
    component = 'pvc_subrogation_manager'
    group = 'pvc_subrogation'
    group_title = _('PVC Subrogation')
    colorname = 'iconbar-purple'

    char_main = u'\uE845' # Subrogation icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        self._assert_pvc_license(controller)
        controller.assert_any_permissions('pvc_request_for_subrogation', 'pvc_manage_subrogations')

class PvcSubrogationRequestIcon(AbstractPvcIcon):

    id = 'icon-pvc-subrogation-request'
    title = _('Request for subrogation')
    icon_index = 82
    component = 'pvc_subrogation_request'
    group = 'pvc_subrogation'
    group_title = _('PVC Subrogation')
    colorname = 'iconbar-purple'

    char_main = u'\uE845' # Subrogation icon
    char_sub = u'\uE846' # Bell icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        self._assert_pvc_license(controller)
        c = PvcMemberController(controller.sa_session, account_id=controller.get_account().id)
        tms = c.get_tbs_member_licenses(permissions=['pvc_request_for_subrogation'])
        if len(tms) == 0:
            raise SecurityException('No TBS members found')
        for tm in tms:
            if tm.is_producer:
                return

        raise SecurityException('Not a producer at any TBS')

class PvcSubrogationPVModuleManagerIcon(AbstractPvcIcon):

    id = 'icon-pvc-pv-module-subrogation-manager'
    title = _('Subrogated PV modules')
    component = 'pvc_pv_module_subrogation_manager'
    icon_index = 80
    group = 'pvc_subrogation'
    group_title = _('PVC Subrogation')
    colorname = 'iconbar-purple'

    char_main = u'\uE845'
    char_sub = u'\uE80a'

    # number icon (unapproved commissioner PV Modules)
    number_type = 'important'
    number_rpc_url = 'pvcycle.subrogation.SubrogationHandler'
    number_rpc_method = 'get_unapproved_commissioner_pv_modules_count'

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('pvc_version_1_access')

        c = PvcSubrogationController(controller.sa_session, account_id=controller.get_account().id, request=controller.request)
        if not c.is_commissioner():
            raise SecurityException('Not a commissioner')

        self._assert_pvc_license(controller)


class PvcSubrogationProductManagerIcon(AbstractPvcIcon):

    id = 'icon-pvc-subrogation-product-manager'
    title = _('Subrogated products')
    component = 'pvc_subrogation_product_manager'
    icon_index = 80
    group = 'pvc_subrogation'
    group_title = _('PVC Subrogation')
    colorname = 'iconbar-purple'

    char_main = u'\uE845'
    char_sub = u'\uE80a'

    @assert_user_logged_in
    def assert_permissions(self, controller):
        c = PvcSubrogationController(controller.sa_session, account_id=controller.get_account().id, request=controller.request)
        if not c.is_commissioner():
            raise SecurityException('Not a commissioner')

        self._assert_pvc_license(controller)


def get_icons():
    return [
        PvcSubrogationManagerIcon(),
        PvcSubrogationRequestIcon(),
        PvcSubrogationPVModuleManagerIcon(),
        PvcSubrogationProductManagerIcon(),
    ]
