# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.
from basesoft.web.core.input.validation import plugins
from basesoft.web.core.forms.handler import FormValidator

from pvcycle.subrogation.controllers import PvcSubrogationController
from pvcycle.core.controllers import PvcMemberController, PvcTBSController

class SubrogationHandler:
    def __init__(self, request, **kwargs):

        self._request = request
        self._sa_session = request.Session()
        self._account_id = request.session['user_id']

        self._controller = PvcSubrogationController(self._sa_session, request=self._request, account_id=self._account_id)

    def validate_request_for_subrogation(self, args):
        form_handler = FormValidator({
            'tbs_id': [(plugins.Required, {})],
            'commissioner_name': [(plugins.Required, {})],
        })
        return form_handler.validate(args)

    def request_for_subrogation(self, bitbucket, params):

        member_id = params['member_id']
        tbs_id = params['tbs_id']

        mc = PvcMemberController(self._sa_session, account_id=self._account_id, request=self._request)
        tbs_member = [tm for tm in mc.get_active_for_tbs(member_id, is_producer=True) if tm.tbs.id==int(tbs_id)].pop()

        self._controller.request_for_subrogation(tbs_member, params['commissioner_name'])
        return True

    def load_subrogation_details(self, _id):
        return self._controller.get_subrogation_state(_id, True)

    def save_subrogation(self, _id, state):
        self._controller.save_subrogation(_id, state)
        return _id

    def load_subrogation(self, _id):
        return self._controller.get_subrogation_state(_id)

    def get_commissioners_by_name(self, subrogation_id, commissioner_name):
        return self._controller.get_available_commissioners_by_name(subrogation_id, commissioner_name)

    def match_commissioner(self, subrogation_id, commissioner_id):
        self._controller.match_commissioner(subrogation_id, commissioner_id)
        return True

    def reject_subrogation(self, subrogation_id, reject_reason):
        self._controller.reject_subrogation(subrogation_id, reject_reason)
        return True

    def load_mandate(self, tbs_id):
        c = PvcTBSController(self._sa_session, account_id=self._account_id, request=self._request)
        return c.load_tbs(tbs_id)['tbs']

    def save_mandate(self, tbs_id, state):
        c = PvcTBSController(self._sa_session, account_id=self._account_id, request=self._request)
        c.save_tbs_texts(tbs_id, state)
        return True

    def get_mandate_replacement_fields(self):
        return self._controller.get_mandate_replacement_fields()

    def get_unapproved_commissioner_pv_modules_count(self):
        return self._controller.get_unapproved_commissioner_pv_modules_count()

    def get_approved_commissioners_for_member(self, license_id, commissioner_id=None, tbs_id=None):
        if license_id is None:
            return []

        return [{
            'value': s.id,
            'label': s.commissioner.member.name
        } for s in self._controller.get_approved_subrogations_for_member(license_id, commissioner_id, tbs_id) if s.commissioner is not None]

    def approve_pv_module(self, _id):
        if not self._controller.has_permissions('pvc_version_1_access'):
            raise DeprecationWarning('This is no longer available (PV CYCLE 2.0)')
        self._controller.approve_pv_module(_id)
        return True

    def reject_pv_module(self, _id, reason):
        if not self._controller.has_permissions('pvc_version_1_access'):
            raise DeprecationWarning('This is no longer available (PV CYCLE 2.0)')
        self._controller.reject_pv_module(_id, reason)
        return True

    def deactivate_product(self, _id):
        self._controller.deactivate_product(_id)
        return True

    def activate_product(self, _id):
        self._controller.activate_product(_id)
        return True

handlers = (SubrogationHandler, )
