# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.

from sqlalchemy.sql.expression import not_, and_, or_

from basesoft.web.contrib.mailing.schedule import HtmlMailTask

from pvcycle.subrogation.models import PvcSubrogation

from basesoft import logging
from pvcycle.subrogation.controllers import PvcSubrogationController
logger = logging.getLogger('pvcycle.subrogation.schedule')

class SubgrogationMailTask(HtmlMailTask):

    INTERVAL = 900 # every 15 minutes

    NEXT_STATUS = {
        PvcSubrogation.MAIL_STATUS_COMMISSIONER_PENDING: (PvcSubrogation.MAIL_STATUS_COMMISSIONER_SENDING, PvcSubrogation.MAIL_STATUS_COMMISSIONER_SENT),
        PvcSubrogation.MAIL_STATUS_SUBROGATOR_PENDING: (PvcSubrogation.MAIL_STATUS_SUBROGATOR_SENDING, PvcSubrogation.MAIL_STATUS_SUBROGATOR_SENT),
        PvcSubrogation.MAIL_STATUS_REJECTION_PENDING: (PvcSubrogation.MAIL_STATUS_REJECTION_SENDING, PvcSubrogation.MAIL_STATUS_REJECTION_SENT)
    }

    def _get_subrogation_to_mail(self):

        _filter = not_(PvcSubrogation.status.in_([PvcSubrogation.STATUS_NEW, PvcSubrogation.STATUS_SUSPENDED]))

        ops = self.NEXT_STATUS.keys()
        _ops_filter = None
        for op in ops:
            _f = PvcSubrogation.mail_status.op('&')(op)==op
            if _ops_filter is None:
                _ops_filter = _f
                continue
            _ops_filter = or_(_ops_filter, _f)

        _filter = and_(_filter, _ops_filter)

        return self.sa_session.query(PvcSubrogation).filter(_filter).all()

    def execute(self):
        subrogations = self._get_subrogation_to_mail()

        for subrogation in subrogations:
            add_status = 0
            rm_status = 0
            for op, next_status in self.NEXT_STATUS.iteritems():
                if subrogation.mail_status & op == op:
                    add_status |= next_status[0]
                    rm_status |= op

            subrogation.mail_status &= ~rm_status
            subrogation.mail_status |= add_status

            self.sa_session.add(subrogation)
            self.sa_session.flush()

        for subrogation in subrogations:
            add_status = 0
            rm_status = 0
            for op, next_status in self.NEXT_STATUS.iteritems():
                op = next_status[0]
                if subrogation.mail_status & op != op:
                    continue

                mail, targets = self._get_mail(subrogation, op)
                if mail is None:
                    self.logger.warn('%s: unable to find mail for subrogation (%s)' % (self, subrogation.id))
                    continue

                add_status |= next_status[1]
                rm_status |= op

                for target in targets:
                    person = target.member.company.contact_person
                    if person is None:
                        self.logger.warn('%s: no person found to deliver subrogation email (%s) for subrogation (%s) for license (%s), aborting', (self, subrogation.id, target.id))
                        continue

                    msg, result = self._build_message(person, mail, context={
                        'subrogation': subrogation
                    }, mail_from=mail.mail_from())

                    self._send_mail(person.email, msg, result)

            subrogation.mail_status &= ~rm_status
            subrogation.mail_status |= add_status

            self.sa_session.add(subrogation)
            self.sa_session.flush()

    def _get_mail(self, subrogation, status):

        c = PvcSubrogationController(self.sa_session)

        mail_map = {
            PvcSubrogation.MAIL_STATUS_REJECTION_SENDING: 'get_subrogation_rejection_mail',
            PvcSubrogation.MAIL_STATUS_SUBROGATOR_SENDING: 'get_subrogator_mandate_mail',
            PvcSubrogation.MAIL_STATUS_COMMISSIONER_SENDING: 'get_commissioner_mandate_mail'
        }

        return getattr(c, mail_map[status])(subrogation)

tasks = (SubgrogationMailTask, )
