# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext as _
from django.utils import simplejson
from django.shortcuts import redirect
from django.shortcuts import render_to_response
from django.template.context import RequestContext

from basesoft.web.contrib.wiki.parser import Parser

from pvcycle.subrogation.controllers import PvcSubrogationController, MandateReplacementFieldsContext
from basesoft.web.core.exceptions import PublicUserException

def view_mandate(request, params):
    account_id = request.session['user_id']
    params = simplejson.loads(params)

    controller = PvcSubrogationController(request.Session(), account_id=account_id, request=request)

    subrogation = controller.get_subrogations(params['subrogation_id']).pop()
    # The mandate has not yet been uploaded by the Subrogator
    if subrogation.mandate_file is not None:
        # redirect to download the file.
        return redirect('/basesoft.web.contrib.file/view/%s/attachment/' % subrogation.mandate_file.file_hash)


    if not params.has_key('print'):
        # redirect to download the pdf from parsable URL.
        if subrogation.subrogator.tbs.mandate_text is None:
            raise PublicUserException(_('Mandate not found for TBS %(tbs)s.' % {'tbs': subrogation.subrogator.tbs.name}))

        params['print'] = True
        return redirect('/basesoft.web.core/download_pdf//pvcycle.subrogation/view_mandate/%s' % simplejson.dumps(params))

    parser = Parser()
    text = parser.render_to_html(subrogation.subrogator.tbs.mandate_text, MandateReplacementFieldsContext(subrogation=subrogation))

    return render_to_response('view_mandate.html', {
        'text': text,
        'params': params,
        'base_requires': ['pvc_contract_scss'],
        'theme_css': 'pvc-contract'
    }, context_instance=RequestContext(request))
