# -*- coding: utf-8 -*-
'''
Created on Nov 6, 2013

@author: marten
'''
import datetime
from datetime import date
from decimal import Decimal
from twisted.trial.unittest import SkipTest

from basesoft.util.virtual_date import set_date, today

from pvcycle.test.selenium.pvc_selenium import PvcSeleniumTestCase
from pvcycle.core.controllers import PvcMemberController
from pvcycle.core.models import PvcTbsMember
from pvcycle.pvmodule.controllers import PvcPVModuleController
from pvcycle.order.controllers import PvcDeclarationOrderController
from pvcycle.pvmodule.models import PvcPVModule, PvcTbsMemberPVModule
from sqlalchemy.sql.expression import and_
from basesoft.web.contrib.i18n.models import Country
from pvcycle.order.models import PvcDeclarationOrder, PvcDeclarationOrderLine
from basesoft.web.core.util.controllers import LicenseConfigController
from pvcycle.order.schedule import NewDeclarationTask, YearlyFeeTask

class TestCase(PvcSeleniumTestCase):
    """
    pvcycle.test.selenium.test_declaration.TestCase
    """

    def setUp(self):
        raise SkipTest('Deprecated with PV CYCLE 2.0')
        super(TestCase, self).setUp()
        self.connect_db()

        set_date(date(2013, 1, 1)) # set date to January 2013

        self.operator = self._create_operator_account()
        self._create_contribution_tax_rates(self.operator)

        set_date(date(2013, 6, 1)) # set date to June 2013

        self._modules = self._fill_pv_modules(self.operator)
        # this TBS has it's setting on MONTHLY!
        self.tbs_account = self._create_tbs_account(self.operator)
        self.accountant_account = self._create_accountant_account(self.tbs_account.default_group.license, self.operator)

        lcc = LicenseConfigController(self.sa_session, scope=self.tbs_account.default_group.license)
        lcc.set('pvc.tbs.declaration.use.weight.for.amount.calculations', False)

        self.payment_methods = self._setup_payment_methods(self.operator, self.tbs_account)

        self._administration_fee = Decimal('250.00')
        self._contribution_fee = Decimal('2.75')

        self.service_level = self._create_tbs_service_level(self.tbs_account, admin_fee=self._administration_fee, contrib_fee=self._contribution_fee, year=today().year)
        self.reset_roe()

    def _create_simple_member_for_declaration(self, modules=None, countries=None, tbs_account=None, service_level=None, ma=None, tm=None):

        if ma is None:
            ma = self._create_member_account()

        # apply at TBS
        if service_level is None:
            service_level = self.service_level
        if tbs_account is None:
            tbs_account = self.tbs_account

        if tm is None:
            tm = self._member_apply_at_tbs(ma, tbs_account, service_level)

        # activate member to automatically create an order entry for the Administration Fee
        mc = PvcMemberController(self.sa_session, account_id=tbs_account.id)
        mc.save_tbs_member(tm.id, {'status': PvcTbsMember.STATUS_APPROVED, 'is_full_member': False, 'is_representative': False})

        if modules is None:
            modules = [self._modules[0], self._modules[1]]

        # now that the TBSMember is approved we should be able to save pv modules for this tbs member
        c = PvcPVModuleController(self.sa_session, account_id=ma.id)
        for module in modules:
            if countries is None:
                c.save_or_update_module_for_tbs_member(tm, tm.tbs.country, module, is_active=True)
                continue
            for country in countries:
                c.save_or_update_module_for_tbs_member(tm, country, module, is_active=True)

        return ma, tm

    def _january_declaration(self, ma=None, tm=None):

        if ma is None or tm is None:
            ma, tm = self._create_simple_member_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)

        # add an open declaration for this member.
        d = dc.add_open_declaration(tm, 2013, month=1)

        self.login(ma.name, 'test_password')

        self.click_iconbar('icon-pvc-outstanding-declarations')
        self.wait_for_loading() # for the custom query

        self.filter_cq(panel_name='pvc-outstanding-declarations', filters=[
            ('Year', 'Equal', ('select', '2013')),
            ('Month', 'Equal', ('select', '1')),
        ])
        self.wait_for_loading() # for the custom query

        self.assert_cq(panel_name='pvc-outstanding-declarations', row_index=0, data={
            'action': 'Declare'
        })

        self.click_button(panel_name='pvc-outstanding-declarations', text='Declare')
        self.wait_for_loading(stable_time=4) # for the details panel

        self.fill_form(panel_name='pvc-declaration-details-panel', values={
            'line_0': 200,
            'line_1': 150,
        })
        return d

    def test_declare(self):
        """
        Tests a simple declare/redeclare.
        """
        self._january_declaration()
        self.click_button(panel_name='pvc-declaration-details-panel', text='Save')

        self.wait_for_loading(times=2)

        self.close_panel(panel_name='pvc-outstanding-declarations')
        self.click_iconbar('icon-pvc-outstanding-declarations')
        self.wait_for_loading() # for the custom query

        self.filter_cq(panel_name='pvc-outstanding-declarations', filters=[
            ('Year', 'Equal', ('select', '2013')),
            ('Month', 'Equal', ('select', '1')),
        ])
        self.wait_for_loading() # for the custom query

        self.assert_cq(panel_name='pvc-outstanding-declarations', row_index=0, data={
            'action': 'Redeclare',
            'total_amount': u'€ 962.50'
        })


    def test_declaration_based_on_weight(self):
        lcc = LicenseConfigController(self.sa_session, scope=self.tbs_account.default_group.license)
        lcc.set('pvc.tbs.declaration.use.weight.for.amount.calculations', True)

        self._january_declaration()

        # weight for line 1 module = 150.76500, quantity = 200, fee = 2.75, total = 82920.75
        # weight for line 2 module = 140.00000, quantity = 150, fee = 2.75, total = 57750.00
        # total = 140670.75

        self.click_button(panel_name='pvc-declaration-details-panel', text='Save')
        self.wait_for_loading(times=2)

        self.close_panel(panel_name='pvc-outstanding-declarations')
        self.click_iconbar('icon-pvc-outstanding-declarations')
        self.wait_for_loading() # for the custom query

        self.filter_cq(panel_name='pvc-outstanding-declarations', filters=[
            ('Year', 'Equal', ('select', '2013')),
            ('Month', 'Equal', ('select', '1')),
        ])
        self.wait_for_loading() # for the custom query
        self.assert_cq(panel_name='pvc-outstanding-declarations', row_index=0, data={
            'action': 'Redeclare',
            'total_amount': u'€ 140,670.75'
        })

    def test_manual_declaration(self):
        """
        Tests a manual declaration
        """
        d = self._january_declaration()

        # SET THE CREATED DATE OF THIS TBS MEMBER TO MATCH THE VIRTUAL DATE
        d.tbs_member.created = datetime.datetime(year=2013, month=1, day=1)
        self.sa_session.add(d)
        self.sa_session.flush()

        self.click_button(panel_name='pvc-declaration-details-panel', text='Cancel')
        self.wait_for_loading(times=2)

        self.close_panel(panel_name='pvc-outstanding-declarations')
        self.click_iconbar('icon-pvc-declaration-manager',hover='icon-pvc-outstanding-declarations')
        self.wait_for_loading() # for the custom query

        self.click_menu(panel_name='pvc-declaration-manager', text="Add declaration")
        self.wait_for_loading() # for the panel

        self.fill_form(panel_name='declaration-select-panel', form_name='tbs_member', values={
            'tbs_id': ('select', d.tbs_member.tbs.name)
        })
        self.wait_for_loading() # for the period select

        self.fill_form(panel_name='declaration-select-panel', form_name='period_select', values={
            'year': ('select', '2013'),
        })
        # depending on year, the months are filled.
        self.fill_form(panel_name='declaration-select-panel', form_name='period_select', values={
            'month': ('select', 'value=2'), # EEPs?
        })

        self.click_button(panel_name='declaration-select-panel', text='Save')
        self.wait_for_loading2() # for the declaration details panel

        self.fill_form(panel_name='pvc-declaration-details-panel', values={
            'line_0': 210,
            'line_1': 160,
        })

        self.click_button(panel_name='pvc-declaration-details-panel', text='Save')
        self.wait_for_loading(times=2)

        self.click_iconbar('icon-pvc-outstanding-declarations')
        self.wait_for_loading() # for the custom query

        self.filter_cq(panel_name='pvc-outstanding-declarations', filters=[
            ('Year', 'Equal', ('select', '2013')),
            ('Month', 'Equal', ('select', '2')),
        ])
        self.wait_for_loading() # for the custom query
        self.assert_cq(panel_name='pvc-outstanding-declarations', row_index=0, data={
            'action': 'Redeclare',
            'total_amount': u'€ 1,017.50'
        })


    def test_manual_declare_multi_company(self):
        """
        Tests a simple declare/redeclare for multiple companies

        pvcycle.test.selenium.test_declaration.TestCase.test_manual_declare_multi_company
        """

        # create second tbs (used for second company)
        tbs_account_2 = self._create_tbs_account(self.operator)
        service_level_2 = self._create_tbs_service_level(tbs_account_2, admin_fee=self._administration_fee, contrib_fee=self._contribution_fee, year=today().year)

        # create member account with its default license
        member_account = self._create_member_account('company 1')
        license_1 = member_account.default_group.license

        # create a second company
        mc = PvcMemberController(self.sa_session, account_id=member_account.id)
        license_2 = mc.save_member_license(self._get_member_license_state('company 2'))

        # apply first company to default tbs
        tbs_member_1 = self._member_apply_at_tbs(member_account, self.tbs_account, self.service_level, member_license_id=license_1.id)

        # apply second company to default tbs
        tbs_member_2 = self._member_apply_at_tbs(member_account, self.tbs_account, self.service_level, member_license_id=license_2.id)

        # apply second company to second tbs
        tbs_member_3 = self._member_apply_at_tbs(member_account, tbs_account_2, service_level_2, member_license_id=license_2.id)

        # update timestamps so the period select works correctly
        tbs_member_1.created = datetime.datetime(year=2013, month=1, day=1)
        tbs_member_2.created = datetime.datetime(year=2013, month=1, day=1)
        tbs_member_3.created = datetime.datetime(year=2013, month=1, day=1)
        self.sa_session.add(tbs_member_1)
        self.sa_session.add(tbs_member_2)
        self.sa_session.flush()

        self._create_simple_member_for_declaration(tbs_account=self.tbs_account, service_level=self.service_level, ma=member_account, tm=tbs_member_1)
        self._create_simple_member_for_declaration(tbs_account=self.tbs_account, service_level=self.service_level, ma=member_account, tm=tbs_member_2)
        self._create_simple_member_for_declaration(tbs_account=tbs_account_2, service_level=service_level_2, ma=member_account, tm=tbs_member_3)

        self.login(member_account.name, 'test_password')

        self.click_iconbar('icon-pvc-declaration-manager',hover='icon-pvc-outstanding-declarations')
        self.wait_for_loading() # for the custom query

        self.click_menu(panel_name='pvc-declaration-manager', text="Add declaration")
        self.wait_for_loading() # for the panel

        self.fill_form(panel_name='declaration-select-panel', form_name='tbs_member', values={
            'license_id': ('select', license_2.name),
        })
        self.wait_for_loading() # for the tbs select

        self.fill_form(panel_name='declaration-select-panel', form_name='tbs_member', values={
            'tbs_id': ('select', tbs_member_3.tbs.name)
        })
        self.wait_for_loading() # for the period select


        self.fill_form(panel_name='declaration-select-panel', form_name='period_select', values={
            'year': ('select', '2013'),
        })
        # depending on year, the months are filled.
        self.fill_form(panel_name='declaration-select-panel', form_name='period_select', values={
            'month': ('select', 'value=2'), # EEPs?
        })

        self.click_button(panel_name='declaration-select-panel', text='Save')
        self.wait_for_loading(stable_time=4) # for the declaration details panel

        self.fill_form(panel_name='pvc-declaration-details-panel', values={
            'line_0': 210,
            'line_1': 160,
        })

        self.click_button(panel_name='pvc-declaration-details-panel', text='Save')
        self.wait_for_loading(times=2)

        self.click_iconbar('icon-pvc-outstanding-declarations')
        self.wait_for_loading() # for the custom query

        self.filter_cq(panel_name='pvc-outstanding-declarations', filters=[
            ('Producer', 'Equal', license_2.name),
            ('TBS', 'Equal', tbs_member_3.tbs.name),
            ('Year', 'Equal', ('select', '2013')),
            ('Month', 'Equal', ('select', '2')),
        ])
        self.wait_for_loading() # for the custom query
        self.assert_cq(panel_name='pvc-outstanding-declarations', row_index=0, data={
            'action': 'Redeclare',
            'total_amount': u'€ 1,017.50'
        })


    def test_checkout(self):
        d = self._january_declaration()

        self.click_button(panel_name='pvc-declaration-details-panel', text='Submit')
        self.wait_for_loading(times=2)

        # select 'bank transfer' payment method and continue
        self.fill_form('pvc-declaration-details-panel-checkout', values={
            'payment_method': ('radio', self.payment_methods[0].id)
        })
        self.click_button(panel_name='pvc-declaration-details-panel-checkout', text='Submit')
        self.wait_for_loading(times=2)

        self.click_button('order-payment_method-panel', text='Mark as submitted')
        self.wait_for_loading()
        self.assert_cq_empty(panel_name='pvc-outstanding-declarations')

        # order should be exportable
        dc = PvcDeclarationOrderController(self.sa_session, account_id=self.accountant_account.id)
        orders = [o['id'] for o in dc.get_unexported_orders_for_merchant(self.tbs_account.default_group.license.id)]
        self.assertTrue(d.id in orders)

    def test_complete_credit_declaration(self):
        d = self._january_declaration()
        self.click_button(panel_name='pvc-declaration-details-panel', text='Submit')
        self.wait_for_loading(times=2)


        self.fill_form('pvc-declaration-details-panel-checkout', values={
            'payment_method': ('radio', self.payment_methods[0].id)
        })
        self.click_button(panel_name='pvc-declaration-details-panel-checkout', text='Submit')
        self.wait_for_loading(times=2)

        self.click_button('order-payment_method-panel', text='Mark as submitted')
        self.wait_for_loading()

        self.close_panel(panel_name='pvc-outstanding-declarations')

        self.click_iconbar('icon-pvc-declaration-manager', hover='icon-pvc-outstanding-declarations')
        self.wait_for_panel('pvc-declaration-manager')

        self.filter_cq(panel_name='pvc-declaration-manager', filters=[
            ('Order ID', 'Equal', d.id),
        ])
        self.wait_for_loading() # for the custom query

        self.assert_cq(panel_name='pvc-declaration-manager', row_index=0, data={
            'action': 'Request'
        })

        self.click_button(panel_name='pvc-declaration-manager', text='Request')
        self.wait_for_loading() # for the details panel

        self.click_button(panel_name='dialog-question', text='Yes')
        self.wait_for_loading() # for the details panel

        self.assert_cq(panel_name='pvc-declaration-manager', row_index=0, data={
            'action': 'Pending'
        })

        self.sa_session.expire(d)

        dc = PvcDeclarationOrderController(self.sa_session, account_id=self.operator.id)
        dc.approve_resubmit_request(d.id)

        self.close_panel(panel_name='pvc-declaration-manager')

        self.click_iconbar('icon-pvc-declaration-manager', hover='icon-pvc-outstanding-declarations')
        self.wait_for_panel('pvc-declaration-manager')

        self.filter_cq(panel_name='pvc-declaration-manager', filters=[
            ('Order ID', 'Equal', d.id),
        ])
        self.wait_for_loading() # for the custom query

        self.assert_cq(panel_name='pvc-declaration-manager', row_index=0, data={
            'action': 'Resubmit'
        })

        self.click_button(panel_name='pvc-declaration-manager', text='Resubmit')
        self.wait_for_loading() # for the details panel

        self.fill_form(panel_name='pvc-declaration-details-panel', values={
            'line_0': 400,
            'line_1': 250,
        })

        self.click_button(panel_name='pvc-declaration-details-panel', text='Save')
        self.wait_for_loading(times=2)

        self.close_panel(panel_name='pvc-declaration-manager')

        credit = self.sa_session.query(PvcDeclarationOrder).filter(PvcDeclarationOrder.parent_order==d).one()

        # now login as accountant
        self.login(self.accountant_account.name, 'test_password')
        # open order manager
        self.click_iconbar('icon-pvc-order-manager', hover='icon-pvc-declaration-manager')
        self.filter_cq(panel_name='pvc-order-manager', filters=[
            ('Order ID', 'Equal', credit.id),
        ])
        self.wait_for_loading() # for the custom query
        self.click_cq('pvc-order-manager', row_index=0, col_name='payment_status')

        # open complete/reject dialog
        self.click_button('order-payment-status-cq', text='Complete or reject order..')
        self.wait_for_loading2()

        # fill form
        self.fill_form('payment-status-details', values={
            'payment_status': ('select', 'Completed'),
            'description': ('textarea', 'i r received moneys')
        })
        self.click_button('payment-status-details', text='Save')

        # close the payment details, should refresh the cq behind it
        self.close_panel('order-payment-status-cq')
        self.wait_for_loading()

        self.close_panel(panel_name='pvc-order-manager')
        self.click_iconbar('icon-pvc-order-manager', hover='icon-pvc-declaration-manager')
        self.wait_for_loading()
        self.filter_cq(panel_name='pvc-order-manager', filters=[
            ('Order ID', 'Equal', credit.id),
        ])
        self.wait_for_loading() # for the custom query

        # status should now be completed
        self.assert_cq(panel_name='pvc-order-manager', row_index=0, data={
            'payment_status': 'Completed'
        })



    def test_declare_additional_line(self):
        d = self._january_declaration()
        self.click_button(panel_name='pvc-declaration-details-panel', text='Add line')
        self.wait_for_loading(times=2)

        forms_data = {
            'pvc-pv-module-form': {
                'supplier': self._modules[2].supplier.name,
                'type': self._modules[2].type.name
            }
        }
        self.fill_forms(panel_name='pvc-pv-module-select', forms_data=forms_data)
        self.fill_forms(panel_name='pvc-pv-module-select', forms_data={
            'pvc-pv-module-countries': {
                'countries': ('fancy-select', [self.tbs_account.default_group.license.country.name], ),
            }
        })
        self.wait_for_loading()

        self.click_button(panel_name='pvc-pv-module-select', text='Save')
        self.wait_for_loading()

        tbs_member_module_id = self.sa_session.query(PvcTbsMemberPVModule).filter(
            and_(
                PvcTbsMemberPVModule.tbs_member==d.tbs_member,
                PvcTbsMemberPVModule.module==self._modules[2],
            )
        ).one().id

        self.fill_form(panel_name='pvc-declaration-details-panel', values={
            'quantity_additional_%s' % tbs_member_module_id: 100,
        })

        self.click_button(panel_name='pvc-declaration-details-panel', text='Save')
        self.wait_for_loading(times=2)

        self.click_button(panel_name='pvc-outstanding-declarations', text='Redeclare')
        self.wait_for_loading() # for the details panel

        self.fill_form(panel_name='pvc-declaration-details-panel', values={
            'line_2': 300,
        })

        self.click_button(panel_name='pvc-declaration-details-panel', text='Save')
        self.wait_for_loading(times=2)

    def test_declare_additional_line_two_countries(self):
        # Reproduce ticket #235

        state = self._get_tbs_license_state()
        state['tbs']['country'] = None
        tbs_account = self._create_tbs_account(self.operator, state=state)

        service_level = self._create_tbs_service_level(tbs_account, admin_fee=self._administration_fee, contrib_fee=self._contribution_fee, year=today().year)

        countries = self.sa_session.query(Country).filter(Country.iso_3166_a2.in_(['NO', 'DK'])).all()
        ma, tm = self._create_simple_member_for_declaration(countries=countries, tbs_account=tbs_account, service_level=service_level)


        d = self._january_declaration(ma, tm)
        self.click_button(panel_name='pvc-declaration-details-panel', text='Add line')
        self.wait_for_loading(times=2)

        # ADD multiple lines
        forms_data = {
            'pvc-pv-module-form': {
                'supplier': self._modules[2].supplier.name,
                'type': self._modules[2].type.name,
            },
        }
        self.fill_forms(panel_name='pvc-pv-module-select', forms_data=forms_data)

        self.fill_forms(panel_name='pvc-pv-module-select', forms_data={
            'pvc-pv-module-countries': {
                'countries': ('fancy-select', ['Sweden', 'Finland'], ),
            }
        })

        self.wait_for_loading()

        self.click_button(panel_name='pvc-pv-module-select', text='Save')
        self.wait_for_loading()

        mm = self.sa_session.query(PvcTbsMemberPVModule).filter(
            and_(
                PvcTbsMemberPVModule.tbs_member==d.tbs_member,
                PvcTbsMemberPVModule.module==self._modules[2],
            )
        ).all()


        # fill out some values
        self.fill_form(panel_name='pvc-declaration-details-panel', values={
            'quantity_additional_%s' % mm[0].id: 101,
            'quantity_additional_%s' % mm[1].id: 102,
        })

        # ADD more lines
        self.click_button(panel_name='pvc-declaration-details-panel', text='Add line')
        self.wait_for_loading(times=2)

        forms_data = {
            'pvc-pv-module-form': {
                'supplier': self._modules[3].supplier.name,
                'type': self._modules[3].type.name,
            },
        }
        self.fill_forms(panel_name='pvc-pv-module-select', forms_data=forms_data)

        self.fill_forms(panel_name='pvc-pv-module-select', forms_data={
            'pvc-pv-module-countries': {
                'countries': ('fancy-select', ['Sweden', 'Poland'], ),
            }
        })

        self.wait_for_loading()

        self.click_button(panel_name='pvc-pv-module-select', text='Save')
        self.wait_for_loading()

        # validate values
        self.assert_form(panel_name='pvc-declaration-details-panel', values={
            'line_0': '200',
            'line_1': '150',
            'quantity_additional_%s' % mm[0].id: '101',
            'quantity_additional_%s' % mm[1].id: '102',
        })

        # fill the other additional lines
        mm = self.sa_session.query(PvcTbsMemberPVModule).filter(
            and_(
                PvcTbsMemberPVModule.tbs_member==d.tbs_member,
                PvcTbsMemberPVModule.module==self._modules[3],
            )
        ).all()


        # fill out some values
        self.fill_form(panel_name='pvc-declaration-details-panel', values={
            'quantity_additional_%s' % mm[0].id: 103,
            'quantity_additional_%s' % mm[1].id: 104,
        })

        # save the declaration.
        self.click_button(panel_name='pvc-declaration-details-panel', text='Save')
        self.wait_for_loading(times=2)

        # validate Declaration in database.
        d = self.sa_session.query(PvcDeclarationOrder).filter(PvcDeclarationOrder.order_id==d.id).one()
        quantities = [0, 0, 101, 102, 103, 104, 150, 200]
        verify = [l.quantity for l in d.lines]
        verify.sort()

        self.assertEqual(quantities, verify)

    def test_declare_additional_line_and_remove(self):
        d = self._january_declaration()
        self.click_button(panel_name='pvc-declaration-details-panel', text='Add line')
        self.wait_for_loading(times=2)

        forms_data = {
            'pvc-pv-module-form': {
                'supplier': self._modules[2].supplier.name,
                'type': self._modules[2].type.name,
            },
        }
        self.fill_forms(panel_name='pvc-pv-module-select', forms_data=forms_data)

        self.fill_forms(panel_name='pvc-pv-module-select', forms_data={
            'pvc-pv-module-countries': {
                'countries': ('fancy-select', [self.tbs_account.default_group.license.country.name], ),
            }
        })
        self.wait_for_loading()

        self.click_button(panel_name='pvc-pv-module-select', text='Save')
        self.wait_for_loading()

        self.click_button(panel_name='pvc-declaration-details-panel', text='Remove line')

        self.click_button(panel_name='pvc-declaration-details-panel', text='Add line')
        self.wait_for_loading(times=2)

        forms_data = {
            'pvc-pv-module-form': {
                'supplier': self._modules[3].supplier.name,
                'type': self._modules[3].type.name,
            },
        }
        self.fill_forms(panel_name='pvc-pv-module-select', forms_data=forms_data)

        self.fill_forms(panel_name='pvc-pv-module-select', forms_data={
            'pvc-pv-module-countries': {
                'countries': ('fancy-select', [self.tbs_account.default_group.license.country.name], ),
            }
        })

        self.wait_for_loading()

        self.click_button(panel_name='pvc-pv-module-select', text='Save')
        self.wait_for_loading()

        tbs_member_module_id = self.sa_session.query(PvcTbsMemberPVModule).filter(
            and_(
                PvcTbsMemberPVModule.tbs_member==d.tbs_member,
                PvcTbsMemberPVModule.module==self._modules[3],
            )
        ).one().id

        self.fill_form(panel_name='pvc-declaration-details-panel', values={
            'quantity_additional_%s' % tbs_member_module_id: 100,
        })

        self.click_button(panel_name='pvc-declaration-details-panel', text='Save')
        self.wait_for_loading(times=2)

    def test_resubmit_after_minimum_contribution_fee(self):
        lcc = LicenseConfigController(self.sa_session, scope=self.tbs_account.default_group.license)
        lcc.set('pvc.tbs.minimum.contribution.fee', 150)

        ma, tm = self._create_simple_member_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)

        pc = PvcDeclarationOrderController(self.sa_session)
        set_date(date(2013, 2, 11))
        pc.close_declarations(tm.tbs)

        dc.request_for_resubmission(d.id)

        self.sa_session.expire(d)
        self.assertEqual(d.resubmit_request, PvcDeclarationOrder.RESUBMIT_REQUEST_PENDING)

        # now the operator approves the resubmit.
        odc = PvcDeclarationOrderController(self.sa_session, account_id=self.operator.id)
        odc.approve_resubmit_request(d.id)

        self.login(ma.name, 'test_password')

        self.click_iconbar('icon-pvc-declaration-manager', hover='icon-pvc-outstanding-declarations')
        self.wait_for_panel('pvc-declaration-manager')

        self.filter_cq(panel_name='pvc-declaration-manager', filters=[
            ('Year', 'Equal', ('select', '2013')),
            ('Month', 'Equal', ('select', '1')),
        ])

        self.click_button(panel_name='pvc-declaration-manager', text='Resubmit')
        self.wait_for_loading()

        self.fill_form(panel_name='pvc-declaration-details-panel', values={
            'line_0': 400,
            'line_1': 250,
        })

        self.click_button(panel_name='pvc-declaration-details-panel', text='Save')
        self.wait_for_loading(times=2)

        self.close_panel(panel_name='pvc-declaration-manager')

    def test_resubmit_after_minimum_contribution_fee_with_additional_line(self):
        lcc = LicenseConfigController(self.sa_session, scope=self.tbs_account.default_group.license)
        lcc.set('pvc.tbs.minimum.contribution.fee', 150)

        ma, tm = self._create_simple_member_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)

        pc = PvcDeclarationOrderController(self.sa_session)
        set_date(date(2013, 2, 11))
        pc.close_declarations(tm.tbs)

        dc.request_for_resubmission(d.id)

        self.sa_session.expire(d)
        self.assertEqual(d.resubmit_request, PvcDeclarationOrder.RESUBMIT_REQUEST_PENDING)

        # now the operator approves the resubmit.
        odc = PvcDeclarationOrderController(self.sa_session, account_id=self.operator.id)
        odc.approve_resubmit_request(d.id)

        self.login(ma.name, 'test_password')

        self.click_iconbar('icon-pvc-declaration-manager', hover='icon-pvc-outstanding-declarations')
        self.wait_for_panel('pvc-declaration-manager')

        self.filter_cq(panel_name='pvc-declaration-manager', filters=[
            ('Year', 'Equal', ('select', '2013')),
            ('Month', 'Equal', ('select', '1')),
        ])

        self.click_button(panel_name='pvc-declaration-manager', text='Resubmit')
        self.wait_for_loading()

        self.click_button(panel_name='pvc-declaration-details-panel', text='Add line')
        self.wait_for_panel('pvc-pv-module-select')
        self.wait_for_loading()

        forms_data = {
            'pvc-pv-module-form': {
                'supplier': self._modules[3].supplier.name,
                'type': self._modules[3].type.name,
            },
        }
        self.fill_forms(panel_name='pvc-pv-module-select', forms_data=forms_data)

        self.fill_forms(panel_name='pvc-pv-module-select', forms_data={
            'pvc-pv-module-countries': {
                'countries': ('fancy-select', [self.tbs_account.default_group.license.country.name], ),
            }
        })

        self.wait_for_loading()

        set_date(date(2013, 7, 1))

        self.click_button(panel_name='pvc-pv-module-select', text='Save')
        self.wait_for_loading()

        tbs_member_module_id = self.sa_session.query(PvcTbsMemberPVModule).filter(
            and_(
                PvcTbsMemberPVModule.tbs_member==d.tbs_member,
                PvcTbsMemberPVModule.module==self._modules[3],
            )
        ).one().id

        self.fill_form(panel_name='pvc-declaration-details-panel', values={
            'quantity_additional_%s' % tbs_member_module_id: 100,
        })

        self.fill_form(panel_name='pvc-declaration-details-panel', values={
            'line_0': 400,
            'line_1': 250,
        })

        self.click_button(panel_name='pvc-declaration-details-panel', text='Save')
        self.wait_for_loading(times=2)

        self.close_panel(panel_name='pvc-declaration-manager')
        self.click_iconbar('icon-pvc-declaration-manager', hover='icon-pvc-outstanding-declarations')
        self.wait_for_panel('pvc-declaration-manager')

        self.filter_cq(panel_name='pvc-declaration-manager', filters=[
            ('Year', 'Equal', ('select', '2013')),
            ('Month', 'Equal', ('select', '1')),
        ])

        self.assert_cq(panel_name='pvc-declaration-manager', row_index=0, data={
            'total_amount': u'€ 150.00'
        })
        self.assert_cq(panel_name='pvc-declaration-manager', row_index=1, data={
            'total_amount': u'€ -150.00'
        })
        self.assert_cq(panel_name='pvc-declaration-manager', row_index=2, data={
            'action': 'Redeclare',
            'total_amount': u'€ 2,062.50'
        })

    def test_resubmit_with_additional_line(self):
        d = self._january_declaration()
        self.click_button(panel_name='pvc-declaration-details-panel', text='Submit')
        self.wait_for_loading(times=2)

        self.fill_form('pvc-declaration-details-panel-checkout', values={
            'payment_method': ('radio', self.payment_methods[0].id)
        })
        self.click_button(panel_name='pvc-declaration-details-panel-checkout', text='Submit')
        self.wait_for_loading(times=2)

        self.click_button('order-payment_method-panel', text='Mark as submitted')
        self.wait_for_loading()

        self.close_panel(panel_name='pvc-outstanding-declarations')

        # auto approved resubmit request (this process gets tested in test_resubmit)
        d.resubmit_request = PvcDeclarationOrder.RESUBMIT_REQUEST_APPROVED
        self.sa_session.add(d)
        self.sa_session.flush()

        self.click_iconbar('icon-pvc-declaration-manager', hover='icon-pvc-outstanding-declarations')
        self.wait_for_panel('pvc-declaration-manager')

        self.filter_cq(panel_name='pvc-declaration-manager', filters=[
            ('Year', 'Equal', ('select', '2013')),
            ('Month', 'Equal', ('select', '1')),
        ])

        self.click_button(panel_name='pvc-declaration-manager', text='Resubmit')
        self.wait_for_loading()

        self.click_button(panel_name='pvc-declaration-details-panel', text='Add line')
        self.wait_for_panel('pvc-pv-module-select')

        self.fill_forms(panel_name='pvc-pv-module-select', forms_data={
            'pvc-pv-module-form': {'supplier': self._modules[3].supplier.name}
        })
        self.wait_for_loading()
        self.fill_forms(panel_name='pvc-pv-module-select', forms_data={
            'pvc-pv-module-form': {'type': self._modules[3].type.name}
        })
        self.wait_for_loading()
        self.fill_forms(panel_name='pvc-pv-module-select', forms_data={
            'pvc-pv-module-countries': {'countries': ('fancy-select', [self.tbs_account.default_group.license.country.name], )}
        })
        self.wait_for_loading()

        self.click_button(panel_name='pvc-pv-module-select', text='Save')
        self.wait_for_loading()

        tbs_member_module_id = self.sa_session.query(PvcTbsMemberPVModule).filter(
            and_(
                PvcTbsMemberPVModule.tbs_member==d.tbs_member,
                PvcTbsMemberPVModule.module==self._modules[3],
            )
        ).one().id

        self.fill_form(panel_name='pvc-declaration-details-panel', values={
            'quantity_additional_%s' % tbs_member_module_id: 100,
        })

        self.click_button(panel_name='pvc-declaration-details-panel', text='Save')
        self.wait_for_loading(times=2)

        self.close_panel(panel_name='pvc-declaration-manager')
        self.click_iconbar('icon-pvc-declaration-manager', hover='icon-pvc-outstanding-declarations')
        self.wait_for_panel('pvc-declaration-manager')

        self.filter_cq(panel_name='pvc-declaration-manager', filters=[
            ('Year', 'Equal', ('select', '2013')),
            ('Month', 'Equal', ('select', '1')),
        ])

        self.assert_cq(panel_name='pvc-declaration-manager', row_index=0, data={
            'total_amount': u'€ 962.50'
        })
        self.assert_cq(panel_name='pvc-declaration-manager', row_index=1, data={
            'total_amount': u'€ -962.50'
        })
        self.assert_cq(panel_name='pvc-declaration-manager', row_index=2, data={
            'action': 'Redeclare',
            'total_amount': u'€ 1,237.50'
        })

    def test_resubmit(self):
        """
        Tests a resubmission (currently done through the order overview).
        """
        d = self._january_declaration()
        self.click_button(panel_name='pvc-declaration-details-panel', text='Submit')
        self.wait_for_loading(times=2)


        self.fill_form('pvc-declaration-details-panel-checkout', values={
            'payment_method': ('radio', self.payment_methods[0].id)
        })
        self.click_button(panel_name='pvc-declaration-details-panel-checkout', text='Submit')
        self.wait_for_loading(times=2)

        self.click_button('order-payment_method-panel', text='Mark as submitted')
        self.wait_for_loading()

        self.close_panel(panel_name='pvc-outstanding-declarations')

        self.click_iconbar('icon-pvc-declaration-manager', hover='icon-pvc-outstanding-declarations')
        self.wait_for_panel('pvc-declaration-manager')

        self.filter_cq(panel_name='pvc-declaration-manager', filters=[
            ('Order ID', 'Equal', d.id),
        ])
        self.wait_for_loading() # for the custom query

        self.assert_cq(panel_name='pvc-declaration-manager', row_index=0, data={
            'action': 'Request'
        })

        self.click_button(panel_name='pvc-declaration-manager', text='Request')
        self.wait_for_loading() # for the details panel

        self.click_button(panel_name='dialog-question', text='Yes')
        self.wait_for_loading() # for the details panel

        self.assert_cq(panel_name='pvc-declaration-manager', row_index=0, data={
            'action': 'Pending'
        })

        self.sa_session.expire(d)

        dc = PvcDeclarationOrderController(self.sa_session, account_id=self.operator.id)
        dc.approve_resubmit_request(d.id)

        self.close_panel(panel_name='pvc-declaration-manager')

        self.click_iconbar('icon-pvc-declaration-manager', hover='icon-pvc-outstanding-declarations')
        self.wait_for_panel('pvc-declaration-manager')

        self.filter_cq(panel_name='pvc-declaration-manager', filters=[
            ('Order ID', 'Equal', d.id),
        ])
        self.wait_for_loading() # for the custom query

        self.assert_cq(panel_name='pvc-declaration-manager', row_index=0, data={
            'action': 'Resubmit'
        })

        self.click_button(panel_name='pvc-declaration-manager', text='Resubmit')
        self.wait_for_loading() # for the details panel

        self.fill_form(panel_name='pvc-declaration-details-panel', values={
            'line_0': 400,
            'line_1': 250,
        })

        self.click_button(panel_name='pvc-declaration-details-panel', text='Save')
        self.wait_for_loading(times=2)

        self.close_panel(panel_name='pvc-declaration-manager')

        self.click_iconbar('icon-pvc-declaration-manager', hover='icon-pvc-outstanding-declarations')
        self.wait_for_panel('pvc-declaration-manager')

    def test_unapproved_pv_modules_in_declarations(self):
        # Ticket #152
        module = self._add_pv_module(self._uniquestring('pvm'), self.operator)
        module.approval_status = PvcPVModule.PV_MODULE_STATUS_PENDING
        self.sa_session.add(module)

        ma, tm = self._create_simple_member_for_declaration(modules=[module])
        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)

        self.login(ma.name, 'test_password')

        self.click_iconbar('icon-pvc-outstanding-declarations')
        self.wait_for_loading() # for the custom query

        self.filter_cq(panel_name='pvc-outstanding-declarations', filters=[
            ('Year', 'Equal', ('select', '2013')),
            ('Month', 'Equal', ('select', '1')),
        ])
        self.wait_for_loading() # for the custom query

        self.assert_cq(panel_name='pvc-outstanding-declarations', row_index=0, data={
            'action': 'Declare'
        })

        self.click_button(panel_name='pvc-outstanding-declarations', text='Declare')
        self.wait_for_loading() # for the details panel

        # assert the text
        self.assertEqual(len(self._driver.find_elements_by_xpath("//div[@class='message-warning']")), 1)

    def test_declaration_remove_line_unapproved_module(self):

        module1 = self._add_pv_module(self._uniquestring('pvm'), self.operator)
        module1.approval_status = PvcPVModule.PV_MODULE_STATUS_PENDING
        self.sa_session.add(module1)

        module2 = self._add_pv_module(self._uniquestring('pvm'), self.operator)
        module2.approval_status = PvcPVModule.PV_MODULE_STATUS_PENDING
        self.sa_session.add(module2)

        ma, tm = self._create_simple_member_for_declaration(modules=[self._modules[0], module1, module2])
        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)

        self.login(ma.name, 'test_password')

        self.click_iconbar('icon-pvc-outstanding-declarations')
        self.wait_for_loading() # for the custom query

        self.filter_cq(panel_name='pvc-outstanding-declarations', filters=[
            ('Year', 'Equal', ('select', '2013')),
            ('Month', 'Equal', ('select', '1')),
        ])
        self.wait_for_loading() # for the custom query

        self.assert_cq(panel_name='pvc-outstanding-declarations', row_index=0, data={
            'action': 'Declare'
        })

        self.click_button(panel_name='pvc-outstanding-declarations', text='Declare')
        self.wait_for_loading() # for the details panel

        self.fill_form(panel_name='pvc-declaration-details-panel', values={
            'line_0': 400
        })

        button = self.find_element_by_css_contains('td#order_action_1 button', 'Remove line')
        button.click()

        self.click_button(panel_name='dialog-question', text='No')
        self.wait_for_loading() # for the details panel

        self.assert_form(panel_name='pvc-declaration-details-panel', values={
            'line_0': u'400'
        })


    def test_declaration_closing_before_specified_date(self):
        d = self._january_declaration()
        self.click_button(panel_name='pvc-declaration-details-panel', text='Submit')
        self.wait_for_loading(times=2)

        # manually change the date
        d.created = datetime.date(2013, 2, 1)
        self.sa_session.add(d)
        self.sa_session.flush()
        self.assertEqual(d.status, PvcDeclarationOrder.STATUS_PENDING)

        self.login(self.operator.name, 'test_password')

        self.click_iconbar('icon-pvc-declaration-manager')
        self.wait_for_loading()

        self.click_menu(panel_name='pvc-declaration-manager', text='Close declarations before specified date')
        self.wait_for_loading()

        set_date(date(2013, 4, 1))

        self.fill_form(panel_name='pvc-declaration-close-panel', values={
            'tbs_id': ('select', d.merchant.name),
            'before': '01/03/2013 00:00'
        })
        self.click_button(panel_name='pvc-declaration-close-panel', text='Close declarations')
        self.wait_for_loading()

        self.sa_session.expire(d)
        self.assertEqual(d.status, PvcDeclarationOrder.STATUS_SUBMITTED)


    def test_declaration_resign(self):
        """
        pvcycle.test.selenium.test_declaration.TestCase.test_declaration_resign
        """

        ma, tm = self._create_simple_member_for_declaration()
        d = self._january_declaration(ma, tm)

        self.click_button(panel_name='pvc-declaration-details-panel', text='Submit')
        self.wait_for_loading(times=2)

        # As operator login and resign the member

        self.login(self.operator.name, 'test_password')
        self.click_iconbar('icon-pvc-member-manager', hover='icon-pvc-accountant-manager')
        self.wait_for_loading()

        self.filter_cq(panel_name='pvc-member-manager', filters=[('TBS', 'Equal', d.tbs_member.tbs.name)])
        self.click_cq('pvc-member-manager', row_index=0, col_name='tbs_member_id_details')
        self.wait_for_loading()

        self.fill_form('pvc-member-details', values={
            'status': ('radio', PvcTbsMember.STATUS_RESIGNING),
        })

        self.click_button(panel_name='pvc-member-details', text='Save')
        self.wait_for_loading()


        # goto jan 2014
        # trigger declarations and fee's..
        # this should create a declaration for december 2013
        set_date(date(2014, 1, 1))
        NewDeclarationTask().execute()
        set_date(date(2014, 1, 12))
        YearlyFeeTask().execute()


        # goto feb 2014 and run the declaration task..
        # this should not trigger a new declaration for this member
        set_date(date(2014, 2, 1))
        NewDeclarationTask().execute()

        # login as member to verify:
        # - no AF for 2014
        # - no declaration for jan 2014

        self.login(ma.name, 'test_password')

        # outstanding declarations
        self.click_iconbar('icon-pvc-outstanding-declarations')
        self.wait_for_loading()

        self.assert_cq('pvc-outstanding-declarations', row_index=0, data={
            'year': '2013', 'month': 'January'
        })
        self.assert_cq('pvc-outstanding-declarations', row_index=1, data={
            'year': '2013', 'month': 'December'
        })
        self.assertEqual(self.cq_rows_length('pvc-outstanding-declarations'), 2)

        # all orders
        self.click_iconbar('icon-pvc-order-manager', hover='icon-pvc-outstanding-declarations')
        self.wait_for_loading()

        self.assert_cq('pvc-order-manager', row_index=0, data={
            'order_type': 'Administrative'
        })
        self.assert_cq('pvc-order-manager', row_index=1, data={
            'order_type': 'Contribution'
        })
        self.assert_cq('pvc-order-manager', row_index=2, data={
            'order_type': 'Contribution'
        })
        self.assertEqual(self.cq_rows_length('pvc-order-manager'), 3)




# below is for running the test from eclipse
if __name__ == '__main__':

    from twisted.scripts.trial import run
    import sys
    sys.argv = ['test_member.py', 'pvcycle.test.selenium.test_declaration.TestCase.test_declaration_remove_line_unapproved_module']
    run()
    sys.exit()
