'''
Created on Nov 4, 2013

@author: marten
'''
from datetime import date
from decimal import Decimal
from twisted.trial.unittest import SkipTest

from basesoft.util.virtual_date import set_date, today

from pvcycle.test.selenium.pvc_selenium import PvcSeleniumTestCase
from pvcycle.core.controllers import PvcMemberController
from pvcycle.core.models import PvcTbsMember
from pvcycle.pvmodule.controllers import PvcPVModuleController
import datetime
from pvcycle.pvmodule.models import PvcPVModule

class TestCase(PvcSeleniumTestCase):
    def setUp(self):
        raise SkipTest('Deprecated with PV CYCLE 2.0')
        super(TestCase, self).setUp()
        self.connect_db()

        self.operator_account = self._create_operator_account()

    def test_create_and_edit_pv_module(self):
        self.login(self.operator_account.name, 'test_password')

        self.click_iconbar('icon-pvc-pv-module-manager', hover='icon-pvc-declaration-manager')
        self.wait_for_loading() # for custom query

        self.click_menu('pvc-pv-module-manager', text='Create PV Module')
        self.wait_for_loading()

        name = self._uniquestring('pvmodule')

        form_data = {
            'supplier': name,
            'type': 'Selenium Module Type',
            'cell_type': 'Selenium Cell Type',
            'nominal_power': 15,
            'weight': '25.25',
            'width': 35,
            'length': 45,
        }

        self.fill_forms(panel_name='pvc-pv-module-details', forms_data={'pvc-pv-module-form': form_data})

        self.click_button(panel_name='pvc-pv-module-details', text='Save')
        self.wait_for_loading()

        # filter the cq to the newly created license
        self.filter_cq(panel_name='pvc-pv-module-manager', filters=[
            ('Brand', 'Equal', name),
        ])

        self.assert_cq(panel_name='pvc-pv-module-manager', row_index=0, data={
            'supplier': name,
            'type': 'Selenium Module Type',
            'cell_type': 'Selenium Cell Type',
            'nominal_power': '15.00',
            'weight': '25.25',
            'width': '35.00',
            'length': '45.00',
            'status': 'Pending',
            'first_approval': self.operator_account.name,
        });

    def test_create_and_edit_pv_module_incorrect_values(self):
        self.login(self.operator_account.name, 'test_password')

        self.click_iconbar('icon-pvc-pv-module-manager', hover='icon-pvc-declaration-manager')
        self.wait_for_loading() # for custom query

        self.click_menu('pvc-pv-module-manager', text='Create PV Module')
        self.wait_for_loading()

        name = self._uniquestring('pvmodule')

        form_data = {
            'supplier': name,
            'type': 'Selenium Module Type',
            'cell_type': 'Selenium Cell Type',
            'nominal_power': 1,
            'weight': '25.25',
            'width': 0,
            'length': 45,
        }

        self.fill_forms(panel_name='pvc-pv-module-details', forms_data={'pvc-pv-module-form': form_data})

        self.click_button(panel_name='pvc-pv-module-details', text='Save')
        self.wait_for_loading2(assert_no_errors=False)
        self.assert_growl_errors_present()


    def test_member_select_pv_modules(self):
        self._create_contribution_tax_rates(self.operator_account)

        set_date(date(2013, 6, 1)) # set date to June 2013

        _modules = self._fill_pv_modules(self.operator_account)
        # this TBS has it's setting on MONTHLY!
        tbs = self._create_tbs_account(self.operator_account)

        _administration_fee = Decimal('250.00')
        _contribution_fee = Decimal('2.75')

        service_level = self._create_tbs_service_level(tbs, admin_fee=_administration_fee, contrib_fee=_contribution_fee, year=today().year)
        ma = self._create_member_account()

        # apply at TBS
        tm = self._member_apply_at_tbs(ma, tbs, service_level)

        # activate member to automatically create an order entry for the Administration Fee
        mc = PvcMemberController(self.sa_session, account_id=tbs.id)
        mc.save_tbs_member(tm.id, {'status': PvcTbsMember.STATUS_APPROVED, 'is_full_member': False, 'is_representative': False})

        self.login(ma.name, 'test_password')
        self.click_iconbar('icon-pvc-pv-module-select', hover='icon-pvc-pv-module-country-manager')
        self.wait_for_loading() # for selection panel

        forms_data = {
            'pvc-pv-module-form': {
                'supplier': _modules[0].supplier.name,
                'type': _modules[0].type.name,
            },

        }
        self.fill_forms(panel_name='pvc-pv-module-select', forms_data=forms_data)

        self.fill_forms(panel_name='pvc-pv-module-select', forms_data={
            'pvc-pv-module-countries': {
                'countries': ('fancy-select', [tbs.default_group.license.country.name], ),
            }
        })
        self.wait_for_loading()

        self.click_button(panel_name='pvc-pv-module-select', text='Save')
        self.wait_for_loading()

        self.click_iconbar('icon-pvc-pv-module-country-manager')
        self.wait_for_loading()

        self.filter_cq(panel_name='pvc-pv-module-country-manager', filters=[
            ('Brand', 'Equal', _modules[0].supplier.name),
        ])

        self.assert_cq(panel_name='pvc-pv-module-country-manager', row_index=0, data={
            'countries': tbs.default_group.license.country.name,
        })

    def test_member_select_reject_pv_module(self):
        self._create_contribution_tax_rates(self.operator_account)

        set_date(date(2013, 6, 1)) # set date to June 2013

        _modules = self._fill_pv_modules(self.operator_account)

        # mark module as rejected
        m = _modules[0]
        m.approval_status = PvcPVModule.PV_MODULE_STATUS_REJECTED
        self.sa_session.add(m)
        self.sa_session.flush()

        # this TBS has it's setting on MONTHLY!
        tbs = self._create_tbs_account(self.operator_account)

        _administration_fee = Decimal('250.00')
        _contribution_fee = Decimal('2.75')

        service_level = self._create_tbs_service_level(tbs, admin_fee=_administration_fee, contrib_fee=_contribution_fee, year=today().year)
        ma = self._create_member_account()

        # apply at TBS
        tm = self._member_apply_at_tbs(ma, tbs, service_level)

        # activate member to automatically create an order entry for the Administration Fee
        mc = PvcMemberController(self.sa_session, account_id=tbs.id)
        mc.save_tbs_member(tm.id, {'status': PvcTbsMember.STATUS_APPROVED, 'is_full_member': False, 'is_representative': False})

        self.login(ma.name, 'test_password')
        self.click_iconbar('icon-pvc-pv-module-select', hover='icon-pvc-pv-module-country-manager')
        self.wait_for_loading() # for selection panel

        forms_data = {
            'pvc-pv-module-form': {
                'supplier': m.supplier.name,
                'type': m.type.name,
            },
        }
        self.fill_forms(panel_name='pvc-pv-module-select', forms_data=forms_data)

        self.fill_forms(panel_name='pvc-pv-module-select', forms_data={
            'pvc-pv-module-countries': {
                'countries': ('fancy-select', [tbs.default_group.license.country.name], ),
            }
        })
        self.wait_for_loading()

        self.click_button(panel_name='pvc-pv-module-select', text='Save')
        self.wait_for_loading(assert_no_errors=False)
        self.assert_growl_errors_present()

    def test_multi_member(self):
        """
        pvcycle.test.selenium.test_pv_module.TestCase.test_multi_member
        """

        self._create_contribution_tax_rates(self.operator_account)
        set_date(date(2013, 6, 1)) # set date to June 2013

        _modules = self._fill_pv_modules(self.operator_account)

        _administration_fee = Decimal('250.00')
        _contribution_fee = Decimal('2.75')

        tbs_account = self._create_tbs_account(self.operator_account)
        service_level = self._create_tbs_service_level(tbs_account, admin_fee=_administration_fee, contrib_fee=_contribution_fee, year=today().year)

        state = self._get_tbs_license_state()
        state['tbs']['country'] = 81 # germany

        tbs_account_2 = self._create_tbs_account(self.operator_account, state=state)
        service_level_2 = self._create_tbs_service_level(tbs_account_2, admin_fee=_administration_fee, contrib_fee=_contribution_fee, year=today().year)

        # create member account with its default license
        member_account = self._create_member_account('company 1')
        license_1 = member_account.default_group.license

        # create a second company
        mc = PvcMemberController(self.sa_session, account_id=member_account.id)
        license_2 = mc.save_member_license(self._get_member_license_state('company 2'))

        # apply first company to default tbs
        tbs_member_1 = self._member_apply_at_tbs(member_account, tbs_account, service_level, member_license_id=license_1.id)

        # apply second company to default tbs
        tbs_member_2 = self._member_apply_at_tbs(member_account, tbs_account, service_level, member_license_id=license_2.id)

        # apply second company to second tbs
        tbs_member_3 = self._member_apply_at_tbs(member_account, tbs_account_2, service_level_2, member_license_id=license_2.id)

        # update timestamps so the period select works correctly
        tbs_member_1.created = datetime.datetime(year=2013, month=1, day=1)
        tbs_member_2.created = datetime.datetime(year=2013, month=1, day=1)
        tbs_member_3.created = datetime.datetime(year=2013, month=1, day=1)
        self.sa_session.add(tbs_member_1)
        self.sa_session.add(tbs_member_2)
        self.sa_session.flush()

        # activate member to automatically create an order entry for the Administration Fee
        mc_tbs_1 = PvcMemberController(self.sa_session, account_id=tbs_account.id)
        mc_tbs_2 = PvcMemberController(self.sa_session, account_id=tbs_account_2.id)
        mc_tbs_1.save_tbs_member(tbs_member_1.id, {'status': PvcTbsMember.STATUS_APPROVED, 'is_full_member': False, 'is_representative': False})
        mc_tbs_1.save_tbs_member(tbs_member_2.id, {'status': PvcTbsMember.STATUS_APPROVED, 'is_full_member': False, 'is_representative': False})
        mc_tbs_2.save_tbs_member(tbs_member_3.id, {'status': PvcTbsMember.STATUS_APPROVED, 'is_full_member': False, 'is_representative': False})

        self.login(member_account.name, 'test_password')

        self.click_iconbar('icon-pvc-pv-module-select', hover='icon-pvc-pv-module-country-manager')
        self.wait_for_loading(stable_time=2) # for selection panel

        self.fill_form('pvc-pv-module-select', values={
            'member_id': ('select', license_2.name),
        })
        self.wait_for_loading()

        forms_data = {
            'pvc-pv-module-form': {
                'supplier': _modules[0].supplier.name,
                'type': _modules[0].type.name,
            },

        }
        self.fill_forms(panel_name='pvc-pv-module-select', forms_data=forms_data)
        self.fill_forms(panel_name='pvc-pv-module-select', forms_data={
            'pvc-pv-module-countries': {
                'countries': ('fancy-select', ['Germany'], ),
            }
        })

        self.wait_for_loading(stable_time=4)

        self.click_button(panel_name='pvc-pv-module-select', text='Save')
        self.wait_for_loading()

        self.click_iconbar('icon-pvc-pv-module-country-manager')
        self.wait_for_loading()

        self.filter_cq(panel_name='pvc-pv-module-country-manager', filters=[
            ('Brand', 'Equal', _modules[0].supplier.name),
        ])

        self.assert_cq(panel_name='pvc-pv-module-country-manager', row_index=0, data={
            'member': license_2.name,
            'countries': 'Germany',
        })

    def test_pvmodule_declaration_overview(self):
        self._create_contribution_tax_rates(self.operator_account)

        set_date(date(2013, 6, 1)) # set date to June 2013

        _modules = self._fill_pv_modules(self.operator_account)
        # this TBS has it's setting on MONTHLY!
        tbs = self._create_tbs_account(self.operator_account)

        _administration_fee = Decimal('250.00')
        _contribution_fee = Decimal('2.75')

        service_level = self._create_tbs_service_level(tbs, admin_fee=_administration_fee, contrib_fee=_contribution_fee, year=today().year)
        ma = self._create_member_account()

        # apply at TBS
        tm = self._member_apply_at_tbs(ma, tbs, service_level)

        # activate member to automatically create an order entry for the Administration Fee
        mc = PvcMemberController(self.sa_session, account_id=tbs.id)
        mc.save_tbs_member(tm.id, {'status': PvcTbsMember.STATUS_PENDING, 'is_full_member': False, 'is_representative': False})

        controller = PvcPVModuleController(self.sa_session, account_id=ma.id)

        controller.save_or_update_module_for_countries(tm.member.id, [tm.tbs.country.id], _modules[0])
        self.sa_session.expire(tm)

        self.login(self.operator_account.name, 'test_password')
        self.click_iconbar('icon-pvc-pv-module-import-manager', hover='icon-pvc-pv-module-manager')
        self.wait_for_loading() # for selection panel

        css = "div[name='pv-module-declaration-cq']  div.bs-custom_query tbody.yui-dt-data tr:nth-child(2) > td.yui-dt-col-member"
        el = self._driver.find_element_by_css_selector(css)

        self.assertNotEqual(el.text, tm.member.name)

# below is for running the test from eclipse
if __name__ == '__main__':

    from twisted.scripts.trial import run
    import sys
    sys.argv = ['test_member.py', 'pvcycle.test.selenium.test_pv_module.TestCase.test_member_edit_countries']
    run()
    sys.exit()

