'''
Created on 25 Jul 2013

@author: martijn
'''

import random

from pvcycle.test.selenium.pvc_selenium import PvcSeleniumTestCase

class TestCase(PvcSeleniumTestCase):

    def setUp(self):

        super(TestCase, self).setUp()
        self.connect_db()


    def test_create_accountant(self):

        rand_id = str(random.random())[2:]

        # login as operator
        self.operator_account = self._create_operator_account()
        self.login(self.operator_account.name, 'test_password')

        # create a TBS license to link to this accountant.
        tbs = self._create_tbs_account(self.operator_account)
        tbs_license = tbs.groups[0].group.license

        # open accountant manager
        self.click_iconbar('icon-pvc-accountant-manager')
        self.wait_for_loading() # for the custom query

        # data for saving and asserting
        forms_data = {
            'tbs_form': {
                'tbs_id': ('select', tbs_license.name),
            },
            'company_form': {
                'name': 'Accountant Company Name ' + rand_id,
                'website': 'http://accountant.com',
                'vat_no': 'BE0893027827'
            },
            'address_form': {
                'street': 'Accountant Street',
                'city': 'Amsterdam',
                'postcode': '1000AA',
                'country': ('select', 'Netherlands'),
                'state': 'NH'
            },
            'person_form': {
                'last_name': 'Accountant Last Name',
                'street': 'Accountant Person Street 1',
                'country': ('select', 'Netherlands'),
            },
        }

        # create new accountant
        self.click_menu('pvc-accountant-manager', text='Create accountant')
        self.wait_for_loading(times=2)

        self.open_fieldset(panel_name='pvc-accountant-details', title='Take back schemes')
        self.fill_forms(panel_name='pvc-accountant-details', forms_data=forms_data)

        self.click_button(panel_name='pvc-accountant-details', text='Save')
        self.wait_for_loading()

        # filter the cq to the newly created license
        self.filter_cq(panel_name='pvc-accountant-manager', filters=[
            ('Name', 'Equal', 'Accountant Company Name ' + rand_id),
        ])

        # verify collins
        self.assert_cq(panel_name='pvc-accountant-manager', row_index=0, data={
            'license_name': 'Accountant Company Name ' + rand_id,
        });

        # open accountant details
        self.click_cq(panel_name='pvc-accountant-manager', row_index=0, col_name='license_name')
        self.wait_for_panel('pvc-accountant-details')
        self.wait_for_loading()

        # assert that values are correctly saved and reloaded
        self.open_fieldset(panel_name='pvc-accountant-details', title='Take back schemes')
        self.assert_forms(panel_name='pvc-accountant-details', forms_data=forms_data)

        # save again so updating works
        self.click_button(panel_name='pvc-accountant-details', text='Save')
        self.wait_for_loading()


# below is for running the test from eclipse
if __name__ == '__main__':

    from twisted.scripts.trial import run
    import sys
    sys.argv = ['test_member.py', 'pvcycle.test.selenium.test_accountant.TestCase.test_create_accountant']
    run()
    sys.exit()


