'''
Created on Feb 27, 2014

@author: marten
'''
from datetime import date
from decimal import Decimal
from twisted.trial.unittest import SkipTest

from basesoft.util.virtual_date import set_date, today

from pvcycle.test.selenium.pvc_selenium import PvcSeleniumTestCase
from pvcycle.core.controllers import PvcMemberController
from pvcycle.core.models import PvcTbsMember
from pvcycle.report.controllers import PvcHistoricalDataController
import datetime

class TestCase(PvcSeleniumTestCase):
    def setUp(self):
        raise SkipTest('Deprecated with PV CYCLE 2.0')

        super(TestCase, self).setUp()
        self.connect_db()

        set_date(date(2014,1,1))

        self.operator = self._create_operator_account()
        self._create_contribution_tax_rates(self.operator)

        # this TBS has it's setting on MONTHLY!
        self.tbs_account = self._create_tbs_account(self.operator)
        self.accountant = self._create_accountant_account(self.tbs_account.default_group.license, self.operator)

        self._administration_fee = Decimal('250.00')
        self._contribution_fee = Decimal('2.75')

        self.service_level = self._create_tbs_service_level(self.tbs_account, admin_fee=self._administration_fee, contrib_fee=self._contribution_fee, year=today().year)

        self.ma = self._create_member_account()

        # apply at TBS
        self.tm = self._member_apply_at_tbs(self.ma, self.tbs_account, self.service_level)

        # activate member to automatically create an order entry for the Administration Fee
        mc = PvcMemberController(self.sa_session, account_id=self.tbs_account.id)
        mc.save_tbs_member(self.tm.id, {'status': PvcTbsMember.STATUS_APPROVED, 'is_full_member': False, 'is_representative': False})

    def _do_update(self):
        self.click_iconbar(icon_id='icon-pvc-historical-data-manager')
        self.wait_for_loading()

        self.filter_cq('pvc-historical-data-manager', filters=(
           ('Member', 'Contains', self.tm.member.name),
           ('TBS', 'Equal', ('select', self.tm.tbs.name)),
        ))
        self.wait_for_loading()

        self.click_cq('pvc-historical-data-manager', row_index=0, col_name='details')
        self.wait_for_loading()

        self.assert_form('pvc-historical-data-details', values={
            'tbs_id': ('select', self.tm.tbs.name),
            'year': ('select', '2012'),
            'country_id': ('select', 'Netherlands'),
            'megawatt': '100.10',
            'quantity': '200',
            'weight': '300.30',
            'b2b': '23.45',
            'b2c': '76.55',
        })

        self.fill_form('pvc-historical-data-details', values={
            'megawatt': '110.1',
            'quantity': '210',
            'weight': '310.3',
            'b2b': '26.99',
            'b2c': '73.01',
        })
        self.click_button('pvc-historical-data-details', text='Save')
        self.wait_for_loading()

        self.assert_cq('pvc-historical-data-manager', row_index=0, data={
            'megawatt': '110.10',
            'quantity': '210',
            'weight': '310.30',
            'b2b': '26.99',
            'b2c': '73.01',
            'country': 'Netherlands'
        })

    def test_batch_creation_by_member(self):
        self.login(self.ma.name, 'test_password')

        self.click_iconbar(icon_id='icon-pvc-historical-data-manager')
        self.wait_for_loading()

        self.click_menu(panel_name='pvc-historical-data-manager', text='Add historical data')
        self.wait_for_loading()

        self.fill_form('pvc-historical-data-batch', values={
            'tbs_id': ('select', self.tm.tbs.name),
        })
        self.wait_for_loading() # YEARS and Countries are loading after TBS selection

        self.fill_form('pvc-historical-data-batch', values={
            'year': ('select', '2010'),
            'countries': ('fancy-select', ('Netherlands', )),
        })

        # 1 line is now added (because only one Country), 155 is the ID of Netherlands
        self.fill_form('pvc-historical-data-batch', values={
            'megawatt_155': '100.1',
            'quantity_155': '110',
            'weight_155': '120.3',
            'b2b_155': 25,
            'b2c_155': 75,
        })
        self.click_button('pvc-historical-data-batch', text='Save')
        self.wait_for_loading()

        # validate through details panel
        self.click_cq('pvc-historical-data-manager', row_index=0, col_name='details')
        self.wait_for_loading()

        self.assert_form('pvc-historical-data-details', values={
            'tbs_id': ('select', self.tm.tbs.name),
            'year': ('select', '2010'),
            'country_id': ('select', 'Netherlands'),
            'megawatt': '100.10',
            'quantity': '110',
            'weight': '120.30',
            'b2b': '25.00',
            'b2c': '75.00',
        })

    def test_zero_creation(self):
        self.login(self.ma.name, 'test_password')

        self.click_iconbar(icon_id='icon-pvc-historical-data-manager')
        self.wait_for_loading()

        self.click_menu(panel_name='pvc-historical-data-manager', text='Add historical data')
        self.wait_for_loading()

        self.fill_form('pvc-historical-data-batch', values={
            'tbs_id': ('select', self.tm.tbs.name),
        })
        self.wait_for_loading() # YEARS and Countries are loading after TBS selection

        self.fill_form('pvc-historical-data-batch', values={
            'year': ('select', '2010'),
            'countries': ('fancy-select', ('Netherlands', )),
        })

        # 1 line is now added (because only one Country), 155 is the ID of Netherlands
        self.fill_form('pvc-historical-data-batch', values={
            'megawatt_155': '0',
            'quantity_155': '0',
            'weight_155': '0',
            'b2b_155': 0,
            'b2c_155': 0,
        })
        self.click_button('pvc-historical-data-batch', text='Save')
        self.wait_for_loading()

        # validate through details panel
        self.click_cq('pvc-historical-data-manager', row_index=0, col_name='details')
        self.wait_for_loading()

        self.assert_form('pvc-historical-data-details', values={
            'tbs_id': ('select', self.tm.tbs.name),
            'year': ('select', '2010'),
            'country_id': ('select', 'Netherlands'),
            'megawatt': '0.00',
            'quantity': '0',
            'weight': '0.00',
            'b2b': '0.00',
            'b2c': '0.00',
        })

    def test_update_by_member(self):
        controller = PvcHistoricalDataController(self.sa_session, account_id=self.ma.id)
        controller.save_or_update_historical_data(None, self.tm.id, 2012, 155, Decimal('100.1'), 200, Decimal('300.3'), Decimal('23.45'), Decimal('76.55'))

        self.login(self.ma.name, 'test_password')

        self._do_update()

    def test_update_by_operator(self):
        controller = PvcHistoricalDataController(self.sa_session, account_id=self.ma.id)
        controller.save_or_update_historical_data(None, self.tm.id, 2012, 155, Decimal('100.1'), 200, Decimal('300.3'), Decimal('23.45'), Decimal('76.55'))

        self.login(self.operator.name, 'test_password')

        self._do_update()

    def test_update_by_tbs(self):
        controller = PvcHistoricalDataController(self.sa_session, account_id=self.ma.id)
        controller.save_or_update_historical_data(None, self.tm.id, 2012, 155, Decimal('100.1'), 200, Decimal('300.3'), Decimal('23.45'), Decimal('76.55'))

        self.login(self.tbs_account.name, 'test_password')

        self._do_update()

    def _do_delete(self):
        self.click_iconbar(icon_id='icon-pvc-historical-data-manager')
        self.wait_for_loading()

        self.filter_cq('pvc-historical-data-manager', filters=(
           ('Member', 'Contains', self.tm.member.name),
           ('TBS', 'Equal', ('select', self.tm.tbs.name)),
        ))
        self.wait_for_loading()

        self.click_cq('pvc-historical-data-manager', row_index=0, col_name='row_select', extra_css=' input')

        self.click_menu(panel_name='pvc-historical-data-manager', text='Delete historical data')
        self.wait_for_loading()

        self.click_button('dialog-question', text='Yes')
        self.wait_for_loading(stable_time=2)

        self.assert_cq_empty('pvc-historical-data-manager')

    def test_deletion_by_operator(self):
        controller = PvcHistoricalDataController(self.sa_session, account_id=self.ma.id)
        controller.save_or_update_historical_data(None, self.tm.id, 2012, 155, Decimal('100.1'), 200, Decimal('300.3'), Decimal('23.45'), Decimal('76.55'))

        self.login(self.operator.name, 'test_password')

        self._do_delete()

    def test_deletion_by_tbs(self):
        controller = PvcHistoricalDataController(self.sa_session, account_id=self.ma.id)
        controller.save_or_update_historical_data(None, self.tm.id, 2012, 155, Decimal('100.1'), 200, Decimal('300.3'), Decimal('23.45'), Decimal('76.55'))

        self.login(self.tbs_account.name, 'test_password')

        self._do_delete()

    def test_multi_member(self):
        """
        pvcycle.test.selenium.test_historical_data.TestCase.test_multi_member
        """

        # create second tbs (used for second company)
        tbs_account_2 = self._create_tbs_account(self.operator)
        service_level_2 = self._create_tbs_service_level(tbs_account_2, admin_fee=self._administration_fee, contrib_fee=self._contribution_fee, year=today().year)

        # create member account with its default license
        member_account = self._create_member_account('company 1')
        license_1 = member_account.default_group.license

        # create a second company
        mc = PvcMemberController(self.sa_session, account_id=member_account.id)
        license_2 = mc.save_member_license(self._get_member_license_state('company 2'))

        # apply first company to default tbs
        tbs_member_1 = self._member_apply_at_tbs(member_account, self.tbs_account, self.service_level, member_license_id=license_1.id)

        # apply second company to default tbs
        tbs_member_2 = self._member_apply_at_tbs(member_account, self.tbs_account, self.service_level, member_license_id=license_2.id)

        # apply second company to second tbs
        tbs_member_3 = self._member_apply_at_tbs(member_account, tbs_account_2, service_level_2, member_license_id=license_2.id)

        # update timestamps so the period select works correctly
        tbs_member_1.created = datetime.datetime(year=2013, month=1, day=1)
        tbs_member_2.created = datetime.datetime(year=2013, month=1, day=1)
        tbs_member_3.created = datetime.datetime(year=2013, month=1, day=1)
        self.sa_session.add(tbs_member_1)
        self.sa_session.add(tbs_member_2)
        self.sa_session.flush()

        # activate member to automatically create an order entry for the Administration Fee
        mc_tbs_1 = PvcMemberController(self.sa_session, account_id=self.tbs_account.id)
        mc_tbs_2 = PvcMemberController(self.sa_session, account_id=tbs_account_2.id)
        mc_tbs_1.save_tbs_member(tbs_member_1.id, {'status': PvcTbsMember.STATUS_APPROVED, 'is_full_member': False, 'is_representative': False})
        mc_tbs_1.save_tbs_member(tbs_member_2.id, {'status': PvcTbsMember.STATUS_APPROVED, 'is_full_member': False, 'is_representative': False})
        mc_tbs_2.save_tbs_member(tbs_member_3.id, {'status': PvcTbsMember.STATUS_APPROVED, 'is_full_member': False, 'is_representative': False})


        self.login(member_account.name, 'test_password')

        self.click_iconbar(icon_id='icon-pvc-historical-data-manager')
        self.wait_for_loading()

        self.click_menu(panel_name='pvc-historical-data-manager', text='Add historical data')
        self.wait_for_loading()

        self.fill_form('pvc-historical-data-batch', values={
            'member_id': ('select', license_2.name),
        })
        self.wait_for_loading()

        self.fill_form('pvc-historical-data-batch', values={
            'tbs_id': ('select', tbs_member_3.tbs.name),
        })
        self.wait_for_loading()

        # YEARS and Countries are loading after TBS selection

        self.fill_form('pvc-historical-data-batch', values={
            'year': ('select', '2010'),
            'countries': ('fancy-select', ('Netherlands', )),
        })

        # 1 line is now added (because only one Country), 155 is the ID of Netherlands
        self.fill_form('pvc-historical-data-batch', values={
            'megawatt_155': '100.1',
            'quantity_155': '110',
            'weight_155': '120.3',
            'b2b_155': 25,
            'b2c_155': 75,
        })
        self.click_button('pvc-historical-data-batch', text='Save')
        self.wait_for_loading()

        # validate through details panel
        self.click_cq('pvc-historical-data-manager', row_index=0, col_name='details')
        self.wait_for_loading()

        self.assert_form('pvc-historical-data-details', values={
            'member_id': ('select', tbs_member_3.member.name),
            'tbs_id': ('select', tbs_member_3.tbs.name),
            'year': ('select', '2010'),
            'country_id': ('select', 'Netherlands'),
            'megawatt': '100.10',
            'quantity': '110',
            'weight': '120.30',
            'b2b': '25.00',
            'b2c': '75.00',
        })

        self.click_button('pvc-historical-data-details', text='Save')
        self.wait_for_loading()


# below is for running the test from eclipse
if __name__ == '__main__':

    from twisted.scripts.trial import run
    import sys
    sys.argv = ['test_historical.py', 'pvcycle.test.selenium.test_historical_data.TestCase.test_deletion_by_operator']
    run()
    sys.exit()
