# -*- coding: utf-8 -*-

'''
Created on 25 Jul 2013

@author: martijn
'''

import random
import time
from datetime import date

from django.conf import settings
from basesoft.web.contrib.account.controllers import AccountController
from basesoft.web.contrib.account.models import Account, AccountActivation
from pvcycle.product.controllers import PvcProductController
from pvcycle.product.models import PvcTbsProduct, PvcTbsProductFee
from pvcycle.test.selenium.pvc_selenium import PvcSeleniumTestCase
from pvcycle.core.controllers import PvcMemberController, PvcTBSController
from pvcycle.core.models import PvcTbsMember, PvcTbsLicense, PvcTbsMemberProofFinancialGuaranteeFile
from basesoft.util.virtual_date import today, set_date
from basesoft.web.core.util.controllers import LicenseConfigController
from pvcycle.order.controllers import PvcOrderController, PvcContributionOrderController
from basesoft.web.contrib.order.controllers import PaymentController
from basesoft.web.contrib.order.models import PaymentMethodBankTransfer
from selenium.common.exceptions import NoSuchElementException
from basesoft.web.contrib.contact.models import Company
from decimal import Decimal

class TestCase(PvcSeleniumTestCase):
    """
    pvcycle.test.selenium.test_member.TestCase
    """

    def setUp(self):
        super(TestCase, self).setUp()
        set_date(date(2014, 2, 2))
        self.connect_db()
        q = """
            UPDATE tblCompany set vat_no = CONCAT(vat_no, '_test') WHERE SUBSTRING(vat_no, -5) != '_test';
        """
        self.sa_session.execute(q)

    def _setup_member(self, tbs_account, membership=False, create_service_level=True):
        rand_id = str(random.random())[2:]
        username = 'pvc_%s' % rand_id

        self.sa_session.flush()

        if create_service_level:

            if membership:
                self._create_memberships(tbs_account)
            else:
                self._create_tbs_service_level(tbs_account)

        # logout
        self.clear_cookies()

        # open homepage
        self._driver.get(settings.SELENIUM['browserURL'])
        self.wait_for_loading(stable_time=3)

        # fill register form
        self.fill_form(form_name='registration-form', values={
            'username': username,
            'email': '%s@mail-test.basesoft.com' % username,
            'password': 'dafshkudsfhjgdfsghj4',
            'password2': 'dafshkudsfhjgdfsghj4',
        })

        # wait for passwowrd check
        self.wait_for_loading()

        # eula
        self.fill_form(form_name='registration-form', values={
            'eula': ('checkbox', True),
        })
        self.wait_for_loading()

        self.click_button(text='Register')
        self.wait_for_loading()
        self.click_button(panel_name='dialog-information', text='OK')

        # fake send email to user
        activation = self.sa_session.query(AccountActivation).filter(
            AccountActivation.account == self.sa_session.query(Account).filter(Account.name == username).one()).one()
        activation.activation_stage = AccountActivation.ACTIVATION_STAGE_PENDING
        self.sa_session.add(activation)
        self.sa_session.flush()

        # activate account
        account = self.sa_session.query(Account).filter(Account.name == username).one()
        ac = AccountController(self.sa_session)
        aa = ac.get_account_activation(account)
        self._driver.get(settings.SELENIUM['browserURL']+'#m=activate-%s' % aa.activation_key)
        self.wait_for_loading();
        self.fill_form(panel_name='registration-completion-panel', form_name='person_form', values={
            'first_name': 'Huweii',
            'last_name': 'Nguyen',
            'preferred_locale': ('select', 'English', ),
            'gender': ('select', 'value=male'),
            'phone': '0201234'
        })
        self.click_button(text='Complete registration')
        self.wait_for_loading()

        # login
        self.fill_form(panel_name='login-dialog', values={
            'username': username,
            'password': 'dafshkudsfhjgdfsghj4',
        })
        self.click_button(panel_name='login-dialog', text='Login')
        self.wait_for_loading(stable_time=5)

    def test_create_chinese_member(self):

        tbs_account = self._create_tbs_account()
        rand_id = str(random.random())[2:]
        self._setup_member(tbs_account)
        # we are now in the producer member login

        # click OK dialog
        self.click_button(panel_name='dialog-information', text='OK')

        # first set China.
        self.fill_forms(panel_name='pvc-member-wizard', forms_data={
            'country-form': {'country': ('select', 'China'),},
        })
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        # company details screen
        company_forms_data = {
            'company_form': {
                'name': 'Member Company Name ' + rand_id,
                'website': 'http://member.com',
                'vat_no': 'AR00000000000'
            },
            'address_form': {
                'street': 'Some Shanghai Street Name',
                'city': 'Shanghai',
                'postcode': 'GA SHA',
            },
            'bank_account_form': {
                'bank_name': "China Bank Huweii",
                'bic': "DSBACNBXSHA",
                'number': "324234234",
                'beneficiary': "China Beneficiary",
            },
            'bank_account_address_form': {
                'street': "BANK STREET",
                'city': "BANK CITY",
                'postcode': "BANK POSTCODE",
                'country': ('select', 'China'),
            }
        }
        self.fill_forms(panel_name='pvc-member-wizard', forms_data=company_forms_data)
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')

        self.wait_for_loading()

        # we stop here and we reload the page and see if the forms reload their data.
        self._driver.get(settings.SELENIUM['browserURL'])
        self.wait_for_loading()

        self.click_button(panel_name='dialog-information', text='OK')
        self.wait_for_loading()

        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        self.assert_forms(panel_name='pvc-member-wizard', forms_data=company_forms_data)

    def test_member_cannot_apply_at_local_tbs(self):
        state = self._get_tbs_license_state()
        state['tbs']['tbs_type'] = PvcTbsLicense.TBS_TYPE_LOCAL
        tbs_account = self._create_tbs_account(state=state)
        tbs = tbs_account.groups[0].group.license

        rand_id = str(random.random())[2:]
        self._setup_member(tbs_account)

        self.click_button(panel_name='dialog-information', text='OK')

        self.fill_forms(panel_name='pvc-member-wizard', forms_data={
            'country-form': {'country': ('select', 'Belgium'),},
        })
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        # company details screen
        company_forms_data = {
            'company_form': {
                'name': 'Member Company Name ' + rand_id,
                'website': 'http://member.com',
                'vat_no': 'RO1630615123457'
            },
            'address_form': {
                'street': 'Member Street',
                'street_2': 'Member Street 2',
                'city': 'Amsterdam',
                'postcode': '1000AA',
                'state': 'NH'
            },
            'bank_account_form': {
                'iban': "GB82WEST12345698765432",
                'bic': "UNCRIT2B912",
            }
        }

        self.fill_forms(panel_name='pvc-member-wizard', forms_data=company_forms_data)
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')

        self.wait_for_loading()

        # TODO: company/account/invoice contact

        person_forms_data = {
            'person_form': {
                'last_name': 'Member Contact Last Name',
                'first_name': 'Member Contact First Name',
                'email': 'test-member@mail-test.basesoft.com',
                'preferred_locale': ('select', 'English'),
                'gender': ('select', 'Male'),
                'phone': '234234235',
            },
            'invoice_person_form': {
                'is_same_person_as': ('checkbox', 'true')
            }
        }
        self.fill_forms(panel_name='pvc-member-wizard', forms_data=person_forms_data)

        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        self.assertRaises(NoSuchElementException, self.fill_form, panel_name='pvc-member-wizard', values={
            'tbs_id': ('select', tbs.name),
        })

    def test_italy_2013_member(self):
        tbs_account = self._create_tbs_account()

        tbs = tbs_account.groups[0].group.license
        lcc = LicenseConfigController(self.sa_session, scope=tbs)
        lcc.set('pvc.tbs.was.active.in.2013', True)

        rand_id = str(random.random())[2:]
        self._setup_member(tbs_account)

        self.click_button(panel_name='dialog-information', text='OK')
        self.fill_forms(panel_name='pvc-member-wizard', forms_data={
            'country-form': {'country': ('select', 'Italy'),},
        })
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        # company details screen
        company_forms_data = {
            'company_form': {
                'name': 'Member Company Name ' + rand_id,
                'website': 'http://member.com',
                'vat_no': 'RO1630615123457'
            },
            'address_form': {
                'street': 'Member Street',
                'street_2': 'Member Street 2',
                'city': 'Rome',
                'postcode': '1000AA',
                'state': 'NH'
            },
            'bank_account_form': {
                'iban': "GB82WEST12345698765432",
                'bic': "UNCRIT2B912",
            }
        }

        self.fill_forms(panel_name='pvc-member-wizard', forms_data=company_forms_data)
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading(assert_no_errors=False)
        # check that error is raised because compulsory fields for italy are not filled in
        self.assert_growl_errors_present()

        italy_form = {
            'company_form': {
                'it_fiscal_code': 'RCCMNL83S18D969H',
                'it_pec_address': 'pec@mail-test.basesoft.com',
            }
        }
        # fill in italy specific fields
        self.fill_forms(panel_name='pvc-member-wizard', forms_data=italy_form)
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        # TODO: company/account/invoice contact

        person_forms_data = {
            'person_form': {
                'last_name': 'Member Contact Last Name',
                'first_name': 'Member Contact First Name',
                'email': 'test-member@mail-test.basesoft.com',
                'preferred_locale': ('select', 'English'),
                'gender': ('select', 'Male'),
                'phone': '234234235',
            },
            'invoice_person_form': {
                'is_same_person_as': ('checkbox', 'true')
            }
        }
        self.fill_forms(panel_name='pvc-member-wizard', forms_data=person_forms_data)

        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        self.fill_form(panel_name='pvc-member-wizard', values={
            'tbs_id': ('select', tbs.name),
            'is_commissioner': ('checkbox', True),
            'selling_technique': ('checkbox', 'value=2'),
        })
        self.wait_for_loading()

        self.fill_form(panel_name='pvc-member-wizard', values={
            'was_member_in_2013': ('radio', 'true'),
        })
        self.fill_form(panel_name='pvc-member-wizard', values={
            'service_level_id': ('select', 'tbs service level 1'),
            'upload_later': ('checkbox', True),
        })

        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading2()

        # finish page
        self.click_button(panel_name='pvc-member-wizard', text='Finish')
        self.wait_for_loading()

        # open progress CQ
        self.click_iconbar('icon-pvc-member-registration-progress')
        self.wait_for_loading()

        # italy member doesnt have to upload document is 2013 member, status pending
        self.assert_cq(panel_name='pvc-member-progress-manager', row_index=0, data={
            'status': 'Pending'
        })

        # assert that italy specific fields are saved correctly
        company_data = self.sa_session.query(Company).filter(Company.name == company_forms_data['company_form']['name']).first()
        self.assertTrue(company_data.it_fiscal_code, italy_form['company_form']['it_fiscal_code'])
        self.assertTrue(company_data.it_pec_address, italy_form['company_form']['it_pec_address'])

        # login as the TBS and verify company details
        self.login(tbs_account.name, 'test_password')
        self.click_iconbar('icon-pvc-member-manager', hover='icon-pvc-tbs-service-level')
        self.click_cq(panel_name='pvc-member-manager', row_index=0, col_name='member')
        self.wait_for_loading()
        self.assert_form(panel_name='master-data-panel', values={
            'name': 'Member Company Name ' + rand_id,
            'it_fiscal_code': italy_form['company_form']['it_fiscal_code'],
            'it_pec_address': italy_form['company_form']['it_pec_address'],
        })

    def test_create_french_member(self):
        """
        pvcycle.test.selenium.test_member.TestCase.test_create_french_member
        """

        operator_account = self._create_operator_account()

        tbs_account = self._create_tbs_account(operator_account)
        rand_id = str(random.random())[2:]
        self._setup_member(tbs_account)

        # click OK dialog
        self.click_button(panel_name='dialog-information', text='OK')

        # first set France.
        self.fill_forms(panel_name='pvc-member-wizard', forms_data={
            'country-form': {'country': ('select', 'France'),},
        })
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        # company details screen
        company_forms_data = {
            'company_form': {
                'name': 'Member Company Name ' + rand_id,
                'website': 'http://member.com',
                'vat_no': 'ATU12345675'
            },
            'address_form': {
                'street': 'Member Street',
                'street_2': 'Member Street 2',
                'city': 'Amsterdam',
                'postcode': '1000AA',
                'state': 'NH'
            },
            'bank_account_form': {
                'iban': "GB82WEST12345698765432",
                'bic': "UNCRIT2B912",
            }

        }

        self.fill_forms(panel_name='pvc-member-wizard', forms_data=company_forms_data)

        # the country change made the following fields available..
        self.fill_forms(panel_name='pvc-member-wizard', forms_data={
            'company_form': {
                'fr_rcs_number': '4857645',
                'fr_rcs_location': 'Orsennes',
                'fr_company_type': ('select', 'Entreprise individuelle'),
                'fr_naf_code': '1234A'
            }
        })

        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        person_forms_data = {
            'person_form': {
                'last_name': 'Member Contact Last Name',
                'first_name': 'Member Contact First Name',
                'email': 'test-member@mail-test.basesoft.com',
                'preferred_locale': ('select', 'English'),
                'gender': ('select', 'Male'),
                'phone': '234234235',
            },
            'invoice_person_form': {
                'is_same_person_as': ('checkbox', True)
            }
        }

        self.fill_forms(panel_name='pvc-member-wizard', forms_data=person_forms_data)
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        #
        # apply tbs page
        #

        # select tbs and wait for loading (for service level select)
        self.fill_form(panel_name='pvc-member-wizard', values={
            'tbs_id': ('select', tbs_account.groups[0].group.license.name),
            'is_commissioner': ('checkbox', True),
            'selling_technique': ('checkbox', 'value=2'),
        })
        self.wait_for_loading()

        # open and close preview contract
        self.click_button(panel_name='pvc-member-wizard', text='Preview and download contract')
        self.close_panel('member-preview-contract')

        self.fill_form(panel_name='pvc-member-wizard', values={
            'service_level_id': ('select', 'tbs service level 1'),
            'upload_contract': ('upload', 'utf8_€.pdf'),
        })

        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading2()

        # finish page
        self.click_button(panel_name='pvc-member-wizard', text='Finish')
        self.wait_for_loading2()


        # get the last company created, it should contain a CIF
        company = self.sa_session.query(Company).filter(Company.is_temp==0).order_by(Company.id.desc()).limit(1).one()

        self.assertEqual(company.fr_rcs_number, '4857645')
        self.assertEqual(company.fr_rcs_location, 'Orsennes')
        self.assertEqual(company.fr_company_type, 'Entreprise individuelle')

        # login as the TBS and verify company details
        self.login(tbs_account.name, 'test_password')
        self.click_iconbar('icon-pvc-member-manager', hover='icon-pvc-tbs-service-level')
        self.click_cq(panel_name='pvc-member-manager', row_index=0, col_name='member')
        self.wait_for_loading()
        self.assert_form(panel_name='master-data-panel', values={
            'name': 'Member Company Name ' + rand_id,
            'fr_rcs_number': '4857645',
            'fr_rcs_location': 'Orsennes',
            'fr_company_type': ('select', 'Entreprise individuelle'),
            'fr_naf_code': '1234A'
        })

        # same as operator
        self.login(operator_account.name, 'test_password')
        self.click_iconbar('icon-pvc-member-manager', hover='icon-pvc-accountant-manager')
        self.wait_for_loading()
        self.filter_cq(panel_name='pvc-member-manager', filters=[
            ('Member', 'Equal', 'Member Company Name ' + rand_id),
        ])
        self.click_cq(panel_name='pvc-member-manager', row_index=0, col_name='member')
        self.wait_for_loading()
        self.assert_form(panel_name='master-data-panel', values={
            'name': 'Member Company Name ' + rand_id,
            'fr_rcs_number': '4857645',
            'fr_rcs_location': 'Orsennes',
            'fr_company_type': ('select', 'Entreprise individuelle'),
            'fr_naf_code': '1234A'
        })

    def test_create_spanish_member(self):

        operator_account = self._create_operator_account()

        tbs_account = self._create_tbs_account(operator_account)
        rand_id = str(random.random())[2:]
        self._setup_member(tbs_account)

        # click OK dialog
        self.click_button(panel_name='dialog-information', text='OK')

        # first set Spain.
        self.fill_forms(panel_name='pvc-member-wizard', forms_data={
            'country-form': {'country': ('select', 'Spain'),},
        })
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        # company details screen
        company_forms_data = {
            'company_form': {
                'name': 'Member Company Name ' + rand_id,
                'website': 'http://member.com',
                'vat_no': 'ATU12345675'
            },
            'address_form': {
                'street': 'Member Street',
                'street_2': 'Member Street 2',
                'city': 'Amsterdam',
                'postcode': '1000AA',
                'state': 'NH'
            },
            'bank_account_form': {
                'iban': "GB82WEST12345698765432",
                'bic': "UNCRIT2B912",
            }

        }

        self.fill_forms(panel_name='pvc-member-wizard', forms_data=company_forms_data)

        # the country change made this field available..
        self.fill_forms(panel_name='pvc-member-wizard', forms_data={
            'company_form': {
                'es_cif_nr': 'J99216582'
            }
        })

        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        person_forms_data = {
            'person_form': {
                'last_name': 'Member Contact Last Name',
                'first_name': 'Member Contact First Name',
                'email': 'test-member@mail-test.basesoft.com',
                'preferred_locale': ('select', 'English'),
                'gender': ('select', 'Male'),
                'phone': '234234235',
            },
            'invoice_person_form': {
                'is_same_person_as': ('checkbox', True)
            }
        }

        self.fill_forms(panel_name='pvc-member-wizard', forms_data=person_forms_data)
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        #
        # apply tbs page
        #

        # select tbs and wait for loading (for service level select)
        self.fill_form(panel_name='pvc-member-wizard', values={
            'tbs_id': ('select', tbs_account.groups[0].group.license.name),
            'is_commissioner': ('checkbox', True),
            'selling_technique': ('checkbox', 'value=2'),
        })
        self.wait_for_loading()

        # open and close preview contract
        self.click_button(panel_name='pvc-member-wizard', text='Preview and download contract')
        self.close_panel('member-preview-contract')

        self.fill_form(panel_name='pvc-member-wizard', values={
            'service_level_id': ('select', 'tbs service level 1'),
            'upload_contract': ('upload', 'utf8_€.pdf'),
        })

        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading2()

        # finish page
        self.click_button(panel_name='pvc-member-wizard', text='Finish')
        self.wait_for_loading2()

        # get the last company created, it should contain a CIF
        company = self.sa_session.query(Company).filter(Company.is_temp==0).order_by(Company.id.desc()).limit(1).one()
        self.assertEqual(company.es_cif_nr, 'J99216582')

        # login as the TBS and verify company details
        self.login(tbs_account.name, 'test_password')
        self.click_iconbar('icon-pvc-member-manager', hover='icon-pvc-tbs-service-level')
        self.click_cq(panel_name='pvc-member-manager', row_index=0, col_name='member')
        self.wait_for_loading()
        self.assert_form(panel_name='master-data-panel', values={'name': 'Member Company Name ' + rand_id, 'es_cif_nr': 'J99216582'})

        # same as operator
        self.login(operator_account.name, 'test_password')
        self.click_iconbar('icon-pvc-member-manager', hover='icon-pvc-accountant-manager')
        self.wait_for_loading()
        self.filter_cq(panel_name='pvc-member-manager', filters=[
            ('Member', 'Equal', 'Member Company Name ' + rand_id),
        ])
        self.click_cq(panel_name='pvc-member-manager', row_index=0, col_name='member')
        self.wait_for_loading()
        self.assert_form(panel_name='master-data-panel', values={'name': 'Member Company Name ' + rand_id, 'es_cif_nr': 'J99216582'})

    def test_german_member(self):
        tbs_account = self._create_tbs_account()

        tbs = tbs_account.groups[0].group.license
        lcc = LicenseConfigController(self.sa_session, scope=tbs)

        rand_id = str(random.random())[2:]
        self._setup_member(tbs_account)

        self.click_button(panel_name='dialog-information', text='OK')
        self.fill_forms(panel_name='pvc-member-wizard', forms_data={
            'country-form': {'country': ('select', 'Germany'),},
        })
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        # company details screen
        company_forms_data = {
            'company_form': {
                'name': 'Member Company Name ' + rand_id,
                'website': 'http://member.com',
                'vat_no': 'RO1630615123457'
            },
            'address_form': {
                'street': 'Member Street',
                'street_2': 'Member Street 2',
                'city': 'Berlin',
                'postcode': '1000AA',
                'state': 'NH'
            },
            'bank_account_form': {
                'iban': "GB82WEST12345698765432",
                'bic': "UNCRIT2B912",
            }
        }

        self.fill_forms(panel_name='pvc-member-wizard', forms_data=company_forms_data)
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading(assert_no_errors=False)

        # check that error is raised because compulsory fields for italy are not filled in
        self.assert_growl_errors_present()

        german_form = {
            'company_form': {
                'de_tax_number': 'RCCMNL83S18D969H',
            }
        }

        # fill in german specific fields
        self.fill_forms(panel_name='pvc-member-wizard', forms_data=german_form)
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        # TODO: company/account/invoice contact

        person_forms_data = {
            'person_form': {
                'last_name': 'Member Contact Last Name',
                'first_name': 'Member Contact First Name',
                'email': 'test-member@mail-test.basesoft.com',
                'preferred_locale': ('select', 'English'),
                'gender': ('select', 'Male'),
                'phone': '234234235',
            },
            'invoice_person_form': {
                'is_same_person_as': ('checkbox', 'true')
            }
        }
        self.fill_forms(panel_name='pvc-member-wizard', forms_data=person_forms_data)

        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        self.fill_form(panel_name='pvc-member-wizard', values={
            'tbs_id': ('select', tbs.name),
            'is_commissioner': ('checkbox', True),
            'selling_technique': ('checkbox', 'value=2'),
        })
        self.wait_for_loading()

        self.fill_form(panel_name='pvc-member-wizard', values={
            'service_level_id': ('select', 'tbs service level 1'),
            'upload_later': ('checkbox', True),
        })

        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading2()

        # finish page
        self.click_button(panel_name='pvc-member-wizard', text='Finish')
        self.wait_for_loading()

        # open progress CQ
        self.click_iconbar('icon-pvc-member-registration-progress')
        self.wait_for_loading()

        # assert that german specific fields are saved correctly
        company_data = self.sa_session.query(Company).filter(Company.name == company_forms_data['company_form']['name']).first()
        self.assertTrue(company_data.de_tax_number, german_form['company_form']['de_tax_number'])

        # login as the TBS and verify company details
        self.login(tbs_account.name, 'test_password')
        self.click_iconbar('icon-pvc-member-manager', hover='icon-pvc-tbs-service-level')
        self.click_cq(panel_name='pvc-member-manager', row_index=0, col_name='member')
        self.wait_for_loading()
        self.assert_form(panel_name='master-data-panel', values={
            'name': 'Member Company Name ' + rand_id,
            'de_tax_number': german_form['company_form']['de_tax_number'],
        })


    def test_create_member(self):
        tbs_account = self._create_tbs_account()
        rand_id = str(random.random())[2:]
        self._setup_member(tbs_account)
        # we are now in the producer member login
        # click OK dialog
        self.click_button(panel_name='dialog-information', text='OK')
        self.fill_forms(panel_name='pvc-member-wizard', forms_data={
            'country-form': {'country': ('select', 'Netherlands'),},
        })
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        # company details screen
        company_forms_data = {
            'company_form': {
                'name': 'Member Company Name ' + rand_id,
                'website': 'http://member.com',
                'vat_no': 'ATU12345675',
                'chamber_of_commerce_nr': 'KVK123',
            },
            'address_form': {
                'street': 'Member Street',
                'street_2': 'Member Street 2',
                'city': 'Amsterdam',
                'postcode': '1000AA',
                'state': 'NH'
            },
            'bank_account_form': {
                'iban': "GB82WEST12345698765432",
                'bic': "UNCRIT2B912",
            }
        }

        self.fill_forms(panel_name='pvc-member-wizard', forms_data=company_forms_data)

        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        # TODO: company/account/invoice contact

        person_forms_data = {
            'person_form': {
                'last_name': 'Member Contact Last Name',
                'first_name': 'Member Contact First Name',
                'email': 'test-member@mail-test.basesoft.com',
                'preferred_locale': ('select', 'English'),
                'gender': ('select', 'Male'),
                'phone': '234234235',
            },
            'invoice_person_form': {
                'is_same_person_as': ('checkbox', True)
            }
        }

        self.fill_forms(panel_name='pvc-member-wizard', forms_data=person_forms_data)
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        #
        # apply tbs page
        #

        # select tbs and wait for loading (for service level select)
        self.fill_form(panel_name='pvc-member-wizard', values={
            'tbs_id': ('select', tbs_account.groups[0].group.license.name),
            'is_commissioner': ('checkbox', True),
            'selling_technique': ('checkbox', 'value=2'),
        })
        self.wait_for_loading()

        # open and close preview contract
        self.click_button(panel_name='pvc-member-wizard', text='Preview and download contract')
        self.close_panel('member-preview-contract')

        self.fill_form(panel_name='pvc-member-wizard', values={
            'service_level_id': ('select', 'tbs service level 1'),
            'upload_contract': ('upload', 'utf8_€.pdf'),
        })

        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading2()

        # finish page
        self.click_button(panel_name='pvc-member-wizard', text='Finish')
        self.wait_for_loading2()

        element = self._driver.find_element_by_css_selector("div#icon-pvc-member-registration-progress")
        self.assertTrue(element.is_displayed())

    def test_create_member_selling_technique_other_countries(self):
        tbs_account = self._create_tbs_account()
        rand_id = str(random.random())[2:]
        self._setup_member(tbs_account)
        # we are now in the producer member login
        # click OK dialog
        self.click_button(panel_name='dialog-information', text='OK')
        self.fill_forms(panel_name='pvc-member-wizard', forms_data={
            'country-form': {'country': ('select', 'Netherlands'),},
        })
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        # company details screen
        company_forms_data = {
            'company_form': {
                'name': 'Member Company Name ' + rand_id,
                'website': 'http://member.com',
                'vat_no': 'ATU12345675',
                'chamber_of_commerce_nr': 'KVK123',
            },
            'address_form': {
                'street': 'Member Street',
                'street_2': 'Member Street 2',
                'city': 'Amsterdam',
                'postcode': '1000AA',
                'state': 'NH'
            },
            'bank_account_form': {
                'iban': "GB82WEST12345698765432",
                'bic': "UNCRIT2B912",
            }
        }

        self.fill_forms(panel_name='pvc-member-wizard', forms_data=company_forms_data)

        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        # TODO: company/account/invoice contact

        person_forms_data = {
            'person_form': {
                'last_name': 'Member Contact Last Name',
                'first_name': 'Member Contact First Name',
                'email': 'test-member@mail-test.basesoft.com',
                'preferred_locale': ('select', 'English'),
                'gender': ('select', 'Male'),
                'phone': '234234235',
            },
            'invoice_person_form': {
                'is_same_person_as': ('checkbox', True)
            }
        }

        self.fill_forms(panel_name='pvc-member-wizard', forms_data=person_forms_data)
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        #
        # apply tbs page
        #

        # select tbs and wait for loading (for service level select)
        self.fill_form(panel_name='pvc-member-wizard', values={
            'tbs_id': ('select', tbs_account.groups[0].group.license.name),
            'is_commissioner': ('checkbox', True),
            'selling_technique': ('checkbox', 'value=16'),
        })
        self.wait_for_loading()

        # open and close preview contract
        self.click_button(panel_name='pvc-member-wizard', text='Preview and download contract')
        self.close_panel('member-preview-contract')

        self.fill_form(panel_name='pvc-member-wizard', values={
            'service_level_id': ('select', 'tbs service level 1'),
            'upload_contract': ('upload', 'utf8_€.pdf'),
        })

        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading(assert_no_errors=False)
        # we should have a growl error for the Sales countries being required
        self.assert_growl_errors_present()

        self.fill_form(panel_name='pvc-member-wizard', values={
            'internet_sales_countries': 'Denmark, Sweden'
        })
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        # finish page
        self.click_button(panel_name='pvc-member-wizard', text='Finish')
        self.wait_for_loading2()

        element = self._driver.find_element_by_css_selector("div#icon-pvc-member-registration-progress")
        self.assertTrue(element.is_displayed())

    def test_create_member_duplicate_vat(self):
        tbs_account = self._create_tbs_account()
        rand_id = str(random.random())[2:]
        self._setup_member(tbs_account)

        # click OK dialog
        self.click_button(panel_name='dialog-information', text='OK')
        self.fill_forms(panel_name='pvc-member-wizard', forms_data={
            'country-form': {'country': ('select', 'Belgium'),},
        })
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        # company details screen
        company_forms_data = {
            'company_form': {
                'name': 'Member Company Name ' + rand_id,
                'website': 'http://member.com',
                'vat_no': 'ATU12345620'
            },
            'address_form': {
                'street': 'Member Street',
                'street_2': 'Member Street 2',
                'city': 'Amsterdam',
                'postcode': '1000AA',
                'state': 'NH'
            },
            'bank_account_form': {
                'iban': "GB82WEST12345698765432",
                'bic': "UNCRIT2B912",
            }
        }

        self.fill_forms(panel_name='pvc-member-wizard', forms_data=company_forms_data)
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        tbs_account = self._create_tbs_account()
        rand_id = str(random.random())[2:]
        self._setup_member(tbs_account)

        # click OK dialog
        self.click_button(panel_name='dialog-information', text='OK')
        self.fill_forms(panel_name='pvc-member-wizard', forms_data={
            'country-form': {'country': ('select', 'Belgium'),},
        })
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        # company details screen
        company_forms_data = {
            'company_form': {
                'name': 'Member Company Name ' + rand_id,
                'website': 'http://member.com',
                'vat_no': 'ATU12345620'
            },
            'address_form': {
                'street': 'Member Street',
                'street_2': 'Member Street 2',
                'city': 'Amsterdam',
                'postcode': '1000AA',
                'state': 'NH'
            },
            'bank_account_form': {
                'iban': "GB82WEST12345698765432",
                'bic': "UNCRIT2B912",
            }
        }

        self.fill_forms(panel_name='pvc-member-wizard', forms_data=company_forms_data)
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading(assert_no_errors=False)

        self.assertRaises(self.failureException, self.assert_no_growl_errors)

    def test_create_member_commissioner_only(self):

        tbs_account = self._create_tbs_account()
        rand_id = str(random.random())[2:]
        self._setup_member(tbs_account)

        # click OK dialog
        self.click_button(panel_name='dialog-information', text='OK')
        self.fill_forms(panel_name='pvc-member-wizard', forms_data={
            'country-form': {'country': ('select', 'Belgium'),},
        })
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        # company details screen
        company_forms_data = {
            'company_form': {
                'name': 'Member Company Name ' + rand_id,
                'website': 'http://member.com',
                'vat_no': 'ATU12345620'
            },
            'address_form': {
                'street': 'Member Street',
                'street_2': 'Member Street 2',
                'city': 'Amsterdam',
                'postcode': '1000AA',
                'state': 'NH'
            },
            'bank_account_form': {
                'iban': "GB82WEST12345698765432",
                'bic': "UNCRIT2B912",
            }
        }

        self.fill_forms(panel_name='pvc-member-wizard', forms_data=company_forms_data)
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        # TODO: company/account/invoice contact

        person_forms_data = {
            'person_form': {
                'last_name': 'Member Contact Last Name',
                'first_name': 'Member Contact First Name',
                'email': 'test-member@mail-test.basesoft.com',
                'preferred_locale': ('select', 'English'),
                'gender': ('select', 'Male'),
                'phone': '234234235',
            },
            'invoice_person_form': {
                'is_same_person_as': ('checkbox', 'true')
            }
        }
        self.fill_forms(panel_name='pvc-member-wizard', forms_data=person_forms_data)

        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        #
        # apply tbs page
        #

        # select tbs and wait for loading (for service level select)
        self.fill_form(panel_name='pvc-member-wizard', values={
            'tbs_id': ('select', tbs_account.groups[0].group.license.name),
            'is_commissioner': ('checkbox', True),
            'is_producer': ('checkbox', False),
        })
        self.wait_for_loading()

        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading2()

        # finish page
        self.click_button(panel_name='pvc-member-wizard', text='Finish')

    def test_member_upgrade_to_producer(self):
        tbs_account = self._create_tbs_account()
        member_account = self._create_member_account()
        service_level = self._create_tbs_service_level(tbs_account)
        tm = self._member_apply_at_tbs(member_account, tbs_account, is_producer=False, is_commissioner=True)

        obligation_start_date = date(2012, 1, 1)

        # activate member to automatically create an order entry for the Administration Fee
        mc = PvcMemberController(self.sa_session, account_id=tbs_account.id)
        mc.save_tbs_member(tm.id, {
            'status': PvcTbsMember.STATUS_APPROVED,
            'is_full_member': False,
            'is_representative': False,
            'obligation_start_date': obligation_start_date})

        self.login(tbs_account.name, 'test_password')

        self.click_iconbar('icon-pvc-member-manager')
        self.wait_for_loading()

        self.click_cq('pvc-member-manager', row_index=0, col_name='row_select', extra_css=' input')

        self.click_menu('pvc-member-manager', text="Enable commissioner as producer")

        self.click_button(panel_name='dialog-question', text='Yes')
        self.wait_for_loading()

        self.logout()

        self.login(member_account.name, 'test_password')
        self.wait_for_loading()

        self.fill_form(panel_name='pvc-member-apply-tbs', values={
            'service_level_id': ('select', service_level.name),
            'upload_contract': ('upload', 'contract.pdf'),
        })
        self.click_button(panel_name='pvc-member-apply-tbs', text='Save')
        self.wait_for_loading()

    def test_apply_tbs_upload_later(self):
        """
        pvcycle.test.selenium.test_member.TestCase.test_apply_tbs_upload_later
        """

        tbs_account = self._create_tbs_account()
        member_account = self._create_member_account()
        service_level = self._create_tbs_service_level(tbs_account)
        self._member_apply_at_tbs(member_account, tbs_account, service_level, upload_contract=False)

        # login as the member
        self.login(member_account.name, 'test_password')

        # expect the 'tbs apply member' panel to come up..
        # but close it since this test does it different
        self.wait_for_panel('pvc-member-apply-tbs')

        # fill form
        self.fill_form(panel_name='pvc-member-apply-tbs', values={
            'upload_contract': ('upload', 'contract.pdf'),
        })

        self.click_button(panel_name='pvc-member-apply-tbs', text='Save')
        self.wait_for_loading(times=4)

        # setup compose
        self.expand_layout_pane(0)
        self.click_palette('Mode')
        self.click_palette('Compose')
        self.wait_for_loading()

        # drag registration widget
        self.click_palette('PVC Member', palette_index=2)
        self.drag_item_from_palette('Registration progress', block_index=0, column_index=0)
        self.wait_for_loading(times=2)

        # expect status to be pending
        self.assert_cq(row_index=0, data={
            'status': 'Pending'
        })

    def test_apply_tbs_after_reject(self):

        tbs_account = self._create_tbs_account()
        member_account = self._create_member_account()
        service_level = self._create_tbs_service_level(tbs_account)
        tbs_member = self._member_apply_at_tbs(member_account, tbs_account, service_level)

        # as the tbs, reject the member
        mc = PvcMemberController(self.sa_session, account_id = tbs_account.id)
        mc.save_tbs_member(tbs_member.id, {
            'status': PvcTbsMember.STATUS_REJECTED,
            'is_full_member': False,
            'reject_reason': 'reject reason',
            'is_representative': False
        })

        # login as the member
        self.login(member_account.name, 'test_password')

        self.expand_layout_pane(0)
        self.click_palette('Mode')
        self.click_palette('Compose')
        self.wait_for_loading2()

        self.click_palette('PVC Member', palette_index=2)
        self.drag_item_from_palette('Registration progress', block_index=0, column_index=0)
        self.wait_for_loading2()

        # expect status to be rejected
        self.assert_cq(row_index=0, data={
            'status': 'Rejected'
        })

        # click on the status link
        self.click_cq(row_index=0, col_name='status')
        self.wait_for_loading()

        # fill form
        self.fill_form(panel_name='pvc-member-apply-tbs', values={
            'upload_contract': ('upload', 'contract.pdf'),
        })
        self.click_button(panel_name='pvc-member-apply-tbs', text='Save')
        self.wait_for_loading2()

        # expect status to be pending
        self.assert_cq(row_index=0, data={
            'status': 'Pending'
        })


    def test_member_apply_2nd_tbs(self):

        operator_account = self._create_operator_account()
        tbs_account = self._create_tbs_account(operator_account)
        tbs_account2 = self._create_tbs_account(operator_account)
        service_level = self._create_tbs_service_level(tbs_account, year=today().year)
        service_level2 = self._create_tbs_service_level(tbs_account2, year=today().year)

        member_account = self._create_member_account()
        tbs_member = self._member_apply_at_tbs(member_account, tbs_account, service_level)

        # now, as the tbs: approve the member
        mc = PvcMemberController(self.sa_session, account_id = tbs_account.id)

        # approve the tbs
        mc.save_tbs_member(tbs_member.id, {
            'status': PvcTbsMember.STATUS_APPROVED,
            'is_full_member': False,
            'is_representative': False
        })

        # login as member
        self.login(member_account.name, 'test_password')

        self.click_iconbar('icon-pvc-member-registration-progress')
        self.wait_for_loading()

        self.click_menu(panel_name='pvc-member-progress-manager', menu_index=0, text='Register at new TBS')
        self.wait_for_loading()

        # fill apply tbs form

        self.fill_form(panel_name='pvc-member-apply-tbs', values={
            'tbs_id': ('select', tbs_account2.groups[0].group.license.name),
            'is_commissioner': ('checkbox', True),
            'selling_technique': ('checkbox', 'value=2'),
        })
        self.wait_for_loading()

        self.fill_form(panel_name='pvc-member-apply-tbs', values={
            'service_level_id': ('select', 'tbs service level 1'),
            'upload_contract': ('upload', 'contract.pdf'),
        })

        self.click_button(panel_name='pvc-member-apply-tbs', text='Save')
        self.wait_for_loading()
        self.close_panel(panel_name='pvc-member-progress-manager')

        self.click_iconbar('icon-pvc-member-registration-progress')
        self.wait_for_loading()

        # expect status row 0 to be approved, row 1 to be pending
        self.assert_cq(panel_name='pvc-member-progress-manager', row_index=0, data={
            'tbs': tbs_account.groups[0].group.license.name,
            'status': 'Approved'
        })
        self.assert_cq(panel_name='pvc-member-progress-manager', row_index=1, data={
            'tbs': tbs_account2.groups[0].group.license.name,
            'status': 'Pending'
        })

    def test_member_apply_2nd_tbs_commissioner_only(self):

        operator_account = self._create_operator_account()
        tbs_account = self._create_tbs_account(operator_account)
        tbs_account2 = self._create_tbs_account(operator_account)
        service_level = self._create_tbs_service_level(tbs_account, year=today().year)
        service_level2 = self._create_tbs_service_level(tbs_account2, year=today().year)

        member_account = self._create_member_account()
        tbs_member = self._member_apply_at_tbs(member_account, tbs_account, service_level)

        # now, as the tbs: approve the member
        mc = PvcMemberController(self.sa_session, account_id = tbs_account.id)

        # approve the tbs
        mc.save_tbs_member(tbs_member.id, {
            'status': PvcTbsMember.STATUS_APPROVED,
            'is_full_member': False,
            'is_representative': False
        })

        # login as member
        self.login(member_account.name, 'test_password')

        self.click_iconbar('icon-pvc-member-registration-progress')
        self.wait_for_loading()

        self.click_menu(panel_name='pvc-member-progress-manager', menu_index=0, text='Register at new TBS')
        self.wait_for_loading()

        # fill apply tbs form

        self.fill_form(panel_name='pvc-member-apply-tbs', values={
            'tbs_id': ('select', tbs_account2.groups[0].group.license.name),
            'is_producer': ('checkbox', False),
            'is_commissioner': ('checkbox', True),
        })
        self.wait_for_loading()

        self.click_button(panel_name='pvc-member-apply-tbs', text='Save')
        self.wait_for_loading()
        self.close_panel(panel_name='pvc-member-progress-manager')

        self.click_iconbar('icon-pvc-member-registration-progress')
        self.wait_for_loading()

        # expect status row 0 to be approved, row 1 to be pending
        self.assert_cq(panel_name='pvc-member-progress-manager', row_index=0, data={
            'tbs': tbs_account.groups[0].group.license.name,
            'status': 'Approved'
        })
        self.assert_cq(panel_name='pvc-member-progress-manager', row_index=1, data={
            'tbs': tbs_account2.groups[0].group.license.name,
            'status': 'Pending'
        })

    def test_member_apply_2nd_company(self):

        operator_account = self._create_operator_account()
        tbs_account = self._create_tbs_account(operator_account)
        service_level = self._create_tbs_service_level(tbs_account, year=today().year)

        member_account = self._create_member_account()
        tbs_member = self._member_apply_at_tbs(member_account, tbs_account, service_level)

        # now, as the tbs: approve the member
        mc = PvcMemberController(self.sa_session, account_id = tbs_account.id)

        # approve the member
        mc.save_tbs_member(tbs_member.id, {
            'status': PvcTbsMember.STATUS_APPROVED,
            'is_full_member': False,
            'is_representative': False
        })

        # login as member
        self.login(member_account.name, 'test_password')

        self.click_iconbar('icon-pvc-member-registration-progress')
        self.wait_for_loading()

        self.click_menu(panel_name='pvc-member-progress-manager', menu_index=0, text='Register a new Company')
        self.wait_for_loading()

        self.fill_forms(panel_name='pvc-member-wizard', forms_data={
            'country-form': {'country': ('select', 'Belgium'),},
        })
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        rand_id = str(random.random())[2:]
        license_1_name = member_account.groups[0].group.license.name
        license_2_name = 'Member Company Name ' + rand_id

        # company details screen
        company_forms_data = {
            'company_form': {
                'name': license_2_name,
                'website': 'http://member.com',
                'vat_no': 'ALK99999999L'
            },
            'address_form': {
                'street': 'Member Street',
                'city': 'Amsterdam',
                'postcode': '1000AA',
                'state': 'NH'
            },
            'bank_account_form': {
                'iban': "GB82WEST12345698765432",
                'bic': "UNCRIT2B912",
            }
        }

        self.fill_forms(panel_name='pvc-member-wizard', forms_data=company_forms_data)
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        # TODO: company/account/invoice contact

        person_forms_data = {
            'person_form': {
                'last_name': 'Member Contact Last Name',
                'first_name': 'Member Contact First Name',
                'email': 'test-member@mail-test.basesoft.com',
                'preferred_locale': ('select', 'English'),
                'gender': ('select', 'Male'),
                'phone': '234234235',
            },
            'invoice_person_form': {
                'is_same_person_as': ('checkbox', 'true')
            }
        }
        self.fill_forms(panel_name='pvc-member-wizard', forms_data=person_forms_data)

        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        #
        # apply tbs page
        #

        # select tbs and wait for loading (for service level select)
        self.fill_form(panel_name='pvc-member-wizard', values={
            'tbs_id': ('select', tbs_account.groups[0].group.license.name),
            'is_commissioner': ('checkbox', True),
            'selling_technique': ('checkbox', 'value=2'),
        })
        self.wait_for_loading()

        # open and close preview contract
        self.click_button(panel_name='pvc-member-wizard', text='Preview and download contract')
        self.close_panel('member-preview-contract')

        self.fill_form(panel_name='pvc-member-wizard', values={
            'service_level_id': ('select', 'tbs service level 1'),
            'upload_contract': ('upload', 'contract.pdf'),
        })

        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading(times=2)

        # finish page
        self.click_button(panel_name='pvc-member-wizard', text='Finish')
        self.wait_for_loading()

        # expect status row 0 to be approved, row 1 to be pending
        self.assert_cq(panel_name='pvc-member-progress-manager', row_index=0, data={
            'tbs': tbs_account.groups[0].group.license.name,
            'member': license_1_name,
            'status': 'Approved'
        })
        self.assert_cq(panel_name='pvc-member-progress-manager', row_index=1, data={
            'tbs': tbs_account.groups[0].group.license.name,
            'member': license_2_name,
            'status': 'Pending'
        })


    def test_member_download_certificate(self):
        """
        pvcycle.test.selenium.test_member.TestCase.test_member_download_certificate
        """

        operator_account = self._create_operator_account()
        tbs_account = self._create_tbs_account(operator_account)
        service_level = self._create_tbs_service_level(tbs_account, year=today().year)

        operator_account = self._create_operator_account()
        accountant_account = self._create_accountant_account(tbs_account.default_group.license, operator_account)

        member_account = self._create_member_account()
        tbs_member = self._member_apply_at_tbs(member_account, tbs_account, service_level)

        # setup payment methods
        pc = PaymentController(self.sa_session, account_id=operator_account.id)
        pm1 = pc.save_or_update_payment_method_for_merchant(PaymentMethodBankTransfer, label='bank transfer', merchant=tbs_account.default_group.license)

        # as the tbs: approve the member
        mc = PvcMemberController(self.sa_session, account_id = tbs_account.id)
        mc.save_tbs_member(tbs_member.id, {
            'status': PvcTbsMember.STATUS_APPROVED,
            'is_full_member': False,
            'is_representative': False
        })

        # activate member to automatically create an order entry for the Administration Fee
        mc = PvcMemberController(self.sa_session, account_id=tbs_account.id)
        mc.save_tbs_member(tbs_member.id, {'status': PvcTbsMember.STATUS_APPROVED, 'is_full_member': False, 'is_representative': False})

        # there should be an order
        poc = PvcOrderController(self.sa_session, account_id=tbs_account.id)
        orders = poc.get()
        self.assertEqual(len(orders), 1)
        order = orders[0]

        # let the accountant approve the order
        accountant_pc = PaymentController(self.sa_session, account_id=accountant_account, payment_method=pm1)
        time.sleep(1); accountant_pc.complete_payment(order.id, "all good")

        # setup a tbs certificate text
        tc = PvcTBSController(self.sa_session, account_id=tbs_account.id)
        tc.save_tbs_certificate_state(None, {
            'tbs_id': tbs_account.default_group.license.id,
            'year': today().year,
            'text': """
            certificate text
            """
        })

        # login as member
        self.login(member_account.name, 'test_password')
        self.wait_for_loading()

        # open member certificate overview
        self.click_iconbar('icon-pvc-member-certificate', hover='icon-pvc-member-registration-progress')
        self.wait_for_loading()

        # assert that the certificate has status 'paid' but not yet approved
        self.assert_cq(panel_name='pvc-member-certificate-manager', row_index=0, data={
            'year': '2014',
            'status': 'Paid',
            'download': 'Approval pending'
        })

        # login as operator and approve
        self.login(tbs_account.name, 'test_password')
        self.wait_for_loading()

        # open member certificate overview
        self.click_iconbar('icon-pvc-member-certificate', hover='icon-pvc-member-manager')
        self.wait_for_loading()

        # click on Approve
        self.click_cq(panel_name='pvc-member-certificate-manager', row_index=0, col_name='status')
        self.wait_for_loading()

        # verify certificate status
        self.assert_cq(panel_name='pvc-member-certificate-manager', row_index=0, data={
            'year': '2014',
            'status': 'Approved / Revoke'
        })

        # login as member
        self.login(member_account.name, 'test_password')
        self.wait_for_loading()

        # open member certificate overview
        self.click_iconbar('icon-pvc-member-certificate', hover='icon-pvc-member-registration-progress')
        self.wait_for_loading()

        # assert that the certificate has status 'approved'
        self.assert_cq(panel_name='pvc-member-certificate-manager', row_index=0, data={
            'year': '2014',
            'status': 'Approved',
            'download': 'Download PDF'
        })

    def _goto_tbs_page(self):
        rand_id = str(random.random())[2:]

        self.click_button(panel_name='dialog-information', text='OK')
        self.fill_forms(panel_name='pvc-member-wizard', forms_data={
            'country-form': {'country': ('select', 'Belgium'),},
        })
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        # company details screen
        company_forms_data = {
            'company_form': {
                'name': 'Member Company Name ' + rand_id,
                'website': 'http://member.com',
                'vat_no': 'ATU12345675'
            },
            'address_form': {
                'street': 'Member Street',
                'street_2': 'Member Street 2',
                'city': 'Amsterdam',
                'postcode': '1000AA',
                'state': 'NH'
            },
            'bank_account_form': {
                'iban': "GB82WEST12345698765432",
                'bic': "UNCRIT2B912",
            }
        }

        self.fill_forms(panel_name='pvc-member-wizard', forms_data=company_forms_data)

        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        # TODO: company/account/invoice contact

        person_forms_data = {
            'person_form': {
                'last_name': 'Member Contact Last Name',
                'first_name': 'Member Contact First Name',
                'email': 'test-member@mail-test.basesoft.com',
                'preferred_locale': ('select', 'English'),
                'gender': ('select', 'Male'),
                'phone': '234234235',
            },
            'invoice_person_form': {
                'is_same_person_as': ('checkbox', True)
            }
        }

        self.fill_forms(panel_name='pvc-member-wizard', forms_data=person_forms_data)
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

    def test_membership_member(self):
        # This will test membership application for Member Apply At TBS.
        state = self._get_tbs_license_state()
        state['configs']['pvc.tbs.use.membership.instead.of.service.level'] = True

        tbs_account = self._create_tbs_account(state=state)

        self._setup_member(tbs_account, membership=True)
        self._goto_tbs_page()

        self.fill_form(panel_name='pvc-member-wizard', values={
            'tbs_id': ('select', tbs_account.groups[0].group.license.name)
        })
        self.wait_for_loading(stable_time=2)

        self.fill_form(panel_name='pvc-member-wizard', values={
            'membership_id': ('select', 'membership_0'),
        })
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        # finish page
        self.click_button(panel_name='pvc-member-wizard', text='Finish')
        self.wait_for_loading()

    def test_member_auto_approve(self):

        set_date(date(2013, 1, 1))

        # This will test membership application for Member Apply At TBS.
        state = self._get_tbs_license_state()
        state['configs']['pvc.tbs.auto.approve.member'] = True
        state['configs']['pvc.tbs.use.membership.instead.of.service.level'] = True

        operator = self._create_operator_account()
        tbs_account = self._create_tbs_account(operator_account=operator, state=state)
        self._create_membership_tax_rates(operator)

        self._setup_member(tbs_account, membership=True)
        self._goto_tbs_page()

        self.fill_form(panel_name='pvc-member-wizard', values={
            'tbs_id': ('select', tbs_account.groups[0].group.license.name)
        })
        self.wait_for_loading(stable_time=2)

        self.fill_form(panel_name='pvc-member-wizard', values={
            'membership_id': ('select', 'membership_0'),
        })
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        # finish page
        self.click_button(panel_name='pvc-member-wizard', text='Finish')
        self.wait_for_loading()

        self.assert_cq(panel_name='pvc-order-manager', row_index=0, data={
            'total_amount': u'€ 12.12'
        })

    def test_member_terms_and_conditions(self):
        state = self._get_tbs_license_state()
        state['configs']['pvc.tbs.require.accepting.of.terms.and.conditions'] = True

        tbs_account = self._create_tbs_account(state=state)

        self._setup_member(tbs_account)
        self._goto_tbs_page()

        self.fill_form(panel_name='pvc-member-wizard', values={
            'tbs_id': ('select', tbs_account.groups[0].group.license.name)
        })
        self.wait_for_loading()

        self.fill_form(panel_name='pvc-member-wizard', values={
            'service_level_id': ('select', 'tbs service level 1'),
            'upload_contract': ('upload', 'contract.pdf'),
            'selling_technique': ('checkbox', 'value=2'),
        })

        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading(assert_no_errors=False)
        self.assert_growl_errors_present()

        self.fill_form(panel_name='pvc-member-wizard', values={
            'terms_and_conditions': ('checkbox', True)
        })
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

    def test_member_company_registration_number(self):
        # This will test company registration number requirement on Member Apply At TBS.
        state = self._get_tbs_license_state()
        state['configs']['pvc.tbs.require.company.registration.number'] = True

        tbs_account = self._create_tbs_account(state=state)

        self._setup_member(tbs_account)
        self._goto_tbs_page()

        self.fill_form(panel_name='pvc-member-wizard', values={
            'tbs_id': ('select', tbs_account.groups[0].group.license.name)
        })
        self.wait_for_loading()

        self.fill_form(panel_name='pvc-member-wizard', values={
            'service_level_id': ('select', 'tbs service level 1'),
            'upload_contract': ('upload', 'contract.pdf'),
            'selling_technique': ('checkbox', 'value=2'),
        })
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading(assert_no_errors=False)
        self.assert_growl_errors_present()

        self.fill_form(panel_name='pvc-member-wizard', values={
            'company_registration_number': 'HUH9182'
        })
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

    def test_member_company_registration_number_commissioner_only(self):
        # This will test company registration number requirement on Member Apply At TBS.
        state = self._get_tbs_license_state()
        state['configs']['pvc.tbs.require.company.registration.number'] = True

        tbs_account = self._create_tbs_account(state=state)

        self._setup_member(tbs_account)
        self._goto_tbs_page()

        self.fill_form(panel_name='pvc-member-wizard', values={
            'tbs_id': ('select', tbs_account.groups[0].group.license.name)
        })
        self.wait_for_loading()

        self.fill_form(panel_name='pvc-member-wizard', values={
            'is_producer': ('checkbox', False),
            'is_commissioner': ('checkbox', True),
        })
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

    def test_member_upload_financial_guarantee_after_reject(self):

        state = self._get_tbs_license_state()
        state['configs']['pvc.tbs.require.proof.of.financial.guarantee'] = True

        tbs_account = self._create_tbs_account(state=state)
        member_account = self._create_member_account()
        service_level = self._create_tbs_service_level(tbs_account)
        tbs_member = self._member_apply_at_tbs(member_account, tbs_account, service_level, financial_guarantee=True)

        # as the tbs, reject the member
        mc = PvcMemberController(self.sa_session, account_id = tbs_account.id)
        mc.save_tbs_member(tbs_member.id, {
            'status': PvcTbsMember.STATUS_REJECTED,
            'is_full_member': False,
            'reject_reason': 'reject reason',
            'is_representative': False
        })

        # login as the member
        self.login(member_account.name, 'test_password')

        self.expand_layout_pane(0)
        self.click_palette('Mode')
        self.click_palette('Compose')
        self.wait_for_loading()

        self.click_palette('PVC Member', palette_index=2)
        self.drag_item_from_palette('Registration progress', block_index=0, column_index=0)
        self.wait_for_loading()

        # expect status to be rejected
        self.assert_cq(row_index=0, data={
            'status': 'Rejected'
        })

        # click on the status link
        self.click_cq(row_index=0, col_name='status')
        self.wait_for_loading()

        # fill form
        self.fill_form(panel_name='pvc-member-apply-tbs', values={
            'upload_contract': ('upload', 'contract.pdf'),
        })
        self.fill_form(panel_name='pvc-member-apply-tbs', values={
            'financial_guarantee': ('upload', 'financial_guarantee.pdf'),
        })
        self.wait_for_loading()

        self.click_button(panel_name='pvc-member-apply-tbs', text='Save')
        self.wait_for_loading()

        # expect status to be pending
        self.assert_cq(row_index=0, data={
            'status': 'Pending'
        })

    def test_member_upload_financial_guarantee(self):
        # Add two TBS accounts and check that the one that has financial guarantee config active
        # requires a file upload in registration wizard and the other one doesn't
        state = self._get_tbs_license_state()
        state1 = self._get_tbs_license_state()
        state['configs']['pvc.tbs.require.proof.of.financial.guarantee'] = True

        operator = self._create_operator_account()
        tbs_account = self._create_tbs_account(operator_account=operator, state=state)
        tbs_account1 = self._create_tbs_account(operator_account=operator, state=state1)

        self._setup_member(tbs_account)
        self._goto_tbs_page()

        self.fill_form(panel_name='pvc-member-wizard', values={
            'tbs_id': ('select', tbs_account1.groups[0].group.license.name)
        })
        self.wait_for_loading()

        self.fill_form(panel_name='pvc-member-wizard', values={
            'tbs_id': ('select', tbs_account.groups[0].group.license.name)
        })
        self.wait_for_loading()

        self.fill_form(panel_name='pvc-member-wizard', values={
            'service_level_id': ('select', 'tbs service level 1'),
            'upload_contract': ('upload', 'contract.pdf'),
            'selling_technique': ('checkbox', 'value=2'),
        })
        self.wait_for_loading()

        self.fill_form(panel_name='pvc-member-wizard', values={
            'financial_guarantee': ('upload', 'financial_guarantee.pdf'),
        })
        self.wait_for_loading()

        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        # finish page
        self.click_button(panel_name='pvc-member-wizard', text='Finish')
        self.wait_for_loading()

        f = self.sa_session.query(PvcTbsMemberProofFinancialGuaranteeFile).order_by(PvcTbsMemberProofFinancialGuaranteeFile.id.desc()).first()
        self.assertEqual(f.polymorphic_type, 'PvcTbsMemberProofFinancialGuaranteeFile')

    def _member_no_vat(self, state):
        tbs_account = self._create_tbs_account(state=state)

        self._setup_member(tbs_account)

        # click OK dialog
        self.click_button(panel_name='dialog-information', text='OK')

        # first set China.
        self.fill_forms(panel_name='pvc-member-wizard', forms_data={
            'country-form': {'country': ('select', 'United Kingdom'),},
        })
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        # company details screen
        company_forms_data = {
            'company_form': {
                'name': 'Member Company Name NO VAT',
                'website': 'http://member.com'
            },
            'address_form': {
                'street': 'Some Shanghai Street Name',
                'city': 'Shanghai',
                'postcode': 'GA SHA',
            },
            'bank_account_form': {
                'iban': "GB82WEST12345698765432",
                'bic': "UNCRIT2B912",
            }
        }
        self.fill_forms(panel_name='pvc-member-wizard', forms_data=company_forms_data)
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        person_forms_data = {
            'person_form': {
                'last_name': 'Member Contact Last Name',
                'first_name': 'Member Contact First Name',
                'email': 'test-member@mail-test.basesoft.com',
                'preferred_locale': ('select', 'English'),
                'gender': ('select', 'Male'),
                'phone': '234234235',
            },
            'invoice_person_form': {
                'is_same_person_as': ('checkbox', True)
            }
        }

        self.fill_forms(panel_name='pvc-member-wizard', forms_data=person_forms_data)
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        self.fill_form(panel_name='pvc-member-wizard', values={
            'tbs_id': ('select', tbs_account.groups[0].group.license.name)
        })
        self.wait_for_loading()

        self.fill_form(panel_name='pvc-member-wizard', values={
            'service_level_id': ('select', 'tbs service level 1'),
            'upload_contract': ('upload', 'contract.pdf'),
            'selling_technique': ('checkbox', 'value=2'),
        })

    def test_member_no_vat_failure(self):
        state = self._get_tbs_license_state()
        state['configs']['pvc.tbs.optional.uk.vat.no'] = False

        self._member_no_vat(state)

        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        # we expect a growl Critical Error here as this TBS does not allow UK companies without VAT numbers.
        self.wait_for_loading(assert_no_errors=False)
        self.assert_growl_errors_present()

    def test_member_no_vat_success(self):
        state = self._get_tbs_license_state()
        state['configs']['pvc.tbs.optional.uk.vat.no'] = True

        self._member_no_vat(state)

        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

    def _member_same_country_as_tbs(self, state, country, coc=None):
        tbs_account = self._create_tbs_account(state=state)

        self._setup_member(tbs_account)

        # click OK dialog
        self.click_button(panel_name='dialog-information', text='OK')
        self.fill_forms(panel_name='pvc-member-wizard', forms_data={
            'country-form': {'country': ('select', country),},
        })
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        # company details screen
        company_forms_data = {
            'company_form': {
                'name': 'Member Company Name NO VAT',
                'website': 'http://member.com',
                'vat_no': 'BE0893027827'
            },
            'address_form': {
                'street': 'Some Shanghai Street Name',
                'city': 'Shanghai',
                'postcode': 'GA SHA',
            },
            'bank_account_form': {
                'iban': "GB82WEST12345698765432",
                'bic': "UNCRIT2B912",
            }
        }
        if coc is not None:
            company_forms_data['company_form']['chamber_of_commerce_nr'] = coc
        self.fill_forms(panel_name='pvc-member-wizard', forms_data=company_forms_data)
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        person_forms_data = {
            'person_form': {
                'last_name': 'Member Contact Last Name',
                'first_name': 'Member Contact First Name',
                'email': 'test-member@mail-test.basesoft.com',
                'preferred_locale': ('select', 'English'),
                'gender': ('select', 'Male'),
                'phone': '234234235',
            },
            'invoice_person_form': {
                'is_same_person_as': ('checkbox', True)
            }
        }

        self.fill_forms(panel_name='pvc-member-wizard', forms_data=person_forms_data)
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        self.fill_form(panel_name='pvc-member-wizard', values={
            'tbs_id': ('select', tbs_account.groups[0].group.license.name)
        })
        self.wait_for_loading()

        self.fill_form(panel_name='pvc-member-wizard', values={
            'service_level_id': ('select', 'tbs service level 1'),
            'upload_contract': ('upload', 'contract.pdf'),
            'selling_technique': ('checkbox', 'value=2'),
        })

    def test_member_same_country_as_tbs_failure(self):
        state = self._get_tbs_license_state()
        state['configs']['pvc.tbs.require.member.in.own.country'] = True

        self._member_same_country_as_tbs(state, 'Belgium')
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        # we expect a growl Critical Error here as this TBS does not allow a Country different than their own.
        self.wait_for_loading(assert_no_errors=False)
        self.assert_growl_errors_present()

    def test_member_same_country_as_tbs_success(self):
        state = self._get_tbs_license_state()
        state['configs']['pvc.tbs.require.member.in.own.country'] = True

        self._member_same_country_as_tbs(state, 'Netherlands', coc='KVK1234')
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

    def test_member_master_data_change_contact_vs_invoice(self):
        """
        Reproduction for bug whereby the "Same is invoice" did not disable invoice contact form behaviour. (#581)
        """
        state = self._get_member_license_state('test comp')
        del state['contact_person_data']['phone']
        state['invoice_person_data']['is_same_person_as'] = True
        state['bank_data']['bic'] = 'FTSBNL2RGOL'
        state['bank_data']['iban'] = 'EE382200221020145685'
        member_account = self._create_member_account('company_name', state)

        tbs_account = self._create_tbs_account()

        service_level = self._create_tbs_service_level(tbs_account)

        self._member_apply_at_tbs(member_account, tbs_account, service_level, upload_contract=True)

        self.login(member_account.name, 'test_password')

        self.click_iconbar(icon_id='icon-master-data')
        self.wait_for_loading()

        self.fill_form(panel_name='master-data-panel', form_name='person_form', values={'phone': '343434'})
        # with buggy code this triggered a growl
        self.click_button(panel_name='master-data-panel', text='Save')
        self.wait_for_loading()

    def test_member_master_data_change(self):
        state = self._get_tbs_license_state()
        state['configs']['pvc.tbs.auto.approve.member'] = True
        state['configs']['pvc.tbs.use.membership.instead.of.service.level'] = True

        operator = self._create_operator_account()
        tbs_account = self._create_tbs_account(operator_account=operator, state=state)
        self._create_membership_tax_rates(operator)

        self._setup_member(tbs_account, membership=True)
        self._goto_tbs_page()

        self.fill_form(panel_name='pvc-member-wizard', values={
            'tbs_id': ('select', tbs_account.groups[0].group.license.name)
        })
        self.wait_for_loading(stable_time=2)

        self.fill_form(panel_name='pvc-member-wizard', values={
            'membership_id': ('select', 'membership_0'),
        })
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        # finish page
        self.click_button(panel_name='pvc-member-wizard', text='Finish')
        self.wait_for_loading()

        self.close_panel(panel_name='pvc-order-manager')
        self.click_iconbar(icon_id='icon-master-data')
        self.wait_for_loading()

        self.fill_form(panel_name='master-data-panel',form_name='person_form', values={
            'last_name': 'Member Last Name UPDATED',
            'first_name': 'Member First Name UPDATED',
            'email': 'test-member-update@mail-test.basesoft.com',
        })

        self.click_button(panel_name='master-data-panel', text='Save')
        self.wait_for_loading()

    def test_multi_member_master_data_change(self):

        operator_account = self._create_operator_account()
        tbs_account = self._create_tbs_account(operator_account)
        service_level = self._create_tbs_service_level(tbs_account, year=today().year)

        member_account = self._create_member_account()
        tbs_member = self._member_apply_at_tbs(member_account, tbs_account, service_level)

        # now, as the tbs: approve the member
        mc = PvcMemberController(self.sa_session, account_id = tbs_account.id)

        # approve the member
        mc.save_tbs_member(tbs_member.id, {
            'status': PvcTbsMember.STATUS_APPROVED,
            'is_full_member': False,
            'is_representative': False
        })

        # login as member
        self.login(member_account.name, 'test_password')

        self.click_iconbar('icon-pvc-member-registration-progress')
        self.wait_for_loading()

        self.click_menu(panel_name='pvc-member-progress-manager', menu_index=0, text='Register a new Company')
        self.wait_for_loading()

        rand_id = str(random.random())[2:]
        license_1_name = member_account.groups[0].group.license.name
        license_2_name = 'Member Company Name ' + rand_id

        self.fill_forms(panel_name='pvc-member-wizard', forms_data={
            'country-form': {'country': ('select', 'Belgium'),},
        })
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        # company details screen
        company_forms_data = {
            'company_form': {
                'name': license_2_name,
                'website': 'http://member.com',
                'vat_no': 'ALK99999999L'
            },
            'address_form': {
                'street': 'Member Street',
                'city': 'Amsterdam',
                'postcode': '1000AA',
                'state': 'NH'
            },
            'bank_account_form': {
                'iban': "GB82WEST12345698765432",
                'bic': "UNCRIT2B912",
            }
        }

        self.fill_forms(panel_name='pvc-member-wizard', forms_data=company_forms_data)
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        # TODO: company/account/invoice contact

        person_forms_data = {
            'person_form': {
                'last_name': 'Member Contact Last Name',
                'first_name': 'Member Contact First Name',
                'email': 'test-member@mail-test.basesoft.com',
                'preferred_locale': ('select', 'English'),
                'gender': ('select', 'Male'),
                'phone': '234234235',
            },
            'invoice_person_form': {
                'is_same_person_as': ('checkbox', 'true')
            }
        }
        self.fill_forms(panel_name='pvc-member-wizard', forms_data=person_forms_data)

        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        #
        # apply tbs page
        #

        # select tbs and wait for loading (for service level select)
        self.fill_form(panel_name='pvc-member-wizard', values={
            'tbs_id': ('select', tbs_account.groups[0].group.license.name),
            'is_commissioner': ('checkbox', True),
            'selling_technique': ('checkbox', 'value=2'),
        })
        self.wait_for_loading()

        # open and close preview contract
        self.click_button(panel_name='pvc-member-wizard', text='Preview and download contract')
        self.close_panel('member-preview-contract')

        self.fill_form(panel_name='pvc-member-wizard', values={
            'service_level_id': ('select', 'tbs service level 1'),
            'upload_contract': ('upload', 'contract.pdf'),
        })

        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading(times=2)

        # finish page
        self.click_button(panel_name='pvc-member-wizard', text='Finish')
        self.wait_for_loading()

        # expect status row 0 to be approved, row 1 to be pending
        self.assert_cq(panel_name='pvc-member-progress-manager', row_index=0, data={
            'tbs': tbs_account.groups[0].group.license.name,
            'member': license_1_name,
            'status': 'Approved'
        })
        self.assert_cq(panel_name='pvc-member-progress-manager', row_index=1, data={
            'tbs': tbs_account.groups[0].group.license.name,
            'member': license_2_name,
            'status': 'Pending'
        })

        # now we open the master data panel
        self.click_iconbar(icon_id='icon-master-data')
        self.wait_for_loading()

        self.fill_form(panel_name='master-data-panel',form_name='license_form', values={
            'license_id': ('select', license_2_name)
        })
        self.wait_for_loading()

    def test_member_is_representative(self):

        operator_account = self._create_operator_account()
        self._create_contribution_tax_rates(operator_account)

        set_date(date(2013, 6, 1)) # set date to June 2013

        # this TBS has it's setting on MONTHLY!
        tbs = self._create_tbs_account(operator_account)

        _administration_fee = Decimal('250.00')
        _contribution_fee = Decimal('2.75')

        service_level = self._create_tbs_service_level(tbs, admin_fee=_administration_fee, contrib_fee=_contribution_fee, year=today().year)
        ma = self._create_member_account()

        # apply at TBS
        tm = self._member_apply_at_tbs(ma, tbs, service_level)

        # activate member to automatically create an order entry for the Administration Fee
        mc = PvcMemberController(self.sa_session, account_id=tbs.id)
        mc.save_tbs_member(tm.id, {'status': PvcTbsMember.STATUS_PENDING, 'is_full_member': False, 'is_representative': False})

        self.login(operator_account.name, 'test_password')

        # open member manager
        self.click_iconbar('icon-pvc-member-manager', hover='icon-pvc-accountant-manager')
        self.wait_for_loading()

        # click Review
        self.filter_cq(panel_name='pvc-member-manager', filters=[('TBS', 'Equal', tm.tbs.name)])
        self.click_cq('pvc-member-manager', row_index=0, col_name='tbs_member_id_details')

        self.fill_form('pvc-member-details', values={
            'is_representative': ('checkbox', True),
            'is_free_rider': ('checkbox', True),
        })

        self.click_button(panel_name='pvc-member-details', text='Save')
        self.wait_for_loading()

        self.close_panel('pvc-member-manager')

        # check that it's saved
        self.click_iconbar('icon-pvc-member-manager', hover='icon-pvc-accountant-manager')
        self.wait_for_loading()

        self.filter_cq(panel_name='pvc-member-manager', filters=[('TBS', 'Equal', tm.tbs.name)])
        self.click_cq('pvc-member-manager', row_index=0, col_name='tbs_member_id_details')
        self.wait_for_loading()

        self.assert_form('pvc-member-details', values={
            'is_representative': ('checkbox', True),
            'is_free_rider': ('checkbox', True),
        })

    def test_member_update_selling_technique(self):

        tbs_account = self._create_tbs_account()
        service_level = self._create_tbs_service_level(tbs_account)

        ma = self._create_member_account()
        tm = self._member_apply_at_tbs(ma, tbs_account, service_level)

        self.login(ma.name, 'test_password')

        self.click_iconbar('icon-pvc-member-registration-progress')
        self.wait_for_loading()

        self.click_cq('pvc-member-progress-manager', row_index=0, col_name='selling_technique')
        self.wait_for_loading()

        self.fill_form('pvc-tbs-member-settings-panel', values={
            'selling_technique': ('checkbox', 'value=4', )
        })
        self.click_button('pvc-tbs-member-settings-panel', text='Save')
        self.wait_for_loading()

        self.assert_cq('pvc-member-progress-manager', row_index=0, data={
            'selling_technique': 'Distributing/reselling rebranded products, Importing'
        })
        self.sa_session.expire(tm)
        self.assertEqual(tm.selling_technique, 6)

    def test_change_service_level(self):

        set_date(date(2015, 1, 1))

        operator_account = self._create_operator_account()
        tbs_account = self._create_tbs_account()

        service_level_1 = self._create_tbs_service_level(tbs_account, name='SL A', year=today().year, admin_fee=Decimal('250.00'), level=1, annex_text='ANNEX A')
        service_level_2 = self._create_tbs_service_level(tbs_account, name='SL B', year=today().year, admin_fee=Decimal('300.00'), level=2, annex_text='ANNEX B')

        rand_id = str(random.random())[2:]
        member_company_name = 'Member Company Name ' + rand_id

        # create member and apply at tbs
        member_account = self._create_member_account(company_name=member_company_name)
        tbs_member = self._member_apply_at_tbs(member_account, tbs_account, service_level=service_level_1)

        # the tbs approves
        mc = PvcMemberController(self.sa_session, account_id = tbs_account.id)
        mc.save_tbs_member(tbs_member.id, {
            'status': PvcTbsMember.STATUS_APPROVED,
            'is_representative': False,
            'is_full_member': False
        })

        # goto 2016 and request new SL
        set_date(date(2016, 1, 1))

        self.login(member_account.name, 'test_password')

        self.click_iconbar(title='Registration progress')

        self.click_cq(panel_name='pvc-member-progress-manager', row_index=0, col_name='service_level')
        self.wait_for_loading()

        self.click_button(panel_name='pvc-tbs-member-settings-panel', text='Request service level change')
        self.wait_for_loading()

        self.fill_form('pvc-change-service-level', values={
            'service_level_id': ('select', 'SL B'),
            'upload_contract': ('upload', 'utf8_€.pdf'),
        })
        self.click_button('pvc-change-service-level', text='Save')
        self.wait_for_loading()

        # login as operator
        self.login(operator_account.name, 'test_password')

        # same as operator
        self.click_iconbar('icon-pvc-member-manager', hover='icon-pvc-accountant-manager')
        self.wait_for_loading()

        self.filter_cq(panel_name='pvc-member-manager', filters=[
            ('Member', 'Equal', member_company_name),
        ])

        self.click_cq('pvc-member-manager', row_index=0, col_name='tbs_member_id_details')
        self.wait_for_loading()

        self.click_button('pvc-member-details', text='Approve change request')
        self.wait_for_loading()

    def test_create_guarantee_forecast(self):

        set_date(date(2015, 1, 1))

        operator_account = self._create_operator_account()
        self._create_contribution_tax_rates(operator_account)

        tbs_account = self._create_tbs_account()
        tbs = tbs_account.default_group.license

        service_level_pv_1 = self._create_tbs_service_level(tbs_account, name='SL PV 1',
            year=today().year, admin_fee=Decimal('300.00'), level=1, product_type=PvcTbsProduct.TYPE_PV_RELATED, guarantee_forecast_min_value=0)
        service_level_pv_2 = self._create_tbs_service_level(tbs_account, name='SL PV 2',
            year=today().year, admin_fee=Decimal('300.00'), level=2, product_type=PvcTbsProduct.TYPE_PV_RELATED, guarantee_forecast_min_value=100)

        service_level_other_1 = self._create_tbs_service_level(tbs_account, name='SL Other 1', year=today().year,
            admin_fee=Decimal('300.00'), level=1, guarantee_forecast_min_value=0)
        service_level_other_2 = self._create_tbs_service_level(tbs_account, name='SL Other 2', year=today().year,
            admin_fee=Decimal('300.00'), level=2, guarantee_forecast_min_value=100)

        # set TBS config for advance invoicing.
        tbs.set_config('pvc.tbs.guarantee.forecast', True)

        # add tbs products
        pc = PvcProductController(self.sa_session, account_id=operator_account.id)
        tbs_pv_product = pc.save_or_update_tbs_product(None, 'tbs pv product', 'desc', tbs.id,
            product_type=PvcTbsProduct.TYPE_PV_RELATED, basis_for_invoicing=PvcTbsProduct.INVOICE_BASED_ON_WEIGHT,
            custom_unit=PvcTbsProduct.CUSTOM_UNIT_NOMINAL_POWER
        )

        rand_id = str(random.random())[2:]
        self._setup_member(tbs_account, create_service_level=False)

        # we are now in the producer member login
        # click OK dialog
        self.click_button(panel_name='dialog-information', text='OK')
        self.fill_forms(panel_name='pvc-member-wizard', forms_data={
            'country-form': {'country': ('select', 'Netherlands'),},
        })
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        # company details screen
        company_forms_data = {
            'company_form': {
                'name': 'Member Company Name ' + rand_id,
                'website': 'http://member.com',
                'vat_no': 'ATU12345675',
                'chamber_of_commerce_nr': 'KVK123',
            },
            'address_form': {
                'street': 'Member Street',
                'street_2': 'Member Street 2',
                'city': 'Amsterdam',
                'postcode': '1000AA',
                'state': 'NH'
            },
            'bank_account_form': {
                'iban': "GB82WEST12345698765432",
                'bic': "UNCRIT2B912",
            }
        }

        self.fill_forms(panel_name='pvc-member-wizard', forms_data=company_forms_data)

        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        person_forms_data = {
            'person_form': {
                'last_name': 'Member Contact Last Name',
                'first_name': 'Member Contact First Name',
                'email': 'test-member@mail-test.basesoft.com',
                'preferred_locale': ('select', 'English'),
                'gender': ('select', 'Male'),
                'phone': '234234235',
            },
            'invoice_person_form': {
                'is_same_person_as': ('checkbox', True)
            }
        }

        self.fill_forms(panel_name='pvc-member-wizard', forms_data=person_forms_data)
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        #
        # apply tbs page
        #

        # select tbs and wait for loading (for service level select)
        self.fill_form(panel_name='pvc-member-wizard', values={
            'tbs_id': ('select', tbs_account.groups[0].group.license.name),
        })
        self.wait_for_loading(stable_time=2)

        self.fill_form(panel_name='pvc-member-wizard', values={
            'selling_technique': ('checkbox', 'value=2'),
        })

        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()


        # finish page
        self.click_button(panel_name='pvc-member-wizard', text='Finish')
        self.wait_for_loading()


        #
        # select forecast guarantee product
        #

        self.click_button(panel_name='dialog-information', text='OK')

        self.fill_form(panel_name='pvc-contribution-extra-line-product-select', values={
            'products': ('fancy-select', ['tbs pv product - desc'])
        })
        self.click_button(panel_name='pvc-contribution-extra-line-product-select', text='Add to declaration')
        self.wait_for_loading()

        #
        # contribution panel
        #

        self.assertTrue(self.get_panel_title('pvc-contribution-details-panel').find('New declaration for 2015') == 0)

        self.fill_form(panel_name='pvc-contribution-details-panel', values={
            'line_0_quantity': 100,
            'line_0_weight': '100',
            'line_0_custom_unit': '100',
        })

        self.fill_form(panel_name='pvc-contribution-details-panel', values={
            'agreement': ('checkbox2', True)
        })
        self.wait_for_loading()

        self.click_button(panel_name='pvc-contribution-details-panel', text='Submit')
        self.wait_for_loading()

        self.assertEquals(self._get_text_from_selector('div[name="pvc-contribution-details-panel-checkout"] .total-amount'), u'€ 0.00')

        self.click_button(panel_name='pvc-contribution-details-panel-checkout', text='Submit')
        self.wait_for_loading(times=2)

        self.fill_form(panel_name='pvc-member-apply-tbs', values={
            'upload_contract': ('upload', 'utf8_€.pdf'),
        })

        self.click_button(panel_name='pvc-member-apply-tbs', text='Save')
        self.wait_for_loading()

        # login as operator
        self.login(operator_account.name, 'test_password')

        # open member manager
        self.click_iconbar('icon-pvc-member-manager', hover='icon-pvc-accountant-manager')
        self.wait_for_loading()

        # click review and approve member
        self.filter_cq(panel_name='pvc-member-manager', filters=[
            ('Member', 'Equal', 'Member Company Name ' + rand_id),
        ])

        self.click_cq('pvc-member-manager', row_index=0, col_name='tbs_member_id_details')

        self.fill_form('pvc-member-details', values={
            'status': ('radio', PvcTbsMember.STATUS_APPROVED),
        })

        self.click_button(panel_name='pvc-member-details', text='Save')
        self.wait_for_loading()

    def test_guarantee_forecast_apply_2nd_company(self):

        set_date(date(2015, 1, 1))

        operator_account = self._create_operator_account()
        tbs_account = self._create_tbs_account(operator_account)
        tbs = tbs_account.default_group.license
        #service_level = self._create_tbs_service_level(tbs_account, year=today().year)

        self._create_contribution_tax_rates(operator_account)

        service_level_pv_1 = self._create_tbs_service_level(tbs_account, name='SL PV 1',
            year=today().year, admin_fee=Decimal('300.00'), level=1, product_type=PvcTbsProduct.TYPE_PV_RELATED, guarantee_forecast_min_value=0)
        service_level_pv_2 = self._create_tbs_service_level(tbs_account, name='SL PV 2',
            year=today().year, admin_fee=Decimal('300.00'), level=2, product_type=PvcTbsProduct.TYPE_PV_RELATED, guarantee_forecast_min_value=100)

        service_level_other_1 = self._create_tbs_service_level(tbs_account, name='SL Other 1', year=today().year,
            admin_fee=Decimal('300.00'), level=1, guarantee_forecast_min_value=0)
        service_level_other_2 = self._create_tbs_service_level(tbs_account, name='SL Other 2', year=today().year,
            admin_fee=Decimal('300.00'), level=2, guarantee_forecast_min_value=100)

        # set TBS config for advance invoicing.
        tbs.set_config('pvc.tbs.guarantee.forecast', True)

        # add tbs products
        pc = PvcProductController(self.sa_session, account_id=operator_account.id)

        products = pc.get_available_products_for_tbs(tbs.id)
        tbs_product = pc.save_or_update_tbs_product(None, 'EEE-TBS-TEST', 'WEEE', tbs.id,
            parent_level_id=products[0]['id'], basis_for_invoicing=PvcTbsProduct.INVOICE_BASED_ON_WEIGHT
        )
        tbs_pv_product = pc.save_or_update_tbs_product(None, 'tbs pv product', 'desc', tbs.id,
            parent_level_id=products[0]['id'], product_type=PvcTbsProduct.TYPE_PV_RELATED, basis_for_invoicing=PvcTbsProduct.INVOICE_BASED_ON_WEIGHT,
            custom_unit=PvcTbsProduct.CUSTOM_UNIT_NOMINAL_POWER
        )

        pc.save_contribution_fee(tbs_product.id, service_level_other_2.id, Decimal('0.025'), today(), PvcTbsProductFee.DIMENSION_NONE)
        pc.save_contribution_fee(tbs_pv_product.id, service_level_pv_2.id, Decimal('0.030'), today(), PvcTbsProductFee.DIMENSION_NONE)

        member_account = self._create_member_account()
        tbs_member = self._member_apply_at_tbs(member_account, tbs_account, service_level=None)
        self.assertEqual(tbs_member.status, PvcTbsMember.STATUS_PENDING)

        cc = PvcContributionOrderController(self.sa_session, account_id=member_account.id)
        order = cc.add_guarantee_forecast_order(tbs_member.id, today().year, [tbs_pv_product.id], is_b2b=None)

        self.assertEqual(order.total_amount_including_vat, Decimal('0'))
        self.assertEqual(order.is_guarantee_forecast, True)

        order = cc.update_contribution(order.id, {
            # pv module
            'line_0_quantity': 1,
            'line_0_weight': Decimal('200.00'),
            'line_0_custom_unit': Decimal('250.00'),
        }, additional_lines={
        })

        # submit should determine the new SL
        cc.submit_order(order.id)

        # expect two service levels to be active for this tbs_member
        self.assertEqual(tbs_member.active_service_level.service_level, None)
        self.assertEqual(tbs_member.active_service_level.pv_service_level, service_level_pv_2)

        # approve member
        mc = PvcMemberController(self.sa_session, account_id=tbs_account.id)
        mc.save_tbs_member(tbs_member.id, {'status': PvcTbsMember.STATUS_APPROVED, 'is_full_member': False, 'is_representative': False})

        # login as member
        self.login(member_account.name, 'test_password')

        self.click_iconbar('icon-pvc-member-registration-progress')
        self.wait_for_loading()

        self.click_menu(panel_name='pvc-member-progress-manager', menu_index=0, text='Register a new Company')
        self.wait_for_loading()

        self.fill_forms(panel_name='pvc-member-wizard', forms_data={
            'country-form': {'country': ('select', 'Belgium'),},
        })
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        rand_id = str(random.random())[2:]
        license_1_name = member_account.groups[0].group.license.name
        license_2_name = 'Member Company Name ' + rand_id

        # company details screen
        company_forms_data = {
            'company_form': {
                'name': license_2_name,
                'website': 'http://member.com',
                'vat_no': 'ALK99999999L'
            },
            'address_form': {
                'street': 'Member Street',
                'city': 'Amsterdam',
                'postcode': '1000AA',
                'state': 'NH'
            },
            'bank_account_form': {
                'iban': "GB82WEST12345698765432",
                'bic': "UNCRIT2B912",
            }
        }

        self.fill_forms(panel_name='pvc-member-wizard', forms_data=company_forms_data)
        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        person_forms_data = {
            'person_form': {
                'last_name': 'Member Contact Last Name',
                'first_name': 'Member Contact First Name',
                'email': 'test-member@mail-test.basesoft.com',
                'preferred_locale': ('select', 'English'),
                'gender': ('select', 'Male'),
                'phone': '234234235',
            },
            'invoice_person_form': {
                'is_same_person_as': ('checkbox', 'true')
            }
        }
        self.fill_forms(panel_name='pvc-member-wizard', forms_data=person_forms_data)

        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading()

        # apply tbs page

        # select tbs and wait for loading (for service level select)
        self.fill_form(panel_name='pvc-member-wizard', values={
            'tbs_id': ('select', tbs_account.groups[0].group.license.name),
            'is_commissioner': ('checkbox', True),
            'selling_technique': ('checkbox', 'value=2'),
        })
        self.wait_for_loading()

        self.click_button(panel_name='pvc-member-wizard', text=u'Next »')
        self.wait_for_loading(times=2)

        # finish page
        self.click_button(panel_name='pvc-member-wizard', text='Finish')
        self.wait_for_loading()

        # fill Guarantee forecast zut

        # select forecast guarantee product
        self.click_button(panel_name='dialog-information', text='OK')
        self.fill_form(panel_name='pvc-contribution-extra-line-product-select', values={
            'products': ('fancy-select', ['tbs pv product - desc'])
        })
        self.click_button(panel_name='pvc-contribution-extra-line-product-select', text='Add to declaration')
        self.wait_for_loading()

        # contribution panel
        self.fill_form(panel_name='pvc-contribution-details-panel', values={
            'line_0_quantity': 100,
            'line_0_weight': '100',
            'line_0_custom_unit': '100',
        })

        self.fill_form(panel_name='pvc-contribution-details-panel', values={
            'agreement': ('checkbox2', True)
        })
        self.wait_for_loading()

        self.click_button(panel_name='pvc-contribution-details-panel', text='Submit')
        self.wait_for_loading()

        self.assertEquals(self._get_text_from_selector('div[name="pvc-contribution-details-panel-checkout"] .total-amount'), u'€ 0.00')

        self.click_button(panel_name='pvc-contribution-details-panel-checkout', text='Submit')
        self.wait_for_loading(times=2)


# below is for running the test from eclipse
if __name__ == '__main__':

    from twisted.scripts.trial import run
    import sys
    sys.argv = ['test_member.py', 'pvcycle.test.selenium.test_member.TestCase.test_create_member']
    run()
    sys.exit()


