# -*- coding: utf-8 -*-

'''
Created on 25 Jul 2013

@author: martijn
'''

import random

from basesoft.web.test.selenium.selenium_testcase import XPath
from pvcycle.test.selenium.pvc_selenium import PvcSeleniumTestCase
from decimal import Decimal
from pvcycle.core.controllers import PvcMemberController
from pvcycle.core.models import PvcTbsMember
from basesoft.web.contrib.license.controllers import LicenseController
from basesoft.util.virtual_date import today

class TestOperator(PvcSeleniumTestCase):
    '''
    tests altering and creation of a license
    '''

    def setUp(self):

        super(TestOperator, self).setUp()
        self.connect_db()

        sysadmin_account = self._create_sysadmin_account()
        self.login(sysadmin_account.name, 'test_password')
        self.click_iconbar('icon-license-manager')
        self.wait_for_panel('license-manager-panel')

    def test_create_operator(self):
        """
        create a new license tru the wizard
        """

        rand_id = str(random.random())[2:]

        # open wizard
        self.click_menu(panel_name='license-manager-panel', text='New license wizard')
        self.wait_for_loading()

        # first page
        self.fill_form(panel_name='license-wizard-panel', values={
            'domain_name':'PVC Domain ' + rand_id,
            'license_name': 'Operator License ' + rand_id,
            'polymorphic_type': ('select', 'PVC Operator'),
            'polymorphic_type': ('select', 'PVC Operator'),
            #'expiration_date': '10/15/2010 00:00'
        })
        self.click_button(panel_name='license-wizard-panel', text=u'Next »')
        self.wait_for_loading()
        self.assert_no_growl_errors()

        # second page
        self.fill_form(panel_name='license-wizard-panel', values={
            'first_name': 'Henk', 'middle_name': 'Inthemiddle', 'last_name': 'Operator',
            'title': 'Operator', 'gender': ('select', 'Male'),
            'email': 'operator@test.com',
            'phone': '001234567', 'fax': '0012345678', 'mobile_phone': '0013456',
            'department': 'operator department', 'position': 'operator',
        })
        self.fill_form(panel_name='license-wizard-panel', form_name='contact-address-form', values={
            'street': 'OperatorStreet 11', 'postcode': '1000AA', 'state': 'NH',
            'city': 'Amsterdam', 'country': ('select', 'Netherlands')
        })
        self.wait_for_loading()
        self.click_button(panel_name='license-wizard-panel', text=u'Next »')
        self.wait_for_loading()
        self.assert_no_growl_errors()

        # third page
        self.fill_form(panel_name='license-wizard-panel', values={
            'name': 'Operator Company', 'vat_no': 'BE0893027827', 'website': 'http://operator.com',
            'number': 'BANK' + rand_id, 'bic': 'BIC' + rand_id, 'iban': 'IBAN' + rand_id
        })
        self.fill_form(panel_name='license-wizard-panel', form_name='company-address-form', values={
            'street': 'OperatorStreet 11', 'postcode': '1000AA', 'state': 'NH',
            'city': 'Amsterdam', 'country': ('select', 'Netherlands')
        })
        self.wait_for_loading()
        self.click_button(panel_name='license-wizard-panel', text=u'Next »')
        self.wait_for_loading()

        # fourth page

        # select from Role.
        self.fill_form(panel_name='license-wizard-panel', values={
            'role': ('select', 'PVC Operator', )
        })
        self.click_button(panel_name='license-wizard-panel', text=u'Next »')
        self.wait_for_loading()

        # fith page
        self.click_button(panel_name='license-wizard-panel', text=u'Next »')
        self.wait_for_loading()

        self.click_button(panel_name='license-wizard-panel', text='Finish')
        self.wait_for_panel_closed(panel_name='license-wizard-panel')

        # filter the cq to the newly created license
        self.filter_cq(panel_name='license-manager-panel', filters=[
            ('License', 'Equal', 'Operator License ' + rand_id),
            ('Type', 'Contains', ('select', 'PvcOperatorLicense')),
        ])

        # verify collins
        self.assert_cq(panel_name='license-manager-panel', row_index=0, data={
            'domain_name': 'PVC Domain ' + rand_id,
            'license_name': 'Operator License ' + rand_id,
            'license_polymorphic_type': 'PvcOperatorLicense',
            'contact': 'Henk Inthemiddle Operator',
            'company_name': 'Operator Company',
        });


class TestCase(PvcSeleniumTestCase):

    def setUp(self):

        super(TestCase, self).setUp()
        self.connect_db()

    def test_master_data_change_approval(self):
        operator_account = self._create_operator_account()
        self._create_contribution_tax_rates(operator_account)

        _modules = self._fill_pv_modules(operator_account)
        # this TBS has it's setting on MONTHLY!
        _administration_fee = Decimal('250.00')
        _contribution_fee = Decimal('2.75')

        cn = self._uniquestring('ma')[:16]
        ncn = self._uniquestring('ma')[:16]

        ma = self._create_member_account(company_name=cn)

        state = self._get_tbs_license_state()
        state['tbs']['country'] = None
        eu_tbs = self._create_tbs_account(operator_account, state)

        service_level = self._create_tbs_service_level(eu_tbs, admin_fee=_administration_fee, contrib_fee=_contribution_fee, year=today().year)

        tm = self._member_apply_at_tbs(ma, eu_tbs, service_level)

        mc = PvcMemberController(self.sa_session, account_id=eu_tbs.id)
        mc.save_tbs_member(tm.id, {'status': PvcTbsMember.STATUS_APPROVED, 'is_full_member': False, 'is_representative': False})

        lc = LicenseController(self.sa_session, account_id=ma.id)

        lc.update_master_data(ma.default_group.license.id, {
            'company': {
                'name': ncn,
            }
        })
        self.login(operator_account.name, 'test_password')

        self.click_iconbar('icon-pvc-member-manager', hover='icon-pvc-accountant-manager')
        self.wait_for_loading()

        self.filter_cq(panel_name='pvc-member-manager', filters=[
            ('Member', 'Equal', cn),
        ])
        self.wait_for_loading()

        self.click_cq(panel_name='pvc-member-manager', row_index=0, col_name='master_data')
        self.wait_for_loading()

        self.click_button(panel_name='master-data-compare-panel', text='Approve changes')
        self.wait_for_loading()

        self.close_panel(panel_name='pvc-member-manager')
        self.click_iconbar('icon-pvc-member-manager', hover='icon-pvc-accountant-manager')
        self.wait_for_loading()

        self.filter_cq(panel_name='pvc-member-manager', filters=[
            ('Member', 'Equal', ncn),
        ])

        self.assert_cq(panel_name='pvc-member-manager', row_index=0, data={
            'member': ncn
        })

    def test_archive_tbs_member(self):
        operator_account = self._create_operator_account()
        cn = self._uniquestring('ma')[:16]
        ma = self._create_member_account(company_name=cn)

        tbs = self._create_tbs_account(operator_account)
        service_level = self._create_tbs_service_level(tbs)

        tm = self._member_apply_at_tbs(ma, tbs, service_level)

        self.login(operator_account.name, 'test_password')

        self.click_iconbar('icon-pvc-member-manager', hover='icon-pvc-accountant-manager')
        self.wait_for_loading()

        self.filter_cq(panel_name='pvc-member-manager',filters=[
            ('Member', 'Equal', cn),
        ])

        self.click_cq(panel_name='pvc-member-manager', row_index=0, col_name='row_select', extra_css=' input')
        self.click_menu(panel_name='pvc-member-manager', text='Archive member')
        self.wait_for_loading()

        self.click_button(panel_name='dialog-question', text='Yes')
        self.wait_for_loading()

        self.assert_cq_empty(panel_name='pvc-member-manager')