# -*- coding: utf-8 -*-
'''
Created on Oct 11, 2013

@author: marten
'''
import time

from pvcycle.test.selenium.pvc_selenium import PvcSeleniumTestCase
from basesoft.util.virtual_date import today
from decimal import Decimal
from pvcycle.core.controllers import PvcMemberController
from pvcycle.core.models import PvcTbsMember
from pvcycle.order.controllers import PvcOrderController
from basesoft.web.contrib.order.controllers import PaymentController

class TestCase(PvcSeleniumTestCase):

    def setUp(self):
        super(TestCase, self).setUp()
        self.connect_db()

    def test_administration_fee(self):
        operator_account = self._create_operator_account()
        tbs_account = self._create_tbs_account(operator_account)
        accountant_account = self._create_accountant_account(tbs_account.default_group.license, operator_account)
        payment_methods = self._setup_payment_methods(operator_account, tbs_account)

        administration_fee = Decimal('375.00')
        service_level = self._create_tbs_service_level(tbs_account, admin_fee=administration_fee, year=today().year)

        # create a member
        ma = self._create_member_account()

        # apply at TBS
        tm = self._member_apply_at_tbs(ma, tbs_account, service_level)

        # activate member to automatically create an order entry for the Administration Fee
        mc = PvcMemberController(self.sa_session, account_id=tbs_account.id)
        mc.save_tbs_member(tm.id, {'status': PvcTbsMember.STATUS_APPROVED, 'is_full_member': False, 'is_representative': False})

        self.login(ma.name, 'test_password')
        self.wait_for_loading()

        self.click_iconbar('icon-pvc-order-manager', hover='icon-pvc-outstanding-contributions')
        self.wait_for_panel('pvc-order-manager')
        self.wait_for_loading()

        self.assert_cq(panel_name='pvc-order-manager', row_index=0, data={
            'client': tm.member.name,
            'merchant': tm.tbs.name,
            'payment_status': 'Pending',
            'total_amount': u'€ 375.00' # tax included
        });

        self.close_panel(panel_name='pvc-order-manager')

        # let the accountant_account attach an invoice
        oc = PvcOrderController(self.sa_session, account_id=accountant_account.id)
        order = oc.get_unexported_orders_for_merchant(tbs_account.default_group.license.id).pop()
        oc.attach_invoice_to_order(order['id'], 'INVOICE_ADM_FEE_%s.txt' % order['id'], 'invoice data...', False)

        self.click_iconbar('icon-pvc-order-manager', hover='icon-pvc-outstanding-contributions')
        self.wait_for_panel('pvc-order-manager')

        # assert invoice can be downloaded
        self.assert_cq(panel_name='pvc-order-manager', row_index=0, data={
            'invoice': 'Download'
        })

        # assert payment status
        self.assert_cq(panel_name='pvc-order-manager', row_index=0, data={
            'payment_status': 'Pending'
        })

        # click on 'checkout'
        self.click_cq(panel_name='pvc-order-manager', row_index=0, col_name='payment_checkout', extra_css=' button')
        self.wait_for_panel('order-details-panel-checkout')
        self.wait_for_loading()

        # select 'bank transfer' payment method and continue
        self.fill_form('order-details-panel-checkout', values={
            'payment_method': ('radio', payment_methods[0].id)
        })
        self.click_button('order-details-panel-checkout', text='Submit')
        self.wait_for_loading()

        # that should show the bank details panel with bank details
        self.wait_for_panel('order-payment_method-panel')
        td = self._driver.find_element_by_css_selector('.bank-details tr:nth-child(2) td:nth-child(2)')
        self.assertTrue(td.text, 'BBBBNLNH')

        # click submit payment
        self.click_button('order-payment_method-panel', text='Mark as submitted')
        self.wait_for_loading2()

        # payment status should now be submitted
        self.assert_cq(panel_name='pvc-order-manager', row_index=0, data={
            'payment_status': 'Submitted'
        })

        # let the accountant complete the order
        self.login(accountant_account.name, 'test_password')

        # open order manager
        self.click_iconbar('icon-pvc-order-manager', hover='icon-pvc-contribution-manager')

        # currrent status should be submitted
        self.assert_cq(panel_name='pvc-order-manager', row_index=0, data={
            'payment_status': 'Submitted'
        })

        # open payment details
        self.click_cq('pvc-order-manager', row_index=0, col_name='payment_status')

        # open complete/reject dialog
        self.click_button('order-payment-status-cq', text='Complete or reject order..')
        self.wait_for_loading2()

        # fill form
        self.fill_form('payment-status-details', values={
            'payment_method': ('select', 'bank transfer'),
            'payment_status': ('select', 'Completed'),
            'description': ('textarea', 'i r received moneys')
        })
        self.click_button('payment-status-details', text='Save')

        # close the payment details, should refresh the cq behind it
        self.close_panel('order-payment-status-cq')
        self.wait_for_loading()

        self.close_panel(panel_name='pvc-order-manager')
        self.click_iconbar('icon-pvc-order-manager', hover='icon-pvc-contribution-manager')
        self.wait_for_loading()

        # status should now be completed
        self.assert_cq(panel_name='pvc-order-manager', row_index=0, data={
            'payment_status': 'Completed'
        })

    def test_administration_fee_ogone(self):
        operator_account = self._create_operator_account()
        tbs_account = self._create_tbs_account(operator_account)
        accountant_account = self._create_accountant_account(tbs_account.default_group.license, operator_account)
        payment_methods = self._setup_payment_methods(operator_account, tbs_account)

        administration_fee = Decimal('375.00')
        service_level = self._create_tbs_service_level(tbs_account, admin_fee=administration_fee, year=today().year)

        # create a member
        ma = self._create_member_account()

        # apply at TBS
        tm = self._member_apply_at_tbs(ma, tbs_account, service_level)

        # activate member to automatically create an order entry for the Administration Fee
        mc = PvcMemberController(self.sa_session, account_id=tbs_account.id)
        mc.save_tbs_member(tm.id, {'status': PvcTbsMember.STATUS_APPROVED, 'is_full_member': False, 'is_representative': False})

        self.login(ma.name, 'test_password')
        self.wait_for_loading()

        self.click_iconbar('icon-pvc-order-manager', hover='icon-pvc-outstanding-contributions')
        self.wait_for_panel('pvc-order-manager')

        # click on 'checkout'
        self.click_cq(panel_name='pvc-order-manager', row_index=0, col_name='payment_checkout', extra_css=' button')
        self.wait_for_panel('order-details-panel-checkout')
        self.wait_for_loading()

        # first ogone payment method completes the payment
        self.fill_form('order-details-panel-checkout', values={
            'payment_method': ('radio', payment_methods[1].id)
        })
        self.click_button('order-details-panel-checkout', text='Submit')
        self.wait_for_loading()

        # that should show the ogone payment panel
        self.wait_for_panel('ogone-payment_method-panel')

        # and just wait till the payment is finished (opens a popup, redirect etc and finally updates the text)
        self.wait_for_selector('#ogone-result-text')
        self.click_button(panel_name='ogone-payment_method-panel', text='Close')

        # payment status should now be completed
        self.filter_cq('pvc-order-manager')
        self.assert_cq(panel_name='pvc-order-manager', row_index=0, data={
            'payment_status': 'Completed'
        })

    def test_membership_fee(self):
        operator_account = self._create_operator_account()

        state = self._get_tbs_license_state()
        state['configs']['pvc.tbs.use.membership.instead.of.service.level'] = True
        state['configs']['pvc.tbs.auto.approve.member'] = True

        tbs_account = self._create_tbs_account(operator_account, state=state)

        membership_fee = Decimal('375.00')
        memberships = self._create_memberships(tbs_account, fee=membership_fee)
        accountant_account = self._create_accountant_account(tbs_account.default_group.license, operator_account)
        payment_methods = self._setup_payment_methods(operator_account, tbs_account)

        # create a member
        ma = self._create_member_account()

        # apply at TBS
        tm = self._member_apply_at_tbs(ma, tbs_account, upload_contract=False, is_producer=False, membership=memberships[0])

        # TODO

    def test_payment_method_archive(self):
        operator_account = self._create_operator_account()

        self.login(operator_account.name, 'test_password')
        self.click_iconbar('icon-pvc-payment-methods', hover='icon-pvc-contribution-manager')
        self.wait_for_loading()

        # add a few payment methods
        tbs_account = self._create_tbs_account(operator_account)
        payment_methods = self._setup_payment_methods(operator_account, tbs_account)

        # verify CQ
        self.filter_cq('pvc-payment-method-manager', filters=[
            ('Merchant', 'Equal', tbs_account.default_group.license.name),
        ])
        self.wait_for_loading()
        self.assert_cq('pvc-payment-method-manager', row_index=0, data={
            'polymorphic_type': 'PaymentMethodBankTransfer'
        })
        self.assert_cq('pvc-payment-method-manager', row_index=1, data={
            'polymorphic_type': 'PaymentMethodOgone'
        })

        # archive one
        self.click_cq('pvc-payment-method-manager', row_index=0, col_name='row_select', extra_css=' input')
        self.click_menu(panel_name='pvc-payment-method-manager', text='Archive payment method')
        self.wait_for_loading()

        self.click_button('dialog-question', text='Yes')
        self.wait_for_loading()

        self.sa_session.expire(payment_methods[0])
        self.assertTrue(payment_methods[0].archived)

# below is for running the test from eclipse
if __name__ == '__main__':

    from twisted.scripts.trial import run
    import sys
    sys.argv = ['test_member.py', 'pvcycle.test.selenium.test_order.TestCase.test_administration_fee']
    run()
    sys.exit()

