# Copyright (c) 2015 BaseSoft B.V.
# See LICENSE for details.

from decimal import Decimal
from datetime import date
from pvcycle.core.models import PvcTbsMember
from selenium.webdriver.support.select import Select

from basesoft.util.virtual_date import today, set_date
from pvcycle.product.controllers import PvcProductController
from pvcycle.product.models import PvcTbsProduct, PvcTbsMemberProduct, PvcTbsProductFee
from pvcycle.test.selenium.pvc_selenium import PvcSeleniumTestCase

class TestCase(PvcSeleniumTestCase):

    def setUp(self):

        super(TestCase, self).setUp()
        self.connect_db()

        self._operator_account = self._create_operator_account()

    def _fill_product_row(self, row_index, label, desc, is_active, basis_for_invoicing, assert_=False, custom_unit=None,
                          product_type=PvcTbsProduct.TYPE_OTHER):
        css = self.wait_for_panel_or_form(panel_name='tbs-product-select')
        css = css + " table.bs-form-manager > tbody > tr:nth-child(%s)" % row_index
        container_el = self._driver.find_element_by_css_selector(css)

        label_el = '%s input[name="label"]' % css
        input_el = self._driver.find_element_by_css_selector(label_el)

        if assert_:
            self.assertEquals(input_el.get_attribute("value"), label)
        else:
            input_el.clear()
            input_el.send_keys(unicode(label))

            if input_el.get_attribute("value") != unicode(label):
                input_el.clear()
                input_el.send_keys(unicode(label))

        desc_el = '%s input[name="description"]' % css
        input_el = self._driver.find_element_by_css_selector(desc_el)

        if assert_:
            self.assertEquals(input_el.get_attribute("value"), desc)
        else:
            input_el.clear()
            input_el.send_keys(unicode(desc))

            if input_el.get_attribute("value") != unicode(desc):
                input_el.clear()
                input_el.send_keys(unicode(desc))

        active_el = '%s input[name="is_active"]' % css
        input_el = self._driver.find_element_by_css_selector(active_el)

        if assert_:
            if is_active:
                self.assertTrue(input_el.is_selected())
            else:
                self.assertFalse(input_el.is_selected())
        else:
            if is_active:
                if not input_el.is_selected():
                    input_el.click()

            else:
                if input_el.is_selected():
                    input_el.click()

        # basis for invoicing

        input_css = "%s select[name='basis_for_invoicing']" %  css
        el = self._driver.find_element_by_css_selector(input_css)
        select_el = Select(el)

        if assert_:
            option = select_el.all_selected_options[0]
            self.assertEqual(option.get_attribute("value"), str(basis_for_invoicing))
        else:
            select_el.select_by_value(str(basis_for_invoicing))

        # custom unit

        custom_unit_value = str('' if custom_unit is None else custom_unit)

        input_css = "%s select[name='custom_unit']" %  css
        el = self._driver.find_element_by_css_selector(input_css)
        select_el = Select(el)

        if assert_:
            option = select_el.all_selected_options[0]
            self.assertEqual(option.get_attribute("value"), custom_unit_value)
        else:
            select_el.select_by_value(custom_unit_value)

        # product type


        input_css = "%s select[name='product_type']" %  css
        el = self._driver.find_element_by_css_selector(input_css)
        select_el = Select(el)

        if assert_:
            option = select_el.all_selected_options[0]
            self.assertEqual(option.get_attribute("value"), str(product_type))
        else:
            select_el.select_by_value(str(product_type))

    def test_products_for_tbs(self):

        tbs = self._create_tbs_account(self._operator_account)

        self.login(self._operator_account.name, 'test_password')

        self.click_iconbar('icon-pvc-tbs-product-select', hover='icon-pvc-tbs-product-manager')
        self.wait_for_loading()

        self.fill_form(panel_name='tbs-product-select', values={
            'tbs_id': ('select', 'value=%s' % tbs.default_group.license.id)
        })
        self.wait_for_loading()

        # expand the EEE categories
        self.click_cq(panel_name='tbs-product-select', row_index=0, col_name="select", extra_css="", is_datatable=True)
        # the child product levels are dynamically loaded, wait for them.
        self.wait_for_loading()

        self._fill_product_row(2, 'EEE UPDATED', 'SELENIUM TESTED DESCRIPTION', True, PvcTbsProduct.INVOICE_BASED_ON_WEIGHT)

        self._fill_product_row(3, 'EEE UPDATED CUSTOM', 'SELENIUM TESTED CUSTOM UNIT', True,
                               PvcTbsProduct.INVOICE_BASED_ON_CUSTOM_UNIT, custom_unit=PvcTbsProduct.CUSTOM_UNIT_NOMINAL_POWER,
                               product_type=PvcTbsProduct.TYPE_PV_RELATED)

        self.click_button(panel_name='tbs-product-select', text='Save')
        self.wait_for_loading()

        self.close_panel(panel_name='tbs-product-select')

        self.click_iconbar('icon-pvc-tbs-product-select', hover='icon-pvc-tbs-product-manager')
        self.wait_for_loading()

        self.fill_form(panel_name='tbs-product-select', values={
            'tbs_id': ('select', 'value=%s' % tbs.default_group.license.id)
        })
        self.wait_for_loading()

        self.click_cq(panel_name='tbs-product-select', row_index=0, col_name="select", extra_css="", is_datatable=True)
        self.wait_for_loading()

        self._fill_product_row(2, 'EEE UPDATED', 'SELENIUM TESTED DESCRIPTION', True, PvcTbsProduct.INVOICE_BASED_ON_WEIGHT, True)
        self._fill_product_row(3, 'EEE UPDATED CUSTOM', 'SELENIUM TESTED CUSTOM UNIT', True, PvcTbsProduct.INVOICE_BASED_ON_CUSTOM_UNIT, True,
                               custom_unit=PvcTbsProduct.CUSTOM_UNIT_NOMINAL_POWER, product_type=PvcTbsProduct.TYPE_PV_RELATED)

        # When a child is added or a child is activated for declaration, the parent in the tree has to be at least inactive.
        controller = PvcProductController(self.sa_session, account_id=self._operator_account.id)
        products = controller.get_available_products_for_tbs(tbs.default_group.license.id)
        self.assertEqual(products[0]['status'], PvcTbsProduct.STATUS_INACTIVE)

    def test_new_product_subcategory(self):

        tbs_account = self._create_tbs_account(self._operator_account)
        tbs = tbs_account.default_group.license

        self.login(self._operator_account.name, 'test_password')

        self.click_iconbar('icon-pvc-tbs-product-select', hover='icon-pvc-tbs-product-manager')
        self.wait_for_loading()

        self.fill_form(panel_name='tbs-product-select', values={
            'tbs_id': ('select', 'value=%s' % tbs.default_group.license.id)
        })
        self.wait_for_loading()

        self.click_cq(panel_name='tbs-product-select', row_index=0, col_name="addsub", extra_css=" a", is_datatable=True)
        self.wait_for_loading()

        self.fill_form('pvc-tbs-product-details', values={
            'weee_label': 'MY FIRST WEEE LABEL %d' % tbs.id,
            'weee_description': 'MY FIRST WEEE DESC %d' % tbs.id,
            'label': 'FRENCH TBS CROISSAINT',
            'description': 'FRENCH TBS LE PAIN',
            'basis_for_invoicing': ('select', 'Quantity'),
        })

        self.click_button(panel_name='pvc-tbs-product-details', text='Save')
        self.wait_for_loading()

        self.close_panel(panel_name='tbs-product-select')
        self.click_iconbar('icon-weee-product-level-manager')
        self.wait_for_loading()

        self.filter_cq('weee-product-level-manager', filters=[(
            'Label', 'Contains', 'MY FIRST WEEE LABEL %d' % tbs.id,
        )])

        self.assert_cq('weee-product-level-manager', row_index=0, data={
            'label': 'MY FIRST WEEE LABEL %d' % tbs.id,
            'description': 'MY FIRST WEEE DESC %d' % tbs.id,
        })

    def test_products_for_member(self):

        tbs_account = self._create_tbs_account(self._operator_account)
        tbs = tbs_account.default_group.license
        service_level = self._create_tbs_service_level(tbs_account, admin_fee=Decimal('20.0'), year=today().year)

        ma = self._create_member_account()
        tm = self._member_apply_at_tbs(ma, tbs_account, service_level=service_level)

        tm.status = PvcTbsMember.STATUS_APPROVED
        self.sa_session.add(tm)
        self.sa_session.flush()

        pc = PvcProductController(self.sa_session, account_id=self._operator_account.id)

        products = pc.get_available_products_for_tbs(tbs.id)
        tbs_product = pc.save_or_update_tbs_product(None, 'EEE-TBS-TEST', 'WEEE', tbs.id,
            parent_level_id=products[0]['id'], basis_for_invoicing=PvcTbsProduct.INVOICE_BASED_ON_WEIGHT
        )
        self.login(ma.name, 'test_password')

        self.click_iconbar('icon-pvc-tbs-member-product-manager')
        self.wait_for_loading()

        self.click_menu('pvc-tbs-member-product-manager', text="Add products")
        self.wait_for_loading()

        self.fill_form('pvc-tbs-member-product-select', values={
            'tbs_id': ('select', 'value=%d' % tm.tbs.id, )
        })
        self.wait_for_loading()

        self.fill_form('pvc-tbs-member-product-select', values={
            'products': ('fancy-select', ('EEE-TBS-TEST - WEEE', ), ),
        })

        self.click_button('pvc-tbs-member-product-select', text='Save')
        self.wait_for_loading()

        self.assert_cq('pvc-tbs-member-product-manager', row_index=0, data={
            'product_label': 'EEE-TBS-TEST',
            'product_desc': 'WEEE'
        })

    def test_brands_for_member(self):
        tbs_account = self._create_tbs_account(self._operator_account)
        tbs = tbs_account.default_group.license
        service_level = self._create_tbs_service_level(tbs_account, admin_fee=Decimal('20.0'), year=today().year)

        tbs.set_config('pvc.tbs.contribution.split.by.brand', True)

        pc = PvcProductController(self.sa_session, account_id=self._operator_account.id)

        products = pc.get_available_products_for_tbs(tbs.id)
        tbs_product = pc.save_or_update_tbs_product(None, 'EEE-TBS-TEST', 'WEEE', tbs.id,
            parent_level_id=products[0]['id'], basis_for_invoicing=PvcTbsProduct.INVOICE_BASED_ON_WEIGHT
        )

        ma = self._create_member_account()
        tm = self._member_apply_at_tbs(ma, tbs_account, service_level=service_level)

        tm.status = PvcTbsMember.STATUS_APPROVED
        self.sa_session.add(tm)
        self.sa_session.flush()

        self.login(ma.name, 'test_password')

        self.click_iconbar('icon-pvc-tbs-member-product-manager')
        self.wait_for_loading()

        self.click_menu('pvc-tbs-member-product-manager', text="Add products")
        self.wait_for_loading()

        self.fill_form('pvc-tbs-member-product-select', values={
            'tbs_id': ('select', 'value=%d' % tm.tbs.id, )
        })
        self.wait_for_loading()

        self.fill_form('pvc-tbs-member-product-select', values={
            'products': ('fancy-select', ('EEE-TBS-TEST - WEEE', ), ),
        })

        self.click_button(panel_name='pvc-tbs-member-product-select', text='Add an additional brand')
        self.fill_form(panel_name='pvc-tbs-member-product-select', rowcontrol_row_index=1, values={'brand': 'My Own Other Brand'})
        self.fill_form(panel_name='pvc-tbs-member-product-select', rowcontrol_row_index=0, values={'brand': 'My Own Brand'})

        self.click_button('pvc-tbs-member-product-select', text='Save')
        self.wait_for_loading()

        self.assert_cq('pvc-tbs-member-product-manager', row_index=0, data={
            'product_label': 'EEE-TBS-TEST',
            'product_desc': 'WEEE',
            'brands': 'My Own Brand,My Own Other Brand'
        })

        # we are going to disable a brand

        # separate CQ
        self.click_toolbar_button('pvc-tbs-member-product-manager', 'separate')
        self.wait_for_loading()

        self.click_cq('pvc-tbs-member-product-manager', row_index=1, col_name='row_select', extra_css=' input')

        self.click_menu('pvc-tbs-member-product-manager', text="Deactivate brand")

        self.click_button(panel_name='dialog-question', text='Yes')
        self.wait_for_loading()

        self.assert_cq('pvc-tbs-member-product-manager', row_index=0, data={
            'product_label': 'EEE-TBS-TEST',
            'product_desc': 'WEEE',
            'brands': 'My Own Brand'
        })

    def test_rename_brand(self):
        self._reset_brands()
        pc = PvcProductController(self.sa_session, account_id=self._operator_account.id)
        p1 = pc._get_or_save_brand('Philips')
        p2 = pc._get_or_save_brand('Filips')

        self.login(self._operator_account.name, 'test_password')

        self.click_iconbar('icon-brand-manager', hover='icon-pvc-tbs-product-manager')
        self.wait_for_loading()

        self.filter_cq('pvc-brand-manager', filters=[(
            'Name', 'Equal',(
                'multiple', (
                    ('input_text', p2.name),
                )
            )
        )])
        self.wait_for_loading()

        name = self._uniquestring('b')

        self.click_cq('pvc-brand-manager', row_index=0, col_name='row_select', extra_css=' input')
        self.click_menu('pvc-brand-manager', text="Rename selected brand")
        self.wait_for_loading()

        target = self._driver.find_element_by_css_selector('input.feedback-input')
        target.clear()
        target.send_keys(name)

        self.click_button(panel_name='dialog-feedback', text='OK')
        self.wait_for_loading()

        self.sa_session.expire(p2)
        self.assertEqual(p2.name, name)

    def test_merge_brands(self):
        self._reset_brands()
        pc = PvcProductController(self.sa_session, account_id=self._operator_account.id)
        p1 = pc._get_or_save_brand('Philips')
        p2 = pc._get_or_save_brand('Samsung')
        p3 = pc._get_or_save_brand('Filips')
        p4 = pc._get_or_save_brand('Sony')
        p5 = pc._get_or_save_brand('Phillllips')

        self.login(self._operator_account.name, 'test_password')

        self.click_iconbar('icon-brand-manager', hover='icon-pvc-tbs-product-manager')
        self.wait_for_loading()

        self.filter_cq('pvc-brand-manager', filters=[(
            'Name', 'Equal',(
                'multiple', (
                    ('input_text', p1.name),
                    ('input_text', p3.name),
                    ('input_text', p5.name),
                )
            )
        )])
        self.wait_for_loading()

        self.click_cq('pvc-brand-manager', row_index=0, col_name='row_select', extra_css=' input')
        self.click_cq('pvc-brand-manager', row_index=1, col_name='row_select', extra_css=' input')
        self.click_cq('pvc-brand-manager', row_index=2, col_name='row_select', extra_css=' input')

        self.click_menu('pvc-brand-manager', text="Merge selected brands")
        self.wait_for_loading()

        self.fill_form('pvc-brand-merge-panel', values={
            'target_brand_id': ('radio', p1.id)
        })
        self.click_button('pvc-brand-merge-panel', text='Merge brands')
        self.wait_for_loading()

        self.assert_cq('pvc-brand-manager', row_index=0, data={'status': '-'})
        self.assert_cq('pvc-brand-manager', row_index=1, data={'status': 'Merge pending'})
        self.assert_cq('pvc-brand-manager', row_index=2, data={'status': 'Merge pending'})

        self.close_panel('pvc-brand-manager')
        self.click_iconbar('icon-brand-manager', hover='icon-pvc-tbs-product-manager')
        self.wait_for_loading()

        self.filter_cq('pvc-brand-manager', filters=[(
            'Name', 'Equal', 'Filips'
        )])
        self.wait_for_loading()

        self.click_cq('pvc-brand-manager', row_index=0, col_name='row_select', extra_css=' input')
        self.click_menu('pvc-brand-manager', text="Cancel merge")

        self.click_button(panel_name='dialog-question', text='Yes')
        self.wait_for_loading()

        self.assert_cq('pvc-brand-manager', row_index=0, data={'status': '-'})

    def test_product_fee(self):

        set_date(date(2013, 1, 1)) # set date to January 2013

        tbs_account = self._create_tbs_account(self._operator_account)
        tbs = tbs_account.default_group.license

        service_level = self._create_tbs_service_level(tbs_account, admin_fee=Decimal('230.23'), year=today().year)

        # add some products for this TBS
        pc = PvcProductController(self.sa_session, account_id=self._operator_account.id)
        products = pc.get_available_products_for_tbs(tbs.id)
        tbs_product = pc.save_or_update_tbs_product(None, 'EEE-TBS-TEST', 'WEEE', tbs.id,
            parent_level_id=products[0]['id'], basis_for_invoicing=PvcTbsProduct.INVOICE_BASED_ON_WEIGHT
        )

        tbs_product2 = pc.save_or_update_tbs_product(None, 'EEE-TBS-TEST-2', 'WEEE-2', tbs.id,
            parent_level_id=products[1]['id'], basis_for_invoicing=PvcTbsProduct.INVOICE_BASED_ON_WEIGHT
        )

        parent_product = pc.get_tbs_product_by_weee_product(tbs_product2.product.parent_level.id, tbs.id)

        self.login(self._operator_account.name, 'test_password')

        self.click_iconbar('icon-pvc-tbs-manager', hover='icon-pvc-accountant-manager')
        self.wait_for_loading(stable_time=2)

        # filter cq for this tbs
        self.filter_cq('pvc-tbs-manager', filters=[
            ('ID', 'Equal', '%d' % tbs.id),
        ])
        self.wait_for_loading()

        self.click_cq('pvc-tbs-manager', row_index=0, col_name='product_fees')
        self.wait_for_loading()

        self.fill_form('pvc-tbs-product-fee-panel', values={
            'month': ('select', 'value=%d' % 2),  # march
            'year': 2013,
            'service_level_id': ('select', 'value=%d' % service_level.id)
        })
        self.wait_for_loading()

        self.fill_form('pvc-tbs-product-fee-panel', values={
            'fee_%d' % tbs_product.id: '34.100',  # own fee,
            'fee_%d' % parent_product.id: '123.34'  # parent level fee
        })

        self.click_button('pvc-tbs-product-fee-panel', text="Save")
        self.wait_for_loading()

        self.assertEqual(tbs_product2.get_fee('2013-03-01', service_level.id), Decimal('123.34'))

    def test_product_fee_with_dimension_split(self):
        set_date(date(2013, 1, 1)) # set date to January 2013

        tbs_account = self._create_tbs_account(self._operator_account)
        tbs = tbs_account.default_group.license
        tbs.set_config('pvc.tbs.require.end.user.distinction', True)

        service_level = self._create_tbs_service_level(tbs_account, admin_fee=Decimal('230.23'), year=today().year)

        # add some products for this TBS
        pc = PvcProductController(self.sa_session, account_id=self._operator_account.id)
        products = pc.get_available_products_for_tbs(tbs.id)
        tbs_product = pc.save_or_update_tbs_product(None, 'EEE-TBS-TEST', 'WEEE', tbs.id,
            parent_level_id=products[0]['id'], basis_for_invoicing=PvcTbsProduct.INVOICE_BASED_ON_WEIGHT
        )

        tbs_product2 = pc.save_or_update_tbs_product(None, 'EEE-TBS-TEST-2', 'WEEE-2', tbs.id,
            parent_level_id=products[1]['id'], basis_for_invoicing=PvcTbsProduct.INVOICE_BASED_ON_WEIGHT
        )

        parent_product = pc.get_tbs_product_by_weee_product(tbs_product2.product.parent_level.id, tbs.id)

        self.login(self._operator_account.name, 'test_password')

        self.click_iconbar('icon-pvc-tbs-manager', hover='icon-pvc-accountant-manager')
        self.wait_for_loading()

        # filter cq for this tbs
        self.filter_cq('pvc-tbs-manager', filters=[(
            'ID', 'Equal', '%d' % tbs.id
        )])
        self.wait_for_loading()

        self.click_cq('pvc-tbs-manager', row_index=0, col_name='product_fees')
        self.wait_for_loading()

        self.fill_form('pvc-tbs-product-fee-panel', values={
            'month': ('select', 'value=%d' % 2),  # march
            'year': 2013,
            'service_level_id': ('select', 'value=%d' % service_level.id)
        })
        self.wait_for_loading()

        self.fill_form('pvc-tbs-product-fee-panel', values={
            'fee_b2b_%d' % tbs_product.id: '34.100',  # own fee,
            'fee_b2c_%d' % tbs_product.id: '88.74',  # own fee,
            'fee_b2b_%d' % parent_product.id: '123.34',  # parent level fee
            'fee_b2c_%d' % tbs_product2.id: '74.235',  # own fee
        })

        self.click_button('pvc-tbs-product-fee-panel', text="Save")
        self.wait_for_loading()

        self.assertEqual(tbs_product.get_fee('2013-03-01', service_level.id, PvcTbsProductFee.DIMENSION_IS_B2B), Decimal('34.100'))
        self.assertEqual(tbs_product.get_fee('2013-03-01', service_level.id, PvcTbsProductFee.DIMENSION_IS_B2C), Decimal('88.74'))
        self.assertEqual(tbs_product2.get_fee('2013-03-01', service_level.id, PvcTbsProductFee.DIMENSION_IS_B2B), Decimal('123.34'))
        self.assertEqual(tbs_product2.get_fee('2013-03-01', service_level.id, PvcTbsProductFee.DIMENSION_IS_B2C), Decimal('74.235'))

