# -*- coding: utf-8 -*-
'''
Created on Jan 31, 2014

@author: marten
'''
from datetime import date, datetime
from twisted.trial.unittest import SkipTest
from sqlalchemy.sql.expression import and_

from basesoft.util.virtual_date import today, set_date
from pvcycle.test.selenium.pvc_selenium import PvcSeleniumTestCase
from pvcycle.subrogation.controllers import PvcSubrogationController
from pvcycle.subrogation.models import PvcSubrogation
from pvcycle.pvmodule.controllers import PvcPVModuleController
from pvcycle.core.controllers import PvcMemberController
from pvcycle.core.models import PvcTbsMember
from pvcycle.order.controllers import PvcDeclarationOrderController
from pvcycle.pvmodule.models import PvcTbsMemberPVModule
from basesoft.web.contrib.order.controllers import PaymentController
from basesoft.web.contrib.order.models import PaymentMethodBankTransfer, OrderPaymentStatus

class TestCase(PvcSeleniumTestCase):
    def setUp(self):
        super(TestCase, self).setUp()
        self.connect_db()

        self.operator = self._create_operator_account()

        self.ma = self._create_member_account('SUBROGATOR')
        self.ta = self._create_tbs_account(self.operator)

        self.service_level = self._create_tbs_service_level(self.ta, year=today().year)

        self.tm = self._member_apply_at_tbs(self.ma, self.ta, self.service_level)
        self.tm.status = self.tm.STATUS_APPROVED
        self.sa_session.add(self.tm)

        self.ca = self._create_member_account('COMMISSIONER')
        self.ctm = self._member_apply_at_tbs(self.ca, self.ta, self.service_level, is_producer=False, is_commissioner=True)
        self.ctm.status = self.tm.STATUS_APPROVED
        self.sa_session.add(self.ctm)
        self.sa_session.flush()


    def _setup_multi_member(self):

        # create second tbs (used for second company)
        tbs_account_2 = self._create_tbs_account(self.operator)
        service_level_2 = self._create_tbs_service_level(tbs_account_2, year=today().year)

        # create member account with its default license
        member_account = self._create_member_account('company 1')
        license_1 = member_account.default_group.license

        # create a second company
        mc = PvcMemberController(self.sa_session, account_id=member_account.id)
        license_2 = mc.save_member_license(self._get_member_license_state('company 2'))

        # apply first company to default tbs
        tbs_member_1 = self._member_apply_at_tbs(member_account, self.ta, self.service_level, member_license_id=license_1.id)

        # apply second company to default tbs
        tbs_member_2 = self._member_apply_at_tbs(member_account, self.ta, self.service_level, member_license_id=license_2.id)

        # apply second company to second tbs
        tbs_member_3 = self._member_apply_at_tbs(member_account, tbs_account_2, service_level_2, member_license_id=license_2.id)

        # activate member to automatically create an order entry for the Administration Fee
        mc_tbs_1 = PvcMemberController(self.sa_session, account_id=self.ta.id)
        mc_tbs_2 = PvcMemberController(self.sa_session, account_id=tbs_account_2.id)
        mc_tbs_1.save_tbs_member(tbs_member_1.id, {'status': PvcTbsMember.STATUS_APPROVED, 'is_full_member': False, 'is_representative': False})
        mc_tbs_1.save_tbs_member(tbs_member_2.id, {'status': PvcTbsMember.STATUS_APPROVED, 'is_full_member': False, 'is_representative': False})
        mc_tbs_2.save_tbs_member(tbs_member_3.id, {'status': PvcTbsMember.STATUS_APPROVED, 'is_full_member': False, 'is_representative': False})

        self._member_account = member_account
        self._tbs_member_1 = tbs_member_1
        self._tbs_member_2 = tbs_member_2
        self._tbs_member_3 = tbs_member_3

    def _request_for_subrogation(self, commissioner_name='COMMY'):
        controller = PvcSubrogationController(self.sa_session, account_id=self.ma.id)
        return controller.request_for_subrogation(self.tm, commissioner_name)

    def test_upgrade_to_commissioner(self):
        producer = self._uniquestring('u')
        ma = self._create_member_account(company_name=producer)
        tm = self._member_apply_at_tbs(ma, self.ta, self.service_level)
        tm.status = self.tm.STATUS_APPROVED
        self.sa_session.add(self.tm)

        self.login(self.operator.name, 'test_password')

        self.click_iconbar('icon-pvc-member-manager', hover='icon-pvc-accountant-manager')
        self.wait_for_loading()

        self.filter_cq('pvc-member-manager', filters=[
            ('Member', 'Equal', producer),
        ])
        self.wait_for_loading()

        self.click_cq(panel_name='pvc-member-manager', row_index=0, col_name='row_select', extra_css=' input')
        self.click_menu(panel_name='pvc-member-manager', text='Enable producer as commissioner')
        self.wait_for_loading()

    def test_apply_for_subrogation(self):
        self.login(self.ma.name, 'test_password')

        self.click_iconbar(icon_id='icon-pvc-subrogation-request', hover='icon-pvc-subrogation-manager')
        self.wait_for_loading()

        self.fill_form('subrogation-request-panel', values={
            'tbs_id': ('select', self.tm.tbs.name),
            'commissioner_name': 'COOMMY'
        })

        self.click_button('subrogation-request-panel', text='Save')
        self.wait_for_loading()

        self.click_iconbar(icon_id='icon-pvc-subrogation-manager')
        self.wait_for_loading()

        self.filter_cq(panel_name='pvc-subrogation-manager', filters=[
            ('Status', 'Equal', ('select', 'New')),
        ])

        self.wait_for_loading() # for the custom query
        self.assert_cq(panel_name='pvc-subrogation-manager', row_index=0, data={
            'commissioner_requested': 'COOMMY'
        })

    def test_apply_for_subrogation_multi_member(self):

        self._setup_multi_member()

        self.login(self._member_account.name, 'test_password')

        # apply for first company and first tbs
        self.click_iconbar(icon_id='icon-pvc-subrogation-request', hover='icon-pvc-subrogation-manager')
        self.wait_for_loading()
        self.fill_form('subrogation-request-panel', values={
            'member_id': ('select', self._tbs_member_1.member.name),
        })
        self.wait_for_loading()
        self.fill_form('subrogation-request-panel', values={
            'tbs_id': ('select', self._tbs_member_1.tbs.name),
            'commissioner_name': 'COOMMY 1'
        })
        self.click_button('subrogation-request-panel', text='Save')
        self.wait_for_loading()

        # apply for first company and second tbs
        self.click_iconbar(icon_id='icon-pvc-subrogation-request', hover='icon-pvc-subrogation-manager')
        self.wait_for_loading()
        self.fill_form('subrogation-request-panel', values={
            'member_id': ('select', self._tbs_member_3.member.name),
        })
        self.wait_for_loading()
        self.fill_form('subrogation-request-panel', values={
            'tbs_id': ('select', self._tbs_member_3.tbs.name),
            'commissioner_name': 'COOMMY 2'
        })
        self.click_button('subrogation-request-panel', text='Save')
        self.wait_for_loading()

        self.click_iconbar(icon_id='icon-pvc-subrogation-manager')
        self.wait_for_loading()

        self.filter_cq(panel_name='pvc-subrogation-manager', filters=[
            ('Status', 'Equal', ('select', 'New')),
        ])
        self.wait_for_loading()
        self.assert_cq(panel_name='pvc-subrogation-manager', row_index=0, data={
            'subrogator': self._tbs_member_1.member.name,
            'tbs_name': self._tbs_member_1.tbs.name,
            'commissioner_requested': 'COOMMY 1'
        })
        self.assert_cq(panel_name='pvc-subrogation-manager', row_index=1, data={
            'subrogator': self._tbs_member_3.member.name,
            'tbs_name': self._tbs_member_3.tbs.name,
            'commissioner_requested': 'COOMMY 2'
        })

    def test_match_commissioner(self):

        self._request_for_subrogation()

        self.login(self.ta.name, 'test_password')

        self.click_iconbar(icon_id='icon-pvc-subrogation-manager')
        self.wait_for_loading()

        self.click_cq(panel_name='pvc-subrogation-manager', row_index=0, col_name='commissioner_requested')
        self.wait_for_loading()

        target = self._driver.find_element_by_css_selector("div.message-warning")
        self.assertEqual(target.text, 'No commissioners found with provided name.')

        self.fill_form('subrogation-match-panel', values={
            'commissioner': 'COMM'
        })

        self.wait_for_loading()

        self.click_cq('subrogation-match-panel', row_index=0, col_name='use', is_datatable=True)

        self.click_button('dialog-question', text='Yes')
        self.wait_for_loading()

        self.assert_cq(panel_name='pvc-subrogation-manager', row_index=0, data={
            'status': 'Pending'
        })

    def test_rejection_by_tbs(self):
        self._request_for_subrogation()

        self.login(self.ta.name, 'test_password')

        self.click_iconbar(icon_id='icon-pvc-subrogation-manager')
        self.wait_for_loading()

        self.click_cq(panel_name='pvc-subrogation-manager', row_index=0, col_name='commissioner_requested')
        self.wait_for_loading()

        target = self._driver.find_element_by_css_selector("div.message-warning")
        self.assertEqual(target.text, 'No commissioners found with provided name.')

        self.click_button('subrogation-match-panel', text='Reject')

        target = self._driver.find_element_by_css_selector('input.feedback-input')
        target.clear()
        target.send_keys(unicode('No match found for provided commissioner name'))

        self.click_button('dialog-feedback', text='OK')
        self.wait_for_loading()

        self.assert_cq(panel_name='pvc-subrogation-manager', row_index=0, data={
            'status': 'Rejected'
        })

    def test_create_mandate(self):
        self.login(self.operator.name, 'test_password')

        self.click_iconbar(icon_id='icon-pvc-tbs-manager', hover='icon-pvc-accountant-manager')
        self.wait_for_loading()

        self.filter_cq(panel_name='pvc-tbs-manager', filters=[
            ('Name', 'Equal', self.tm.tbs.name),
        ])
        self.wait_for_loading()

        self.click_cq('pvc-tbs-manager', row_index=0, col_name='mandate')
        self.wait_for_loading()

        self.fill_form('pvc-subrogation-mandate-panel', values={
            'mandate_text': ('textarea', 'Selenium Mandate Test Text')
        })
        self.wait_for_loading()

        self.click_button('pvc-subrogation-mandate-panel', text='Save and close')
        self.wait_for_loading()

        self.sa_session.expire(self.tm.tbs)
        self.assertEqual(self.tm.tbs.mandate_text, 'Selenium Mandate Test Text')

    def test_subrogator_mandate(self):
        subrogation = self._request_for_subrogation('Comm')

        controller = PvcSubrogationController(self.sa_session, account_id=self.ta.id)
        controller.match_commissioner(subrogation.id, self.ctm.id)

        self.login(self.ma.name, 'test_password')

        self.click_iconbar(icon_id='icon-pvc-subrogation-manager')
        self.wait_for_loading()
        self.assert_cq(panel_name='pvc-subrogation-manager', row_index=0, data={
            'status': 'Pending'
        })

        self.click_cq(panel_name='pvc-subrogation-manager', row_index=0, col_name='details')
        self.wait_for_loading()

        self.fill_form(panel_name='pvc-subrogation-details', values={
            'upload_mandate': ('upload', 'contract.pdf'),
        })

        self.click_button('pvc-subrogation-details', text='Save')
        self.wait_for_loading()

        self.assert_cq(panel_name='pvc-subrogation-manager', row_index=0, data={
            'status': 'Producer signed'
        })

    def test_commissioner_mandate(self):
        subrogation = self._request_for_subrogation('Comm')

        controller = PvcSubrogationController(self.sa_session, account_id=self.ta.id)
        controller.match_commissioner(subrogation.id, self.ctm.id)

        upload_mandate = self._create_uploaded_file('upload_mandate.pdf', 'upload_mandate contents')
        controller = PvcSubrogationController(self.sa_session, account_id=self.ma.id)
        subrogation = controller.save_subrogation(subrogation.id, {'upload_mandate': upload_mandate.file_hash})

        self.login(self.ca.name, 'test_password')

        self.click_iconbar(icon_id='icon-pvc-subrogation-manager', hover="icon-pvc-subrogation-product-manager")
        self.wait_for_loading()
        self.assert_cq(panel_name='pvc-subrogation-manager', row_index=0, data={
            'status': 'Producer signed'
        })

        self.click_cq(panel_name='pvc-subrogation-manager', row_index=0, col_name='details')
        self.wait_for_loading()

        self.fill_form(panel_name='pvc-subrogation-details', values={
            'upload_mandate': ('upload', 'contract.pdf'),
        })

        self.click_button('pvc-subrogation-details', text='Save')
        self.wait_for_loading()

        self.assert_cq(panel_name='pvc-subrogation-manager', row_index=0, data={
            'status': 'Commissioner signed'
        })

    def test_approval_by_tbs(self):
        subrogation = self._request_for_subrogation()
        controller = PvcSubrogationController(self.sa_session, account_id=self.ta.id)
        subrogation = controller.match_commissioner(subrogation.id, self.ctm.id)

        upload_mandate = self._create_uploaded_file('upload_mandate.pdf', 'upload_mandate contents')
        controller = PvcSubrogationController(self.sa_session, account_id=self.ma.id)
        subrogation = controller.save_subrogation(subrogation.id, {'upload_mandate': upload_mandate.file_hash})

        upload_mandate = self._create_uploaded_file('upload_mandate.pdf', 'upload_mandate contents')
        controller = PvcSubrogationController(self.sa_session, account_id=self.ca.id)
        subrogation = controller.save_subrogation(subrogation.id, {'upload_mandate': upload_mandate.file_hash})

        self.login(self.ta.name, 'test_password')

        self.click_iconbar(icon_id='icon-pvc-subrogation-manager')
        self.wait_for_loading()

        self.click_cq(panel_name='pvc-subrogation-manager', row_index=0, col_name='details')
        self.wait_for_loading()

        self.fill_form(panel_name='pvc-subrogation-details', values={
            'status': ('radio', PvcSubrogation.STATUS_ACTIVE)
        })

        self.click_button('pvc-subrogation-details', text='Save')
        self.wait_for_loading()

        self.assert_cq(panel_name='pvc-subrogation-manager', row_index=0, data={
            'status': 'Active'
        })

    def test_suspension_by_tbs(self):
        subrogation = self._request_for_subrogation()
        controller = PvcSubrogationController(self.sa_session, account_id=self.ta.id)
        subrogation = controller.match_commissioner(subrogation.id, self.ctm.id)

        upload_mandate = self._create_uploaded_file('upload_mandate.pdf', 'upload_mandate contents')
        controller = PvcSubrogationController(self.sa_session, account_id=self.ma.id)
        subrogation = controller.save_subrogation(subrogation.id, {'upload_mandate': upload_mandate.file_hash})

        upload_mandate = self._create_uploaded_file('upload_mandate.pdf', 'upload_mandate contents')
        controller = PvcSubrogationController(self.sa_session, account_id=self.ca.id)
        subrogation = controller.save_subrogation(subrogation.id, {'upload_mandate': upload_mandate.file_hash})

        controller = PvcSubrogationController(self.sa_session, account_id=self.ta.id)
        controller.approve_subrogation(subrogation.id)

        self.login(self.ta.name, 'test_password')

        self.click_iconbar(icon_id='icon-pvc-subrogation-manager')
        self.wait_for_loading()

        self.assert_cq(panel_name='pvc-subrogation-manager', row_index=0, data={
            'status': 'Active'
        })

        self.click_cq(panel_name='pvc-subrogation-manager', row_index=0, col_name='details')
        self.wait_for_loading()

        self.fill_form(panel_name='pvc-subrogation-details', values={
            'status': ('radio', PvcSubrogation.STATUS_SUSPENDED)
        })

        self.click_button('pvc-subrogation-details', text='Save')
        self.wait_for_loading()

        self.assert_cq(panel_name='pvc-subrogation-manager', row_index=0, data={
            'status': 'Suspended'
        })

    def test_subrogator_select_pv_modules_save_and_next(self):
        raise SkipTest('Deprecated with PV CYCLE 2.0')

        _modules = self._fill_pv_modules(self.operator)

        # create and approve Subrogation
        subrogation = self._request_for_subrogation('Comm')

        controller = PvcSubrogationController(self.sa_session, account_id=self.ta.id)
        controller.match_commissioner(subrogation.id, self.ctm.id)

        subrogation.status = PvcSubrogation.STATUS_ACTIVE
        self.sa_session.add(subrogation)
        self.sa_session.flush()

        # Subrogator logs in and selects PV Modules
        self.login(self.ma.name, 'test_password')

        self.click_iconbar('icon-pvc-pv-module-select', hover='icon-pvc-pv-module-country-manager')
        self.wait_for_loading() # for selection panel

        forms_data = {
            'pvc-pv-module-form': {
                'supplier': _modules[0].supplier.name,
                'type': _modules[0].type.name,
            },
        }
        self.fill_forms(panel_name='pvc-pv-module-select', forms_data=forms_data)
        self.fill_forms(panel_name='pvc-pv-module-select', forms_data={
            'pvc-pv-module-countries': {
                'subrogation_id': ('select', self.ctm.member.name), # assigns them to the the Commissioner
                'countries': ('fancy-select', [self.ta.default_group.license.country.name], ),
            }
        })
        self.wait_for_loading()

        self.click_button(panel_name='pvc-pv-module-select', text='Save and next')
        self.wait_for_loading()

        forms_data = {
            'pvc-pv-module-form': {
                'supplier': _modules[1].supplier.name,
                'type': _modules[1].type.name,
            },
        }
        self.fill_forms(panel_name='pvc-pv-module-select', forms_data=forms_data)
        self.fill_forms(panel_name='pvc-pv-module-select', forms_data={
            'pvc-pv-module-countries': {
                'subrogation_id': ('select', self.ctm.member.name), # assigns them to the the Commissioner
                'countries': ('fancy-select', [self.ta.default_group.license.country.name], ),
            }
        })
        self.wait_for_loading()

        self.click_button(panel_name='pvc-pv-module-select', text='Save')
        self.wait_for_loading()

    def test_subrogator_select_pv_modules(self):
        raise SkipTest('Deprecated with PV CYCLE 2.0')

        _modules = self._fill_pv_modules(self.operator)

        # create and approve Subrogation
        subrogation = self._request_for_subrogation('Comm')

        controller = PvcSubrogationController(self.sa_session, account_id=self.ta.id)
        controller.match_commissioner(subrogation.id, self.ctm.id)

        subrogation.status = PvcSubrogation.STATUS_ACTIVE
        self.sa_session.add(subrogation)
        self.sa_session.flush()

        # Subrogator logs in and selects PV Modules
        self.login(self.ma.name, 'test_password')

        self.click_iconbar('icon-pvc-pv-module-select', hover='icon-pvc-pv-module-country-manager')
        self.wait_for_loading() # for selection panel

        forms_data = {
            'pvc-pv-module-form': {
                'supplier': _modules[0].supplier.name,
                'type': _modules[0].type.name,
            },
        }
        self.fill_forms(panel_name='pvc-pv-module-select', forms_data=forms_data)
        self.fill_forms(panel_name='pvc-pv-module-select', forms_data={
            'pvc-pv-module-countries': {
                'subrogation_id': ('select', self.ctm.member.name), # assigns them to the the Commissioner
                'countries': ('fancy-select', [self.ta.default_group.license.country.name], ),
            }
        })
        self.wait_for_loading()

        self.click_button(panel_name='pvc-pv-module-select', text='Save')
        self.wait_for_loading()

        # assert the custom query
        self.click_iconbar('icon-pvc-pv-module-country-manager')
        self.wait_for_loading()

        self.filter_cq(panel_name='pvc-pv-module-country-manager', filters=[
            ('Brand', 'Equal', _modules[0].supplier.name),
        ])

        self.assert_cq(panel_name='pvc-pv-module-country-manager', row_index=0, data={
            'commissioner': self.ctm.member.name,
        })

    def test_commissioner_approval_of_pv_module(self):
        raise SkipTest('Deprecated with PV CYCLE 2.0')

        # create and approve Subrogation
        _modules = self._fill_pv_modules(self.operator)

        # create and approve Subrogation
        subrogation = self._request_for_subrogation('Comm')

        controller = PvcSubrogationController(self.sa_session, account_id=self.ta.id)
        controller.match_commissioner(subrogation.id, self.ctm.id)

        subrogation.status = PvcSubrogation.STATUS_ACTIVE
        self.sa_session.add(subrogation)
        self.sa_session.flush()

        # Add PV Modules for Commissioner by Subrogator
        pvc = PvcPVModuleController(self.sa_session, account_id=self.ma.id)
        modules = pvc.save_or_update_module_for_countries(self.tm.member.id, [self.tm.tbs.country.id], _modules[0], subrogation.id)

        # Commissioner logs in
        self.login(self.ca.name, 'test_password')

        # Validate one outstanding approval request for PV Module
        self.click_iconbar('icon-pvc-pv-module-subrogation-manager')

        # assert CQ
        self.assert_cq(panel_name='pvc-subrogated-pv-module-manager', row_index=0, data={
            'subrogator': self.tm.member.name,
            'approval_status': 'Pending'
        })

        self.click_button(panel_name='pvc-subrogated-pv-module-manager', text='Approve')
        self.wait_for_loading() # for the dialog

        self.click_button(panel_name='dialog-question', text='Yes')
        self.wait_for_loading() # for cq

        # assert status.
        self.assert_cq(panel_name='pvc-subrogated-pv-module-manager', row_index=0, data={
            'subrogator': self.tm.member.name,
            'approval_status': 'Approved'
        })

    def test_declaration_subrogation_split(self):
        raise SkipTest('Deprecated with PV CYCLE 2.0')
        # create and approve Subrogation
        _modules = self._fill_pv_modules(self.operator)

        # create and approve Subrogation
        subrogation = self._request_for_subrogation('Comm')

        controller = PvcSubrogationController(self.sa_session, account_id=self.ta.id)
        controller.match_commissioner(subrogation.id, self.ctm.id)

        subrogation.status = PvcSubrogation.STATUS_ACTIVE
        self.sa_session.add(subrogation)
        self.sa_session.flush()

        # Add PV Modules for Subrogator
        pvc = PvcPVModuleController(self.sa_session, account_id=self.ma.id)
        pvc.save_or_update_module_for_countries(self.tm.member.id, [self.tm.tbs.country.id], _modules[0])
        pvc.save_or_update_module_for_countries(self.tm.member.id, [self.tm.tbs.country.id], _modules[1])

        dc = PvcDeclarationOrderController(self.sa_session, account_id=self.ma.id)
        d = dc.add_open_declaration(self.tm, 2013, month=1)

        # Subrogator logs in and selects PV Modules
        self.login(self.ma.name, 'test_password')

        self.click_iconbar('icon-pvc-outstanding-declarations')
        self.wait_for_loading() # for the custom query

        self.filter_cq(panel_name='pvc-outstanding-declarations', filters=[
            ('Year', 'Equal', ('select', '2013')),
            ('Month', 'Equal', ('select', '1')),
        ])
        self.wait_for_loading() # for the custom query

        self.click_button(panel_name='pvc-outstanding-declarations', text='Declare')
        self.wait_for_loading() # for the details panel

        self.fill_form(panel_name='pvc-declaration-details-panel', values={
            'line_0': 400,
            'line_1': 500,
        })

        self.fill_form(panel_name='pvc-declaration-details-panel', values={
            'subrogation_line_1': ('select', 'COMMISSIONER'),
        })
        self.click_button(panel_name='pvc-declaration-details-panel', text='Save')

        self.wait_for_loading(times=2)

        self.close_panel(panel_name='pvc-outstanding-declarations')
        self.click_iconbar('icon-pvc-outstanding-declarations')
        self.wait_for_loading() # for the custom query

        self.filter_cq(panel_name='pvc-outstanding-declarations', filters=[
            ('Year', 'Equal', ('select', '2013')),
            ('Month', 'Equal', ('select', '1')),
        ])

        self.assert_cq(panel_name='pvc-outstanding-declarations', row_index=0, data={
            'action': 'Redeclare',
        })
        self.assert_cq(panel_name='pvc-outstanding-declarations', row_index=1, data={
            'action': 'Redeclare',
            'commissioner': 'COMMISSIONER',
        })

    def test_declaration_subrogation_split_on_resubmit(self):
        raise SkipTest('Deprecated with PV CYCLE 2.0')
        # create and approve Subrogation
        _modules = self._fill_pv_modules(self.operator)

        # create and approve Subrogation
        subrogation = self._request_for_subrogation('Comm')

        controller = PvcSubrogationController(self.sa_session, account_id=self.ta.id)
        controller.match_commissioner(subrogation.id, self.ctm.id)

        subrogation.status = PvcSubrogation.STATUS_ACTIVE
        self.sa_session.add(subrogation)
        self.sa_session.flush()

        # Add PV Modules for Subrogator
        pvc = PvcPVModuleController(self.sa_session, account_id=self.ma.id)
        pvc.save_or_update_module_for_countries(self.tm.member.id, [self.tm.tbs.country.id], _modules[0])
        pvc.save_or_update_module_for_countries(self.tm.member.id, [self.tm.tbs.country.id], _modules[1])

        dc = PvcDeclarationOrderController(self.sa_session, account_id=self.ma.id)
        d = dc.add_open_declaration(self.tm, 2013, month=1)
        d = dc.update_declaration(d.id, {'line_0': 400, 'line_1': 500})
        dc.submit_order(d.id)
        d.resubmit_request = PvcDeclarationOrder.RESUBMIT_REQUEST_APPROVED
        self.sa_session.add(d)
        self.sa_session.flush()

        # Subrogator logs in and selects PV Modules
        self.login(self.ma.name, 'test_password')

        self.click_iconbar('icon-pvc-declaration-manager',hover='icon-pvc-outstanding-declarations')
        self.wait_for_loading() # for the custom query

        self.filter_cq(panel_name='pvc-declaration-manager', filters=[
            ('Year', 'Equal', ('select', '2013')),
            ('Month', 'Equal', ('select', '1')),
        ])
        self.wait_for_loading() # for the custom query

        self.click_button(panel_name='pvc-declaration-manager', text='Resubmit')
        self.wait_for_loading() # for the details panel

        self.fill_form(panel_name='pvc-declaration-details-panel', values={
            'line_0': 400,
            'line_1': 500,
        })

        self.fill_form(panel_name='pvc-declaration-details-panel', values={
            'subrogation_line_1': ('select', 'COMMISSIONER'),
        })
        self.click_button(panel_name='pvc-declaration-details-panel', text='Save')

        self.wait_for_loading(times=2)

        self.close_panel(panel_name='pvc-declaration-manager')
        self.click_iconbar('icon-pvc-outstanding-declarations')
        self.wait_for_loading() # for the custom query

        self.filter_cq(panel_name='pvc-outstanding-declarations', filters=[
            ('Year', 'Equal', ('select', '2013')),
            ('Month', 'Equal', ('select', '1')),
        ])

        self.assert_cq(panel_name='pvc-outstanding-declarations', row_index=0, data={
            'action': 'Redeclare',
        })
        self.assert_cq(panel_name='pvc-outstanding-declarations', row_index=1, data={
            'action': 'Redeclare',
            'commissioner': 'COMMISSIONER',
        })

    def test_declaration_subrogation_add_line(self):
        raise SkipTest('Deprecated with PV CYCLE 2.0')
        # create and approve Subrogation
        _modules = self._fill_pv_modules(self.operator)

        # create and approve Subrogation
        subrogation = self._request_for_subrogation('Comm')

        controller = PvcSubrogationController(self.sa_session, account_id=self.ta.id)
        controller.match_commissioner(subrogation.id, self.ctm.id)

        subrogation.status = PvcSubrogation.STATUS_ACTIVE
        self.sa_session.add(subrogation)
        self.sa_session.flush()

        # Add PV Modules for Subrogator
        pvc = PvcPVModuleController(self.sa_session, account_id=self.ma.id)
        pvc.save_or_update_module_for_countries(self.tm.member.id, [self.tm.tbs.country.id], _modules[0])
        pvc.save_or_update_module_for_countries(self.tm.member.id, [self.tm.tbs.country.id], _modules[1])
        m = pvc.save_or_update_module_for_countries(self.tm.member.id, [self.tm.tbs.country.id], _modules[2], subrogation.id)[0]
        m.commissioner_approval_status = PvcTbsMemberPVModule.COMMISSIONER_APPROVAL_STATUS_APPROVED
        self.sa_session.add(m)
        self.sa_session.flush()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=self.ma.id)
        d = dc.add_open_declaration(self.tm, 2013, month=1)
        self.assertEqual(len(d), 2)

        # add one additional module for subrogation
        rv = pvc.save_or_update_module_for_countries(self.tm.member.id, [self.tm.tbs.country.id], _modules[3], subrogation.id)[0]
        rv.commissioner_approval_status = PvcTbsMemberPVModule.COMMISSIONER_APPROVAL_STATUS_APPROVED

        # add one additional module for producer
        pvc.save_or_update_module_for_countries(self.tm.member.id, [self.tm.tbs.country.id], _modules[2])

        # Subrogator logs in
        self.login(self.ma.name, 'test_password')

        self.click_iconbar('icon-pvc-outstanding-declarations')
        self.wait_for_loading() # for the custom query

        self.filter_cq(panel_name='pvc-outstanding-declarations', filters=[
            ('Year', 'Equal', ('select', '2013')),
            ('Month', 'Equal', ('select', '1')),
            ('Commissioner', 'Equal', 'COMMISSIONER'),
        ])
        self.wait_for_loading() # for the custom query

        # we should have one declaration for the provided COMMISSIONER
        self.click_button(panel_name='pvc-outstanding-declarations', text='Declare')
        self.wait_for_loading() # for the details panel

        self.click_button(panel_name='pvc-declaration-details-panel', text='Add line')
        self.wait_for_loading() # for the pv module select panel

        forms_data = {
            'pvc-pv-module-form': {
                'supplier': _modules[3].supplier.name,
                'type': _modules[3].type.name,
            },
        }
        self.fill_forms(panel_name='pvc-pv-module-select', forms_data=forms_data)
        self.fill_forms(panel_name='pvc-pv-module-select', forms_data={
            'pvc-pv-module-countries': {
                'countries': ('fancy-select', [self.tm.tbs.country.name], ),
            }
        })
        self.wait_for_loading()

        self.click_button(panel_name='pvc-pv-module-select', text='Save')
        self.wait_for_loading()

        self.fill_form(panel_name='pvc-declaration-details-panel', values={
            'line_0': 400,
        })

        tbs_member_module_id = self.sa_session.query(PvcTbsMemberPVModule).filter(
            and_(
                PvcTbsMemberPVModule.tbs_member==self.tm,
                PvcTbsMemberPVModule.subrogation==subrogation,
                PvcTbsMemberPVModule.module==_modules[3],
            )
        ).one().id

        self.fill_form(panel_name='pvc-declaration-details-panel', values={
            'quantity_additional_%s' % tbs_member_module_id: 100,
        })

        self.click_button(panel_name='pvc-declaration-details-panel', text='Save')

        self.wait_for_loading(times=2)

        self.close_panel(panel_name='pvc-outstanding-declarations')
        self.click_iconbar('icon-pvc-outstanding-declarations')
        self.wait_for_loading() # for the custom query

        self.filter_cq(panel_name='pvc-outstanding-declarations', filters=[
            ('Year', 'Equal', ('select', '2013')),
            ('Month', 'Equal', ('select', '1')),
        ])

    def test_declaration_subrogation_split_with_add_line(self):
        raise SkipTest('Deprecated with PV CYCLE 2.0')
        # create and approve Subrogation
        _modules = self._fill_pv_modules(self.operator)

        # create and approve Subrogation
        subrogation = self._request_for_subrogation('Comm')

        controller = PvcSubrogationController(self.sa_session, account_id=self.ta.id)
        controller.match_commissioner(subrogation.id, self.ctm.id)

        subrogation.status = PvcSubrogation.STATUS_ACTIVE
        self.sa_session.add(subrogation)
        self.sa_session.flush()

        # Add PV Modules for Subrogator
        pvc = PvcPVModuleController(self.sa_session, account_id=self.ma.id)
        pvc.save_or_update_module_for_countries(self.tm.member.id, [self.tm.tbs.country.id], _modules[0])
        pvc.save_or_update_module_for_countries(self.tm.member.id, [self.tm.tbs.country.id], _modules[1])

        dc = PvcDeclarationOrderController(self.sa_session, account_id=self.ma.id)
        d = dc.add_open_declaration(self.tm, 2013, month=1)

        # Subrogator logs in and selects PV Modules
        self.login(self.ma.name, 'test_password')

        self.click_iconbar('icon-pvc-outstanding-declarations')
        self.wait_for_loading() # for the custom query

        self.filter_cq(panel_name='pvc-outstanding-declarations', filters=[
            ('Year', 'Equal', ('select', '2013')),
            ('Month', 'Equal', ('select', '1')),
        ])
        self.wait_for_loading() # for the custom query

        self.click_button(panel_name='pvc-outstanding-declarations', text='Declare')
        self.wait_for_loading() # for the details panel

        # add a line
        self.click_button(panel_name='pvc-declaration-details-panel', text='Add line')
        self.wait_for_loading() # for the pv module select panel

        forms_data = {
            'pvc-pv-module-form': {
                'supplier': _modules[3].supplier.name,
                'type': _modules[3].type.name,
            },
        }
        self.fill_forms(panel_name='pvc-pv-module-select', forms_data=forms_data)
        self.fill_forms(panel_name='pvc-pv-module-select', forms_data={
            'pvc-pv-module-countries': {
                'countries': ('fancy-select', [self.tm.tbs.country.name], ),
            }
        })
        self.wait_for_loading()

        self.click_button(panel_name='pvc-pv-module-select', text='Save')
        self.wait_for_loading()

        self.fill_form(panel_name='pvc-declaration-details-panel', values={
            'line_0': 400,
            'line_1': 500,
        })

        tbs_member_module_id = self.sa_session.query(PvcTbsMemberPVModule).filter(
            and_(
                PvcTbsMemberPVModule.tbs_member==self.tm,
                PvcTbsMemberPVModule.module==_modules[3],
            )
        ).one().id

        self.fill_form(panel_name='pvc-declaration-details-panel', values={
            'quantity_additional_%s' % tbs_member_module_id: 100,
        })

        self.fill_form(panel_name='pvc-declaration-details-panel', values={
            'subrogation_line_1': ('select', 'COMMISSIONER'),
        })
        self.click_button(panel_name='pvc-declaration-details-panel', text='Save')

        self.wait_for_loading(times=2)

        self.close_panel(panel_name='pvc-outstanding-declarations')
        self.click_iconbar('icon-pvc-outstanding-declarations')
        self.wait_for_loading() # for the custom query

        self.filter_cq(panel_name='pvc-outstanding-declarations', filters=[
            ('Year', 'Equal', ('select', '2013')),
            ('Month', 'Equal', ('select', '1')),
        ])

        self.assert_cq(panel_name='pvc-outstanding-declarations', row_index=0, data={
            'action': 'Redeclare',
            'total_amount': u'€ 5,561.72', # line of 100 + line of 400
        })
        self.assert_cq(panel_name='pvc-outstanding-declarations', row_index=1, data={
            'action': 'Redeclare',
            'commissioner': 'COMMISSIONER',
            'total_amount': u'€ 5,561.72', # line of 500
        })

    def test_manual_declaration_subrogation(self):
        raise SkipTest('Deprecated with PV CYCLE 2.0')

        set_date(date(2013, 1, 1)) # set date to January 2013

        # create and approve Subrogation
        _modules = self._fill_pv_modules(self.operator)

        # SET THE CREATED DATE OF THIS TBS MEMBER TO MATCH THE VIRTUAL DATE
        self.tm.created = datetime(year=2013, month=1, day=1)
        self.sa_session.add(self.tm)
        self.sa_session.flush()


        # create and approve Subrogation
        subrogation = self._request_for_subrogation('Comm')

        controller = PvcSubrogationController(self.sa_session, account_id=self.ta.id)
        controller.match_commissioner(subrogation.id, self.ctm.id)

        subrogation.status = PvcSubrogation.STATUS_ACTIVE
        self.sa_session.add(subrogation)
        self.sa_session.flush()

        # Add PV Modules for Subrogator
        pvc = PvcPVModuleController(self.sa_session, account_id=self.ma.id)
        pvc.save_or_update_module_for_countries(self.tm.member.id, [self.tm.tbs.country.id], _modules[0])
        pvc.save_or_update_module_for_countries(self.tm.member.id, [self.tm.tbs.country.id], _modules[1])
        m = pvc.save_or_update_module_for_countries(self.tm.member.id, [self.tm.tbs.country.id], _modules[2], subrogation.id)[0]
        m.commissioner_approval_status = PvcTbsMemberPVModule.COMMISSIONER_APPROVAL_STATUS_APPROVED
        self.sa_session.add(m)
        self.sa_session.flush()

        set_date(date(2013, 6, 1))

        # Subrogator logs in and selects PV Modules
        self.login(self.ma.name, 'test_password')

        self.click_iconbar('icon-pvc-declaration-manager',hover='icon-pvc-outstanding-declarations')
        self.wait_for_loading() # for the custom query

        self.click_menu(panel_name='pvc-declaration-manager', text="Add declaration")
        self.wait_for_loading() # for the panel

        self.fill_form(panel_name='declaration-select-panel', form_name='tbs_member', values={
            'tbs_id': ('select', self.tm.tbs.name)
        })
        self.wait_for_loading() # for the period select

        self.fill_form(panel_name='declaration-select-panel', form_name='period_select', values={
            'year': ('select', '2013'),
        })
        # depending on year, the months are filled.
        self.fill_form(panel_name='declaration-select-panel', form_name='period_select', values={
            'month': ('select', 'value=1'),
        })
        self.click_button(panel_name='declaration-select-panel', text='Save')
        self.wait_for_loading() # for the outstanding declaration panel

        self.filter_cq(panel_name='pvc-outstanding-declarations', filters=[
            ('Year', 'Equal', ('select', '2013')),
            ('Month', 'Equal', ('select', '1')),
        ])
        # we should have two outstanding declarations.
        self.wait_for_loading() # for the custom query
        self.assert_cq(panel_name='pvc-outstanding-declarations', row_index=0, data={
            'action': 'Declare',
        })
        self.assert_cq(panel_name='pvc-outstanding-declarations', row_index=1, data={
            'action': 'Declare',
        })

    def test_declaration_producer_mark_as_submitted(self):
        raise SkipTest('Deprecated with PV CYCLE 2.0')
        # create and approve Subrogation
        _modules = self._fill_pv_modules(self.operator)

        # create and approve Subrogation
        subrogation = self._request_for_subrogation('Comm')

        controller = PvcSubrogationController(self.sa_session, account_id=self.ta.id)
        controller.match_commissioner(subrogation.id, self.ctm.id)

        subrogation.status = PvcSubrogation.STATUS_ACTIVE
        self.sa_session.add(subrogation)
        self.sa_session.flush()

        # Add PV Modules for Subrogator
        pvc = PvcPVModuleController(self.sa_session, account_id=self.ma.id)
        pvc.save_or_update_module_for_countries(self.tm.member.id, [self.tm.tbs.country.id], _modules[0])
        pvc.save_or_update_module_for_countries(self.tm.member.id, [self.tm.tbs.country.id], _modules[1])
        m = pvc.save_or_update_module_for_countries(self.tm.member.id, [self.tm.tbs.country.id], _modules[2], subrogation.id)[0]
        m.commissioner_approval_status = PvcTbsMemberPVModule.COMMISSIONER_APPROVAL_STATUS_APPROVED
        self.sa_session.add(m)
        self.sa_session.flush()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=self.ma.id)
        d = dc.add_open_declaration(self.tm, 2013, month=1)
        self.assertEqual(len(d), 2)

        # Subrogator logs in
        self.login(self.ma.name, 'test_password')

        self.click_iconbar('icon-pvc-outstanding-declarations')
        self.wait_for_loading() # for the custom query

        self.filter_cq(panel_name='pvc-outstanding-declarations', filters=[
            ('Year', 'Equal', ('select', '2013')),
            ('Month', 'Equal', ('select', '1')),
            ('Commissioner', 'Equal', 'COMMISSIONER'),
        ])
        self.wait_for_loading() # for the custom query

        # Submit declaration
        # we should have one declaration for the provided COMMISSIONER
        self.click_button(panel_name='pvc-outstanding-declarations', text='Declare')
        self.wait_for_loading() # for the details panel

        self.fill_form(panel_name='pvc-declaration-details-panel', values={
            'line_0': 400,
        })

        self.click_button(panel_name='pvc-declaration-details-panel', text='Submit')
        self.wait_for_loading()

        self.close_panel(panel_name='pvc-outstanding-declarations')

        self.click_iconbar('icon-pvc-declaration-manager',hover='icon-pvc-outstanding-declarations')
        self.wait_for_loading() # for the custom query

        self.filter_cq(panel_name='pvc-declaration-manager', filters=[
            ('Year', 'Equal', ('select', '2013')),
            ('Month', 'Equal', ('select', '1')),
            ('Commissioner', 'Equal', 'COMMISSIONER'),
        ])
        self.wait_for_loading() # for the custom query

        # The Declaration should be marked as submitted and thus a request for Resubmission should be possible
        self.assert_cq(panel_name='pvc-declaration-manager', row_index=0, data={
            'action': 'Request',
        })

    def test_declaration_commissioner_add_payment(self):
        raise SkipTest('Deprecated with PV CYCLE 2.0')
        pc = PaymentController(self.sa_session, account_id=self.operator.id)
        pm1 = pc.save_or_update_payment_method_for_merchant(PaymentMethodBankTransfer, label='bank transfer', merchant=self.tm.tbs.default_group.license)

        # create and approve Subrogation
        _modules = self._fill_pv_modules(self.operator)

        # create and approve Subrogation
        subrogation = self._request_for_subrogation('Comm')

        controller = PvcSubrogationController(self.sa_session, account_id=self.ta.id)
        controller.match_commissioner(subrogation.id, self.ctm.id)

        subrogation.status = PvcSubrogation.STATUS_ACTIVE
        self.sa_session.add(subrogation)
        self.sa_session.flush()

        # Add PV Modules for Subrogator
        pvc = PvcPVModuleController(self.sa_session, account_id=self.ma.id)
        m = pvc.save_or_update_module_for_countries(self.tm.member.id, [self.tm.tbs.country.id], _modules[2], subrogation.id)[0]
        m.commissioner_approval_status = PvcTbsMemberPVModule.COMMISSIONER_APPROVAL_STATUS_APPROVED
        self.sa_session.add(m)
        self.sa_session.flush()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=self.ma.id)
        d = dc.add_open_declaration(self.tm, 2013, month=1)
        d = dc.update_declaration(d.id, {'line_0': 225})
        dc.submit_order(d.id)

        # commissioner log in
        self.login(self.ca.name, 'test_password')
        self.click_iconbar('icon-pvc-declaration-manager',hover='icon-pvc-outstanding-declarations')
        self.wait_for_loading() # for the custom query

        self.filter_cq(panel_name='pvc-declaration-manager', filters=[
            ('Year', 'Equal', ('select', '2013')),
            ('Month', 'Equal', ('select', '1'))
        ])
        self.wait_for_loading() # for the custom query

        self.click_cq(panel_name='pvc-declaration-manager', row_index=0, col_name='details')
        self.wait_for_loading() # for the details panel

        self.click_button(panel_name='pvc-declaration-details-panel', text='Pay')
        self.wait_for_loading()

        # select 'bank transfer' payment method and continue
        self.fill_form('pvc-declaration-details-panel-checkout', values={
            'payment_method': ('radio', pm1.id)
        })
        self.click_button(panel_name='pvc-declaration-details-panel-checkout', text='Submit')
        self.wait_for_loading()

        self.click_button('order-payment_method-panel', text='Mark as submitted')
        self.wait_for_loading()

        ps = d.get_current_payment_status()
        self.assertEqual(ps, OrderPaymentStatus.PAYMENT_SUBMITTED)
