# coding=utf-8
'''
Created on 25 Jul 2013

@author: martijn
'''

import random
import datetime
import xmlrpclib
from xmlrpclib import Fault

from dateutil.relativedelta import relativedelta
from django.conf import settings

from basesoft.util.virtual_date import set_date, today
from pvcycle.core.controllers import PvcMemberController
from pvcycle.product.models import PvcTbsProduct
from pvcycle.test.selenium.pvc_selenium import PvcSeleniumTestCase
from pvcycle.core.models import PvcTbsMember, PvcTbsServiceLevel, PvcTbsLicense
from basesoft.web.core.util.controllers import LicenseConfigController
from decimal import Decimal

class TestCase(PvcSeleniumTestCase):

    def setUp(self):
        super(TestCase, self).setUp()
        self.connect_db()

        # reset vat_no
        q = """
            UPDATE tblCompany set vat_no = CONCAT(vat_no, '_test') WHERE SUBSTRING(vat_no, -5) != '_test';
        """
        self.sa_session.execute(q)

        # reset virtual date
        set_date(datetime.datetime.now().date())

    def test_create_tbs(self):

        # enable the task to disable pv modules.

        rand_id = str(random.random())[2:]

        # login as operator
        self.operator_account = self._create_operator_account()
        self.login(self.operator_account.name, 'test_password')

        # open tbs manager
        self.click_iconbar('icon-pvc-tbs-manager', hover='icon-pvc-accountant-manager')
        self.wait_for_loading() # for custom query

        # data for saving and asserting
        forms_data = {
            'tbs_info_form': {
                'tbs_type': ('radio', '1'), # subsidiary
            },
            'company_form': {
                'name': 'TBS Company Name ' + rand_id,
                'website': 'http://tbs.com',
                'vat_no': 'BE0893027827'
            },
            'bank_account_form': {
                'iban': "IL620108000000099999999",
                'bic': "DEUTDEFF",
                'number': "324234234",
            },
            'address_form': {
                'street': 'TBS Street',
                'city': 'Amsterdam',
                'postcode': '1000AA',
                'country': ('select', 'Netherlands'),
                'state': 'NH'
            },
            'person_form': {
                'first_name': 'TBS First Name',
                'last_name': 'TBS Last Name',
                'email': 'tbs-test@mail-test.basesoft.com',
                'preferred_locale': ('select', 'English'),
                'gender': ('select', 'Female'),
                'phone': '3459764356'
            }
        }

        # create new tbs
        self.click_menu('pvc-tbs-manager', text='Create TBS')
        self.wait_for_loading()

        self.fill_forms(panel_name='pvc-tbs-details', forms_data=forms_data)

        self.click_button(panel_name='pvc-tbs-details', text='Save')
        self.wait_for_loading()

        self.open_fieldset('pvc-tbs-settings', title='TBS declaration settings')

        self.fill_forms('pvc-tbs-settings', forms_data={
            'license_config_form': {
                'pvc.tbs.declaration.period': ('select', 'Monthly'),
            }
        })
        self.click_button(panel_name='pvc-tbs-settings', text='Save')
        self.wait_for_loading()

        # filter the cq to the newly created license
        self.filter_cq(panel_name='pvc-tbs-manager', filters=[
            ('Name', 'Equal', 'TBS Company Name ' + rand_id),
        ])

        # verify collins
        self.assert_cq(panel_name='pvc-tbs-manager', row_index=0, data={
            'license_name': 'TBS Company Name ' + rand_id,
        });

        # open tbs details
        self.click_cq(panel_name='pvc-tbs-manager', row_index=0, col_name='license_name')
        self.wait_for_panel('pvc-tbs-details')
        self.wait_for_loading()

        # assert that values are correctly saved and reloaded
        self.assert_forms(panel_name='pvc-tbs-details', forms_data=forms_data)

        # save again so updating works
        self.click_button(panel_name='pvc-tbs-details', text='Save')
        self.wait_for_loading()

        self.click_cq(panel_name='pvc-tbs-manager', row_index=0, col_name='settings')
        self.wait_for_loading()

        self.open_fieldset('pvc-tbs-settings', title='TBS declaration settings')

        self.assert_forms(panel_name='pvc-tbs-settings', forms_data={
            'license_config_form': {
                'pvc.tbs.declaration.period': ('select', 'Monthly'),
            }
        })
        self.wait_for_loading()

        self.click_button(panel_name='pvc-tbs-settings', text='Save')
        self.wait_for_loading()

    def test_update_license_configs(self):
        """
        Tests the updating of license configs through the Preferences panel (iqportal #3772)
        """
        operator_account = self._create_operator_account()
        tbs_account = self._create_tbs_account(operator_account)
        self.login(tbs_account.name, 'test_password')

        # open the Preferences panel
        self.click_iconbar('icon-settings')
        self.wait_for_loading()

        license = tbs_account.groups[0].group.license

        # open the fieldset for the license we just created.
        self.open_fieldset(panel_name='account-preferences', title='License configs for %s' % license.name)
        self.open_fieldset('account-preferences', title='TBS declaration settings')
        self.open_fieldset('account-preferences', title='TBS registration settings')

        # assert the declaration period to be set to monthly.
        self.assert_forms(panel_name='account-preferences', forms_data={'%s' % license.id: {
            'pvc.tbs.declaration.period': ('select', 'Monthly'),
            'pvc.tbs.membership.valid.for.years': '3',
        }})

        new_data = {
            '%s' % license.id: {
                'pvc.tbs.declaration.period': ('select', 'Yearly'),
                'pvc.tbs.membership.valid.for.years': '200',
                'pvc.tbs.declaration.closing.day': '365' # this should error.
            }
        }
        self.fill_forms(panel_name='account-preferences', forms_data=new_data)

        # close the fieldset for the license to be able to press the save button.
        self.click_fieldset(panel_name='account-preferences', title='License configs for %s' % license.name)

        self.click_button(panel_name='account-preferences', text='Save')
        self.wait_for_loading(assert_no_errors=False)
        self.assert_growl_errors_present()

        # open the fieldset for the license in order to submit new values.
        self.open_fieldset(panel_name='account-preferences', title='License configs for %s' % license.name)

        new_data['%s' % license.id]['pvc.tbs.declaration.closing.day'] = '364'
        self.fill_forms(panel_name='account-preferences', forms_data=new_data)
        self.click_button(panel_name='account-preferences', text='Save')
        self.wait_for_loading()

        self.click_button(panel_name='account-preferences', text='Cancel')

        self.click_iconbar('icon-settings')
        self.wait_for_loading()

        # open the fieldset for the license we just created.
        self.open_fieldset(panel_name='account-preferences', title='License configs for %s' % license.name)
        self.open_fieldset('account-preferences', title='TBS declaration settings')
        self.open_fieldset('account-preferences', title='TBS registration settings')

        # assert the declaration period to be set to yearly.
        self.assert_forms(panel_name='account-preferences', forms_data=new_data)

    def test_tbs_set_fee(self):

        operator_account = self._create_operator_account()
        tbs_account = self._create_tbs_account(operator_account)
        self._attach_permissions_to_group(tbs_account.groups[0].group, 'pvc_tbs_service_level')

        self.login(tbs_account.name, 'test_password')

        self.click_iconbar('icon-pvc-tbs-service-level', hover='icon-pvc-member-manager')
        self.click_menu('pvc-tbs-service-level-manager', text='Create a new Service Level')

        self.fill_form(panel_name='pvc-service-level-details', values={
            'tbs_id': ('select', tbs_account.groups[0].group.license.name),
            'level': 1,
            'name': 'test service level',
            'status': ('select', "value=%s" % PvcTbsServiceLevel.STATUS_DISABLED_FOR_NEW_REGISTRATIONS),
            'contract_annex_text': ('textarea', 'contract text'),
        })

        # admin fee for current year
        self.fill_form(panel_name='pvc-service-level-details', rowcontrol_name='rc_administration_fees', rowcontrol_row_index=0, values={
            'start_year': today().year,
            'fee': 15,
        })

        # admin fee for next year
        self.click_button(panel_name='pvc-service-level-details', text='Add Administration Fee')
        self.fill_form(panel_name='pvc-service-level-details', rowcontrol_name='rc_administration_fees', rowcontrol_row_index=1, values={
            'start_year': (today() + datetime.timedelta(days=365)).year,
            'fee': 16,
        })

        # contrib fee for next month
        next_month = today() + relativedelta(months=1)
        first_next_month = datetime.date(next_month.year, next_month.month, 1)

        self.click_button(panel_name='pvc-service-level-details', text='Save')
        self.wait_for_loading()

        # assert fee active for current year
        self.filter_cq('pvc-tbs-service-level-manager', filters=[])
        self.assert_cq('pvc-tbs-service-level-manager', row_index=0, data={
            'current_administration_fee': '15.00000'
        })

        # advance a year
        set_date(today() + datetime.timedelta(days=365))

        # assert fee next year
        self.click_button('pvc-tbs-service-level-manager', text='Update')
        self.wait_for_loading()
        self.assert_cq('pvc-tbs-service-level-manager', row_index=0, data={
            'current_administration_fee': '16.00000'
        })

    def test_tbs_set_fee_guarantee_forecast(self):

        operator_account = self._create_operator_account()
        tbs_account = self._create_tbs_account(operator_account)
        self._attach_permissions_to_group(tbs_account.groups[0].group, 'pvc_tbs_service_level')

        tbs_account.default_group.license.set_config('pvc.tbs.guarantee.forecast', True)

        self.login(tbs_account.name, 'test_password')

        self.click_iconbar('icon-pvc-tbs-service-level', hover='icon-pvc-member-manager')
        self.click_menu('pvc-tbs-service-level-manager', text='Create a new Service Level')

        self.fill_form(panel_name='pvc-service-level-details', values={
            'tbs_id': ('select', tbs_account.groups[0].group.license.name),
            'level': 1,
            'name': 'test service level',
            'status': ('select', "value=%s" % PvcTbsServiceLevel.STATUS_DISABLED_FOR_NEW_REGISTRATIONS),
            'contract_annex_text': ('textarea', 'contract text'),
        })

        self.wait_for_loading()
        self.fill_form(panel_name='pvc-service-level-details', values={
            'product_type': ('select', 'value=%s' % PvcTbsProduct.TYPE_PV_RELATED),
            'guarantee_forecast_min_value': 1000
        })

        # admin fee for current year
        self.fill_form(panel_name='pvc-service-level-details', rowcontrol_name='rc_administration_fees', rowcontrol_row_index=0, values={
            'start_year': today().year,
            'fee': 15,
        })

        # admin fee for next year
        self.click_button(panel_name='pvc-service-level-details', text='Add Administration Fee')
        self.fill_form(panel_name='pvc-service-level-details', rowcontrol_name='rc_administration_fees', rowcontrol_row_index=1, values={
            'start_year': (today() + datetime.timedelta(days=365)).year,
            'fee': 16,
        })

        # contrib fee for next month
        next_month = today() + relativedelta(months=1)
        first_next_month = datetime.date(next_month.year, next_month.month, 1)

        self.click_button(panel_name='pvc-service-level-details', text='Save')
        self.wait_for_loading()

    def test_tbs_alter_fee(self):
        from datetime import date
        operator = self._create_operator_account()
        self._create_contribution_tax_rates(operator)

        set_date(date(2013, 6, 1)) # set date to June 2013

        # this TBS has it's setting on MONTHLY!
        tbs_account = self._create_tbs_account(operator)

        uname = self._uniquestring('sl')
        service_level = self._create_tbs_service_level(tbs_account, name=uname, admin_fee=Decimal('250.00'), contrib_fee=Decimal('2.75'), year=today().year)

        self.login(operator.name, 'test_password')

        self.click_iconbar('icon-pvc-tbs-service-level', hover='icon-pvc-accountant-manager')
        self.wait_for_loading()

        self.filter_cq(panel_name='pvc-tbs-service-level-manager', filters=[
            ('Service level name', 'Equal', uname),
        ])
        self.wait_for_loading()

        self.click_cq(panel_name='pvc-tbs-service-level-manager', row_index=0, col_name='row_select', extra_css=' input')

        self.click_menu(panel_name='pvc-tbs-service-level-manager', text='Alter fee for service level')
        self.wait_for_loading()

        self.fill_form(panel_name='pvc-tbs-service-level-alteration-panel', values={
            'contribution_fee': '0.05500',
            'administration_fee': '55.00000',
        })
        self.click_button(panel_name='pvc-tbs-service-level-alteration-panel', text='Update fee')

        self.click_button(panel_name='dialog-question', text='Yes')
        self.wait_for_loading()

        self.sa_session.expire(service_level)
        self.assertEqual(service_level.get_current_administration_fee(), Decimal('55.00000'))
        self.assertEqual(service_level.get_current_contribution_fee(), Decimal('0.05500'))

    def test_tbs_set_membership_level_fee(self):
        operator_account = self._create_operator_account()
        tbs_account = self._create_tbs_account(operator_account)

        lcc = LicenseConfigController(self.sa_session, scope=tbs_account.default_group.license)
        lcc.set('pvc.tbs.use.membership.instead.of.service.level', True)

        self._attach_permissions_to_group(tbs_account.groups[0].group, 'pvc_tbs_membership_level')

        self.login(tbs_account.name, 'test_password')

        self.click_iconbar('icon-pvc-tbs-membership-level', hover='icon-pvc-member-manager')
        self.click_menu('pvc-tbs-membership-manager', text='Create a new Membership level')
        self.wait_for_loading()

        self.fill_form(panel_name='pvc-membership-details', values={
            'tbs_id': ('select', tbs_account.groups[0].group.license.name),
            'name': 'test membership level',
        })

        # fee for current year
        self.fill_form(panel_name='pvc-membership-details', rowcontrol_name='rc_membership_fees', rowcontrol_row_index=0, values={
            'start_year': today().year,
            'fee': 15,
        })

        # fee for 3 years from now
        self.click_button(panel_name='pvc-membership-details', text='Add membership fee')
        self.fill_form(panel_name='pvc-membership-details', rowcontrol_name='rc_membership_fees', rowcontrol_row_index=1, values={
            'start_year': today().year+3,
            'fee': 16,
        })

        self.click_button(panel_name='pvc-membership-details', text='Save')
        self.wait_for_loading()

    def test_terms_and_conditions(self):
        operator_account = self._create_operator_account()
        tbs = self._create_tbs_account(operator_account).default_group.license

        self.login(operator_account.name, 'test_password')

        self.click_iconbar('icon-pvc-tbs-manager', hover='icon-pvc-accountant-manager')
        self.wait_for_loading()

        self.filter_cq('pvc-tbs-manager', filters=[
            ('Name', 'Equal', tbs.name),
        ])
        self.wait_for_loading()

        self.click_cq('pvc-tbs-manager', row_index=0, col_name='terms_conditions')
        self.wait_for_loading()

        self.fill_form(panel_name='pvc-tbs-tac-panel', values={
            'terms_and_conditions_text': ('textarea', 'Aap, Noot, Mies', )
        })
        self.click_button(panel_name='pvc-tbs-tac-panel', text='Save and close')
        self.wait_for_loading()

        self.click_cq('pvc-tbs-manager', row_index=0, col_name='terms_conditions')
        self.wait_for_loading()

    def test_tbs_approve_member(self):

        operator_account = self._create_operator_account()
        tbs_account = self._create_tbs_account(operator_account)
        service_level = self._create_tbs_service_level(tbs_account, year=today().year)

        # create a member and let it apply at the tbs
        member_account = self._create_member_account()
        self.sa_session.expire(member_account)
        self.sa_session.flush()
        self._member_apply_at_tbs(member_account, tbs_account, service_level)

        # login as the tbs
        self.login(tbs_account.name, 'test_password')
        self.wait_for_loading()

        # click the member manager
        self.click_iconbar('icon-pvc-member-manager', hover='icon-pvc-tbs-service-level')

        # click the member name
        self.click_cq(panel_name='pvc-member-manager', row_index=0, col_name='member')
        self.wait_for_loading(stable_time=4)

        # update company name and save
        self.fill_form(panel_name='master-data-panel', values={'iban': 'IS140159260076545510730339', 'bic': 'INNDNL2UDDE', 'name': 'company name updated'})
        self.click_button(panel_name='master-data-panel', text='Save')
        self.wait_for_loading()

        # open 'details'
        self.click_cq(panel_name='pvc-member-manager', row_index=0, col_name='tbs_member_id_details')
        self.wait_for_loading()

        # approve member
        self.fill_form(panel_name='pvc-member-details', values={
            'status': ('radio', PvcTbsMember.STATUS_APPROVED)
        })
        self.click_button(panel_name='pvc-member-details', text='Save')
        self.wait_for_loading(times=2)

        # TODO: fill in voting rights

        # assert overview
        self.assert_cq(panel_name='pvc-member-manager', row_index=0, data={
            'status': 'Approved',
        })

    def test_member_periodicity_change(self):
        oa = self._create_operator_account()
        ta = self._create_tbs_account(oa)
        sl = self._create_tbs_service_level(ta, admin_fee=Decimal('249.95'), year=today().year)
        ma = self._create_member_account()

        # apply at TBS
        tm = self._member_apply_at_tbs(ma, ta, sl)

        # activate member to automatically create an order entry for the Administration Fee
        mc = PvcMemberController(self.sa_session, account_id=ta.id)
        mc.save_tbs_member(tm.id, {
            'status': PvcTbsMember.STATUS_APPROVED,
            'is_full_member': False,
            'is_representative': False
        })

        tm.declaration_periodicity = PvcTbsLicense.DECLARATION_PERIOD_QUARTERLY
        tm.declaration_closing_day = 21
        self.sa_session.add(tm)
        self.sa_session.flush()
        self.sa_session.expire(tm)

        self.login(ta.name, 'test_password')

        self.click_iconbar('icon-pvc-member-manager', hover='icon-pvc-tbs-service-level')
        self.wait_for_loading()

        self.click_cq(panel_name='pvc-member-manager', row_index=0, col_name='tbs_member_id_details')
        self.wait_for_loading()

        self.fill_form(panel_name='pvc-member-details', values={
            'declaration_periodicity': ('select', 'value=%d' % PvcTbsLicense.DECLARATION_PERIOD_MONTHLY)
        })

        self.click_button(panel_name='pvc-member-details', text='Save')
        self.wait_for_loading()

        # no declarations yet, periodicity can be changed directly.
        self.assertEqual(tm.declaration_periodicity, PvcTbsLicense.DECLARATION_PERIOD_MONTHLY)

        self.click_cq(panel_name='pvc-member-manager', row_index=0, col_name='tbs_member_id_details')
        self.wait_for_loading()

    def test_upload_of_contract_through_backoffice(self):
        # See ticket #184
        tbs_account = self._create_tbs_account()
        member_account = self._create_member_account()
        service_level = self._create_tbs_service_level(tbs_account)

        tbs = tbs_account.groups[0].group.license
        lcc = LicenseConfigController(self.sa_session, scope=tbs)
        lcc.set('pvc.tbs.was.active.in.2013', True)

        tbs_member = self._member_apply_at_tbs(member_account, tbs_account, service_level, upload_contract=False, was_member_in_2013=True)
        self.assertEqual(len(tbs_member.contracts), 0)

        # login as the tbs
        self.login(tbs_account.name, 'test_password')
        self.wait_for_loading()

        # click the member manager
        self.click_iconbar('icon-pvc-member-manager', hover='icon-pvc-tbs-service-level')
        self.wait_for_loading()

        # open 'details'
        self.click_cq(panel_name='pvc-member-manager', row_index=0, col_name='tbs_member_id_details')
        self.wait_for_loading2()

        self.fill_form(panel_name='pvc-member-details', values={
            'upload_contract': ('upload', 'contract.pdf'),
        })
        self.click_button(panel_name='pvc-member-details', text='Save')
        self.wait_for_loading(times=2)

        self.sa_session.expire(tbs_member)
        self.assertTrue(len(tbs_member.contracts) > 0)

    def test_upload_of_contract_through_backoffice_non_italy(self):
        # See ticket #262
        tbs_account = self._create_tbs_account()
        member_account = self._create_member_account()
        service_level = self._create_tbs_service_level(tbs_account)

        tbs_member = self._member_apply_at_tbs(member_account, tbs_account, service_level, upload_contract=False, was_member_in_2013=True)
        self.assertEqual(len(tbs_member.contracts), 0)

        self.login(tbs_account.name, 'test_password')
        self.wait_for_loading()

        # click the member manager
        self.click_iconbar('icon-pvc-member-manager', hover='icon-pvc-tbs-service-level')
        self.wait_for_loading()

        # open 'details'
        self.click_cq(panel_name='pvc-member-manager', row_index=0, col_name='tbs_member_id_details')
        self.wait_for_loading()

        self.fill_form(panel_name='pvc-member-details', values={
            'upload_contract': ('upload', 'contract.pdf'),
        })
        self.click_button(panel_name='pvc-member-details', text='Save')
        self.wait_for_loading()

        self.sa_session.expire(tbs_member)
        self.assertTrue(len(tbs_member.contracts) > 0)
        self.assertEqual(tbs_member.status, PvcTbsMember.STATUS_PENDING)

    def test_certificate(self):
        """
        pvcycle.test.selenium.test_tbs.TestCase.test_certificate
        """

        operator_account = self._create_operator_account()
        tbs_account = self._create_tbs_account(operator_account)
        self._attach_permissions_to_group(tbs_account.groups[0].group, 'pvc_tbs_service_level')

        self.login(tbs_account.name, 'test_password')

        self.click_iconbar('icon-pvc-tbs-certificate-manager', hover='icon-pvc-member-manager')

        self.click_menu('pvc-tbs-certificate-manager', text='Create a new Certificate')
        self.wait_for_loading()

        self.fill_form('pvc-certificate-details', values={
            'tbs_id': ('select', tbs_account.default_group.license.name),
            'year': today().year,
            'text': ('textarea',
                     u""""
::
= Attestato 2014* =

::

::PV CYCLE Italia S.r.l. attesta che la società::

::
= @member:name =

== @member:vat_no ==

 ::

::è membro di PV CYCLE Italia S.r.l. che gestisce per i propri membri la
raccolta e il riciclaggio a livello nazionale dei moduli fotovoltaici a fine vita.::


::www.pvcycle.org::

[[BR]]

|| Jan Clyncke || || Milano ||
|| Direttore Generale || || @certificate:issue_date ||
|| INSERT SIGNATURE IMAGE HERE || || ||


::,,* La validità del presente attestato può essere confermata esclusivamente sul sito http://extranet.pvcycle.org/ attraverso il numero di verifica @certificate:hash,, ::


::[[Footer(PV CYCLE Italia srl, Piazzale Biancamano 8, 20121 Milano, CF/P.IVA IT 120 185 71005)]]::
                    """)
        })
        self.click_button(panel_name='pvc-certificate-details', text='Save and close')
        self.wait_for_loading()

    def test_approval_of_commissioner_only(self):
        # issue whereby the commissioner only still showed and required CONTRACT + Service Level (#431)
        operator_account = self._create_operator_account()
        tbs_account = self._create_tbs_account(operator_account)

        # create a member and let it apply at the tbs
        member_account = self._create_member_account()
        self.sa_session.expire(member_account)
        self.sa_session.flush()
        self._member_apply_at_tbs(member_account, tbs_account, service_level=None, upload_contract=False, is_producer=False, is_commissioner=True)

        # login as the tbs
        self.login(tbs_account.name, 'test_password')
        self.wait_for_loading()

        # click the member manager
        self.click_iconbar('icon-pvc-member-manager', hover='icon-pvc-tbs-service-level')

        # click the member name
        self.click_cq(panel_name='pvc-member-manager', row_index=0, col_name='member')
        self.wait_for_loading(stable_time=4)

        # update company name, iban and swift/bic and save
        self.fill_form(panel_name='master-data-panel', values={'iban': 'IS140159260076545510730339', 'bic': 'INNDNL2UDDE', 'name': 'company name updated'})
        self.click_button(panel_name='master-data-panel', text='Save')
        self.wait_for_loading()

        # open 'details'
        self.click_cq(panel_name='pvc-member-manager', row_index=0, col_name='tbs_member_id_details')
        self.wait_for_loading()

        # approve member
        self.fill_form(panel_name='pvc-member-details', values={
            'status': ('radio', PvcTbsMember.STATUS_APPROVED)
        })
        self.click_button(panel_name='pvc-member-details', text='Save')
        self.wait_for_loading(times=2)

        # assert overview
        self.assert_cq(panel_name='pvc-member-manager', row_index=0, data={
            'status': 'Approved',
        });

# below is for running the test from eclipse
if __name__ == '__main__':

    from twisted.scripts.trial import run
    import sys
    sys.argv = ['test_tbs.py', 'pvcycle.test.selenium.test_tbs.TestCase.test_create_tbs']
    run()
    sys.exit()



