'''
Created on Nov 4, 2013

@author: marten
'''
from basesoft.web.contrib.notification.models import Notification

import suds

from decimal import Decimal
from datetime import date

from basesoft.util.virtual_date import today, set_date, now
from basesoft.web.core.exceptions import PublicUserException, PermissionDeniedException
from basesoft.web.core.util.controllers import LicenseConfigController
from basesoft.web.contrib.notification.controllers import NotificationController

from pvcycle.test.unit import pvc_unittest
from pvcycle.core.controllers import PvcMemberController, PvcTBSController
from pvcycle.core.exceptions import PvcPVModuleNotApprovedException
from pvcycle.core.models import PvcTbsMember, PvcTbsLicense
from pvcycle.order.controllers import PvcDeclarationOrderController,\
    PvcOrderController
from pvcycle.order.models import PvcDeclarationOrder, PvcDeclarationOrderLine
from pvcycle.pvmodule.models import PvcPVModule, PvcTbsMemberPVModule
from pvcycle.pvmodule.controllers import PvcPVModuleController
from basesoft.web.contrib.order.controllers import PaymentController
from basesoft.web.contrib.order.models import PaymentMethodBankTransfer, OrderPaymentStatus
from pvcycle.subrogation.controllers import PvcSubrogationController
from pvcycle.subrogation.models import PvcSubrogation
from sqlalchemy.sql.expression import and_

from django.conf import settings

class TestCase(pvc_unittest.PvcTestCase):
    """
    pvcycle.test.unit.test_declaration.TestCase
    """

    def setUp(self):
        self.connect_db()

        set_date(date(2013, 1, 1)) # set date to January 2013
        self.reset_roe()

        self.operator = self._create_operator_account()
        self._create_contribution_tax_rates(self.operator)

        set_date(date(2013, 6, 1)) # set date to June 2013

        self._modules = self._fill_pv_modules(self.operator)
        # this TBS has it's setting on MONTHLY!
        self.tbs_account = self._create_tbs_account(self.operator)
        self.accountant = self._create_accountant_account(self.tbs_account.default_group.license, self.operator)

        self._administration_fee = Decimal('250.00')
        self._contribution_fee = Decimal('2.75')

        self.service_level = self._create_tbs_service_level(self.tbs_account, admin_fee=self._administration_fee, contrib_fee=self._contribution_fee, year=today().year)

    def _create_simple_member_for_declaration(self, modules=None):
        ma = self._create_member_account()

        # apply at TBS
        tm = self._member_apply_at_tbs(ma, self.tbs_account, self.service_level)

        # activate member to automatically create an order entry for the Administration Fee
        mc = PvcMemberController(self.sa_session, account_id=self.tbs_account.id)
        mc.save_tbs_member(tm.id, {'status': PvcTbsMember.STATUS_APPROVED, 'is_full_member': False, 'is_representative': False})

        if modules is None:
            modules = [self._modules[0], self._modules[1]]

        # now that the TBSMember is approved we should be able to save pv modules for this tbs member
        c = PvcPVModuleController(self.sa_session, account_id=ma.id)
        for module in modules:
            c.save_or_update_module_for_tbs_member(tm, tm.tbs.country, module, is_active=True)
        return ma, tm

    def _add_member_module(self, ma, tm, module):
        c = PvcPVModuleController(self.sa_session, account_id=ma.id)
        return c.save_or_update_module_for_tbs_member(tm, tm.tbs.country, module, is_active=True)

    def _get_declarations_for_tbs(self, tbs, year, month=None, quarter=None, status=None):
        """
        returns all declaration objects for the given tbs and date params.
        Moved from PvcDeclarationController as it was only used in the test cases.
        """

        where = and_(
            PvcTbsMember.tbs==tbs,
            PvcDeclarationOrder.year==year,
            PvcDeclarationOrder.quarter==quarter,
            PvcDeclarationOrder.month==month
        )

        if status != None:
            where = and_(where, PvcDeclarationOrder.status == status)

        return self.sa_session.query(PvcDeclarationOrder).join(PvcTbsMember, PvcDeclarationOrder.tbs_member_id==PvcTbsMember.id).filter(where).all()

    def test_add_declaration(self):
        ma, tm = self._create_simple_member_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        # we cannot add a declaration per quarter
        self.assertRaises(PublicUserException, dc.add_open_declaration, tm, 2013, None, 1)

        # we cannot add a declaration for June:
        #self.assertRaises(, c.add_open_declaration, tm, 2013, month=6, quarter=None)

        # we should now be able to add a new Declaration for January 2013
        d = dc.add_open_declaration(tm, 2013, month=1)
        self.assertEqual(d.tbs_member, tm)
        self.assertEqual(len(d.lines), 2)

    def test_disable_as_producer(self):
        ma, tm = self._create_simple_member_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        # we cannot add a declaration per quarter
        self.assertRaises(PublicUserException, dc.add_open_declaration, tm, 2013, None, 1)

        tm.is_commissioner = True
        self.sa_session.add(tm)
        self.sa_session.flush()

        # we should now be able to add a new Declaration for January 2013
        d = dc.add_open_declaration(tm, 2013, month=1)

        # it should be impossible to disable this producer
        mc = PvcMemberController(self.sa_session, account_id=self.operator.id)
        self.assertRaises(PublicUserException, mc.disable_producer, tm.id)

    def test_declare(self):
        ma, tm = self._create_simple_member_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)
        self.assertEqual(PvcDeclarationOrder.STATUS_NEW, d.status)

        # we need to supply information for ALL the lines.
        self.assertRaises(PublicUserException, dc.update_declaration, d.id, {'line_0': 200})

        # we cant supply negative values
        self.assertRaises(PublicUserException, dc.update_declaration, d.id, {'line_0': -200, 'line_1': 150})

        d = dc.update_declaration(d.id, {'line_0': 200, 'line_1': 150})
        self.assertEqual(PvcDeclarationOrder.STATUS_PENDING, d.status)

        self.assertEqual(d.lines[0].quantity, 200)
        self.assertEqual(d.lines[1].quantity, 150)

    def test_max_line_quantity(self):
        ma, tm = self._create_simple_member_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)
        self.assertEqual(PvcDeclarationOrder.STATUS_NEW, d.status)

        # we need to supply information for ALL the lines.
        self.assertRaises(PublicUserException, dc.update_declaration, d.id, {'line_0': PvcDeclarationOrderLine.MAX_LINE_QUANTITY+1, 'line_1': 150})

    def test_additional_line(self):
        # This tests additional lines selected by the producer, ticket #156
        ma, tm = self._create_simple_member_for_declaration()

        module = self._add_pv_module(self._uniquestring('pvm'), self.operator)
        self.sa_session.add(module)

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)

        mm = self._add_member_module(ma, tm, module)

        # Add an extra PV Module
        # Add PV Module to Declaration (check duplicates, check correct TBS)
        d = dc.update_declaration(d.id, {'line_0': 200, 'line_1': 150}, {'tbs_member_module_%s' % mm.id: mm.id, 'quantity_additional_%s' % mm.id: 200})

        self.assertEqual(len(d.lines), 3)
        self.assertEqual(d.lines[2].tbs_member_module, mm)
        self.assertEqual(d.lines[2].quantity, 200)

    def test_additional_line_rejected_module(self):
        # This tests additional lines selected by the producer, ticket #156
        ma, tm = self._create_simple_member_for_declaration()

        module = self._add_pv_module(self._uniquestring('pvm'), self.operator)
        self.sa_session.add(module)

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)

        mm = self._add_member_module(ma, tm, module)
        # mark module as reject
        mm.module.approval_status = PvcPVModule.PV_MODULE_STATUS_REJECTED
        self.sa_session.add(mm.module)
        self.sa_session.flush()

        # Add PV Module to Declaration (check duplicates, check correct TBS)
        self.assertRaises(PublicUserException, dc.update_declaration, d.id, {'line_0': 200, 'line_1': 150}, {'tbs_member_module_%s' % mm.id: mm.id, 'quantity_additional_%s' % mm.id: 200})

    def test_additional_line_duplicate_module(self):
        ma, tm = self._create_simple_member_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)

        mm = d.lines[0].tbs_member_module

        self.assertRaises(PublicUserException, dc.update_declaration, d.id, {'line_0': 200, 'line_1': 150}, {'tbs_member_module_%s' % mm.id: mm.id, 'quantity_additional_%s' % mm.id: 200})

    def test_additional_line_in_resubmit(self):
        ma, tm = self._create_simple_member_for_declaration()
        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        # insert new declaration
        d = dc.add_open_declaration(tm, 2013, month=1)
        # declare the declaration
        d = dc.update_declaration(d.id, {'line_0': 200, 'line_1': 150})
        # submit the declaration (for export)
        d = dc.submit_order(d.id)

        module = self._add_pv_module(self._uniquestring('pvm'), self.operator)
        self.sa_session.add(module)

        mm = self._add_member_module(ma, tm, module)

        # set approval status to APPROVED (this is tested in test_resubmit)
        d.resubmit_request = PvcDeclarationOrder.RESUBMIT_REQUEST_APPROVED
        self.sa_session.add(d)
        self.sa_session.flush()

        resubmit = dc.update_declaration(d.id, {'line_0': 200, 'line_1': 150}, {'tbs_member_module_%s' % mm.id: mm.id, 'quantity_additional_%s' % mm.id: 200})
        credit = resubmit.parent_order
        self.assertEqual(len(credit.lines), 2)

        self.assertNotEqual(d.id, resubmit.id)

        self.assertEqual(len(resubmit.lines), 3)
        self.assertEqual(resubmit.lines[2].tbs_member_module, mm)
        self.assertEqual(resubmit.lines[2].quantity, 200)

    def test_export(self):
        ma, tm = self._create_simple_member_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)
        d = dc.update_declaration(d.id, {'line_0': 200, 'line_1': 150})

        # we can still redeclare as the declaration has not been submitted
        d = dc.update_declaration(d.id, {'line_0': 210, 'line_1': 155})

        # assert that the display_date is not set untill its submitted
        self.assertEquals(d.display_date, None)

        dc.submit_order(d.id)

        # now it must be set
        self.assertEquals(d.display_date.year, now().year)

        adc = PvcDeclarationOrderController(self.sa_session, account_id=self.accountant.id)
        states = adc.get_unexported_orders_for_merchant(tm.tbs.id)
        ids = [s['id'] for s in states]
        self.assertTrue(d.id in ids)

    def test_export_including_admin_order(self):
        ma, tm = self._create_simple_member_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)
        d = dc.update_declaration(d.id, {'line_0': 200, 'line_1': 150})

        # we can still redeclare as the declaration has not been submitted
        d = dc.update_declaration(d.id, {'line_0': 210, 'line_1': 155})

        dc.submit_order(d.id)

        adc = PvcDeclarationOrderController(self.sa_session, account_id=self.accountant.id)
        states = adc.get_unexported_orders_for_merchant(tm.tbs.id, True)
        self.assertEqual(len(states), 2)

    def test_export_including_minimum_contribution_fee(self):
        lcc = LicenseConfigController(self.sa_session, scope=self.tbs_account.default_group.license)
        lcc.set('pvc.tbs.minimum.contribution.fee', 150)

        ma, tm = self._create_simple_member_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        dc.add_open_declaration(tm, 2013, month=1)

        pc = PvcDeclarationOrderController(self.sa_session)
        set_date(date(2013, 2, 11))
        pc.close_declarations(tm.tbs)

        adc = PvcDeclarationOrderController(self.sa_session, account_id=self.accountant.id)
        states = adc.get_unexported_orders_for_merchant(tm.tbs.id)
        self.assertEqual(len(states), 1)
        self.assertEqual(len(states[0]['lines']), 3)
        self.assertEqual(states[0]['total_amount_excluding_vat'], Decimal('150.00000'))

    def test_resubmit_request_rejection(self):
        ma, tm = self._create_simple_member_for_declaration()
        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        # insert new declaration
        d = dc.add_open_declaration(tm, 2013, month=1)
        # declare the declaration
        d = dc.update_declaration(d.id, {'line_0': 200, 'line_1': 150})
        # submit the declaration (for export)
        d = dc.submit_order(d.id)
        # make a request for resubmission
        dc.request_for_resubmission(d.id)

        # check that notification is sent and sender is as expected
        notif = self.sa_session.query(Notification).order_by(Notification.id.desc()).first()
        self.assertEqual(notif.sender, tm.member.default_group)

        # get the notification count of the member account
        nc = NotificationController(self.sa_session, account_id=ma.id)
        cnt = nc.get_unread_notifications_count()

        # now the operator approves the resubmit.
        odc = PvcDeclarationOrderController(self.sa_session, account_id=self.operator.id)
        odc.reject_resubmit_request(d.id, "I don't like you.")

        # assert that the member received a new notification.
        self.assertEqual(nc.get_unread_notifications_count(), cnt+1)

        self.sa_session.expire(d)
        self.assertEqual(d.resubmit_request, None)

    def test_resubmit_cancellation(self):
        ma, tm = self._create_simple_member_for_declaration()
        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        # insert new declaration
        d = dc.add_open_declaration(tm, 2013, month=1)
        # declare the declaration
        d = dc.update_declaration(d.id, {'line_0': 200, 'line_1': 150})
        # submit the order
        d = dc.submit_order(d.id)
        # we do a request for resubmission
        dc.request_for_resubmission(d.id)

        self.sa_session.expire(d)
        self.assertEqual(d.resubmit_request, PvcDeclarationOrder.RESUBMIT_REQUEST_PENDING)

        # operator time
        odc = PvcDeclarationOrderController(self.sa_session, account_id=self.operator.id)
        # lets try to cancel the request.
        self.assertRaises(PublicUserException, odc.cancel_resubmit_request, d.id)

        # now the operator approves the resubmit.
        odc.approve_resubmit_request(d.id)

        self.sa_session.expire(d)
        self.assertEqual(d.resubmit_request, PvcDeclarationOrder.RESUBMIT_REQUEST_APPROVED)

        # now the operator cancels the request
        odc.cancel_resubmit_request(d.id)
        self.sa_session.expire(d)
        self.assertEqual(d.resubmit_request, None)

    def test_resubmit(self):
        ma, tm = self._create_simple_member_for_declaration()
        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        # insert new declaration
        d = dc.add_open_declaration(tm, 2013, month=1)
        # declare the declaration
        d = dc.update_declaration(d.id, {'line_0': 200, 'line_1': 150})

        # try to do a request for resubmission, this should fail as the declaration is still outstanding
        self.assertRaises(PublicUserException, dc.request_for_resubmission, d.id)

        # submit the declaration (for export)
        d = dc.submit_order(d.id)

        # try to resubmit the declaration, this should trigger an error
        self.assertRaises(PublicUserException, dc.update_declaration, d.id, {'line_0': 225, 'line_1': 175})

        # count notifications for operator
        nc = NotificationController(self.sa_session, account_id=self.operator.id)
        cnt = nc.get_unread_notifications_count()

        # make a request for resubmission
        dc.request_for_resubmission(d.id)

        # check that notification is sent and sender is as expected
        notif = self.sa_session.query(Notification).order_by(Notification.id.desc()).first()
        self.assertEqual(notif.sender, tm.member.default_group)

        self.sa_session.expire(d)
        self.assertEqual(d.resubmit_request, PvcDeclarationOrder.RESUBMIT_REQUEST_PENDING)

        # assert that the operator received a new notification.
        self.assertEqual(nc.get_unread_notifications_count(), cnt+1)

        # try to request another time, this should raise an exception
        self.assertRaises(PublicUserException, dc.request_for_resubmission, d.id)

        # get the notification count of the member account
        nc = NotificationController(self.sa_session, account_id=ma.id)
        cnt = nc.get_unread_notifications_count()

        # now the operator approves the resubmit.
        odc = PvcDeclarationOrderController(self.sa_session, account_id=self.operator.id)
        odc.approve_resubmit_request(d.id)

        # assert that the member received a new notification.
        self.assertEqual(nc.get_unread_notifications_count(), cnt+1)

        # now that we have an approved resubmit request on a submitted declaration, an update on this declaration will do a resubmit.
        resubmit = dc.update_declaration(d.id, {'line_0': 225, 'line_1': 175})
        self.sa_session.expire(d)
        self.assertEqual(d.resubmit_request, None)
        # the resubmit's parent is the credited order.
        credit = resubmit.parent_order

        self.assertEqual(credit.parent_order, d) # our initial order should be the same as the first submit

        self.assertEqual(credit.lines[0].quantity, -200)
        self.assertEqual(credit.lines[1].quantity, -150)
        self.assertEqual(credit.status, PvcDeclarationOrder.STATUS_PENDING)

        self.assertEqual(resubmit.lines[0].quantity, 225)
        self.assertEqual(resubmit.lines[1].quantity, 175)

        # resubmit and verify status of credit.
        resubmit = dc.submit_order(resubmit.id)

        credit = resubmit.parent_order
        self.assertEqual(credit.status, PvcDeclarationOrder.STATUS_SUBMITTED)

        # we had a problem with self referential #332, verify with actual sql.
        result = self.sa_session.execute("""
            SELECT o.id FROM tblOrder as o
            WHERE o.id = %s
            AND o.parent_order_id = %s
        """ % (credit.id, d.id)).fetchone()
        self.assertEqual(result[0], credit.id)

    def test_resubmit_with_payment(self):
        """
        Resubmit test case for IQPortal Ticket #4024
        """
        ma, tm = self._create_simple_member_for_declaration()

        pc = PaymentController(self.sa_session, account_id=self.operator.id)
        pm1 = pc.save_or_update_payment_method_for_merchant(PaymentMethodBankTransfer, merchant=self.tbs_account.default_group.license, label='bank transfer')

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        # insert new declaration
        d = dc.add_open_declaration(tm, 2013, month=1)
        # declare the declaration
        d = dc.update_declaration(d.id, {'line_0': 200, 'line_1': 150})
        # submit the declaration (for export)
        d = dc.submit_order(d.id)

        # set approval status to APPROVED (this is tested in test_resubmit)
        d.resubmit_request = PvcDeclarationOrder.RESUBMIT_REQUEST_APPROVED
        self.sa_session.add(d)
        self.sa_session.flush()

        # now that we have an submitted declaration, an update on this declaration will do a resubmit.
        resubmit = dc.update_declaration(d.id, {'line_0': 225, 'line_1': 175})
        # the resubmit's parent is the credited order.
        credit = resubmit.parent_order

        pc = PaymentController(self.sa_session, payment_method=pm1, account_id=ma.id)
        pc.submit_payment(resubmit.id, 'marky mark and the funky bunch.')

        c = dc.get(credit.id).pop()
        self.assertEqual(c.status, PvcDeclarationOrder.STATUS_SUBMITTED)

    def test_credit_payment_soap(self):

        ma, tm = self._create_simple_member_for_declaration()

        pc = PaymentController(self.sa_session, account_id=self.operator.id)
        pm1 = pc.save_or_update_payment_method_for_merchant(PaymentMethodBankTransfer, merchant=self.tbs_account.default_group.license, label='bank transfer')

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)
        d = dc.update_declaration(d.id, {'line_0': 200, 'line_1': 150})
        d = dc.submit_order(d.id)

        # set approval status to APPROVED (this is tested in test_resubmit)
        d.resubmit_request = PvcDeclarationOrder.RESUBMIT_REQUEST_APPROVED
        self.sa_session.add(d)
        self.sa_session.flush()

        # now that we have an submitted declaration, an update on this declaration will do a resubmit.
        resubmit = dc.update_declaration(d.id, {'line_0': 225, 'line_1': 175})
        credit = resubmit.parent_order

        # use the soap service to complete the payment
        self._start_django()
        client = suds.client.Client('%spvcycle.core/api/wsdl' % settings.BASE_URL, cache=None)
        client.service.login(self.accountant.name, 'test_password')
        client.service.update_order_payment(credit.id, OrderPaymentStatus.PAYMENT_COMPLETED, '', 'credit paid', now())

        # assert payment status
        self.sa_session.expire(credit)
        self.assertEqual(credit.get_current_payment_status(), OrderPaymentStatus.PAYMENT_COMPLETED)


    def test_resubmit_same_quantities(self):
        # Tests resubmission of exact same quantities (#162)

        ma, tm = self._create_simple_member_for_declaration()
        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        # insert new declaration
        d = dc.add_open_declaration(tm, 2013, month=1)
        # declare the declaration
        d = dc.update_declaration(d.id, {'line_0': 200, 'line_1': 150})
        # submit the declaration (for export)
        d = dc.submit_order(d.id)

        # set approval status to APPROVED (this is tested in test_resubmit)
        d.resubmit_request = PvcDeclarationOrder.RESUBMIT_REQUEST_APPROVED
        self.sa_session.add(d)
        self.sa_session.flush()

        r = dc.update_declaration(d.id, {'line_0': 200, 'line_1': 150})
        self.assertEqual(d.id, r.id)

    def test_declaration_based_on_weight(self):
        ma, tm = self._create_simple_member_for_declaration()

        lcc = LicenseConfigController(self.sa_session, scope=tm.tbs)
        lcc.set('pvc.tbs.declaration.use.weight.for.amount.calculations', True)

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        # insert new declaration
        d = dc.add_open_declaration(tm, 2013, month=1)
        d = dc.update_declaration(d.id, {'line_0': 200, 'line_1': 150})
        self.assertEqual(d.lines[0].line_amount_excluding_vat, self._modules[0].weight*200*Decimal('2.75'))

        self.assertEqual(d.total_amount_excluding_vat, self._modules[0].weight*200*Decimal('2.75')+self._modules[1].weight*150*Decimal('2.75'))

    def test_unapproved_pv_modules_in_declarations(self):
        # Ticket #152
        module = self._add_pv_module(self._uniquestring('pvm'), self.operator)
        module.approval_status = PvcPVModule.PV_MODULE_STATUS_PENDING
        self.sa_session.add(module)

        ma, tm = self._create_simple_member_for_declaration(modules=[module])
        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)
        self.assertRaises(PvcPVModuleNotApprovedException, dc._assert_ready_for_approval, d)

        # test ticket #157 - Notification to Operator.
        nc = NotificationController(self.sa_session, account_id=self.operator.id)
        cnt = nc.get_unread_notifications_count()

        pc = PvcDeclarationOrderController(self.sa_session)
        set_date(date(2013, 2, 11))
        pc.close_declarations(tm.tbs)

        # check that notification is sent and sender is as expected
        notif = self.sa_session.query(Notification).order_by(Notification.id.desc()).first()
        self.assertEqual(notif.sender, tm.tbs.default_group)

        self.assertEqual(cnt+1, nc.get_unread_notifications_count())

    def test_close_declaration_non_compliant(self):
        ma, tm = self._create_simple_member_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)

        pc = PvcDeclarationOrderController(self.sa_session)
        set_date(date(2013, 2, 11))
        pc.close_declarations(tm.tbs)

        self.sa_session.expire(d)
        self.assertEqual(d.is_compliant, PvcDeclarationOrder.IS_NON_COMPLIANT)

    def test_close_declaration_compliant_minimum_contribution_fee(self):
        lcc = LicenseConfigController(self.sa_session, scope=self.tbs_account.default_group.license)
        lcc.set('pvc.tbs.minimum.contribution.fee', 150)

        ma, tm = self._create_simple_member_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)

        d = dc.update_declaration(d.id, {'line_0': 200, 'line_1': 150})
        self.assertEqual(d.status, PvcDeclarationOrder.STATUS_PENDING)

        pc = PvcDeclarationOrderController(self.sa_session)
        set_date(date(2013, 2, 11))
        pc.close_declarations(tm.tbs)

        self.sa_session.expire(d)
        self.assertEqual(len(d.lines), 2)
        self.assertEqual(d.status, PvcDeclarationOrder.STATUS_SUBMITTED)
        self.assertEqual(d.is_compliant, PvcDeclarationOrder.IS_COMPLIANT)

    def test_close_declaration_non_compliant_minimum_contribution_fee(self):
        lcc = LicenseConfigController(self.sa_session, scope=self.tbs_account.default_group.license)
        lcc.set('pvc.tbs.minimum.contribution.fee', 150)

        ma, tm = self._create_simple_member_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)

        pc = PvcDeclarationOrderController(self.sa_session)
        set_date(date(2013, 2, 11))
        pc.close_declarations(tm.tbs)

        self.sa_session.expire(d)
        self.assertEqual(d.is_compliant, PvcDeclarationOrder.IS_NON_COMPLIANT)

        self.assertEqual(len(d.lines), 3)
        self.assertEqual(d.invoiced_amount, 150)

    def test_minimum_contribution_fee_too_many_lines(self):
        lcc = LicenseConfigController(self.sa_session, scope=self.tbs_account.default_group.license)
        lcc.set('pvc.tbs.minimum.contribution.fee', 150)

        ma, tm = self._create_simple_member_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)

        pc = PvcDeclarationOrderController(self.sa_session)
        set_date(date(2013, 2, 11)) # go somewhere in the future on the 11th day for submission
        pc.close_declarations(tm.tbs)
        #check that non_compliant status is correctly assigned (not paid, not approved)
        self.sa_session.expire(d)
        self.assertEqual(d.is_compliant, PvcDeclarationOrder.IS_NON_COMPLIANT)

        self.assertEqual(len(d.lines), 3)
        self.assertEqual(d.invoiced_amount, 150)

        set_date(date(2013, 7, 11)) # go somewhere in the future on the 11th day

        # request for resubmission
        dc.request_for_resubmission(d.id)

        self.sa_session.expire(d)
        self.assertEqual(d.resubmit_request, PvcDeclarationOrder.RESUBMIT_REQUEST_PENDING)

        # now the operator approves the resubmit.
        odc = PvcDeclarationOrderController(self.sa_session, account_id=self.operator.id)
        odc.approve_resubmit_request(d.id)

        # the last line should be blackholed but a new line should be added because new amount is less than min CF.
        resubmit = dc.update_declaration(d.id, {'line_0': 0, 'line_1': 0, 'line_2': 150})
        #submit order to check amount to be paid
        dc.submit_order(resubmit.id)
        #check that there are no new order lines
        self.assertEqual(len(resubmit.lines), 3)
        self.assertEqual(resubmit.invoiced_amount, 150)
        self.assertEqual(resubmit.lines[2].line_amount_excluding_vat, 150)

    def test_minimum_contribution_fee_tbs_with_weight(self):
        lcc = LicenseConfigController(self.sa_session, scope=self.tbs_account.default_group.license)
        lcc.set('pvc.tbs.minimum.contribution.fee', 150)
        lcc.set('pvc.tbs.declaration.use.weight.for.amount.calculations', True)

        ma, tm = self._create_simple_member_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)

        pc = PvcDeclarationOrderController(self.sa_session)
        set_date(date(2013, 2, 11)) # go somewhere in the future on the 11th day for submission
        pc.close_declarations(tm.tbs)
        #check that non_compliant status is correctly assigned (not paid, not approved)
        self.sa_session.expire(d)
        self.assertEqual(d.is_compliant, PvcDeclarationOrder.IS_NON_COMPLIANT)

        self.assertEqual(len(d.lines), 3)
        self.assertEqual(d.invoiced_amount, 150)

        set_date(date(2013, 3, 11)) # go somewhere in the future on the 11th day for submission
        # request for resubmission
        dc.request_for_resubmission(d.id)

        # check that notification is sent and sender is as expected
        notif = self.sa_session.query(Notification).order_by(Notification.id.desc()).first()
        self.assertEqual(notif.sender, tm.member.default_group)

        self.sa_session.expire(d)
        self.assertEqual(d.resubmit_request, PvcDeclarationOrder.RESUBMIT_REQUEST_PENDING)

        # now the operator approves the resubmit.
        odc = PvcDeclarationOrderController(self.sa_session, account_id=self.operator.id)
        odc.approve_resubmit_request(d.id)

        #resubmit with amount less than min CF and check for recalculated min CF
        resubmit = dc.update_declaration(d.id, {'line_0': 0, 'line_1': 0})
        pc.close_declarations(tm.tbs)
        #check that new line has been created due to amount < min fee
        self.assertEqual(len(resubmit.lines), 3)
        self.assertEqual(resubmit.invoiced_amount, 150)
        self.assertEqual(resubmit.lines[2].line_amount_excluding_vat, 150)

        set_date(date(2013, 4, 11))
        # request for resubmission
        dc.request_for_resubmission(resubmit.id)

        self.sa_session.expire(resubmit)
        self.assertEqual(resubmit.resubmit_request, PvcDeclarationOrder.RESUBMIT_REQUEST_PENDING)

        # now the operator approves the resubmit.
        odc = PvcDeclarationOrderController(self.sa_session, account_id=self.operator.id)
        odc.approve_resubmit_request(resubmit.id)

        #resubmit with amount bigger than min CF and check invoiced amount
        resubmitted = dc.update_declaration(resubmit.id, {'line_0': 0, 'line_1': 10})
        #submit order to check amount to be paid
        dc.submit_order(resubmitted.id)
        #check that there are no new order lines
        self.assertEqual(len(resubmitted.lines), 2)
        self.assertEqual(resubmitted.invoiced_amount, 3850)

    def test_minimum_contribution_fee_resubmit_additional_line(self):
        lcc = LicenseConfigController(self.sa_session, scope=self.tbs_account.default_group.license)
        lcc.set('pvc.tbs.minimum.contribution.fee', 150)

        ma, tm = self._create_simple_member_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)

        pc = PvcDeclarationOrderController(self.sa_session)
        set_date(date(2013, 2, 11)) # go somewhere in the future on the 11th day for submission
        pc.close_declarations(tm.tbs)
        #check that non_compliant status is correctly assigned (not paid, not approved)
        self.sa_session.expire(d)
        self.assertEqual(d.is_compliant, PvcDeclarationOrder.IS_NON_COMPLIANT)

        self.assertEqual(len(d.lines), 3)
        self.assertEqual(d.invoiced_amount, 150)

        set_date(date(2013, 7, 11)) # go somewhere in the future

        # request for resubmission
        dc.request_for_resubmission(d.id)

        # check that notification is sent and sender is as expected
        notif = self.sa_session.query(Notification).order_by(Notification.id.desc()).first()
        self.assertEqual(notif.sender, tm.member.default_group)

        self.sa_session.expire(d)
        self.assertEqual(d.resubmit_request, PvcDeclarationOrder.RESUBMIT_REQUEST_PENDING)

        module = self._add_pv_module(self._uniquestring('pvm'), self.operator)
        self.sa_session.add(module)

        mm = self._add_member_module(ma, tm, module)

        #approve the new module, need a second party
        operator1 = self._create_operator_account()
        pvmc = PvcPVModuleController(self.sa_session, account_id=operator1.id)
        pvmc.approve_module(module.id)

        # now the operator approves the resubmit.
        odc = PvcDeclarationOrderController(self.sa_session, account_id=self.operator.id)
        odc.approve_resubmit_request(d.id)

        resubmit = dc.update_declaration(d.id, {'line_0': 0, 'line_1': 10}, additional_lines={'tbs_member_module_%s' % mm.id: mm.id, 'quantity_additional_%s' % mm.id: 0})
        #submit order to check amount to be paid
        dc.submit_order(resubmit.id)
        #check that new line has been created due to amount < min fee
        self.assertEqual(len(resubmit.lines), 4)
        self.assertEqual(resubmit.invoiced_amount, 150)
        self.assertEqual(resubmit.lines[3].line_amount_excluding_vat, 122.5)


    def test_minimum_contribution_fee_zero_resubmit(self):
        lcc = LicenseConfigController(self.sa_session, scope=self.tbs_account.default_group.license)
        lcc.set('pvc.tbs.minimum.contribution.fee', 150)

        ma, tm = self._create_simple_member_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)

        pc = PvcDeclarationOrderController(self.sa_session)
        set_date(date(2013, 2, 11)) # go somewhere in the future on the 11th day for submission
        pc.close_declarations(tm.tbs)
        #check that non_compliant status is correctly assigned (not paid, not approved)
        self.sa_session.expire(d)
        self.assertEqual(d.is_compliant, PvcDeclarationOrder.IS_NON_COMPLIANT)

        self.assertEqual(len(d.lines), 3)
        self.assertEqual(d.invoiced_amount, 150)
        set_date(date(2013, 3, 11)) # go somewhere in the future on the 11th day for submission
        # request for resubmission
        dc.request_for_resubmission(d.id)

        # check that notification is sent and sender is as expected
        notif = self.sa_session.query(Notification).order_by(Notification.id.desc()).first()
        self.assertEqual(notif.sender, tm.member.default_group)

        self.sa_session.expire(d)
        self.assertEqual(d.resubmit_request, PvcDeclarationOrder.RESUBMIT_REQUEST_PENDING)

        # now the operator approves the resubmit.
        odc = PvcDeclarationOrderController(self.sa_session, account_id=self.operator.id)
        odc.approve_resubmit_request(d.id)

        resubmit = dc.update_declaration(d.id, {'line_0': 0, 'line_1': 0})

        self.assertEqual(resubmit.is_compliant, PvcDeclarationOrder.IS_COMPLIANT)
        #submit order to check amount to be paid
        dc.submit_order(resubmit.id)
        self.sa_session.expire(resubmit)
        self.assertEqual(resubmit.is_compliant, PvcDeclarationOrder.IS_COMPLIANT)
        #check that new line has been created due to amount < min fee
        self.assertEqual(len(resubmit.lines), 3)
        self.assertEqual(resubmit.total_amount_excluding_vat, 150)

    def test_minimum_contribution_fee_too_many_lines_with_bigger_declared_amount(self):
        lcc = LicenseConfigController(self.sa_session, scope=self.tbs_account.default_group.license)
        lcc.set('pvc.tbs.minimum.contribution.fee', 150)

        ma, tm = self._create_simple_member_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)

        pc = PvcDeclarationOrderController(self.sa_session)
        set_date(date(2013, 2, 11)) # go somewhere in the future on the 11th day for submission
        pc.close_declarations(tm.tbs)
        #check that non_compliant status is correctly assigned (not paid, not approved)
        self.sa_session.expire(d)
        self.assertEqual(d.is_compliant, PvcDeclarationOrder.IS_NON_COMPLIANT)

        self.assertEqual(len(d.lines), 3)
        self.assertEqual(d.invoiced_amount, 150)

        set_date(date(2013, 7, 11)) # go somewhere in the future

        # request for resubmission
        dc.request_for_resubmission(d.id)

        # check that notification is sent and sender is as expected
        notif = self.sa_session.query(Notification).order_by(Notification.id.desc()).first()
        self.assertEqual(notif.sender, tm.member.default_group)

        # check that notification is sent and sender is as expected
        notif = self.sa_session.query(Notification).order_by(Notification.id.desc()).first()
        self.assertEqual(notif.sender, tm.member.default_group)

        self.sa_session.expire(d)
        self.assertEqual(d.resubmit_request, PvcDeclarationOrder.RESUBMIT_REQUEST_PENDING)

        # now the operator approves the resubmit.
        odc = PvcDeclarationOrderController(self.sa_session, account_id=self.operator.id)
        odc.approve_resubmit_request(d.id)

        # the last line should be blackholed.
        resubmit = dc.update_declaration(d.id, {'line_0': 0, 'line_1': 200, 'line_2': 150})
        #submit order to check amount to be paid
        dc.submit_order(resubmit.id)
        #check that there are no new order lines
        self.assertEqual(len(resubmit.lines), 2)
        self.assertEqual(resubmit.invoiced_amount, 550)


    def test_minimum_contribution_fee_resubmit_additional_line_with_bigger_declared_amount(self):
        lcc = LicenseConfigController(self.sa_session, scope=self.tbs_account.default_group.license)
        lcc.set('pvc.tbs.minimum.contribution.fee', 150)

        ma, tm = self._create_simple_member_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)

        pc = PvcDeclarationOrderController(self.sa_session)
        set_date(date(2013, 2, 11)) # go somewhere in the future on the 11th day for submission
        pc.close_declarations(tm.tbs)
        #check that non_compliant status is correctly assigned (not paid, not approved)
        self.sa_session.expire(d)
        self.assertEqual(d.is_compliant, PvcDeclarationOrder.IS_NON_COMPLIANT)

        self.assertEqual(len(d.lines), 3)
        self.assertEqual(d.invoiced_amount, 150)

        set_date(date(2013, 7, 11))

        # request for resubmission
        dc.request_for_resubmission(d.id)

        # check that notification is sent and sender is as expected
        notif = self.sa_session.query(Notification).order_by(Notification.id.desc()).first()
        self.assertEqual(notif.sender, tm.member.default_group)

        self.sa_session.expire(d)
        self.assertEqual(d.resubmit_request, PvcDeclarationOrder.RESUBMIT_REQUEST_PENDING)

        module = self._add_pv_module(self._uniquestring('pvm'), self.operator)
        self.sa_session.add(module)

        mm = self._add_member_module(ma, tm, module)

        #approve the new module
        operator1 = self._create_operator_account()
        pvmodule = PvcPVModuleController(self.sa_session, account_id=operator1.id)
        pvmodule.approve_module(module.id)

        # now the operator approves the resubmit.
        odc = PvcDeclarationOrderController(self.sa_session, account_id=self.operator.id)
        odc.approve_resubmit_request(d.id)

        resubmit = dc.update_declaration(d.id, {'line_0': 0, 'line_1': 0}, additional_lines={'tbs_member_module_%s' % mm.id: mm.id, 'quantity_additional_%s' % mm.id: 200})
        #submit order to check amount to be paid
        dc.submit_order(resubmit.id)
        #check that there are no new order lines
        self.assertEqual(len(resubmit.lines), 3)
        self.assertEqual(resubmit.invoiced_amount, 550)

    def test_nested_resubmit_of_minimum_contribution_fee(self):
        lcc = LicenseConfigController(self.sa_session, scope=self.tbs_account.default_group.license)
        lcc.set('pvc.tbs.minimum.contribution.fee', 150)

        ma, tm = self._create_simple_member_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)

        pc = PvcDeclarationOrderController(self.sa_session)
        set_date(date(2013, 2, 11)) # go somewhere in the future on the 11th day for submission
        pc.close_declarations(tm.tbs)
        #check that non_compliant status is correctly assigned (not paid, not approved)
        self.sa_session.expire(d)
        self.assertEqual(d.is_compliant, PvcDeclarationOrder.IS_NON_COMPLIANT)

        self.assertEqual(len(d.lines), 3)
        self.assertEqual(d.invoiced_amount, 150)

        set_date(date(2013, 3, 11)) # go somewhere in the future on the 11th day

        # request for resubmission
        dc.request_for_resubmission(d.id)

        # check that notification is sent and sender is as expected
        notif = self.sa_session.query(Notification).order_by(Notification.id.desc()).first()
        self.assertEqual(notif.sender, tm.member.default_group)

        self.sa_session.expire(d)
        self.assertEqual(d.resubmit_request, PvcDeclarationOrder.RESUBMIT_REQUEST_PENDING)

        # now the operator approves the resubmit.
        odc = PvcDeclarationOrderController(self.sa_session, account_id=self.operator.id)
        odc.approve_resubmit_request(d.id)

        resubmit = dc.update_declaration(d.id, {'line_0': 0, 'line_1': 200})
        set_date(date(2013, 5, 11)) # go somewhere in the future on the 11th day
        pc.close_declarations(tm.tbs)

        self.sa_session.expire(resubmit)

        self.assertEqual(len(resubmit.lines), 2)
        self.assertEqual(resubmit.invoiced_amount, 550)

        set_date(date(2013, 7, 11)) # go somewhere in the future on the 11th day

        # request for resubmission
        dc.request_for_resubmission(resubmit.id)

        self.sa_session.expire(resubmit)

        # now the operator approves the resubmit.
        odc.approve_resubmit_request(resubmit.id)

        resubmitted = dc.update_declaration(resubmit.id, {'line_0': 0, 'line_1': 10})
        #submit order to check amount to be paid
        dc.submit_order(resubmitted.id)
        #check that there are no new order lines
        self.assertEqual(len(resubmitted.lines), 3)
        self.assertEqual(resubmitted.invoiced_amount, 150)
        self.assertEqual(resubmitted.lines[2].line_amount_excluding_vat, 122.5)

    def test_close_declaration_after_resubmit_non_compliant(self):
        """
        Issue whereby a resubmit of the same quantity (0) on a non compliant order would trigger the minimum contribution fee.

        This only happens in edge cases whereby the TBS didnt have minimum contrib fee at first, but later enabled it.
        """

        ma, tm = self._create_simple_member_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)

        pc = PvcDeclarationOrderController(self.sa_session)
        set_date(date(2013, 2, 11))
        pc.close_declarations(tm.tbs)

        self.sa_session.expire(d)
        self.assertEqual(d.is_compliant, PvcDeclarationOrder.IS_NON_COMPLIANT)

        self.assertEqual(len(d.lines), 2)
        self.assertEqual(d.invoiced_amount, 0)

        lcc = LicenseConfigController(self.sa_session, scope=self.tbs_account.default_group.license)
        lcc.set('pvc.tbs.minimum.contribution.fee', 150)

        # request for resubmission
        dc.request_for_resubmission(d.id)

        # check that notification is sent and sender is as expected
        notif = self.sa_session.query(Notification).order_by(Notification.id.desc()).first()
        self.assertEqual(notif.sender, tm.member.default_group)

        self.sa_session.expire(d)
        self.assertEqual(d.resubmit_request, PvcDeclarationOrder.RESUBMIT_REQUEST_PENDING)

        # now the operator approves the resubmit.
        odc = PvcDeclarationOrderController(self.sa_session, account_id=self.operator.id)
        odc.approve_resubmit_request(d.id)

        resubmit = dc.update_declaration(d.id, {'line_0': 0, 'line_1': 0})

        self.assertEqual(resubmit.is_compliant, PvcDeclarationOrder.IS_COMPLIANT)
        self.assertEqual(len(resubmit.lines), 2)

        # assert exception is thrown when trying to submit the order (order is submitted twice)
        self.assertRaises(Exception, dc.submit_order, resubmit.id)


    def test_close_declaration_compliant(self):
        ma, tm = self._create_simple_member_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)

        d = dc.update_declaration(d.id, {'line_0': 200, 'line_1': 150})
        self.assertEqual(d.status, PvcDeclarationOrder.STATUS_PENDING)

        pc = PvcDeclarationOrderController(self.sa_session)
        set_date(date(2013, 2, 11))
        pc.close_declarations(tm.tbs)

        self.sa_session.expire(d)
        self.assertEqual(d.status, PvcDeclarationOrder.STATUS_SUBMITTED)
        self.assertEqual(d.is_compliant, PvcDeclarationOrder.IS_COMPLIANT)

    def test_close_declaration_zero_declare_compliant(self):
        ma, tm = self._create_simple_member_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)

        d = dc.update_declaration(d.id, {'line_0': 0, 'line_1': 0})
        self.assertEqual(d.status, PvcDeclarationOrder.STATUS_PENDING)

        pc = PvcDeclarationOrderController(self.sa_session)
        set_date(date(2013, 2, 11))
        pc.close_declarations(tm.tbs)

        self.sa_session.expire(d)
        self.assertEqual(d.status, PvcDeclarationOrder.STATUS_SUBMITTED)
        self.assertEqual(d.is_compliant, PvcDeclarationOrder.IS_COMPLIANT)

    def test_close_declaration_before_specified_date(self):
        ma, tm = self._create_simple_member_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)

        d = dc.update_declaration(d.id, {'line_0': 200, 'line_1': 150})
        self.assertEqual(d.status, PvcDeclarationOrder.STATUS_PENDING)

        # We have to manually update the creation time stamp;
        d.created = date(2013, 2, 1)
        self.sa_session.add(d)
        self.sa_session.flush()

        pc = PvcDeclarationOrderController(self.sa_session, account_id=self.operator.id)
        set_date(date(2013, 2, 12))

        from datetime import datetime
        self.assertRaises(PublicUserException, pc.close_declarations_created_before_specified_date, tm.tbs, datetime.fromordinal(today().toordinal()))

        # go one month further...
        set_date(date(2013, 3, 12))
        dt = date(2013, 2, 12)

        declarations = pc.close_declarations_created_before_specified_date(tm.tbs, datetime.fromordinal(dt.toordinal()))

        self.assertTrue(d in declarations)
        self.assertEqual(len(declarations), 1)

    def test_removal_of_lines_on_reject_of_pv_module(self):
        # Ticket #388
        module = self._add_pv_module(self._uniquestring('pvm'), self.operator)
        module.approval_status = PvcPVModule.PV_MODULE_STATUS_PENDING
        self.sa_session.add(module)

        ma, tm = self._create_simple_member_for_declaration(modules=[module, self._modules[0]])
        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)

        self.assertTrue(module in [l.tbs_member_module.module for l in d.lines])

        pvc = PvcPVModuleController(self.sa_session, account_id=self.operator)
        pvc.reject_module(module.id)

        self.sa_session.expire(d)
        self.assertEqual(len(d.lines), 1)
        self.assertFalse(module in [l.tbs_member_module.module for l in d.lines])

        # now we test that Submitted orders do not have their lines removed
        module = self._add_pv_module(self._uniquestring('pvm'), self.operator)
        module.approval_status = PvcPVModule.PV_MODULE_STATUS_APPROVED
        self.sa_session.add(module)
        ma, tm = self._create_simple_member_for_declaration(modules=[module, self._modules[1]])
        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)
        d = dc.update_declaration(d.id, {'line_0': 200, 'line_1': 150})
        # submit the declaration (for export)
        d = dc.submit_order(d.id)

        pvc = PvcPVModuleController(self.sa_session, account_id=self.operator)
        pvc.reject_module(module.id)

        # rejection should not have affected this declaration.
        self.assertEqual(len(d.lines), 2)
        self.assertTrue(module in [l.tbs_member_module.module for l in d.lines])

    def test_removal_of_lines_on_reject_multi_country(self):

        state = self._get_tbs_license_state()
        state['tbs']['country'] = None
        eu_tbs = self._create_tbs_account(self.operator, state)

        ma = self._create_member_account()

        eu_sl = self._create_tbs_service_level(eu_tbs, admin_fee=self._administration_fee, contrib_fee=self._contribution_fee, year=today().year)
        tm = self._member_apply_at_tbs(ma, eu_tbs, eu_sl)

        mc = PvcMemberController(self.sa_session, account_id=eu_tbs.id)
        mc.save_tbs_member(tm.id, {'status': PvcTbsMember.STATUS_APPROVED, 'is_full_member': False, 'is_representative': False})

        mc = PvcMemberController(self.sa_session, account_id=ma.id)
        _map = mc.get_member_countries(ma.default_group.license.id, True)
        possibilities = [country for _tm, country in _map.values() if _tm == tm]

        module = self._add_pv_module(self._uniquestring('pvm'), self.operator)
        module.approval_status = PvcPVModule.PV_MODULE_STATUS_PENDING
        self.sa_session.add(module)

        # add two modules, one for 3 countries, one for 2 countries
        controller = PvcPVModuleController(self.sa_session, account_id=ma.id)
        controller.save_or_update_module_for_countries(tm.member.id, [possibilities[0].id, possibilities[1].id, possibilities[2].id], module)
        controller.save_or_update_module_for_countries(tm.member.id, [possibilities[0].id, possibilities[1].id], self._modules[1])

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)
        self.assertEqual(len(d.lines), 5)

        pvc = PvcPVModuleController(self.sa_session, account_id=self.operator)
        pvc.reject_module(module.id)

        self.sa_session.expire(d)
        self.assertEqual(len(d.lines), 2)

    def test_schedule_new_declarations_monthly(self):
        """
        pvcycle.test.unit.test_declaration.TestCase.test_schedule_new_declarations_monthly
        """

        pc = PvcDeclarationOrderController(self.sa_session)
        ma, tm = self._create_simple_member_for_declaration()

        # its june, trigger new declrataions
        # and assert new declarations for month may
        pc.trigger_new_declarations(tm.tbs)
        self.assertTrue( len(self._get_declarations_for_tbs(tm.tbs, year=2013, month=5)) > 0 )

        # goto closing day
        set_date(date(2013, 6, 11))
        pc.close_declarations(tm.tbs)
        declarations = self._get_declarations_for_tbs(tm.tbs, year=2013, month=5, status=PvcDeclarationOrder.STATUS_SUBMITTED)
        self.assertTrue(len(declarations) == 1), 'its: %s' % len(declarations)

        # goto jan 2014
        set_date(date(2014, 1, 1))

        # its january, trigger new declrataions
        # and assert new declarations for december 2013
        pc.trigger_new_declarations(tm.tbs)
        self.assertTrue( len(self._get_declarations_for_tbs(tm.tbs, year=2013, month=12)) > 0 )

        # goto closing day
        set_date(date(2014, 1, 11))
        pc.close_declarations(tm.tbs)
        declarations = self._get_declarations_for_tbs(tm.tbs, year=2013, month=12, status=PvcDeclarationOrder.STATUS_SUBMITTED)
        self.assertTrue(len(declarations) == 1)

        # goto feburary and assert next closing date is the 11'th
        set_date(date(2014, 2, 1))
        pc.trigger_new_declarations(tm.tbs)
        self.assertEqual(pc.get_next_closing_date(tm.tbs), date(2014, 2, 11) )


    def test_schedule_new_declarations_quarterly(self):

        # change declaration period to quarterly
        lcc = LicenseConfigController(self.sa_session, scope=self.tbs_account.default_group.license)
        lcc.set('pvc.tbs.declaration.period', PvcTbsLicense.DECLARATION_PERIOD_QUARTERLY )

        pc = PvcDeclarationOrderController(self.sa_session)
        ma, tm = self._create_simple_member_for_declaration()

        # set date to quarter 3, trigger and assert declarations for quarter 2
        set_date(date(2013, 7, 1))
        pc.trigger_new_declarations(tm.tbs)
        self.assertTrue( len(self._get_declarations_for_tbs(tm.tbs, year=2013, quarter=2)) > 0 )

        # goto closing day
        set_date(date(2013, 7, 11))
        pc.close_declarations(tm.tbs)
        declarations = self._get_declarations_for_tbs(tm.tbs, year=2013, quarter=2, status=PvcDeclarationOrder.STATUS_SUBMITTED)
        self.assertTrue(len(declarations) == 1, 'its: %s' % len(declarations))

        # set date to quarter 1 2014, trigger and assert declarations for quarter 4 2013
        set_date(date(2014, 1, 1))
        pc.trigger_new_declarations(tm.tbs)
        self.assertTrue( len(self._get_declarations_for_tbs(tm.tbs, year=2013, quarter=4)) > 0 )

        # goto closing day
        set_date(date(2014, 1, 11))
        pc.close_declarations(tm.tbs)
        declarations = self._get_declarations_for_tbs(tm.tbs, year=2013, quarter=4, status=PvcDeclarationOrder.STATUS_SUBMITTED)
        self.assertTrue(len(declarations) == 1), 'its: %s' % len(declarations)

        # goto feb
        set_date(date(2014, 2, 1))
        self.assertEqual(pc.get_next_closing_date(tm.tbs), date(2014, 4, 11) )


    def test_schedule_new_declarations_yearly(self):

        # change declaration period to yearly
        lcc = LicenseConfigController(self.sa_session, scope=self.tbs_account.default_group.license)
        lcc.set('pvc.tbs.declaration.period', PvcTbsLicense.DECLARATION_PERIOD_YEARLY )

        pc = PvcDeclarationOrderController(self.sa_session)
        ma, tm = self._create_simple_member_for_declaration()

        # set date to jan 2014, trigger declarations for 2013
        set_date(date(2014, 1, 1))
        pc.trigger_new_declarations(tm.tbs)
        self.assertTrue( len(self._get_declarations_for_tbs(tm.tbs, year=2013)) == 1 )

        # goto closing day
        set_date(date(2014, 1, 11))
        pc.close_declarations(tm.tbs)
        declarations = self._get_declarations_for_tbs(tm.tbs, year=2013, status=PvcDeclarationOrder.STATUS_SUBMITTED)
        self.assertTrue(len(declarations) == 1)

        # goto feburary and assert next closing date is in 2015
        set_date(date(2014, 2, 1))
        self.assertEqual(pc.get_next_closing_date(tm.tbs), date(2015, 1, 11) )

    def test_manual_declarations_monthly(self):
        lcc = LicenseConfigController(self.sa_session, scope=self.tbs_account.default_group.license)
        lcc.set('pvc.tbs.declaration.period', PvcTbsLicense.DECLARATION_PERIOD_MONTHLY )

        pc = PvcDeclarationOrderController(self.sa_session)
        ma, tm = self._create_simple_member_for_declaration()
        self._update_creation_date(tm, date(2014, 1, 1))

        set_date(date(2014, 1, 2))
        self.assertEqual(pc.get_undeclared_periods(tm), {})

        set_date(date(2014, 2, 1))
        self.assertEqual(pc.get_undeclared_periods(tm), {})

        set_date(date(2014, 2, 2))
        self.assertEqual(pc.get_undeclared_periods(tm), {2014: {'months': [1]}})

    def test_manual_declarations_quarterly(self):
        lcc = LicenseConfigController(self.sa_session, scope=self.tbs_account.default_group.license)
        lcc.set('pvc.tbs.declaration.period', PvcTbsLicense.DECLARATION_PERIOD_QUARTERLY )

        pc = PvcDeclarationOrderController(self.sa_session)
        ma, tm = self._create_simple_member_for_declaration()
        self._update_creation_date(tm, date(2014, 1, 1))

        set_date(date(2014, 2, 1))
        self.assertEqual(pc.get_undeclared_periods(tm), {})

        set_date(date(2014, 3, 1))
        self.assertEqual(pc.get_undeclared_periods(tm), {})

        set_date(date(2014, 3, 2))
        self.assertEqual(pc.get_undeclared_periods(tm), {})

        set_date(date(2014, 4, 1))
        self.assertEqual(pc.get_undeclared_periods(tm), {})

        set_date(date(2014, 4, 2))
        self.assertEqual(pc.get_undeclared_periods(tm), {2014: {'quarters': [1]}})

    def test_manual_declarations_yearly(self):
        lcc = LicenseConfigController(self.sa_session, scope=self.tbs_account.default_group.license)
        lcc.set('pvc.tbs.declaration.period', PvcTbsLicense.DECLARATION_PERIOD_YEARLY )

        pc = PvcDeclarationOrderController(self.sa_session)
        ma, tm = self._create_simple_member_for_declaration()
        self._update_creation_date(tm, date(2014, 1, 1))

        set_date(date(2014, 2, 1))
        self.assertEqual(pc.get_undeclared_periods(tm), {})

        set_date(date(2014, 2, 2))
        self.assertEqual(pc.get_undeclared_periods(tm), {})

        set_date(date(2015, 1, 1))
        self.assertEqual(pc.get_undeclared_periods(tm), {})

        set_date(date(2015, 1, 2))
        self.assertEqual(pc.get_undeclared_periods(tm), {2014: {}})

    def test_manual_declarations_monthly_with_declared_periods(self):
        lcc = LicenseConfigController(self.sa_session, scope=self.tbs_account.default_group.license)
        lcc.set('pvc.tbs.declaration.period', PvcTbsLicense.DECLARATION_PERIOD_MONTHLY )

        pc = PvcDeclarationOrderController(self.sa_session)
        ma, tm = self._create_simple_member_for_declaration()
        self._update_creation_date(tm, date(2014, 1, 1))

        set_date(date(2014, 7, 1))

        pc.add_open_declaration(tm, 2014, month=4)

        self.assertEqual(pc.get_undeclared_periods(tm), {2014: {'months': [1,2,3,5]}})

        pc.add_open_declaration(tm, 2014, month=1)
        pc.add_open_declaration(tm, 2014, month=2)
        pc.add_open_declaration(tm, 2014, month=3)
        pc.add_open_declaration(tm, 2014, month=5)

        self.assertEqual(pc.get_undeclared_periods(tm), {})

    def test_manual_declarations_quarterly_with_declared_periods(self):
        lcc = LicenseConfigController(self.sa_session, scope=self.tbs_account.default_group.license)
        lcc.set('pvc.tbs.declaration.period', PvcTbsLicense.DECLARATION_PERIOD_QUARTERLY )

        pc = PvcDeclarationOrderController(self.sa_session)
        ma, tm = self._create_simple_member_for_declaration()
        self._update_creation_date(tm, date(2014, 1, 1))

        set_date(date(2015, 1, 1))

        pc.add_open_declaration(tm, 2014, quarter=3)

        self.assertEqual(pc.get_undeclared_periods(tm), {2014: {'quarters': [1,2,4]}})

        pc.add_open_declaration(tm, 2014, quarter=1)
        pc.add_open_declaration(tm, 2014, quarter=2)
        pc.add_open_declaration(tm, 2014, quarter=4)

        self.assertEqual(pc.get_undeclared_periods(tm), {})

    def test_manual_declarations_yearly_with_declared_periods(self):
        lcc = LicenseConfigController(self.sa_session, scope=self.tbs_account.default_group.license)
        lcc.set('pvc.tbs.declaration.period', PvcTbsLicense.DECLARATION_PERIOD_YEARLY )

        pc = PvcDeclarationOrderController(self.sa_session)
        ma, tm = self._create_simple_member_for_declaration()
        self._update_creation_date(tm, date(2014, 1, 1))

        set_date(date(2017, 1, 2))

        pc.add_open_declaration(tm, 2015)

        self.assertEqual(pc.get_undeclared_periods(tm), {2014: {}, 2016: {}})

    def test_remove_line_exception(self):
        # this tests ticket #386
        ma, tm = self._create_simple_member_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)
        # cant remove approved modules
        self.assertRaises(PublicUserException, dc.remove_line, d.id, d.lines[0].id)

    def test_remove_line(self):
        # this tests ticket #386

        # add new unapproved module
        module = self._add_pv_module(self._uniquestring('pvm'), self.operator)
        module.approval_status = PvcPVModule.PV_MODULE_STATUS_PENDING
        self.sa_session.add(module)

        ma, tm = self._create_simple_member_for_declaration(modules=[module, self._modules[0], self._modules[1]])
        # trigger new declaration
        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)
        remove_line = None
        for line in d.lines:
            if line.tbs_member_module.module == module:
                remove_line = line
                break

        # remove declaration line with unapproved module.
        d = dc.remove_line(d.id, remove_line.id)

        # validate.
        self.assertEqual(len(d.lines), 2)

    def test_remove_line_rejected_module(self):
        # if a line gets removed with a rejected module, the tbs member module should be set to OBSOLETE.
        module = self._add_pv_module(self._uniquestring('pvm'), self.operator)
        module.approval_status = PvcPVModule.PV_MODULE_STATUS_REJECTED
        self.sa_session.add(module)

        ma, tm = self._create_simple_member_for_declaration(modules=[module, self._modules[0], self._modules[1]])
        # trigger new declaration
        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)
        remove_line = None
        for line in d.lines:
            if line.tbs_member_module.module == module:
                remove_line = line
                break

        tbs_member_module = remove_line.tbs_member_module
        d = dc.remove_line(d.id, remove_line.id)

        self.sa_session.expire(tbs_member_module)
        self.assertEqual(tbs_member_module.is_active, PvcTbsMemberPVModule.IS_OBSOLETE)

    def test_remove_line_unapproved_module_disable_for_future(self):
        # this tests when the user removes a line which they dont want to have triggered in the future anymore
        module = self._add_pv_module(self._uniquestring('pvm'), self.operator)
        module.approval_status = PvcPVModule.PV_MODULE_STATUS_PENDING
        self.sa_session.add(module)

        ma, tm = self._create_simple_member_for_declaration(modules=[module, self._modules[0], self._modules[1]])
        # trigger new declaration
        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)

        remove_line = None
        for line in d.lines:
            if line.tbs_member_module.module == module:
                remove_line = line
                break

        tbs_member_module = remove_line.tbs_member_module

        # remove declaration line with unapproved module.
        d = dc.remove_line(d.id, remove_line.id, PvcTbsMemberPVModule.IS_INACTIVE)
        self.sa_session.expire(tbs_member_module)
        self.assertEqual(tbs_member_module.is_active, PvcTbsMemberPVModule.IS_INACTIVE)

        d = dc.add_open_declaration(tm, 2013, month=2)
        modules = [line.tbs_member_module.module for line in d.lines]
        self.assertTrue(module not in modules)

    def test_soap_get_previous_exported_contribution_order(self):
        """
        This tests the re-getting of a previously exported Declaration order.
        """
        ma, tm = self._create_simple_member_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d = dc.add_open_declaration(tm, 2013, month=1)
        d = dc.update_declaration(d.id, {'line_0': 200, 'line_1': 150})

        # we can still redeclare as the declaration has not been submitted
        d = dc.update_declaration(d.id, {'line_0': 210, 'line_1': 155})

        # assert that the display_date is not set untill its submitted
        self.assertEquals(d.display_date, None)

        dc.submit_order(d.id)

        # now it must be set
        self.assertEquals(d.display_date.year, now().year)

        adc = PvcDeclarationOrderController(self.sa_session, account_id=self.accountant.id)
        states = adc.get_unexported_orders_for_merchant(tm.tbs.id)
        ids = [s['id'] for s in states]
        self.assertTrue(d.id in ids)

        self.assertEqual(states[0]['properties'][3]['key'], 'producer_id')
        self.assertTrue(states[0]['properties'][3]['value_int'] > 0)


        self._start_django()
        self.client = suds.client.Client('%spvcycle.core/api/wsdl' % settings.BASE_URL, cache=None)
        self.client.service.login(self.accountant.name, 'test_password')

        exported = self.client.service.get_order_by_id(d.id)
        self.assertEqual(exported.id, d.id)
        self.assertEqual(exported.client_id, tm.member.id)

    def test_fee_change_in_the_past(self):
        """
        Due to competitor behavior PV CYCLE requested a change whereby they could lower the Contribution Fee in the past
        For background information please check the related ticket #590
        """
        # create a few Declarations for different members
        set_date(date(2013, 7, 1))

        # 1. create and do nothing for previous period, THIS DECLARATION SHOULD NOT BE AFFECTED.
        ma, tm = self._create_simple_member_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d1 = dc.add_open_declaration(tm, 2013, month=6)

        # move forward a month and close declarations
        pc = PvcDeclarationOrderController(self.sa_session)
        set_date(date(2013, 7, 11))
        pc.close_declarations(tm.tbs)

        # set new fee for service level starting March (so declarations over february)
        sl = tm.active_service_level.service_level

        tc = PvcTBSController(self.sa_session, account_id=self.operator.id)
        state = tc.get_service_level_state(sl.id)
        state['contribution_fees'].append({
            'start_date': date(2013, 8, 1),
            'fee': Decimal('0.015'),
        })
        tc.save_service_level_state(sl.id, state=state)

        set_date(date(2013, 8, 1))

        # 2. create and Declare actual quantities
        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d2 = dc.add_open_declaration(tm, 2013, month=7)
        d2 = dc.update_declaration(d2.id, {'line_0': 100, 'line_1': 200})

        # 3. create and Submit actual quantities
        ma, tm = self._create_simple_member_for_declaration()
        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d3 = dc.add_open_declaration(tm, 2013, month=7)
        d3 = dc.update_declaration(d3.id, {'line_0': 100, 'line_1': 200})
        d3 = dc.submit_order(d3.id)

        # 4. create and Declare 0 quantities
        ma, tm = self._create_simple_member_for_declaration()
        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d4 = dc.add_open_declaration(tm, 2013, month=7)
        d4 = dc.update_declaration(d4.id, {'line_0': 0, 'line_1': 0})

        # 5. create and Submit 0 quantities
        ma, tm = self._create_simple_member_for_declaration()
        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d5 = dc.add_open_declaration(tm, 2013, month=7)
        d5 = dc.update_declaration(d5.id, {'line_0': 0, 'line_1': 0})
        d5 = dc.submit_order(d5.id)

        # 6. create and do nothing
        ma, tm = self._create_simple_member_for_declaration()
        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma.id)
        d6 = dc.add_open_declaration(tm, 2013, month=7)

        # change Contribution Fee for attached service level
        affected = tc.update_service_level_contribution_fee(sl.id, date(2013, 8, 1), Decimal('0.005'))

        # validate starting fee 2.75 for 1
        for l in d1.lines:
            self.assertEqual(l.price_per_unit, Decimal('2.75000'))

        # validate fees 0.015 for 2, 3, 4, 5
        self.sa_session.expire_all()
        for d in [d2, d3, d4, d5]:
            for l in d.lines:
                self.assertEqual(l.price_per_unit, Decimal('0.015'))

        # validate new fee for 6
        for l in d6.lines:
            self.assertEqual(l.price_per_unit, Decimal('0.005'))


    def test_declaration_resigning(self):
        """
        pvcycle.test.unit.test_declaration.TestCase.test_declaration_resigning
        """

        pc = PvcDeclarationOrderController(self.sa_session)
        ma, tm = self._create_simple_member_for_declaration()

        # goto jan 2014
        set_date(date(2014, 1, 1))

        # its january, trigger new declrataions
        # and assert new declarations for december 2013
        pc.trigger_new_declarations(tm.tbs)
        self.assertTrue( len(self._get_declarations_for_tbs(tm.tbs, year=2013, month=12)) > 0 )

        # goto closing day
        set_date(date(2014, 1, 11))
        pc.close_declarations(tm.tbs)
        declarations = self._get_declarations_for_tbs(tm.tbs, year=2013, month=12, status=PvcDeclarationOrder.STATUS_SUBMITTED)
        self.assertTrue(len(declarations) == 1)

        # goto feburary and assert next closing date is the 11'th
        set_date(date(2014, 2, 1))
        pc.trigger_new_declarations(tm.tbs)
        self.assertEqual(pc.get_next_closing_date(tm.tbs), date(2014, 2, 11) )

        # goto closing day
        set_date(date(2014, 2, 11))
        pc.close_declarations(tm.tbs)
        declarations = self._get_declarations_for_tbs(tm.tbs, year=2014, month=1, status=PvcDeclarationOrder.STATUS_SUBMITTED)
        self.assertTrue(len(declarations) == 1)

        # resign the member, he should still get declarations for whole 2014
        mc = PvcMemberController(self.sa_session, account_id=self.operator.id)
        mc.save_tbs_member(tm.id, {'status': PvcTbsMember.STATUS_RESIGNING, 'is_full_member': False, 'is_representative': False})

        # march.. expect to still get declarations
        set_date(date(2014, 3, 1))
        pc.trigger_new_declarations(tm.tbs)
        self.assertTrue( len(self._get_declarations_for_tbs(tm.tbs, year=2014, month=2)) > 0 )

        set_date(date(2014, 3, 11))
        pc.close_declarations(tm.tbs)
        declarations = self._get_declarations_for_tbs(tm.tbs, year=2014, month=2, status=PvcDeclarationOrder.STATUS_SUBMITTED)
        self.assertTrue(len(declarations) == 1)

        # goto jan 2015 and trigger yearly tasks
        set_date(date(2015, 1, 1))
        pc.trigger_new_declarations(tm.tbs)
        self.assertTrue( len(self._get_declarations_for_tbs(tm.tbs, year=2014, month=2)) > 0 )

        # close declarations for dec 2014
        set_date(date(2015, 1, 11))
        pc.close_declarations(tm.tbs)
        declarations = self._get_declarations_for_tbs(tm.tbs, year=2014, month=12, status=PvcDeclarationOrder.STATUS_SUBMITTED)
        self.assertTrue(len(declarations) == 1)

        # trigger yearly AF orders for current tbs
        # this also sets the member status to 'resigned'
        set_date(date(2015, 1, 12))
        PvcOrderController(self.sa_session).trigger_yearly_administrative_fees(tm.tbs)

        # goto feb 2015
        set_date(date(2015, 2, 1))
        pc.trigger_new_declarations(tm.tbs)

        # expect no declarations have been triggered for jan
        pc.trigger_new_declarations(tm.tbs)
        self.assertEqual(len(self._get_declarations_for_tbs(tm.tbs, year=2015, month=1)), 0)



class ManualDeclarationTestCase(pvc_unittest.PvcTestCase):
    def setUp(self):
        self.connect_db()

        set_date(date(2013, 1, 1)) # set date to January 2013

    def test_monthly(self):
        pc = PvcDeclarationOrderController(self.sa_session)
        self.assertEqual(pc._get_possible_periods_for_monthly_declarations(2013), None)

        set_date(date(2013, 2, 1)) # set date to February 2013
        self.assertEqual(pc._get_possible_periods_for_monthly_declarations(2013), None)

        set_date(date(2013, 2, 2)) # set date to 2nd February 2013
        self.assertEqual(pc._get_possible_periods_for_monthly_declarations(2013), {2013: {'months': [1]}})

        set_date(date(2014, 2, 2)) # set date to 2nd February 2014
        self.assertEqual(pc._get_possible_periods_for_monthly_declarations(2013), {2013: {'months': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12]}, 2014: {'months': [1]}})

    def test_quarterly(self):
        pc = PvcDeclarationOrderController(self.sa_session)
        self.assertEqual(pc._get_possible_periods_for_quarterly_declarations(2013), None)

        set_date(date(2013, 4, 1)) # set date to February 2013
        self.assertEqual(pc._get_possible_periods_for_quarterly_declarations(2013), None)

        set_date(date(2013, 4, 2)) # set date to 2nd February 2013
        self.assertEqual(pc._get_possible_periods_for_quarterly_declarations(2013), {2013: {'quarters': [1]}})

        set_date(date(2014, 1, 2)) # set date to 2nd February 2014
        self.assertEqual(pc._get_possible_periods_for_quarterly_declarations(2013), {2013: {'quarters': [1, 2, 3, 4]}})

    def test_yearly(self):
        pc = PvcDeclarationOrderController(self.sa_session)
        self.assertEqual(pc._get_possible_periods_for_yearly_declarations(2013), None)

        set_date(date(2014, 1, 1)) # set date to February 2013
        self.assertEqual(pc._get_possible_periods_for_yearly_declarations(2013), None)

        set_date(date(2014, 1, 2)) # set date to 2nd February 2014
        self.assertEqual(pc._get_possible_periods_for_yearly_declarations(2013), {2013: {}})

class SubrogationDeclarationTestCase(pvc_unittest.PvcTestCase):
    def setUp(self):
        self.connect_db()

        set_date(date(2013, 1, 1)) # set date to January 2013

        self.operator = self._create_operator_account()
        self._create_contribution_tax_rates(self.operator)

        set_date(date(2013, 6, 1)) # set date to June 2013

        self._modules = self._fill_pv_modules(self.operator)
        # this TBS has it's setting on MONTHLY!
        self.ta = self._create_tbs_account(self.operator)
        self.accountant = self._create_accountant_account(self.ta.default_group.license, self.operator)

        self._administration_fee = Decimal('250.00')
        self._contribution_fee = Decimal('2.75')

        self.service_level = self._create_tbs_service_level(self.ta, admin_fee=self._administration_fee, contrib_fee=self._contribution_fee, year=today().year)

        self.ma = self._create_member_account('SUBROGATOR')
        self.tm = self._member_apply_at_tbs(self.ma, self.ta, self.service_level)
        self.tm.status = self.tm.STATUS_APPROVED
        self.sa_session.add(self.tm)

        self.ca = self._create_member_account('COMMISSIONER')
        self.ctm = self._member_apply_at_tbs(self.ca, self.ta, self.service_level, is_producer=False, is_commissioner=True)
        self.ctm.status = self.tm.STATUS_APPROVED
        self.sa_session.add(self.ctm)
        self.sa_session.flush()

    def _add_member_module(self, ma, tm, module):
        c = PvcPVModuleController(self.sa_session, account_id=ma.id)
        return c.save_or_update_module_for_tbs_member(tm, tm.tbs.country, module, is_active=True)

    def _create_subrogation(self):
        controller = PvcSubrogationController(self.sa_session, account_id=self.ma.id)
        subrogation = controller.request_for_subrogation(self.tm, 'commissioner_name')
        # Set subrogation status active.
        subrogation.status = PvcSubrogation.STATUS_ACTIVE
        subrogation.commissioner = self.ctm
        self.sa_session.add(subrogation)
        self.sa_session.flush()
        return subrogation

    def _prepare_for_declaration(self, modules=None, subrogation=None, approve=True):

        if modules is None:
            modules = [self._modules[0], self._modules[1]]

        rv = []
        # now that the TBSMember is approved we should be able to save pv modules for this tbs member
        c = PvcPVModuleController(self.sa_session, account_id=self.ma.id)
        for module in modules:
            m = c.save_or_update_module_for_tbs_member(self.tm, self.tm.tbs.country, module, is_active=True, subrogation_id=subrogation)
            rv.append(m)
            if subrogation is not None and approve:
                m.commissioner_approval_status = PvcTbsMemberPVModule.COMMISSIONER_APPROVAL_STATUS_APPROVED
                self.sa_session.add(m)
        self.sa_session.flush()
        return rv

    def test_subrogate_complete_order(self):
        subrogation = self._create_subrogation()
        self.sa_session.expire(self.tm)

        self._prepare_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=self.ma.id)
        d = dc.add_open_declaration(self.tm, 2013, month=1)
        self.assertEqual(PvcDeclarationOrder.STATUS_NEW, d.status)

        declarations = dc.update_declaration(d.id, {'line_0': 225, 'line_1': 175}, subrogate={'line_0': subrogation.id, 'line_1': subrogation.id})
        self.assertEqual(len(declarations), 1)

    def test_split_order(self):
        subrogation = self._create_subrogation()
        self.sa_session.expire(self.tm)

        self._prepare_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=self.ma.id)
        d = dc.add_open_declaration(self.tm, 2013, month=1)
        self.assertEqual(PvcDeclarationOrder.STATUS_NEW, d.status)

        declarations = dc.update_declaration(d.id, {'line_0': 225, 'line_1': 175}, subrogate={'line_1': subrogation.id})

        subrogated = declarations[0]
        original = declarations[1]

        # both should have status pending
        self.assertEqual(subrogated.status, PvcDeclarationOrder.STATUS_PENDING)
        self.assertEqual(original.status, PvcDeclarationOrder.STATUS_PENDING)

        # The producer should NOT have privileges to PAYe for the new Declaration.
        self.assertRaises(PermissionDeniedException, dc.assert_privileges_on_object, subrogated, PvcDeclarationOrder.PRIV_PAY)

        # The producer SHOULD have privileges to PAY for the original Declaration.
        self.assertTrue(dc.has_privileges_on_object(original, PvcDeclarationOrder.PRIV_PAY))

        # assert that the original order only has one line left
        self.assertEqual(len(original.lines), 1)

        # Make sure we cannot subrogate an already subrogated Declaration
        self.assertRaises(PublicUserException, dc.update_declaration, subrogated.id, {'line_0': 13}, subrogate={'line_0': subrogation.id})

        # now check if we have pay privs as COMMISSIONER
        dc = PvcDeclarationOrderController(self.sa_session, account_id=self.ca.id)
        self.assertTrue(dc.has_privileges_on_object(subrogated, PvcDeclarationOrder.PRIV_PAY))

        # Also make sure we have no UPDATE privs on the order (that is still the PRODUCERS task)
        self.assertRaises(PermissionDeniedException, dc.assert_privileges_on_object, subrogated, PvcDeclarationOrder.PRIV_UPDATE)

    def test_split_order_on_resubmit(self):
        subrogation = self._create_subrogation()
        self.sa_session.expire(self.tm)

        self._prepare_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=self.ma.id)
        d = dc.add_open_declaration(self.tm, 2013, month=1)
        self.assertEqual(PvcDeclarationOrder.STATUS_NEW, d.status)

        d = dc.update_declaration(d.id, {'line_0': 225, 'line_1': 175})
        dc.submit_order(d.id)

        d.resubmit_request = PvcDeclarationOrder.RESUBMIT_REQUEST_APPROVED
        self.sa_session.add(d)

        d = dc.update_declaration(d.id, {'line_0': 1400, 'line_1': 1400}, subrogate={'line_0': subrogation.id})

        self.assertEqual(len(d), 2)
        subrogated, produced = d
        self.assertEqual(len(subrogated.lines), 1)
        self.assertEqual(len(produced.lines), 1)

    def test_split_order_on_resubmit_same_quantities(self):
        subrogation = self._create_subrogation()
        self.sa_session.expire(self.tm)

        self._prepare_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=self.ma.id)
        d = dc.add_open_declaration(self.tm, 2013, month=1)
        self.assertEqual(PvcDeclarationOrder.STATUS_NEW, d.status)

        d = dc.update_declaration(d.id, {'line_0': 225, 'line_1': 175})
        dc.submit_order(d.id)

        d.resubmit_request = PvcDeclarationOrder.RESUBMIT_REQUEST_APPROVED
        self.sa_session.add(d)

        d = dc.update_declaration(d.id, {'line_0': 225, 'line_1': 175}, subrogate={'line_0': subrogation.id})

        self.assertEqual(len(d), 2)
        subrogated, produced = d
        self.assertEqual(len(subrogated.lines), 1)
        self.assertEqual(len(produced.lines), 1)

    def test_split_order_with_new_line(self):
        subrogation = self._create_subrogation()
        self.sa_session.expire(self.tm)

        self._prepare_for_declaration()

        dc = PvcDeclarationOrderController(self.sa_session, account_id=self.ma.id)
        d = dc.add_open_declaration(self.tm, 2013, month=1)
        self.assertEqual(PvcDeclarationOrder.STATUS_NEW, d.status)

        module = self._add_pv_module(self._uniquestring('pvm'), self.operator)
        self.sa_session.add(module)

        mm = self._add_member_module(self.ma, self.tm, module)

        # Add an extra PV Module
        # Add PV Module to Declaration (check duplicates, check correct TBS)

        declarations = dc.update_declaration(d.id, {'line_0': 225, 'line_1': 175}, additional_lines={'tbs_member_module_%s' % mm.id: mm.id, 'quantity_additional_%s' % mm.id: 200}, subrogate={'line_1': subrogation.id})

        original = declarations[1]
        self.assertEqual(len(original.lines), 2)

    def test_split_order_already_exist_for_period(self):
        subrogation = self._create_subrogation()
        self.sa_session.expire(self.tm)

        self._prepare_for_declaration()
        # add some pv modules for subrogation
        self._prepare_for_declaration(modules=[self._modules[2], self._modules[3]], subrogation=subrogation)

        dc = PvcDeclarationOrderController(self.sa_session, account_id=self.ma.id)
        # trigger new declarations
        declarations = dc.add_open_declaration(self.tm, 2013, month=1)

        # we should have two declarations
        self.assertEqual(len(declarations), 2)

        # we cannot subrogate this period declaration to this subrogation, as there already is a declaration for that combination.
        self.assertRaises(PublicUserException, dc.update_declaration, declarations[0].id, {'line_0': 225, 'line_1': 175}, subrogate={'line_1': subrogation.id})

        self.assertEqual(len(dc.get_available_subrogations_for_order(declarations[0].id)), 0)

    def test_new_declarations(self):
        subrogation = self._create_subrogation()
        self.sa_session.expire(self.tm)

        producer_modules = self._prepare_for_declaration()
        # add some pv modules for subrogation
        subrogated_modules = self._prepare_for_declaration(modules=[self._modules[2], self._modules[3]], subrogation=subrogation)

        dc = PvcDeclarationOrderController(self.sa_session, account_id=self.ma.id)
        # trigger new declarations
        declarations = dc.add_open_declaration(self.tm, 2013, month=1)

        # we should have two declarations
        self.assertEqual(len(declarations), 2)

        subrogated = declarations[1]
        producer = declarations[0]

        if declarations[0].producer != declarations[0].tbs_member:
            subrogated = declarations[0]
            producer = declarations[1]

        # have the Producer declare the own declaration.
        d = dc.update_declaration(producer.id, {'line_0': 225, 'line_1': 175})
        self.assertEqual(len(d.lines), 2)
        self.assertTrue(d.lines[0].tbs_member_module in producer_modules)

        # now have them declare the Subrogated declaration.
        d = dc.update_declaration(subrogated.id, {'line_0': 111, 'line_1': 222})
        self.assertEqual(len(d.lines), 2)
        self.assertTrue(d.lines[0].tbs_member_module in subrogated_modules)

    def test_ticket_692(self):
        subrogation = self._create_subrogation()
        modules = [self._modules[0], self._modules[1], self._modules[2]]
        self._prepare_for_declaration(modules=modules)

        dc = PvcDeclarationOrderController(self.sa_session, account_id=self.ma.id)
        # trigger new declarations
        declaration = dc.add_open_declaration(self.tm, 2013, month=1)

        declarations = dc.update_declaration(declaration.id, {'line_0': 225, 'line_1': 175, 'line_2': 180}, subrogate={'line_1': subrogation.id})

        # we should have two declarations
        self.assertEqual(len(declarations), 2)

        producer = declarations[0]
        subrogated = declarations[1]

        if declarations[0].producer != declarations[0].tbs_member:
            producer = declarations[1]
            subrogated = declarations[0]

        self.assertEqual(len(producer.lines), 2)
        self.assertEqual(producer.lines[0].quantity, 225)
        self.assertEqual(producer.lines[1].quantity, 180)

        self.assertEqual(len(subrogated.lines), 1)
        self.assertEqual(subrogated.lines[0].quantity, 175)

    def test_submit_declaration_by_producer(self):
        subrogation = self._create_subrogation()
        self.sa_session.expire(self.tm)

        producer_modules = self._prepare_for_declaration()
        # add some pv modules for subrogation
        subrogated_modules = self._prepare_for_declaration(modules=[self._modules[2], self._modules[3]], subrogation=subrogation)

        dc = PvcDeclarationOrderController(self.sa_session, account_id=self.ma.id)
        # trigger new declarations
        declarations = dc.add_open_declaration(self.tm, 2013, month=1)

        # we should have two declarations
        self.assertEqual(len(declarations), 2)

        producer = declarations[0]

        if declarations[0].producer != declarations[0].tbs_member:
            producer = declarations[1]

        # have the Producer declare the own declaration.
        d = dc.update_declaration(producer.id, {'line_0': 225, 'line_1': 175})
        d = dc.submit_order(d.id)
        self.assertEqual(d.status, PvcDeclarationOrder.STATUS_SUBMITTED)

    def test_request_for_resubmission_failure_by_commissioner(self):
        subrogation = self._create_subrogation()
        self.sa_session.expire(self.tm)

        producer_modules = self._prepare_for_declaration()
        # add some pv modules for subrogation
        subrogated_modules = self._prepare_for_declaration(modules=[self._modules[2], self._modules[3]], subrogation=subrogation)

        dc = PvcDeclarationOrderController(self.sa_session, account_id=self.ma.id)
        # trigger new declarations
        declarations = dc.add_open_declaration(self.tm, 2013, month=1)

        # we should have two declarations
        self.assertEqual(len(declarations), 2)

        subrogated = declarations[0]

        if declarations[0].producer == declarations[0].tbs_member:
            subrogated = declarations[1]

        # have the Producer declare the own declaration.
        d = dc.update_declaration(subrogated.id, {'line_0': 225, 'line_1': 175})
        d = dc.submit_order(d.id)

        dc = PvcDeclarationOrderController(self.sa_session, account_id=self.ca.id)
        self.assertRaises(PublicUserException, dc.request_for_resubmission, d.id)

    def test_pay_declaration_by_commissioner(self):

        pc = PaymentController(self.sa_session, account_id=self.operator.id)
        pm1 = pc.save_or_update_payment_method_for_merchant(PaymentMethodBankTransfer, label='bank transfer', merchant=self.tm.tbs.default_group.license)

        subrogation = self._create_subrogation()
        self.sa_session.expire(self.tm)

        producer_modules = self._prepare_for_declaration()
        # add some pv modules for subrogation
        subrogated_modules = self._prepare_for_declaration(modules=[self._modules[2], self._modules[3]], subrogation=subrogation)

        dc = PvcDeclarationOrderController(self.sa_session, account_id=self.ma.id)
        # trigger new declarations
        declarations = dc.add_open_declaration(self.tm, 2013, month=1)

        # we should have two declarations
        self.assertEqual(len(declarations), 2)

        subrogated = declarations[1]
        producer = declarations[0]

        if declarations[0].producer != declarations[0].tbs_member:
            subrogated = declarations[0]
            producer = declarations[1]

        # have the Producer declare the own declaration.
        d = dc.update_declaration(producer.id, {'line_0': 225, 'line_1': 175})
        accountant_pc = PaymentController(self.sa_session, account_id=self.ca.id, payment_method=pm1)
        import time
        time.sleep(1)
        self.assertRaises(PublicUserException, accountant_pc.submit_payment, d.id, "commissioner paid")

        d = dc.update_declaration(subrogated.id, {'line_0': 111, 'line_1': 222})
        accountant_pc.submit_payment(d.id, "commissioner paid")

        ps = d.get_current_payment_status()
        self.assertEqual(ps, OrderPaymentStatus.PAYMENT_SUBMITTED)

    def test_close_declaration_unapproved_pv_module(self):
        subrogation = self._create_subrogation()
        self.sa_session.expire(self.tm)

        producer_modules = self._prepare_for_declaration()
        # add some pv modules for subrogation
        subrogated_modules = self._prepare_for_declaration(modules=[self._modules[2], self._modules[3]], subrogation=subrogation)

        dc = PvcDeclarationOrderController(self.sa_session, account_id=self.ma.id)
        # trigger new declarations
        declarations = dc.add_open_declaration(self.tm, 2013, month=1)

        subrogated = declarations[1]
        producer = declarations[0]

        if declarations[0].producer != declarations[0].tbs_member:
            subrogated = declarations[0]
            producer = declarations[1]

        subrogated_modules[0].commissioner_approval_status = PvcTbsMemberPVModule.COMMISSIONER_APPROVAL_STATUS_PENDING
        self.sa_session.add(subrogated_modules[0])

        self.assertRaises(PvcPVModuleNotApprovedException, dc._assert_ready_for_approval, subrogated)

        # test ticket #157 - Notification to Operator.
        nc = NotificationController(self.sa_session, account_id=self.operator.id)
        cnt = nc.get_unread_notifications_count()

        pc = PvcDeclarationOrderController(self.sa_session)
        set_date(date(2013, 2, 11))
        pc.close_declarations(self.tm.tbs)

        self.assertEqual(cnt+1, nc.get_unread_notifications_count())

    def test_remove_line_rejected_module_by_commissioner(self):
        subrogation = self._create_subrogation()
        self.sa_session.expire(self.tm)

        self._prepare_for_declaration()
        # add some pv modules for subrogation
        subrogated_modules = self._prepare_for_declaration(modules=[self._modules[2], self._modules[3]], subrogation=subrogation)

        dc = PvcDeclarationOrderController(self.sa_session, account_id=self.ma.id)
        # trigger new declarations
        declarations = dc.add_open_declaration(self.tm, 2013, month=1)

        subrogated = declarations[1]

        if declarations[0].producer != declarations[0].tbs_member:
            subrogated = declarations[0]

        # reject one pv module as commissioner
        sc = PvcSubrogationController(self.sa_session, account_id=self.ca.id)
        sc.reject_pv_module(subrogated_modules[0].id, 'Cuz a rabbit')

        self.sa_session.expire(subrogated)
        self.assertEqual(len(subrogated.lines), 1)

    def test_add_line_with_rejected_pv_module_by_commissioner(self):
        subrogation = self._create_subrogation()
        self.sa_session.expire(self.tm)

        self._prepare_for_declaration()
        # add some pv modules for subrogation
        self._prepare_for_declaration(modules=[self._modules[2], self._modules[3]], subrogation=subrogation)

        dc = PvcDeclarationOrderController(self.sa_session, account_id=self.ma.id)
        # trigger new declarations
        declarations = dc.add_open_declaration(self.tm, 2013, month=1)

        # add an additional PV Module and have the commissioner reject it.
        more_modules = self._prepare_for_declaration(modules=[self._modules[4]], subrogation=subrogation)
        mm = more_modules[0]

        # reject one pv module as commissioner
        sc = PvcSubrogationController(self.sa_session, account_id=self.ca.id)
        sc.reject_pv_module(mm.id, 'Cuz a rabbit')

        subrogated = declarations[1]

        if declarations[0].producer != declarations[0].tbs_member:
            subrogated = declarations[0]

        self.assertRaises(PublicUserException, dc.update_declaration, subrogated.id, {'line_0': 225, 'line_1': 175}, additional_lines={'tbs_member_module_%s' % mm.id: mm.id, 'quantity_additional_%s' % mm.id: 200})


# below is for running the test from eclipse
if __name__ == '__main__':

    from twisted.scripts.trial import run
    import sys
    sys.argv = ['test_declaration.py', 'pvcycle.test.unit.test_declaration.SubrogationDeclarationTestCase.test_split_order']
    run()
    sys.exit()
