'''
Created on Nov 11, 2013

@author: marten
'''
import os

from datetime import date
from decimal import Decimal

from basesoft.util.virtual_date import set_date, today
from basesoft.web.contrib.i18n.models import Country
from basesoft.web.core.exceptions import PublicSecurityException, PublicUserException

from pvcycle.test.unit import pvc_unittest
from pvcycle.core.controllers import PvcMemberController, PvcNotificationController, PvcTBSController
from pvcycle.core.models import PvcTbsMember
from pvcycle.pvmodule.controllers import PvcPVModuleController
from basesoft.web.contrib.file.models import ImportFileType, ImportFile
from pvcycle.pvmodule.schedule import ImportPVModuleTask
from twisted.trial.unittest import SkipTest
from pvcycle.pvmodule.models import PvcPVModule, PvcTbsMemberPVModule
from pvcycle.subrogation.controllers import PvcSubrogationController
from pvcycle.subrogation.models import PvcSubrogation
from basesoft.web.contrib.notification.controllers import NotificationController
from basesoft.web.contrib.license.controllers import LicenseController

class TestCase(pvc_unittest.PvcTestCase):
    def setUp(self):
        self.connect_db()

        set_date(date(2013, 1, 1)) # set date to January 2013

        self.reset_roe()

        self.operator = self._create_operator_account()
        self._create_contribution_tax_rates(self.operator)

        set_date(date(2013, 6, 1)) # set date to June 2013

        self._modules = self._fill_pv_modules(self.operator)
        # this TBS has it's setting on MONTHLY!
        self.tbs = self._create_tbs_account(self.operator)
        self.accountant = self._create_accountant_account(self.tbs, self.operator)

        self._administration_fee = Decimal('250.00')
        self._contribution_fee = Decimal('2.75')

        self.service_level = self._create_tbs_service_level(self.tbs, admin_fee=self._administration_fee, contrib_fee=self._contribution_fee, year=today().year)

        self.ma = self._create_member_account()

        # apply at TBS
        self.tm = self._member_apply_at_tbs(self.ma, self.tbs, self.service_level)

        # activate member to automatically create an order entry for the Administration Fee
        mc = PvcMemberController(self.sa_session, account_id=self.tbs.id)
        mc.save_tbs_member(self.tm.id, {'status': PvcTbsMember.STATUS_APPROVED, 'is_full_member': False, 'is_representative': False})

        self.controller = PvcPVModuleController(self.sa_session, account_id=self.ma.id)

    def test_simple_pv_module_for_country(self):
        self.controller.save_or_update_module_for_countries(self.tm.member.id, [self.tm.tbs.country.id], self._modules[0])
        self.sa_session.expire(self.tm)

        self.assertTrue(self._modules[0] in [m.module for m in self.tm.modules])

    def test_country_not_available_for_member(self):
        """
        This test adding a PV Module to a country which is not connected to this Member.
        """
        c = self.sa_session.query(Country).filter(Country.id!=self.tbs.default_group.license.country.id).first()
        self.assertRaises(PublicSecurityException, self.controller.save_or_update_module_for_countries, self.tm.member.id, [c.id], self._modules[0])

    def test_pv_module_already_selected_with_different_country(self):
        # Ticket #210
        state = self._get_tbs_license_state()
        state['tbs']['country'] = None
        eu_tbs = self._create_tbs_account(self.operator, state)

        eu_sl = self._create_tbs_service_level(eu_tbs, admin_fee=self._administration_fee, contrib_fee=self._contribution_fee, year=today().year)
        tm = self._member_apply_at_tbs(self.ma, eu_tbs, eu_sl)

        mc = PvcMemberController(self.sa_session, account_id=eu_tbs.id)
        mc.save_tbs_member(tm.id, {'status': PvcTbsMember.STATUS_APPROVED, 'is_full_member': False, 'is_representative': False})

        mc = PvcMemberController(self.sa_session, account_id=self.ma.id)
        _map = mc.get_member_countries(self.ma.default_group.license.id, True)
        possibilities = [country for _tm, country in _map.values() if _tm == tm]

        self.controller.save_or_update_module_for_countries(tm.member.id, [possibilities[0].id], self._modules[0])

        self.controller.save_or_update_module_for_countries(tm.member.id, [possibilities[1].id], self._modules[0])

    def test_eu_tbs_countries(self):
        state = self._get_tbs_license_state()
        state['tbs']['country'] = None
        eu_tbs = self._create_tbs_account(self.operator, state)

        eu_sl = self._create_tbs_service_level(eu_tbs, admin_fee=self._administration_fee, contrib_fee=self._contribution_fee, year=today().year)
        tm = self._member_apply_at_tbs(self.ma, eu_tbs, eu_sl)

        mc = PvcMemberController(self.sa_session, account_id=eu_tbs.id)
        mc.save_tbs_member(tm.id, {'status': PvcTbsMember.STATUS_APPROVED, 'is_full_member': False, 'is_representative': False})

        mc = PvcMemberController(self.sa_session, account_id=self.ma.id)
        _map = mc.get_member_countries(self.ma.default_group.license.id, True)
        possibilities = [country for _tm, country in _map.values() if _tm == tm]

        mms = self.controller.save_or_update_module_for_countries(tm.member.id, [possibilities[0].id], self._modules[0])
        self.assertEqual(mms[0].country, possibilities[0])

    def test_countries_for_disabled_tbs(self):
        raise SkipTest('This test is no longer relevant as per implementation of Rest of Europe being managable by Operator')

        # make sure there are no Swiss TBS's for this test.
        self.sa_session.execute('UPDATE tblPvcTbsLicense SET country_id = 155 WHERE country_id = 211')

        state = self._get_tbs_license_state()
        state['tbs']['country'] = 211 # SWITZERLAND
        non_eu_tbs = self._create_tbs_account(self.operator, state)

        state = self._get_tbs_license_state()
        state['tbs']['country'] = None
        eu_tbs = self._create_tbs_account(self.operator, state)

        eu_sl = self._create_tbs_service_level(eu_tbs, admin_fee=self._administration_fee, contrib_fee=self._contribution_fee, year=today().year)
        tm = self._member_apply_at_tbs(self.ma, eu_tbs, eu_sl)

        mc = PvcMemberController(self.sa_session, account_id=eu_tbs.id)
        mc.save_tbs_member(tm.id, {'status': PvcTbsMember.STATUS_APPROVED, 'is_full_member': False, 'is_representative': False})

        country = self.sa_session.query(Country).filter(Country.id==211).first()

        mc = PvcMemberController(self.sa_session, account_id=self.ma.id)
        countries = mc.get_member_countries(self.ma.default_group.license.id)

        self.assertTrue(country not in countries)

        # deactive the non_eu_tbs
        lc = LicenseController(self.sa_session, account_id=self.operator.id)
        lc.update_license_active_state(non_eu_tbs.default_group.license.id, active=False)

        mc = PvcMemberController(self.sa_session, account_id=self.ma.id)
        countries = mc.get_member_countries(self.ma.default_group.license.id)

        self.assertTrue(country in countries)

    def test_four_eye_approval(self):
        nc = PvcNotificationController(self.sa_session, account_id=self.operator.id)
        cnt = nc.get_unread_notifications_count()

        name = self._uniquestring('pvmodule')
        module = self.controller.get_or_create_module(name, 'Type A', 'Cell Type A', 100, 10, 25, 5)

        self.assertEqual(module.first_approval_by, None)
        self.assertEqual(cnt+1, nc.get_unread_notifications_count())

        c = PvcPVModuleController(self.sa_session, account_id=self.operator.id)
        m = c.approve_module(module.id)

        self.assertEqual(module.id, m.id)
        self.assertEqual(m.first_approval_by, self.operator)

        # we can only approve one time with the same account
        self.assertRaises(PublicSecurityException, c.approve_module, module.id)

        second_operator = self._create_operator_account()

        c = PvcPVModuleController(self.sa_session, account_id=second_operator.id)
        m = c.approve_module(module.id)

        self.assertEqual(module.id, m.id)
        self.assertEqual(m.first_approval_by, self.operator)
        self.assertEqual(m.second_approval_by, second_operator)

    def test_negative_and_zero_values(self):
        # We do not allow negative or 0 values, however we do allow zero for nominal power (Tickets #212, #462)
        name = self._uniquestring('pvmodule')
        self.assertRaises(AssertionError, self.controller.get_or_create_module, name, 'Type A', 'Cell Type A', -100, 20, 25, 5)
        self.assertRaises(AssertionError, self.controller.get_or_create_module, name, 'Type A', 'Cell Type A', 100, -20, 25, 5)
        self.assertRaises(AssertionError, self.controller.get_or_create_module, name, 'Type A', 'Cell Type A', 100, 20, -25, 5)
        self.assertRaises(AssertionError, self.controller.get_or_create_module, name, 'Type A', 'Cell Type A', 100, 20, 25, -5)

        self.assertRaises(AssertionError, self.controller.get_or_create_module, name, 'Type A', 'Cell Type A', 100, 0, 25, 5)
        self.assertRaises(AssertionError, self.controller.get_or_create_module, name, 'Type A', 'Cell Type A', 100, 20, 0, 5)
        self.assertRaises(AssertionError, self.controller.get_or_create_module, name, 'Type A', 'Cell Type A', 100, 20, 25, 0)

    def test_ticket_382(self):
        # test public user exception for duplicate PV Module supplier & type combination

        s = self._uniquestring('sup')
        t = self._uniquestring('type')
        t2 = self._uniquestring('type')

        # can only be done as OPERATOR.
        controller = PvcPVModuleController(self.sa_session, account_id=self.operator.id)
        first = controller.save_or_update_module(None, s, t, 'dontmatter', 1, 1, 1, 1)
        second = controller.save_or_update_module(None, s, t2, 'alsodontmatter', 1, 1, 1, 1)

        self.assertNotEqual(first, second)

        # now try to update second
        self.assertRaises(PublicUserException, controller.save_or_update_module, second.id, s, t, 'alsodontmatter', 1, 1, 1, 1)

    def test_pv_module_select_with_subrogation(self):

        # create commissioner.
        ca = self._create_member_account('COMMISSIONER')
        ctm = self._member_apply_at_tbs(ca, self.tbs, self.service_level, is_producer=False, is_commissioner=True)
        ctm.status = self.tm.STATUS_APPROVED
        self.sa_session.add(ctm)
        self.sa_session.flush()

        # add subrogation.
        controller = PvcSubrogationController(self.sa_session, account_id=self.ma.id)
        subrogation = controller.request_for_subrogation(self.tm, 'commissioner_name')
        # Set subrogation status active.
        subrogation.commissioner = ctm
        self.sa_session.add(subrogation)
        self.sa_session.flush()

        # assert that you can not use an inactive subrogation
        self.assertRaises(PublicUserException, self.controller.save_or_update_module_for_countries, self.tm.member.id, [self.tm.tbs.country.id], self._modules[0], subrogation.id)

        subrogation.status = PvcSubrogation.STATUS_ACTIVE
        subrogation.commissioner = ctm
        self.sa_session.add(subrogation)
        self.sa_session.flush()

        # save pv modules for countries + subrogation.
        modules = self.controller.save_or_update_module_for_countries(self.tm.member.id, [self.tm.tbs.country.id], self._modules[0], subrogation.id)
        # verify.
        self.assertEqual(modules[0].subrogation, subrogation)
        self.assertEqual(modules[0].commissioner_approval_status, PvcTbsMemberPVModule.COMMISSIONER_APPROVAL_STATUS_PENDING)

        pvc = PvcSubrogationController(self.sa_session, account_id=ca.id)
        self.assertEqual(pvc.get_unapproved_commissioner_pv_modules_count(), 1)

        # Lets approve the module
        pvm = pvc.approve_pv_module(modules[0].id)
        self.sa_session.expire(pvm)

        self.assertEqual(pvm.commissioner_approval_status, PvcTbsMemberPVModule.COMMISSIONER_APPROVAL_STATUS_APPROVED)

        # save another pv module for countries + subrogation.
        modules = self.controller.save_or_update_module_for_countries(self.tm.member.id, [self.tm.tbs.country.id], self._modules[1], subrogation.id)
        # verify.
        self.assertEqual(modules[0].subrogation, subrogation)
        self.assertEqual(modules[0].commissioner_approval_status, PvcTbsMemberPVModule.COMMISSIONER_APPROVAL_STATUS_PENDING)

        # count notifications of Subrogator account
        nc = NotificationController(self.sa_session, account_id=self.ma.id)
        cnt = nc.get_unread_notifications_count()

        pvm = pvc.reject_pv_module(modules[0].id, 'cuz i can')
        self.sa_session.expire(pvm)

        self.assertEqual(pvm.commissioner_approval_status, PvcTbsMemberPVModule.COMMISSIONER_APPROVAL_STATUS_REJECTED)

        # check that the Subrogator has a new notification
        self.assertEqual(nc.get_unread_notifications_count(), cnt+1)

    def test_pv_module_deactivate_and_reselect(self):
        state = self._get_tbs_license_state()
        state['tbs']['country'] = None
        eu_tbs = self._create_tbs_account(self.operator, state)

        eu_sl = self._create_tbs_service_level(eu_tbs, admin_fee=self._administration_fee, contrib_fee=self._contribution_fee, year=today().year)
        tm = self._member_apply_at_tbs(self.ma, eu_tbs, eu_sl)

        mc = PvcMemberController(self.sa_session, account_id=eu_tbs.id)
        mc.save_tbs_member(tm.id, {'status': PvcTbsMember.STATUS_APPROVED, 'is_full_member': False, 'is_representative': False})

        mc = PvcMemberController(self.sa_session, account_id=self.ma.id)
        _map = mc.get_member_countries(self.ma.default_group.license.id, True)
        possibilities = [country for _tm, country in _map.values() if _tm == tm]

        # select for 3 countries
        mms = self.controller.save_or_update_module_for_countries(tm.member.id, [possibilities[0].id, possibilities[1].id, possibilities[2].id], self._modules[0])

        self.controller.deactivate_module(self._modules[0].id, tm.id, None, [possibilities[1].id])
        self.assertTrue(possibilities[1] not in self.controller.get_countries_for_module(self._modules[0].id, tm.id, None))

        self.controller.save_or_update_module_for_countries(tm.member.id, [possibilities[1].id], self._modules[0])
        self.assertTrue(possibilities[1] in self.controller.get_countries_for_module(self._modules[0].id, tm.id, None))

    def test_roe_disable_task(self):
        state = self._get_tbs_license_state()
        state['tbs']['country'] = None
        eu_tbs = self._create_tbs_account(self.operator, state)
        eu_sl = self._create_tbs_service_level(eu_tbs, admin_fee=self._administration_fee, contrib_fee=self._contribution_fee, year=today().year)
        tm = self._member_apply_at_tbs(self.ma, eu_tbs, eu_sl)

        mc = PvcMemberController(self.sa_session, account_id=eu_tbs.id)
        mc.save_tbs_member(tm.id, {'status': PvcTbsMember.STATUS_APPROVED, 'is_full_member': False, 'is_representative': False})

        mc = PvcMemberController(self.sa_session, account_id=self.ma.id)
        _map = mc.get_member_countries(self.ma.default_group.license.id, True)
        possibilities = [country for _tm, country in _map.values() if _tm == tm]

        # select for 3 countries
        mms = self.controller.save_or_update_module_for_countries(tm.member.id, [possibilities[0].id, possibilities[1].id, possibilities[2].id], self._modules[0])

        tc = PvcTBSController(self.sa_session, account_id=self.operator.id)
        countries = tc.get_available_tbs_countries()
        ncountries = []
        for c in countries:
            if c != possibilities[1]:
                ncountries.append(c.id)

        tc.save_roe_countries(ncountries)

        self.controller.disable_pv_modules_for_all_non_eu_countries()

        self.assertTrue(possibilities[1] not in self.controller.get_countries_for_module(self._modules[0].id, tm.id, None))

class PhotonImportTestCase(pvc_unittest.PvcTestCase):
    def setUp(self):
        self.connect_db()

        self.operator = self._create_operator_account()
        self.controller = PvcPVModuleController(self.sa_session, account_id=self.operator.id)

    def _save_import_file(self, file_path, _file):
        # make temp file from photon test.
        from basesoft.web.contrib.file.controllers import AnonymousFileController
        afc = AnonymousFileController(self.sa_session)

        with file(file_path) as f:
            data = f.read()

        _type = self.sa_session.query(ImportFileType).filter(ImportFileType.name=='pvcycle.pvmodule').one()

        _file = afc.save_file_for_target_group(data, _file, 'Photon DB import', self.operator.default_group, ImportFile, {
            'status': ImportFile.STATUS_NEW,
            'settings': '{}',
            'type': _type
        })
        return _file

    def test_simple_import_photon_db(self):
        csv_file = os.path.join(os.path.dirname(__file__), 'files', 'photon_test.csv')

        _file = self._save_import_file(csv_file, 'photon_test.csv')

        nc = PvcNotificationController(self.sa_session, account_id=self.operator.id)
        cnt = nc.get_unread_notifications_count()

        t = ImportPVModuleTask()
        t.execute()

        self.sa_session.expire(_file)
        self.assertEqual(_file.status, ImportFile.STATUS_SUCCESS)
        self.assertEqual(int(_file.result), 14)

        # assert notification.
        self.assertEqual(cnt+1, nc.get_unread_notifications_count())

        # supplier: type : [nominal power, length, width, weight]
        expected = {
            'Eni S.p.A.': {
                'PL 120 B' : [Decimal(115), Decimal(1180), Decimal(1180), Decimal(15)],
                'PL 8/65' : [Decimal(67), Decimal(1215), Decimal(555), Decimal('8.5')],
                'P 5/43': [Decimal(45), Decimal(995), Decimal(450), Decimal(6)],
            },
            'BP Solar': {
                'BP 275 F': [Decimal(75), Decimal(1188), Decimal(530), Decimal('7.5')]
            },
            'alfasolar Vertriebs-GmbH': {
                'alfa 55 S': [Decimal(55), Decimal(1000), Decimal('45.7'), Decimal(5)]
            },
            'Shell Solar GmbH (jetzt bei Solarworld)': {
                'RSM 50 Photowatt': [Decimal(48), Decimal(1006), Decimal(478), Decimal('5.9')]
            },
            'AstroPower': {
                'APX-80 (AP 8225)': [Decimal(80), Decimal(1476), Decimal(660), Decimal('11.9')],
                'APX-90 (AP 9225)': [Decimal(90), Decimal('163.3'), Decimal(660), Decimal('11.9')],
                'AP-1106': [Decimal(110), Decimal(1476), Decimal(660), Decimal('11.9')],
                'AP-1206': [Decimal(120), Decimal(1476), Decimal(660), Decimal('11.9')],
            },
            'Schott Solar AG': {
                '50-ALF/17': [Decimal(50), Decimal(965), Decimal(452), Decimal(6)],
                '300-DGF/50-B': [Decimal('28.5'), Decimal(1892), Decimal(1282), Decimal('48.5')],
                '300-DGF/50': [Decimal(300), Decimal(1892), Decimal(1282), Decimal('48.5')],
                'ASE-300-DG-FT / 17 V, 300 W': [Decimal(300), Decimal(1892), Decimal(1283), Decimal('48.5')],
            }

        }

        result = self.sa_session.query(PvcPVModule).order_by(PvcPVModule.last_updated.desc()).limit(14).all()
        for m in result:
            self.assertEqual(expected[m.supplier.name][m.type.name], [m.nominal_power, m.length, m.width, m.weight], 'Failed on %s - %s' % (m.supplier.name, m.type.name))

    def test_with_failure(self):
        csv_file = os.path.join(os.path.dirname(__file__), 'files', 'photon_test_2.csv')

        _file = self._save_import_file(csv_file, 'photon_test_2.csv')

        t = ImportPVModuleTask()
        t.execute()

        self.sa_session.expire(_file)
        self.assertEqual(_file.status, ImportFile.STATUS_SUCCESS)
        self.assertEqual(int(_file.result), 12)
        self.assertTrue(_file.failure_file is not None)

    def test_import_real_photon_db(self):
        raise SkipTest
        xlsx_file = os.path.join(os.path.dirname(__file__), 'files', 'photon.csv')

        _file = self._save_import_file(xlsx_file, 'photon.csv')

        t = ImportPVModuleTask()
        t.execute()

        self.sa_session.expire(_file)
        self.assertEqual(_file.status, ImportFile.STATUS_SUCCESS)
        self.assertEqual(int(_file.result), 45553)


# below is for running the test from eclipse
if __name__ == '__main__':

    from twisted.scripts.trial import run
    import sys
    sys.argv = ['test_pv_module.py', 'pvcycle.test.unit.test_pv_module.PhotonImportTestCase.test_simple_import_photon_db']
    run()
    sys.exit()

