'''
Created on Jul 22, 2013

@author: marten
'''

import datetime
from pvcycle.product.models import PvcBrand, PvcTbsProduct

from sqlalchemy.sql.expression import and_
from dateutil.relativedelta import relativedelta

from decimal import Decimal
from random import choice

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.contrib.contact.models import Person
from basesoft.web.contrib.license.controllers import LicenseController
from basesoft.util.virtual_date import today
from basesoft.web.contrib.i18n.models import Country, Locale

from pvcycle.core.controllers import PvcTBSController, PvcMemberController, PvcAccountantController
from pvcycle.core.models import PvcTbsLicense, PvcTbsMember, PvcROECountry, PvcTbsServiceLevel
from basesoft.web.contrib.order.controllers import TaxController,\
    GLAccountController, PaymentController
from basesoft.web.core.exceptions import PublicUserException
from pvcycle.pvmodule.controllers import PvcPVModuleController
from pvcycle.pvmodule.models import PvcPVModule, PvcPVModuleSupplier,\
    PvcPVModuleType
from basesoft.web.contrib.order.models import TaxClassCountry,\
    PaymentMethodBankTransfer, PaymentMethod, PaymentMethodOgone

from basesoft.web.contrib.license.models import Domain
from basesoft.web.contrib.account.controllers import AccountController
from basesoft.web.contrib.account.models import AccountActivation, Account
from basesoft.web.contrib.contact.controllers import SimpleContactController
from basesoft.web.test.unit.test_abstract_controller import MockRequest


class PvcTestCase(iqportal_unittest.IQPortalTestCase):

    def _get_address_data(self):
        return {
            'country': 155,
        }

    def _get_operator_license_state(self):
        return {
            'license': {
                'name': u'test_pvc_operator_license',
                'expiration_date': None,
                'is_default': True,
                'polymorphic_type': 'PvcOperatorLicense',
                },
            'domain': {'name': 'PV CYCLE'},
            'contact_person': {
                'first_name': u'Marten',
                'last_name': u'Koedam',
                'gender': Person.MALE,
                'email': u'marten@basesoft.com',
                'position': u'developer',
                'department': u'development',
                'phone': '+31206847087',
                'birth_date': '1981-10-30',
                'address_data': self._get_address_data()
                },
            'company': {
                'name': self._uniquestring('PVCCompany'),
                'vat_no': 'NL810449493B01',
                'website': u'http://www.basesoft.com',
                },
            'company_address': {
                'address': self._get_address_data(),
                'is_invoice': True,
                'is_delivery': False
                },
            'company_bank_account': {
                'bank_account': {
                    'number': self._uniquestring('99'),
                    'iban': self._uniquestring('NL'),
                    'bic':'BBBBNLNH'},
                'is_default': True
                },
            'group': {'name': u'admins'},
            'permissions': [],
            'usage_options': []
        }

    def _get_tbs_license_state(self):
        return {
            'tbs': {
                'tbs_type': PvcTbsLicense.TBS_TYPE_SUBSIDIARY,
                'country': 155,
                'contract_text': """
                    ''tbs''' contract text
                    @other:current_date
                """,
            },
            'configs': {
                'pvc.tbs.declaration.period': PvcTbsLicense.DECLARATION_PERIOD_MONTHLY,
            },
            'contact_person': {
                'first_name': u'Marten',
                'last_name': u'Koedam',
                'gender': Person.MALE,
                'email': u'marten@basesoft.com',
                'position': u'developer',
                'department': u'development',
                'phone': '+31206847087',
                'birth_date': '1981-10-30',
                'address_data': self._get_address_data()
                },
            'company': {
                'name': self._uniquestring('TbsCompany')[:40],
                'vat_no': 'NL810449493B01',
                'website': u'http://www.basesoft.com',
                },
            'company_address': {
                'address': self._get_address_data(),
                'is_invoice': True,
                'is_delivery': False
                },
            'company_bank_account': {
                'bank_account': {
                    'number': self._uniquestring('99'),
                    'iban': self._uniquestring('NL'),
                    'bic':'BBBBNLNH'},
                'is_default': True
                },
        }

    def _get_accountant_license_state(self):
        return {
            'contact_person': {
                'first_name': u'Marten',
                'last_name': u'Koedam',
                'gender': Person.MALE,
                'email': u'marten@basesoft.com',
                'position': u'developer',
                'department': u'development',
                'phone': '+31206847087',
                'birth_date': '1981-10-30',
                'address_data': self._get_address_data()
                },
            'company': {
                'name': self._uniquestring('AccountantCompany')[:40],
                'vat_no': 'NL810449493B01',
                'website': u'http://www.basesoft.com',
                },
            'company_address': {
                'address': self._get_address_data(),
                'is_invoice': True,
                'is_delivery': False
                },
            'company_bank_account': {
                'bank_account': {
                    'number': self._uniquestring('99'),
                    'iban': self._uniquestring('NL'),
                    'bic':'BBBBNLNH'},
                'is_default': True
                },
        }

    def _create_sysadmin_account(self):

        # sysadmin
        sysadmin_account = self._create_active_account()
        sysadmin_group = sysadmin_account.groups[0].group
        self._attach_permissions_to_group(sysadmin_group,
            'view_dashboard',
            'view_domain', 'view_license', 'view_all_domains', 'view_group', 'create_domain', 'create_license',
            'distribute_group_permissions', 'distribute_license_permissions', 'manage_roles',
            'scheduler_manager', 'manage_iqportal_roles',
            'pvc_operator_license',
        )

        return sysadmin_account

    def _create_operator_account(self):

        # sysadmin
        sysadmin_account = self._create_sysadmin_account()

        # operator license
        lc = LicenseController(self.sa_session, account_id=sysadmin_account.id)
        operator_license = lc.save_license_from_state(self._get_operator_license_state())

        # operator user
        operator_account = self._create_active_account(license=operator_license)
        operator_group = operator_account.groups[0].group
        self._attach_permissions_from_role(operator_group, 'PVC Operator')

        self._create_tax_rates(operator_account)

        return operator_account

    def _create_tbs_account(self, operator_account=None, state=None):

        if operator_account is None:
            operator_account = self._create_operator_account()
        if state is None:
            state = self._get_tbs_license_state()

        tbsc = PvcTBSController(self.sa_session, account_id=operator_account.id)
        tbs_license = tbsc.save_tbs(state)

        tbs_account = self._create_active_account(license=tbs_license, group=tbs_license.groups[0])

        # assign GL accounts
        c = GLAccountController(self.sa_session, account_id = operator_account.id)
        gl_account = c.save_gl_account_state(None, {'merchant_id': tbs_account.default_group.license.id, 'code': 'GL_ADMIN_FEE', 'account_number': 'GL123456'})
        gl_account = c.save_gl_account_state(None, {'merchant_id': tbs_account.default_group.license.id, 'code': 'GL_CONTRIB_FEE', 'account_number': 'GL789012'})
        gl_account = c.save_gl_account_state(None, {'merchant_id': tbs_account.default_group.license.id, 'code': 'GL_MEMBER_FEE', 'account_number': 'GL987653'})
        gl_account = c.save_gl_account_state(None, {'merchant_id': tbs_account.default_group.license.id, 'code': 'GL_FORFEIT_FEE', 'account_number': 'GL_forfeit123'})
        gl_account = c.save_gl_account_state(None, {'merchant_id': tbs_account.default_group.license.id, 'code': 'GL_FORECAST_FEE', 'account_number': 'GLF0R3CAS7'})

        return tbs_account

    def _create_accountant_account(self, tbs, operator_account=None):
        if operator_account is None:
            operator_account = self._create_operator_account()
        if type(tbs) != list:
            tbs = [tbs]

        ac = PvcAccountantController(self.sa_session, account_id=operator_account.id)
        state = self._get_tbs_license_state()
        state['company']['name'] = self._uniquestring('Accountant')[:40]
        state['tbs_licenses'] = [{'tbs_id': t.id, 'start_date': today(), 'end_date': None} for t in tbs]
        license = ac.save_accountant(state)
        accountant_account = self._create_active_account(license=license, group=license.groups[0])
        #self._attach_permissions_to_group(accountant_account.default_group, 'soap_login', 'pvc_view_orders')
        return accountant_account

    def _create_memberships(self, tbs_account, name=None, cnt=3, year=None, fee=None):
        tbs_controller = PvcTBSController(account_id=tbs_account.id)

        if year is None:
            year = today().year

        if fee is None:
            fee = Decimal('12.123')

        if name is None:
            name = 'membership'

        rv = []
        for i in range(cnt):
            rv.append(tbs_controller.save_membership_state(None, {
                'tbs_id': tbs_account.groups[0].group.license.id,
                'name': '%s_%s' % (name, i),
                'membership_fees': [{
                    'start_year': year,
                    'fee': fee*(i+1),
                }],
            }))

        return rv

    def _create_tbs_service_level(self, tbs_account, name='tbs service level 1', admin_fee=None, contrib_fee=None,
                                  year=None, annex_text=None, status=None, level=1, product_type=PvcTbsProduct.TYPE_OTHER,
                                  guarantee_forecast_min_value=None):

        if status is None:
            status = PvcTbsServiceLevel.STATUS_ACTIVE

        if year is None:
            year = (today() + datetime.timedelta(days=365)).year

        if admin_fee is None:
            admin_fee = Decimal('11.12345')

        if contrib_fee is None:
            contrib_fee = Decimal('11.12345')

        if annex_text is None:
            annex_text = "contract '''annex''' text"

        self._attach_permissions_to_group(tbs_account.groups[0].group,
            'pvc_tbs_service_level',
        )

        tbs_controller = PvcTBSController(account_id=tbs_account.id)

        # create a service level
        service_level = tbs_controller.save_service_level_state(None, {
            'tbs_id': tbs_account.groups[0].group.license.id,
            'name': name,
            'level': level,
            'status': status,
            'product_type': product_type,
            'guarantee_forecast_min_value': guarantee_forecast_min_value,
            'contract_annex_text': annex_text,
            'administration_fees': [{
                'start_year': year,
                'fee': admin_fee,
            }],
            'contribution_fees': [{
                'start_date': datetime.date(year, 1, 1),
                'fee': contrib_fee
            }],
        })

        return service_level

    def _get_member_license_state(self, company_name):
        return {
            'dbID': None,
            'company': {
                'name': company_name,
                'website': 'http://member.com',
                'vat_no': 'BE0893027827'
            },
            'address_data': {
                'street': 'Some Shanghai Street Name',
                'city': 'Shanghai',
                'postcode': 'GA SHA',
                'country': self.sa_session.query(Country).filter(Country.iso_3166_a2=='BE').one().id
            },
            'bank_data': {
                'iban': "IBAN92748283423",
                'bic': "BIC974234",
                'number': "324234234",
            },
            'contact_person_data': {
                'last_name': 'Member Contact Last Name',
                'first_name': 'Member Contact First Name',
                'email': 'contact-person-pvc@mail-test.basesoft.com',
                'preferred_locale': self.sa_session.query(Locale).filter(Locale.locale=='en_GB').one().id,
                'gender': 'Male',
                'phone': '234234235',
            },
            'invoice_person_data': {
                'last_name': 'InvoiceLastName',
                'first_name': 'InvoiceFirstName',
                'email': 'invoice-person-pvc@mail-test.basesoft.com',
                'preferred_locale': self.sa_session.query(Locale).filter(Locale.locale=='en_GB').one().id,
                'gender': 'Male',
                'phone': '234234235',
            }
        }

    def _create_member_account(self, company_name=None, state=None):

        if company_name is None:
            company_name = 'member company'

        if state is None:
            state = self._get_member_license_state(company_name)

        username = self._uniquestring('u')
        email = '%s@mail-test.basesoft.com' % username

        ac = AccountController(self.sa_session, request=MockRequest())
        account = ac.create_inactive_account(username, 'test_password', email)

        # fake send email to user
        activation = self.sa_session.query(AccountActivation).filter(
            AccountActivation.account == self.sa_session.query(Account).filter(Account.name == username).one()).one()
        activation.activation_stage = AccountActivation.ACTIVATION_STAGE_PENDING
        self.sa_session.add(activation)
        self.sa_session.flush()

        # activate account
        ac = AccountController(self.sa_session)
        aa = ac.get_account_activation(account)
        self.assertRaises(PublicUserException, ac.activate_account, aa.activation_key)

        # add person info
        locale = self.sa_session.query(Locale).filter(Locale.locale=='en_GB').one()
        scc = SimpleContactController(self.sa_session, account_id=aa.account.id, check_active=False)
        person_state = {
            'preferred_locale': locale.id,
            'last_name': 'Nguyen',
            'first_name': 'Huie',
        }

        # update the person preferred locale
        person = scc._save_or_update_person(account.person.id, person_state)
        self.assertEqual(person.preferred_locale, locale)
        self.assertEqual(person.preferred_locale.locale, scc.config.get('basesoft.web.contrib.locale'))

        account = ac.activate_account(aa.activation_key)

        # back to pvc..
        mc = PvcMemberController(self.sa_session, account_id=account.id)

        # set new license_id in state
        state['dbID'] = account.default_group.license.id

        # save company info
        mc.save_member_license(state)

        return account

    def _member_apply_at_tbs(self, member_account, tbs_account, service_level=None, upload_contract=True,
                             was_member_in_2013=False, is_producer=True, is_commissioner=False, membership=None,
                             member_license_id=None, financial_guarantee=False):

        mc = PvcMemberController(self.sa_session, account_id=member_account.id)

        if member_license_id is None:
            member_license_id = member_account.groups[0].group.license.id

        tbs_license_id = tbs_account.groups[0].group.license.id

        service_level_id = None
        if service_level is not None:
            service_level_id = service_level.id

        membership_id = None
        if membership is not None:
            membership_id = membership.id

        if upload_contract:
            upload_contract = self._create_uploaded_file('upload_contract.pdf', 'upload_contract contents')
        else:
            upload_contract = None

        if financial_guarantee:
            financial_guarantee = self._create_uploaded_file('financial_guarantee.pdf', 'financial_guarantee contents')
        else:
            financial_guarantee = None

        selling_technique = 2

        tbs_member = mc.apply_at_tbs(tbs_license_id, member_license_id, None, service_level_id, None, is_commissioner, is_producer, upload_contract,
                                     was_member_in_2013=was_member_in_2013, membership_id=membership_id,
                                     financial_guarantee=financial_guarantee, selling_technique=selling_technique)

        return tbs_member

    def _create_multiple_producers_at_tbs(self, tbs_account, service_levels, cnt=10, approve=True):

        for i in range(0, cnt):
            ma = self._create_member_account('member %s' % i)
            tbs_member = self._member_apply_at_tbs(ma, tbs_account, choice(service_levels))
            if approve:
                tbs_member.status = PvcTbsMember.STATUS_APPROVED
                self.sa_session.add(tbs_member)

        self.sa_session.flush()


    def _create_tax_rates(self, operator_account):

        c = TaxController(self.sa_session, account_id = operator_account.id)

        admin_tax = None
        for tc in c.get_tax_classes():
            if tc.code == 'TAX_ADMIN_FEE':
                admin_tax = tc
                break

        tid = None
        if admin_tax is None:
            admin_tax = c.save_tax_class_state(None, {'name': 'Administrative Fee Tax', 'code': 'TAX_ADMIN_FEE'})

        start_date = today()
        country_nl = self.sa_session.query(Country).filter(Country.name == 'Netherlands').one()
        tcc = self.sa_session.query(TaxClassCountry).filter(TaxClassCountry.tax_class_id == admin_tax.id, TaxClassCountry.country_id == country_nl.id).first()
        if tcc is not None:
            tid = tcc.id
            start_date = today()+relativedelta(days=1)

        try:
            cr = c.get_current_rate(country_nl, 'TAX_ADMIN_FEE')
        except PublicUserException:
            cr = None
        if cr is None:
            c.save_country_tax_state(tid, {
                'country_id': 155,
                'tax_class_id': admin_tax.id,
                'tax_rates': [{
                    'start_date': start_date,
                    'rate': Decimal(21, 0)
                }]
            })

    def _create_contribution_tax_rates(self, operator_account, country=None, rate=None):

        c = TaxController(self.sa_session, account_id = operator_account.id)

        admin_tax = None
        for tc in c.get_tax_classes():
            if tc.code == 'TAX_CONTRIB_FEE':
                admin_tax = tc
                break

        tid = None
        if admin_tax is None:
            admin_tax = c.save_tax_class_state(None, {'name': 'Contribution Fee Tax', 'code': 'TAX_CONTRIB_FEE'})

        start_date = today()
        if country is None:
            country = self.sa_session.query(Country).filter(Country.name == 'Netherlands').one()
        tcc = self.sa_session.query(TaxClassCountry).filter(TaxClassCountry.tax_class_id == admin_tax.id, TaxClassCountry.country_id == country.id).first()
        if tcc is not None:
            tid = tcc.id
            start_date = today()+relativedelta(days=1)

        if rate is None:
            rate = Decimal(21, 0)

        try:
            cr = c.get_current_rate(country, 'TAX_CONTRIB_FEE')
        except PublicUserException:
            cr = None
        if cr is None:
            c.save_country_tax_state(tid, {
                'country_id': country.id,
                'tax_class_id': admin_tax.id,
                'tax_rates': [{
                    'start_date': start_date,
                    'rate': rate
                }]
            })

    def _create_membership_tax_rates(self, operator_account, country_name=None):

        c = TaxController(self.sa_session, account_id = operator_account.id)

        admin_tax = None
        for tc in c.get_tax_classes():
            if tc.code == 'TAX_MEMBER_FEE':
                admin_tax = tc
                break

        tid = None
        if admin_tax is None:
            admin_tax = c.save_tax_class_state(None, {'name': 'Membership Fee Tax', 'code': 'TAX_MEMBER_FEE'})

        start_date = today()
        if country_name is None:
            country_name = 'Netherlands'
        country = self.sa_session.query(Country).filter(Country.name == country_name).one()
        tcc = self.sa_session.query(TaxClassCountry).filter(TaxClassCountry.tax_class_id == admin_tax.id, TaxClassCountry.country_id == country.id).first()
        if tcc is not None:
            tid = tcc.id
            start_date = today()+relativedelta(days=1)

        try:
            cr = c.get_current_rate(country, 'TAX_MEMBER_FEE')
        except PublicUserException:
            cr = None
        if cr is None:
            c.save_country_tax_state(tid, {
                'country_id': country.id,
                'tax_class_id': admin_tax.id,
                'tax_rates': [{
                    'start_date': start_date,
                    'rate': Decimal(21, 0)
                }]
            })


    def _fill_pv_modules(self, operator_account=None):
        """
        Inserts a few approved PV Modules
        """
        if operator_account is None:
            operator_account = self._create_operator_account()

        secondary = self._create_operator_account()

        c = PvcPVModuleController(self.sa_session, account_id=operator_account.id)
        c2 = PvcPVModuleController(self.sa_session, account_id=secondary.id)

        # id, supplier, _type, cell_type, nominal_power, weight, width, length
        modules = [
            [None, 'Supplier A', 'Type A', 'Cell Type A', 100, '150.765', 10, 15],
            [None, 'Supplier B', 'Type B', 'Cell Type B', 110, 140, 11, 14],
            [None, 'Supplier C', 'Type C', 'Cell Type A', 120, 160, 12, 13],
            [None, 'Supplier D', 'Type D', 'Cell Type C', 130, 140, 13, 12],
            [None, 'Supplier A', 'Type E', 'Cell Type D', 110, 160, 14, 11],
            [None, 'Supplier B', 'Type F', 'Cell Type E', 110, 150, 15, 10],
            [None, 'Supplier C', 'Type G', 'Cell Type F', 120, 140, 14, 11],
            [None, 'Supplier D', 'Type H', 'Cell Type A', 130, 150, 13, 12],
            [None, 'Supplier A', 'Type I', 'Cell Type F', 140, 160, 12, 13],
            [None, 'Supplier B', 'Type J', 'Cell Type F', 150, 160, 11, 12],
            [None, 'Supplier C', 'Type K', 'Cell Type E', 160, 150, 10, 11],
            [None, 'Supplier A', 'Type L', 'Cell Type D', 180, 140, 11, 12],
            [None, 'Supplier B', 'Type M', 'Cell Type C', 190, 160, 12, 13],
            [None, 'Supplier A', 'Type N', 'Cell Type B', 110, 140, 13, 14],
            [None, 'Supplier A', 'Type O', 'Cell Type A', 120, 160, 14, 15],
        ]

        rv = []
        for module in modules:
            m = self.sa_session.query(PvcPVModule).join(PvcPVModuleSupplier).join(PvcPVModuleType).filter(
                and_(PvcPVModuleSupplier.name==module[1], PvcPVModuleType.name==module[2])).first()
            if m is not None:
                module[0] = m.id
            m = c.save_or_update_module(*module)

            c2.approve_module(m.id)
            rv.append(m)

        return rv

    def _add_pv_module(self, supplier_name, operator_account=None):
        if operator_account is None:
            operator_account = self._create_operator_account()

        c = PvcPVModuleController(self.sa_session, account_id=operator_account.id)
        return c.get_or_create_module(supplier_name, 'Type A', 'Cell Type A', 10, 20, 30, 40)

    def _setup_payment_methods(self, operator_account, tbs_account):

        merchant = tbs_account.default_group.license
        self._attach_permissions_to_group(operator_account.default_group, 'manage_payment_methods')

        # see if payment method exists for pvcycle domain, else create it
        pc = PaymentController(self.sa_session, account_id=operator_account.id)
        if len(pc.get_payment_methods(merchant=merchant)) == 0:
            pm = pc.save_or_update_payment_method_for_merchant(PaymentMethodBankTransfer, merchant=merchant, label="bank transfer", **{'priority': 1})
            pm = pc.save_or_update_payment_method_for_merchant(PaymentMethodOgone, merchant=merchant, label='online payment', **{
                'priority': 2,
                'psp_id': 'selenium_accept',
                'sha_in': 'Bier1234',
                'sha_out': 'Tuut1234',
                })


        return pc.get_payment_methods(merchant=merchant)

    def _update_creation_date(self, obj, newdate):
        from datetime import datetime as dt
        obj.created = dt.combine(newdate, dt.min.time())
        self.sa_session.add(obj)
        self.sa_session.flush()

    def reset_roe(self):

        ac = PvcTBSController(self.sa_session)
        countries = ac.get_efta_countries()
        self.sa_session.query(PvcROECountry).delete()

        for country in countries:
            c = PvcROECountry()
            c.country = country
            self.sa_session.add(c)

    def _reset_brands(self):
        for b in self.sa_session.query(PvcBrand).all():
            b.merge_brand = None
            b.merge_status = None
            self.sa_session.add(b)

        self.sa_session.flush()