'''
Created on Jul 22, 2013

@author: marten
'''
from basesoft.web.contrib.license.controllers import LicenseController

from pvcycle.test.unit import pvc_unittest
from pvcycle.core.models import PvcAccountantLicense
from pvcycle.core.controllers import PvcAccountantController, PvcTBSController
from basesoft.web.core.exceptions import PermissionDeniedException

class PvcAccountantTestCase(pvc_unittest.PvcTestCase):
    def setUp(self):

        self.connect_db()

        # sysadmin
        self.sysadmin_account = self._create_active_account()
        self.sysadmin_group = self.sysadmin_account.groups[0].group
        self._attach_permissions_to_group(self.sysadmin_group,
            'view_license', 'view_all_domains', 'view_group', 'create_domain', 'create_license',
            'distribute_license_permissions',
            'pvc_operator_license'
        )

        # operator user
        self.operator_account = self._create_operator_account()

        self.controller = PvcAccountantController(account_id=self.operator_account.id)

    def test_create_accountant(self):
        """
        This tests the creation of Accountant(s) by the operator.
        """

        # saving
        accountant_license = self.controller.save_accountant(self._get_accountant_license_state())
        self.assertTrue(isinstance(accountant_license, PvcAccountantLicense))

        # loading
        accountant_state = self.controller.load_accountant(accountant_license.id)
        self.assertTrue(accountant_state['company']['name'].startswith('AccountantCompany'))

        # accountant user (if needed)
        self.tbs_account = self._create_active_account(license=accountant_license)
        self.tbs_group = self.tbs_account.groups[0].group

    def test_ticket_59(self):
        """
        Creating an accountant without TBS' being available (or with no TBS selected)
        """
        state = self._get_accountant_license_state()
        state['tbs_licenses'] = [{'tbs_id': ''}]
        accountant_license = self.controller.save_accountant(state)
        self.assertTrue(isinstance(accountant_license, PvcAccountantLicense))

    def test_create_accountant_tbs_link(self):
        """
        This tests the creation and updating of an Accountant by the operator with linkes Take Back Schemes.
        """

        # we first create a TBS License so that we can link it to the accountant.
        tbs_controller = PvcTBSController(self.sa_session, account_id=self.operator_account.id)
        tbs1 = tbs_controller.save_tbs(self._get_tbs_license_state())

        # accountant state including the TBS.
        state = self._get_accountant_license_state()
        state['tbs_licenses'] = [{
            'tbs_id': tbs1.id,
            'start_date': None,
            'end_date': None
        }]
        # save the accountant state with the TBS
        accountant = self.controller.save_accountant(state)

        # assert that the previously created TBS is linked to the Accountant
        self.assertTrue(tbs1 in accountant.schemes)

        # add another TBS license and add it to the accountant as well, set a start date
        tbs2 = tbs_controller.save_tbs(self._get_tbs_license_state())
        state = self._get_accountant_license_state()
        state['dbID'] = accountant.id
        state['tbs_licenses'] = [{
            'tbs_id': tbs1.id,
            'start_date': None,
            'end_date': None
        }, {
            'tbs_id': tbs2.id,
            'start_date': None,
            'end_date': None
        }]

        # update the accountant
        accountant = self.controller.save_accountant(state)

        # assert that both TBS are linked to the accountant and that the new link has a start date set.
        self.assertEqual(len(accountant.schemes), 2)

        # remove the first TBS from the state
        state = self._get_accountant_license_state()
        state['dbID'] = accountant.id
        state['tbs_licenses'] = [{
            'tbs_id': tbs2.id,
            'start_date': None,
            'end_date': None
        }]

        # update the accountant
        accountant = self.controller.save_accountant(state)

        # assert that the first TBS is no longer attached to this license.
        self.assertEqual(len(accountant.schemes), 1)
        self.assertTrue(tbs2 in accountant.schemes)



