'''
Created on Jun 23, 2014

@author: rado
'''
from decimal import Decimal

from basesoft.util.virtual_date import today

from pvcycle.test.unit import pvc_unittest
from pvcycle.core.controllers import PvcMemberController
from pvcycle.core.models import PvcTbsMember
from pvcycle.pvmodule.controllers import PvcPVModuleController
from pvcycle.order.models import PvcDeclarationOrder
from sqlalchemy.sql.expression import and_
from pvcycle.order.controllers import PvcDeclarationOrderController
from django.conf import settings
from iqportal.iqube.core.models import DataSource
from iqportal.iqube.core.controllers import ProjectController,\
    IQubeRoleController, IQubeRole
from pvcycle.order.schedule import AggregatePvcDeclarationTask
from iqportal.iqube.xmla.result import MdxException
from pvcycle.report.controllers import PvcHistoricalDataController
from pvcycle.order.iqube_sync import PvcDeclarationDataCubeRole
from twisted.trial.unittest import SkipTest
from pvcycle.report.iqube_sync import HistoricalDataCubeRole

class TestCase(pvc_unittest.PvcTestCase):

    def setUp(self):

        self.connect_db()

        self.operator = self._create_operator_account()
        self._attach_permissions_to_group(self.operator.groups[0].group, 'view_datasource')

        self._modules = self._fill_pv_modules(self.operator)

    def _get_declarations_for_tbs(self, tbs, year, month=None, quarter=None, status=None):
        """
        returns all declaration objects for the given tbs and date params.
        Moved from PvcDeclarationController as it was only used in the test cases.
        """

        where = and_(
            PvcTbsMember.tbs==tbs,
            PvcDeclarationOrder.year==year,
            PvcDeclarationOrder.quarter==quarter,
            PvcDeclarationOrder.month==month
        )

        if status != None:
            where = and_(where, PvcDeclarationOrder.status == status)

        return self.sa_session.query(PvcDeclarationOrder).join(PvcTbsMember, PvcDeclarationOrder.tbs_member_id==PvcTbsMember.id).filter(where).all()

    def _create_simple_member_for_declaration(self, modules=None):

        ma = self._create_member_account()
        tbs_account = self._create_tbs_account(self.operator)
        administration_fee = Decimal('250.00')
        contribution_fee = Decimal('2.75')
        service_level = self._create_tbs_service_level(tbs_account, admin_fee=administration_fee, contrib_fee=contribution_fee, year=today().year)
        # apply at TBS
        tm = self._member_apply_at_tbs(ma, tbs_account, service_level)

        # activate member to automatically create an order entry for the Administration Fee
        mc = PvcMemberController(self.sa_session, account_id=tbs_account.id)
        mc.save_tbs_member(tm.id, {'status': PvcTbsMember.STATUS_APPROVED, 'is_full_member': False, 'is_representative': False})

        if modules is None:
            modules = [self._modules[0], self._modules[1]]

        # now that the TBSMember is approved we should be able to save pv modules for this tbs member
        c = PvcPVModuleController(self.sa_session, account_id=ma.id)
        for module in modules:
            c.save_or_update_module_for_tbs_member(tm, tm.tbs.country, module, is_active=True)

        return ma, tm, tbs_account

    def test_declaration_cube_roles(self):

        # create TBS account
        member_account, tbs_member, tbs_account = self._create_simple_member_for_declaration(modules=None)

        # create declaration
        dc = PvcDeclarationOrderController(self.sa_session, account_id=member_account.id)
        d = dc.add_open_declaration(tbs_member, 2013, month=1)

        name = '%s.PvcOrderData' % settings.SETTINGS_MODULE
        ds = self.sa_session.query(DataSource).filter(DataSource.name==name).one()
        role = PvcDeclarationDataCubeRole(self.sa_session, ds)

        # assert operator role
        role_list = role.get_role_list(self.operator)
        self.assertEqual(len(role_list), 1)
        self.assertEqual(role_list[0]['cube'], 'PvcDeclarationDataCube')
        self.assertEqual(role_list[0]['access'], IQubeRole.ACCESS_ALL)
        self.assertEqual(role_list[0]['type'], IQubeRole.MEMBER_TYPE_CUBE)

        # assert tbs role
        role_list = role.get_role_list(tbs_account)
        self.assertEqual(len(role_list), 7) # cube, 1 tbs, 1 member and 1 producer = 7 entries
        self.assertEqual(role_list[0]['cube'], 'PvcDeclarationDataCube')
        self.assertEqual(role_list[0]['access'], IQubeRole.ACCESS_ALL)
        self.assertEqual(role_list[0]['type'], IQubeRole.MEMBER_TYPE_CUBE)
        # first entry is cube, second is top levels for TBS, Member and Producer, 4th is TBS id, 5th is Member id
        self.assertEqual(role_list[4]['access'], IQubeRole.ACCESS_ALL)
        self.assertEqual(role_list[4]['type'], IQubeRole.MEMBER_TYPE_MEMBER)
        self.assertEqual(role_list[4]['unique_name'], '[TBS].[%s]' % tbs_member.tbs.id)
        self.assertEqual(role_list[5]['access'], IQubeRole.ACCESS_ALL)
        self.assertEqual(role_list[5]['type'], IQubeRole.MEMBER_TYPE_MEMBER)
        self.assertEqual(role_list[5]['unique_name'], '[Member].[%s]' % tbs_member.member.id)

        # assert member role (should be same as TBS)
        role_list = role.get_role_list(member_account)
        self.assertEqual(len(role_list), 7) # cube, 1 tbs, 1 member and 1 producer = 7 entries
        self.assertEqual(role_list[0]['cube'], 'PvcDeclarationDataCube')
        self.assertEqual(role_list[0]['access'], IQubeRole.ACCESS_ALL)
        self.assertEqual(role_list[0]['type'], IQubeRole.MEMBER_TYPE_CUBE)
        # first entry is cube, second is top levels for TBS, Member and Producer, 4th is TBS id, 5th is Member id
        self.assertEqual(role_list[4]['access'], IQubeRole.ACCESS_ALL)
        self.assertEqual(role_list[4]['type'], IQubeRole.MEMBER_TYPE_MEMBER)
        self.assertEqual(role_list[4]['unique_name'], '[TBS].[%s]' % tbs_member.tbs.id)
        self.assertEqual(role_list[5]['access'], IQubeRole.ACCESS_ALL)
        self.assertEqual(role_list[5]['type'], IQubeRole.MEMBER_TYPE_MEMBER)
        self.assertEqual(role_list[5]['unique_name'], '[Member].[%s]' % tbs_member.member.id)



    def test_historical_cube_roles(self):

        #create accounts in different groups to test that they don't see their data
        ma1, tm1, ta1 = self._create_simple_member_for_declaration(modules=None)
        ma2, tm2, ta2 = self._create_simple_member_for_declaration(modules=None)

        historical_controller = PvcHistoricalDataController(self.sa_session, account_id=self.operator)
        name = '%s.PVCHistoricalData' % settings.SETTINGS_MODULE

        ds = self.sa_session.query(DataSource).filter(DataSource.name==name).one()
        #grant cube access to operator
        #cube = 'HistoricalDataCube'

        #create data for each account
        historical_controller.save_or_update_historical_data(None, tm1.id, 2012, 155, Decimal('100.1'), 200, Decimal('300.3'), Decimal('23.45'), Decimal('76.55'))
        historical_controller.save_or_update_historical_data(None, tm2.id, 2013, 155, Decimal('200.1'), 300, Decimal('400.3'), Decimal('13.45'), Decimal('86.55'))


        role = HistoricalDataCubeRole(self.sa_session, ds)

        # assert operator role
        role_list = role.get_role_list(self.operator.id)
        self.assertEqual(len(role_list), 1)
        self.assertEqual(role_list[0]['cube'], 'HistoricalDataCube')
        self.assertEqual(role_list[0]['access'], IQubeRole.ACCESS_ALL)
        self.assertEqual(role_list[0]['type'], IQubeRole.MEMBER_TYPE_CUBE)

        # assert ma1 account
        role_list = role.get_role_list(ma1.id)
        self.assertEqual(len(role_list), 5) # cube, tbs and member levels = 5
        self.assertEqual(role_list[0]['cube'], 'HistoricalDataCube')
        self.assertEqual(role_list[0]['access'], IQubeRole.ACCESS_ALL)
        self.assertEqual(role_list[0]['type'], IQubeRole.MEMBER_TYPE_CUBE)
        # 0 = Cube, 1 = TBS, 2 = Member, 3 = TBS.id, 4 = Member.id
        self.assertEqual(role_list[3]['access'], IQubeRole.ACCESS_ALL)
        self.assertEqual(role_list[3]['type'], IQubeRole.MEMBER_TYPE_MEMBER)
        self.assertEqual(role_list[3]['unique_name'], '[TBS].[%s]' % tm1.tbs.id)
        self.assertEqual(role_list[4]['access'], IQubeRole.ACCESS_ALL)
        self.assertEqual(role_list[4]['type'], IQubeRole.MEMBER_TYPE_MEMBER)
        self.assertEqual(role_list[4]['unique_name'], '[Member].[%s]' % tm1.member.id)

        # assert ta1 account (should be equal to ma1)
        role_list = role.get_role_list(ta1.id)
        self.assertEqual(len(role_list), 5) # cube, tbs and member levels = 5
        self.assertEqual(role_list[0]['cube'], 'HistoricalDataCube')
        self.assertEqual(role_list[0]['access'], IQubeRole.ACCESS_ALL)
        self.assertEqual(role_list[0]['type'], IQubeRole.MEMBER_TYPE_CUBE)
        # 0 = Cube, 1 = TBS, 2 = Member, 3 = TBS.id, 4 = Member.id
        self.assertEqual(role_list[3]['access'], IQubeRole.ACCESS_ALL)
        self.assertEqual(role_list[3]['type'], IQubeRole.MEMBER_TYPE_MEMBER)
        self.assertEqual(role_list[3]['unique_name'], '[TBS].[%s]' % tm1.tbs.id)
        self.assertEqual(role_list[4]['access'], IQubeRole.ACCESS_ALL)
        self.assertEqual(role_list[4]['type'], IQubeRole.MEMBER_TYPE_MEMBER)
        self.assertEqual(role_list[4]['unique_name'], '[Member].[%s]' % tm1.member.id)

        # assert ma2 account
        role_list = role.get_role_list(ma2.id)
        self.assertEqual(len(role_list), 5) # cube, tbs and member levels = 5
        self.assertEqual(role_list[0]['cube'], 'HistoricalDataCube')
        self.assertEqual(role_list[0]['access'], IQubeRole.ACCESS_ALL)
        self.assertEqual(role_list[0]['type'], IQubeRole.MEMBER_TYPE_CUBE)
        # 0 = Cube, 1 = TBS, 2 = Member, 3 = TBS.id, 4 = Member.id
        self.assertEqual(role_list[3]['access'], IQubeRole.ACCESS_ALL)
        self.assertEqual(role_list[3]['type'], IQubeRole.MEMBER_TYPE_MEMBER)
        self.assertEqual(role_list[3]['unique_name'], '[TBS].[%s]' % tm2.tbs.id)
        self.assertEqual(role_list[4]['access'], IQubeRole.ACCESS_ALL)
        self.assertEqual(role_list[4]['type'], IQubeRole.MEMBER_TYPE_MEMBER)
        self.assertEqual(role_list[4]['unique_name'], '[Member].[%s]' % tm2.member.id)

        # assert ta2 account (should be equal to ma2)
        role_list = role.get_role_list(ta2.id)
        self.assertEqual(len(role_list), 5) # cube, tbs and member levels = 5
        self.assertEqual(role_list[0]['cube'], 'HistoricalDataCube')
        self.assertEqual(role_list[0]['access'], IQubeRole.ACCESS_ALL)
        self.assertEqual(role_list[0]['type'], IQubeRole.MEMBER_TYPE_CUBE)
        # 0 = Cube, 1 = TBS, 2 = Member, 3 = TBS.id, 4 = Member.id
        self.assertEqual(role_list[3]['access'], IQubeRole.ACCESS_ALL)
        self.assertEqual(role_list[3]['type'], IQubeRole.MEMBER_TYPE_MEMBER)
        self.assertEqual(role_list[3]['unique_name'], '[TBS].[%s]' % tm2.tbs.id)
        self.assertEqual(role_list[4]['access'], IQubeRole.ACCESS_ALL)
        self.assertEqual(role_list[4]['type'], IQubeRole.MEMBER_TYPE_MEMBER)
        self.assertEqual(role_list[4]['unique_name'], '[Member].[%s]' % tm2.member.id)
