'''
Created on 8 Aug 2013

@author: martijn
'''
from datetime import date

import random
from decimal import Decimal
from basesoft.web.contrib.file.models import File
from pvcycle.order.models import PvcOrder
from pvcycle.order.models import PvcOrderLineAdministrationFee

from pvcycle.test.unit import pvc_unittest
from pvcycle.core.controllers import PvcMemberController, ContractReplacementFieldsContext,\
    ServiceLevelReplacementFieldsContext

from basesoft.web.contrib.account.controllers import AccountController
from basesoft.web.contrib.account.models import AccountActivation, Account
from basesoft.web.test.unit.test_abstract_controller import MockRequest

from basesoft.web.core.exceptions import PublicUserException
from basesoft.web.contrib.contact.controllers import SimpleContactController
from basesoft.web.contrib.i18n.models import Locale, Country
from basesoft.web.core.util.controllers import LicenseConfigController
from basesoft.web.contrib.wiki.parser import Parser
from basesoft.util.virtual_date import today, set_date

from pvcycle.core.models import PvcTbsMember, PvcTbsMemberServiceLevel, PvcTbsMemberCertificate, PvcTbsLicense, \
    PvcTbsServiceLevel, PvcTbsMemberProofFinancialGuaranteeFile
from pvcycle.core.exceptions import PvcCertificateNotFoundException, PvcCertificateInvalidException
from pvcycle.order.controllers import PvcOrderController
from basesoft.web.contrib.notification.controllers import NotificationController

class TestCase(pvc_unittest.PvcTestCase):

    def setUp(self):
        self.connect_db()

    def test_member_account(self):

        rand_id = str(random.random())[2:]
        username = 'user_%s' % rand_id
        email = '%s@mail-test.basesosft.com' % username

        ac = AccountController(self.sa_session, request=MockRequest())
        account = ac.create_inactive_account(username, 'temp_password', email)

        # fake send email to user
        activation = self.sa_session.query(AccountActivation).filter(
            AccountActivation.account == self.sa_session.query(Account).filter(Account.name == username).one()).one()
        activation.activation_stage = AccountActivation.ACTIVATION_STAGE_PENDING
        self.sa_session.add(activation)
        self.sa_session.flush()

        # activate account
        ac = AccountController(self.sa_session)
        aa = ac.get_account_activation(account)
        self.assertRaises(PublicUserException, ac.activate_account, aa.activation_key)

        # add person info
        locale = self.sa_session.query(Locale).filter(Locale.locale=='en_GB').one()

        scc = SimpleContactController(self.sa_session, account_id=aa.account.id, check_active=False)
        state = {
            'preferred_locale': locale.id,
            'last_name': 'Nguyen',
            'first_name': 'Huie',
        }

        # update the person preferred locale
        person = scc._save_or_update_person(account.person.id, state)
        self.assertEqual(person.preferred_locale, locale)
        self.assertEqual(person.preferred_locale.locale, scc.config.get('basesoft.web.contrib.locale'))

        account = ac.activate_account(aa.activation_key)

        # controller
        mc = PvcMemberController(self.sa_session, account_id=account.id)

        # currently the license is a normal one
        self.assertEqual(account.groups[0].group.license.polymorphic_type , 'License')
        self.assertEqual(account.groups[0].group.license.company, None)

        # company state
        company_state = {
            'dbID': account.groups[0].group.license.id,
            'company': {
                'name': 'test company',
                'vat_no': 'BE0893027827'
            },
            'address_data': {
                'street': 'Urkerweg 1234'
            },
            'bank_data': {
                'number': 1234567,
                'iban': "iban",
                'bic': 'bic'
            },

        }

        # save company info
        mc.save_member_license(company_state)
        c = account.groups[0].group.license.company
        self.assertTrue(c.contact_person is None)
        self.assertEqual(c.addresses[0].address.street, 'Urkerweg 1234')
        self.assertEqual(c.bank_accounts[0].bank_account.number, '1234567')

        contact_state = {
            'dbID': account.groups[0].group.license.id,
            'contact_person_data': {
                'first_name': 'Contact Person First',
                'last_name': 'Contact Person Name',
            },
            'invoice_person_data': {
                'is_same_person_as': True
            }
        }

        mc.save_member_license(contact_state)
        c = account.groups[0].group.license.company

        # assert a contact person is set for this company
        self.assertEqual(c.contact_person.last_name, 'Contact Person Name')
        self.assertEqual(c.contact_person.first_name, 'Contact Person First')

        self.assertEqual(c.contact_person, c.invoice_person)

        # assert that the new license is now a PvcMemberLicense
        self.assertEqual(account.default_group.license.polymorphic_type , 'PvcMemberLicense')
        member_license_id = account.default_group.license.id

        # setup a tbs with service level
        tbs_account = self._create_tbs_account()

        nc = NotificationController(self.sa_session, account_id=tbs_account.id)
        cnt = nc.get_unread_notifications_count()

        tbs_license_id = tbs_account.default_group.license.id
        service_level = self._create_tbs_service_level(tbs_account)

        upload_contract = self._create_uploaded_file('upload_contract.pdf', 'upload_contract contents')

        is_commissioner = True
        is_producer = True

        # apply at tbs
        mc.apply_at_tbs(tbs_license_id, member_license_id, None, service_level.id, None, is_commissioner, is_producer, upload_contract)
        self.assertEqual(cnt+1, nc.get_unread_notifications_count())

        # assert member at tbs
        tbs_members = self.sa_session.query(PvcTbsMember).filter(PvcTbsMember.member_id == member_license_id).all()
        self.assertEqual(len(tbs_members), 1)
        tbs_member = tbs_members[0]

        # assert service level
        self.assertEqual(self.sa_session.query(PvcTbsMemberServiceLevel).filter(PvcTbsMemberServiceLevel.tbs_member_id == tbs_member.id).count(), 1)

        # TODO: assert current/next year fee

    def test_italy_member_of_2013(self):
        tbs_account = self._create_tbs_account()
        member_account = self._create_member_account()
        service_level = self._create_tbs_service_level(tbs_account)

        tbs = tbs_account.groups[0].group.license
        lcc = LicenseConfigController(self.sa_session, scope=tbs)
        lcc.set('pvc.tbs.was.active.in.2013', True)

        tbs_member = self._member_apply_at_tbs(member_account, tbs_account, service_level, upload_contract=False, was_member_in_2013=True)
        self.assertTrue(tbs_member.was_member_in_2013)
        self.assertEqual(tbs_member.status, PvcTbsMember.STATUS_PENDING)

        # now test for TBS that doesnt have the config set
        tbs_account = self._create_tbs_account()
        member_account = self._create_member_account()
        service_level = self._create_tbs_service_level(tbs_account)

        tbs_member = self._member_apply_at_tbs(member_account, tbs_account, service_level, upload_contract=False, was_member_in_2013=True)
        self.assertFalse(tbs_member.was_member_in_2013)
        self.assertEqual(tbs_member.status, PvcTbsMember.STATUS_INCOMPLETE)

    def test_certificate(self):
        tbs_account = self._create_tbs_account()
        member_account = self._create_member_account()
        service_level = self._create_tbs_service_level(tbs_account)
        tbs_member = self._member_apply_at_tbs(member_account, tbs_account, service_level, upload_contract=False)
        tbs_member.status = PvcTbsMember.STATUS_APPROVED

        c = PvcOrderController(self.sa_session)

        # Insert Certificate manual, test scenarios only, normally this is done on Order Payment.
        certificate = PvcTbsMemberCertificate()
        certificate.year = today().year
        certificate.tbs_member = tbs_member
        certificate.status = PvcTbsMemberCertificate.STATUS_APPROVED
        certificate.hash = c.generate_unique_hash()
        self.sa_session.add(certificate)

        c = PvcMemberController(self.sa_session)
        self.assertEqual(certificate, c.validate_certificate(certificate.hash))

        self.assertRaises(PvcCertificateNotFoundException, c.validate_certificate, 'wontwork')

        certificate.year = today().year-1
        self.sa_session.add(certificate)
        self.sa_session.flush()

        self.assertRaises(PvcCertificateInvalidException, c.validate_certificate, certificate.hash)

        certificate.year = today().year
        certificate.status = PvcTbsMemberCertificate.STATUS_REVOKED
        self.sa_session.add(certificate)
        self.sa_session.flush()

        self.assertRaises(PvcCertificateInvalidException, c.validate_certificate, certificate.hash)

        # #453: suspend member, certificate shouldnt be valid anymore
        tbs_member.status = PvcTbsMember.STATUS_SUSPENDED
        certificate.year = today().year
        certificate.status = PvcTbsMemberCertificate.STATUS_APPROVED
        self.sa_session.add(certificate)
        self.sa_session.flush()
        self.assertRaises(PvcCertificateInvalidException, c.validate_certificate, certificate.hash)


    def test_apply_at_local_tbs(self):
        # test for ticket #391
        state = self._get_tbs_license_state()
        state['tbs']['tbs_type'] = PvcTbsLicense.TBS_TYPE_LOCAL
        tbs_account = self._create_tbs_account(state=state)

        member_account = self._create_member_account()
        service_level = self._create_tbs_service_level(tbs_account)
        self.assertRaises(PublicUserException, self._member_apply_at_tbs, member_account, tbs_account, service_level, upload_contract=False)

    def test_member_commissioner_only(self):
        tbs_account = self._create_tbs_account()
        member_account = self._create_member_account()
        service_level = None
        tbs = tbs_account.groups[0].group.license

        tbs_member = self._member_apply_at_tbs(
            member_account, tbs_account, service_level,
            is_producer=False, is_commissioner=True, upload_contract=False)

        self.assertFalse(tbs_member.is_producer)
        self.assertTrue(tbs_member.is_commissioner)

        self.assertEqual(tbs_member.status, PvcTbsMember.STATUS_PENDING)

        self.assertEqual(len(tbs_member.service_levels), 0)

    def test_disable_as_producer(self):
        tbs_account = self._create_tbs_account()
        service_level = self._create_tbs_service_level(tbs_account)
        member_account = self._create_member_account()

        tm = self._member_apply_at_tbs(member_account, tbs_account, service_level, is_producer=True, is_commissioner=True)
        tm.status = PvcTbsMember.STATUS_APPROVED
        self.sa_session.add(tm)
        self.sa_session.flush()

        operator = self._create_operator_account()

        mc = PvcMemberController(self.sa_session, account_id=operator.id)
        mc.disable_producer(tm.id)

        self.sa_session.expire(tm)
        self.assertFalse(tm.is_producer)

    def test_disable_service_level_for_registration(self):
        tbs_account = self._create_tbs_account()
        service_level = self._create_tbs_service_level(tbs_account)

        # create another service level and disable it.
        service_level1 = self._create_tbs_service_level(tbs_account, name='Duex', status=PvcTbsServiceLevel.STATUS_DISABLED_FOR_NEW_REGISTRATIONS, level=2)
        member_account = self._create_member_account()

        self.assertRaises(PublicUserException, self._member_apply_at_tbs, member_account, tbs_account, service_level1, is_producer=True, is_commissioner=True)
        # and this one shouldnt fail
        self._member_apply_at_tbs(member_account, tbs_account, service_level, is_producer=True, is_commissioner=True)

    def test_contract_output(self):
        state = self._get_tbs_license_state()
        state['tbs']['contract_text'] = """
        Contract for @member:name with VAT NO @member:vat_no

        Address:

        @member_address:street
        @member_address:street_2
        @member_address:postcode @member_address:city
        @country:name
        """

        tbs_account = self._create_tbs_account(state=state)
        tbs = tbs_account.groups[0].group.license
        member_account = self._create_member_account()
        member = member_account.groups[0].group.license
        service_level = self._create_tbs_service_level(tbs_account, annex_text="""

        ::SL Contract::

        This is a contract for @member:name with an ADMIN fee of @service_level:current_admin_fee

        The contribution fee is set to @service_level:current_contrib_fee

        """, year=today().year)

        parser = Parser()

        contract_text = parser.render_to_html(tbs.contract_text, ContractReplacementFieldsContext(member=member, tbs=tbs))
        self.assertEqual("""<p>\n        Contract for member company with VAT NO BE0893027827 <br />\n</p>\n<p>\n        Address: <br />\n</p>\n<p>\n        Some Shanghai Street Name <br />\n         <br />\n        GA SHA Shanghai <br />\n        Belgium <br />\n         <br />\n</p>\n""", contract_text)

        sl = Parser().render_to_html(service_level.contract_annex_text, ServiceLevelReplacementFieldsContext(member=member, service_level=service_level))
        self.assertEqual("""<p>\n        <center>SL Contract</center> <br />\n</p>\n<p>\n        This is a contract for member company with an ADMIN fee of 11.12 <br />\n</p>\n<p>\n        The contribution fee is set to 11.12 <br />\n</p>\n<p>\n         <br />\n</p>\n""", sl)



    def test_member_upload_later(self):

        tbs_account = self._create_tbs_account()
        tbs_license_id = tbs_account.default_group.license.id
        member_account = self._create_member_account()
        service_level = self._create_tbs_service_level(tbs_account)

        # apply without contract
        tbs_member = self._member_apply_at_tbs(member_account, tbs_account, service_level, upload_contract=False)

        # assert status is unfinished
        self.assertEquals(tbs_member.status, PvcTbsMember.STATUS_INCOMPLETE)

        # upload a new contract
        member_license_id = member_account.groups[0].group.license.id
        upload_contract = self._create_uploaded_file('upload_contract.pdf', 'upload_contract contents')
        mc = PvcMemberController(self.sa_session, account_id=member_account.id)
        mc.apply_at_tbs(tbs_license_id, member_license_id, tbs_member.id, service_level.id, None, is_commissioner=False, is_producer=True, upload_contract=upload_contract)

    def test_member_second_company(self):

        tbs_account = self._create_tbs_account()
        member_account = self._create_member_account()
        service_level = self._create_tbs_service_level(tbs_account)
        tbs_member = self._member_apply_at_tbs(member_account, tbs_account, service_level, upload_contract=False)

        # current license is license1
        license1_id = member_account.groups[0].group.license.id

        # now create 2nd company, pass in license_id = None and expect new license

        company_state = {
            'dbID': None, # new license
            'company': {
                'name': 'test company',
                'vat_no': 'BE0893027827',
            },
            'address_data': {
                'street': 'Urkerweg 1234'
            },
            'bank_data': {
                'number': 1234567,
                'iban': "iban",
                'bic': 'bic'
            },

        }

        mc = PvcMemberController(self.sa_session, account_id=member_account.id)
        license_new = mc.save_member_license(company_state)

        # expect different license id
        self.assertNotEqual(license1_id, license_new.id)

        # save person info for the new company
        contact_state = {
            'dbID': license_new.id,
            'contact_person_data': {
                'first_name': 'Contact Person First',
                'last_name': 'Contact Person Name',
            },
            'invoice_person_data': {
                'is_same_person_as': True
            }
        }

        mc = PvcMemberController(self.sa_session, account_id=member_account.id)
        mc.save_member_license(contact_state)

    def test_apply_at_dts_tbs(self):
        """
        This is a test case for the UK DTS implementation.
        """
        state = self._get_tbs_license_state()
        state['configs']['pvc.tbs.use.membership.instead.of.service.level'] = True
        state['configs']['pvc.tbs.auto.approve.member'] = True

        tbs_account = self._create_tbs_account(state=state)
        tbs = tbs_account.default_group.license

        memberships = self._create_memberships(tbs_account)

        member_account = self._create_member_account()

        mc = PvcMemberController(self.sa_session, account_id=member_account.id)

        tbs_member = mc.apply_at_tbs(tbs.id, member_account.default_group.license.id, None, None, None, False, False, None, membership_id=memberships[0].id)
        self.assertTrue(mc.has_permissions('save_dashboard'))
        self.assertFalse(mc.has_permissions('pvc_select_pv_modules'))

        self.assertEqual(tbs_member.status, PvcTbsMember.STATUS_APPROVED)

    def test_apply_with_company_reg_number(self):
        """
        This is a test case for the UK DTS implementation.
        """
        state = self._get_tbs_license_state()
        state['configs']['pvc.tbs.use.membership.instead.of.service.level'] = True
        state['configs']['pvc.tbs.require.company.registration.number'] = True

        tbs_account = self._create_tbs_account(state=state)
        tbs = tbs_account.default_group.license

        memberships = self._create_memberships(tbs_account)

        member_account = self._create_member_account()

        mc = PvcMemberController(self.sa_session, account_id=member_account.id)

        self.assertRaises(AssertionError, mc.apply_at_tbs, tbs.id, member_account.default_group.license.id, None, None, None, False, True, None, membership_id=memberships[0].id)

        tm = mc.apply_at_tbs(tbs.id, member_account.default_group.license.id, None, None, None, False, True, None, membership_id=memberships[0].id, company_reg_no='ABCD999')

        self.assertEqual(tm.company_registration_number, 'ABCD999')

    def test_apply_with_no_uk_vat(self):
        state = self._get_tbs_license_state()
        state['configs']['pvc.tbs.optional.uk.vat.no'] = False

        tbs_account = self._create_tbs_account(state=state)
        tbs = tbs_account.default_group.license
        service_level = self._create_tbs_service_level(tbs_account)

        # first we test at the TBS where VAT for UK is NOT optional
        state = self._get_member_license_state('test member')
        state['company']['vat_no'] = None
        state['address_data']['country'] = self.sa_session.query(Country).filter(Country.iso_3166_a2=='GB').one().id

        member_account = self._create_member_account(state=state)
        mc = PvcMemberController(self.sa_session, account_id=member_account.id)
        self.assertRaises(PublicUserException, mc.apply_at_tbs, tbs.id, member_account.default_group.license.id, None, service_level.id, None, True, False, None)

        state = self._get_tbs_license_state()
        state['configs']['pvc.tbs.optional.uk.vat.no'] = True

        tbs_account = self._create_tbs_account(state=state)
        tbs = tbs_account.default_group.license
        service_level = self._create_tbs_service_level(tbs_account)
        # now it should not raise an exception
        mc.apply_at_tbs(tbs.id, member_account.default_group.license.id, None, service_level.id, None, True, False, None)

    def test_apply_at_tbs_with_same_country_requirement(self):
        state = self._get_tbs_license_state()
        state['configs']['pvc.tbs.require.member.in.own.country'] = True

        tbs_account = self._create_tbs_account(state=state)
        tbs = tbs_account.default_group.license
        service_level = self._create_tbs_service_level(tbs_account)

        # first we test with different countries
        state = self._get_member_license_state('test member')
        state['address_data']['country'] = self.sa_session.query(Country).filter(Country.iso_3166_a2=='GB').one().id

        member_account = self._create_member_account(state=state)
        mc = PvcMemberController(self.sa_session, account_id=member_account.id)
        # Different countries will raise an exception
        self.assertRaises(PublicUserException, mc.apply_at_tbs, tbs.id, member_account.default_group.license.id, None, service_level.id, None, True, False, None)

        # now we test with the same country as the TBS (Netherlands)
        state = self._get_member_license_state('test member')
        state['address_data']['country'] = self.sa_session.query(Country).filter(Country.iso_3166_a2=='NL').one().id

        member_account = self._create_member_account(state=state)
        mc = PvcMemberController(self.sa_session, account_id=member_account.id)
        mc.apply_at_tbs(tbs.id, member_account.default_group.license.id, None, service_level.id, None, True, False, None)

    def test_archive_tbs_member(self):
        from decimal import Decimal
        tbs_account = self._create_tbs_account()
        service_level = self._create_tbs_service_level(tbs_account, admin_fee=Decimal('250.00'))
        member_account = self._create_member_account()
        tm = self._member_apply_at_tbs(member_account, tbs_account, service_level=service_level)
        tm.status = PvcTbsMember.STATUS_APPROVED
        self.sa_session.add(tm)

        mc = PvcMemberController(self.sa_session, account_id=tbs_account.id)

        self.assertRaises(PublicUserException, mc.archive_tbs_member, tm.id)

        tm.status = PvcTbsMember.STATUS_SUSPENDED

        mc.archive_tbs_member(tm.id)

        self.sa_session.expire(tm)
        self.assertTrue(tm.archived)

    def test_member_upload_proof_of_financial_guarantee(self):

        tbs_account = self._create_tbs_account()
        tbs = tbs_account.default_group.license
        tbs_license_id = tbs_account.default_group.license.id
        member_account = self._create_member_account()
        operator_account = self._create_operator_account()
        service_level = self._create_tbs_service_level(tbs_account)

        lcc = LicenseConfigController(self.sa_session, scope=tbs)
        lcc.set('pvc.tbs.require.proof.of.financial.guarantee', True)

        # apply at tbs
        tbs_member = self._member_apply_at_tbs(member_account, tbs_account, service_level, upload_contract=False, financial_guarantee=True)

        # assert file is saved and is the same as the linked to the tbs member
        f = self.sa_session.query(PvcTbsMemberProofFinancialGuaranteeFile).order_by(PvcTbsMemberProofFinancialGuaranteeFile.id.desc()).first()
        self.assertEquals(f, tbs_member.guarantee_file)

        # get the file as operator = no errors
        oc = PvcMemberController(sa_session=self.sa_session, account_id=operator_account.id)
        proof = oc.get_proof_of_financial_guarantee(tbs_member.id)
        self.assertEqual(proof[0], tbs_member.guarantee_file)

        # get the file as a tbs at which member has applied = no errors
        oc = PvcMemberController(sa_session=self.sa_session, account_id=tbs_account.id)
        proof = oc.get_proof_of_financial_guarantee(tbs_member.id)
        self.assertEqual(proof[0], tbs_member.guarantee_file)

        # get the file as a different tbs = raise error
        tbs_account = self._create_tbs_account()
        oc = PvcMemberController(sa_session=self.sa_session, account_id=tbs_account.id)
        self.assertRaises(IndexError, oc.get_proof_of_financial_guarantee, tbs_member.id)

    def test_member_change_service_level(self):

        set_date(date(2015, 1, 1))

        operator_account = self._create_operator_account()
        tbs_account = self._create_tbs_account()
        member_account = self._create_member_account()

        service_level_1 = self._create_tbs_service_level(tbs_account, name='SL A', year=today().year, admin_fee=Decimal('250.00'), level=1)
        service_level_2 = self._create_tbs_service_level(tbs_account, name='SL B', year=today().year, admin_fee=Decimal('300.00'), level=2)

        tbs_member = self._member_apply_at_tbs(member_account, tbs_account, service_level_1)

        # tbs approves the member
        mc = PvcMemberController(self.sa_session, account_id=tbs_account.id)
        mc.save_tbs_member(tbs_member.id, {'status': PvcTbsMember.STATUS_APPROVED, 'is_full_member': False, 'is_representative': False})

        # member requests to change service level
        set_date(date(2015, 1, 2))
        upload_contract = self._create_uploaded_file('upload_contract.pdf', 'upload_contract contents')
        c = PvcMemberController(sa_session=self.sa_session, account_id=member_account.id)
        member_service_level = c.request_change_service_level(tbs_member.id, service_level_2.id, upload_contract)
        self.assertEqual(member_service_level.status, PvcTbsMemberServiceLevel.STATUS_PENDING)

        # assert current service level is still 1
        self.assertEqual(tbs_member._get_active_service_level().service_level.id, service_level_1.id)

        # peek in feb and expect SL1 to be active (cuz not yet approved)
        set_date(date(2015, 2, 1))
        self.assertEqual(tbs_member._get_active_service_level().service_level.id, service_level_1.id)
        set_date(date(2015, 1, 2))

        # operator rejects the request
        c_o = PvcMemberController(sa_session=self.sa_session, account_id=operator_account.id)
        c_o.reject_change_service_level(member_service_level.id)

        # member re-requests to change service level
        upload_contract = self._create_uploaded_file('upload_contract.pdf', 'upload_contract contents')
        c = PvcMemberController(sa_session=self.sa_session, account_id=member_account.id)
        member_service_level = c.request_change_service_level(tbs_member.id, service_level_2.id, upload_contract)

        # operator approves the request
        c_o = PvcMemberController(sa_session=self.sa_session, account_id=operator_account.id)
        c_o.approve_change_service_level(member_service_level.id)
        self.assertEqual(member_service_level.status, PvcTbsMemberServiceLevel.STATUS_APPROVED)

        # goto feb and expect SL2 to be active
        set_date(date(2015, 2, 1))
        self.assertEqual(tbs_member._get_active_service_level().service_level.id, service_level_2.id)

        # expect to have a new AF order with 50 eurie for the second order
        poc = PvcOrderController(self.sa_session, account_id=member_account.id)
        orders = poc.get()
        self.assertEqual(len(orders), 2)
        order = orders[1]
        self.assertTrue(isinstance(order, PvcOrder))
        self.assertEqual(len(order.lines), 1)
        self.assertTrue(isinstance(order.lines[0], PvcOrderLineAdministrationFee))
        self.assertEqual(order.lines[0].price_per_unit, Decimal('50.0000'))
        self.assertEqual(order.total_amount_excluding_vat, Decimal('50.0000'))
        self.assertEqual(order.lines[0].line_amount_including_vat, Decimal('50.0000'))

        # downgrade back to SL1
        set_date(date(2015, 2, 2))
        upload_contract = self._create_uploaded_file('upload_contract.pdf', 'upload_contract contents')
        member_service_level = c.request_change_service_level(tbs_member.id, service_level_1.id, upload_contract)
        c_o.approve_change_service_level(member_service_level.id)

        # expect no new invoice
        self.assertEqual(len(poc.get()), 2)

        # assert current service level is still 2
        self.assertEqual(tbs_member._get_active_service_level().service_level.id, service_level_2.id)

        # goto december and expect SL2 still to be active
        set_date(date(2015, 12, 1))
        self.assertEqual(tbs_member._get_active_service_level().service_level.id, service_level_2.id)

        # goto jan 2016 and expect SL1 to be active
        set_date(date(2016, 1, 1))
        self.assertEqual(tbs_member._get_active_service_level().service_level.id, service_level_1.id)

        # TODO: test periodicity quarterly / yearly

# below is for running the test from eclipse
if __name__ == '__main__':

    from twisted.scripts.trial import run
    import sys
    sys.argv = ['test_member.py', 'pvcycle.test.unit.test_member']
    run()
    sys.exit()

