'''
Created on Jul 24, 2014

@author: rado
'''
from iqportal.iqube.test.unit.test_configurable_widget_reporters import AbstractConfigurableWidgetReporterTestCase
from basesoft.web.core.util.controllers import AbstractController
from iqportal.iqube.core.models import DataSourceWidget, DataSource
from pvcycle.core.controllers import PvcMemberController
from pvcycle.core.models import PvcTbsMember
from pvcycle.pvmodule.controllers import PvcPVModuleController
from decimal import Decimal
from basesoft.util.virtual_date import today
from pvcycle.test.unit import pvc_unittest
from pvcycle.order.controllers import PvcDeclarationOrderController
from pvcycle.order.schedule import AggregatePvcDeclarationTask
from django.conf import settings

class TestCase(AbstractConfigurableWidgetReporterTestCase, pvc_unittest.PvcTestCase):
    DATASOURCE = 'PvcOrderData'

    def setUp(self):

        self.connect_db()
        self._start_django()

        self.account = self._create_operator_account()
        self.group = self.account.groups[0].group
        self._attach_permissions_to_group(self.group,
                'view_datasource', 'view_datasource_widget', 'create_mdx_result', 'view_mdx_result',
                )
        widget = self.sa_session.query(DataSourceWidget).filter(DataSourceWidget.label == 'Declaration result (old)').one()
        self._ds = self.sa_session.query(DataSource).filter(DataSource.name == '%s.PvcOrderData' % settings.SETTINGS_MODULE).one()

        c = AbstractController(sa_session=self.sa_session, account_id=None)
        c.grant_privilege_to(widget, privileges=DataSourceWidget.all_privileges, group=True, target_group=self.group, check_own_privs=False)

        self.set_datasource()

        self._modules = self._fill_pv_modules(self.account)

    def _create_simple_member_for_declaration(self, modules=None):

        ma = self._create_member_account()
        tbs_account = self._create_tbs_account(self.account)
        administration_fee = Decimal('250.00')
        contribution_fee = Decimal('2.75')
        service_level = self._create_tbs_service_level(tbs_account, admin_fee=administration_fee, contrib_fee=contribution_fee, year=today().year)
        # apply at TBS
        tm = self._member_apply_at_tbs(ma, tbs_account, service_level)

        # activate member to automatically create an order entry for the Administration Fee
        mc = PvcMemberController(self.sa_session, account_id=tbs_account.id)
        mc.save_tbs_member(tm.id, {'status': PvcTbsMember.STATUS_APPROVED, 'is_full_member': False, 'is_representative': False})

        if modules is None:
            modules = [self._modules[0], self._modules[1]]

        # now that the TBSMember is approved we should be able to save pv modules for this tbs member
        c = PvcPVModuleController(self.sa_session, account_id=ma.id)
        for module in modules:
            c.save_or_update_module_for_tbs_member(tm, tm.tbs.country, module, is_active=True)

        return ma, tm

    def test_aggregation_update(self):

        import time

        ma1, tm1 = self._create_simple_member_for_declaration(modules=[self._modules[0]])
        ma2, tm2 = self._create_simple_member_for_declaration(modules=[self._modules[0]])
        # trigger new declarations
        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma1.id)
        d1 = dc.add_open_declaration(tm1, 2013, month=1)
        d2 = dc.add_open_declaration(tm2, 2013, month=1)

        AggregatePvcDeclarationTask().execute()

        q = """
            SELECT order_id, last_updated FROM aggPvcDeclaration
            WHERE order_id IN (%(orders)s)
        """ % {'orders': '%d, %d' % (d1.id, d2.id)}

        orders = self.sa_session.execute(q).fetchall()
        self.assertEqual(len(orders), 2)

        verify = {}
        for o in orders:
            verify[o['order_id']] = o['last_updated']

        # sleep so that last updated is definitely different.
        time.sleep(2)

        # update a field on d2
        d2.auto_submitted = 1 if d2.auto_submitted == 0 else 0
        self.sa_session.add(d2)
        self.sa_session.flush()

        # rerun aggregate
        AggregatePvcDeclarationTask().execute()

        # the second order should be updated in the aggregation table
        orders = self.sa_session.execute(q).fetchall()
        self.assertEqual(len(orders), 2)
        for o in orders:
            if o['order_id'] == d2.id:
                self.assertNotEqual(verify[d2.id], o['last_updated'])
            else:
                self.assertEqual(verify[d1.id], o['last_updated'])

    def test_pvc_declaration_reporter_with_mdx_formula(self):

        reporter, applied_config = self._get_configurable_widget_defaults('Declaration result (old)')
        ma1, tm1 = self._create_simple_member_for_declaration(modules=None)
        ma2, tm2 = self._create_simple_member_for_declaration(modules=None)
        # trigger new declarations
        dc = PvcDeclarationOrderController(self.sa_session, account_id=ma1.id)
        dc.add_open_declaration(tm1, 2013, month=1)
        dc.add_open_declaration(tm2, 2013, month=1)

        AggregatePvcDeclarationTask().execute()

        dimensions = {
            'TBS': ['[TBS].[TBS].[%s]'%tm1.tbs.id, '[TBS].[TBS].[%s]'%tm2.tbs.id],
            'Is Full Member': ['[Is Full Member].[0]'],
            'Country' : ['[Country].[%s]'%tm1.tbs.country.id],
            'Member': ['[Member].[Member].Members'],
            'Measures': ["[Measures].[Amount (excl. vat)]"]
        }
        applied_config = self._smorgle_applied_config(applied_config, dimensions, 'PvcDeclarationDataCube')
        applied_config['select_axis'] = 'rows'
        applied_config['count_opts'] = 'TopCount'
        applied_config['number_of_members'] = '5'
        # move the dimensions to the correct axis
        measures_index = applied_config['dimensions']['Measures']
        applied_config['axes_%s' % measures_index] = 'columns'
        member_index = applied_config['dimensions']['Member']
        applied_config['axes_%s' % member_index] = 'columns'
        applied_config['column_order'] = ['Measures', 'Member']

        member_index = applied_config['dimensions']['TBS']
        applied_config['axes_%s' % member_index] = 'rows'
        member_index = applied_config['dimensions']['Is Full Member']
        applied_config['axes_%s' % member_index] = 'rows'
        member_index = applied_config['dimensions']['Country']
        applied_config['axes_%s' % member_index] = 'rows'
        applied_config['row_order'] = ['TBS', 'Is Full Member', 'Country']

        mdx = self._get_mdx(reporter.get_mdx_config(applied_config))
        self.assertMdx("""
            WITH SET [Static Count] AS TopCount(CROSSJOIN(CROSSJOIN({[TBS].[%s],[TBS].[%s]}, {[Is Full Member].[0]}), {[Country].[%s]}), 5, [Measures].[Amount (excl. vat)])
            SELECT NON EMPTY CROSSJOIN({[Measures].[Amount (excl. vat)]}, {[Member].[Member].Members}) ON COLUMNS,
                NON EMPTY {[Static Count]} ON ROWS
            FROM [PvcDeclarationDataCube]
        """% (tm1.tbs.id, tm2.tbs.id, tm1.tbs.country.id), mdx[0])
