'''
Created on Jan 30, 2014

@author: marten
'''
from basesoft.util.virtual_date import today
from basesoft.web.contrib.i18n.models import Locale
from basesoft.web.contrib.notification.models import Notification
from basesoft.web.core.exceptions import PublicUserException, PublicSecurityException, PermissionDeniedException

from pvcycle.test.unit import pvc_unittest
from pvcycle.core.controllers import PvcNotificationController, PvcMemberController
from pvcycle.subrogation.controllers import PvcSubrogationController
from pvcycle.subrogation.models import PvcSubrogation
from pvcycle.subrogation.mailing import MailController
from pvcycle.subrogation.schedule import SubgrogationMailTask
from pvcycle.pvmodule.models import PvcTbsMemberPVModule
from pvcycle.pvmodule.controllers import PvcPVModuleController

class TestCase(pvc_unittest.PvcTestCase):

    def setUp(self):
        self.connect_db()

        self.operator = self._create_operator_account()
        self.locale = self.session.query(Locale).filter_by(locale='en_GB').one()

        self.ma = self._create_member_account('SUBROGATOR')
        self.ta = self._create_tbs_account(self.operator)

        self.service_level = self._create_tbs_service_level(self.ta, year=today().year)

        self.tm = self._member_apply_at_tbs(self.ma, self.ta, self.service_level)
        self.tm.status = self.tm.STATUS_APPROVED
        self.sa_session.add(self.tm)

        self.ca = self._create_member_account('COMMISSIONER')
        self.ctm = self._member_apply_at_tbs(self.ca, self.ta, self.service_level, is_producer=False, is_commissioner=True)
        self.ctm.status = self.tm.STATUS_APPROVED
        self.sa_session.add(self.ctm)
        self.sa_session.flush()

    def _setup_mails(self):
        # TODO: mandate mails.
        controller = MailController(self.session, self.locale, account_id=self.operator.id)
        controller.save_state(self._get_rejection_mail_state())
        controller.save_state(self._get_subrogator_mandate_mail_state())
        controller.save_state(self._get_commissioner_mandate_mail_state())

    def _get_rejection_mail_state(self):
        return {
            "mail": {
                "recipients": ['domain_%s' % self.ma.default_group.license.domain.id],
                "mail_from_text": "MockMailer",
                "name": u"Subrogation rejected",
                "objectName": u"PvcSubrogationRejectionMail",
                "dbID": None,

                "locales": {
                    'en_GB': {
                        'subject': u"Rejection of Subrogation",
                        'content': u'Dear <img  src="/basesoft.web.core/rte/make_image/Person%3ALast%20Name" title="person.last_name" class="replacement-field ">. Reject reason <img  class="replacement-field " title="subrogation.reject_reason" src="/basesoft.web.core/rte/make_image/Subrogation%3AReject%20Reason"> '
                    },
                },
                "fallbackLocale": {'value': u'en_GB'},
            }
        }

    def _get_subrogator_mandate_mail_state(self):
        return {
            "mail": {
                "recipients": ['domain_%s' % self.ma.default_group.license.domain.id],
                "mail_from_text": "MockMailer",
                "name": u"Subrogation mandate signature required",
                "objectName": u"PvcSubrogationSubrogatorMandateMail",
                "dbID": None,

                "locales": {
                    'en_GB': {
                        'subject': u"Mandate signature required of Subrogator",
                        'content': u'Dear <img  src="/basesoft.web.core/rte/make_image/Person%3ALast%20Name" title="person.last_name" class="replacement-field ">. etc. '
                    },
                },
                "fallbackLocale": {'value': u'en_GB'},
            }
        }

    def _get_commissioner_mandate_mail_state(self):
        return {
            "mail": {
                "recipients": ['domain_%s' % self.ma.default_group.license.domain.id],
                "mail_from_text": "MockMailer",
                "name": u"Commissioner mandate signature required",
                "objectName": u"PvcSubrogationCommissionerMandateMail",
                "dbID": None,

                "locales": {
                    'en_GB': {
                        'subject': u"Mandate signature required of Commissioner",
                        'content': u'Dear <img  src="/basesoft.web.core/rte/make_image/Person%3ALast%20Name" title="person.last_name" class="replacement-field ">. etc. '
                    },
                },
                "fallbackLocale": {'value': u'en_GB'},
            }
        }

    def _request_for_subrogation(self, commissioner_name='COMMY'):
        controller = PvcSubrogationController(self.sa_session, account_id=self.ma.id)
        return controller.request_for_subrogation(self.tm, commissioner_name)

    def test_upgrade_to_commissioner(self):
        mc = PvcMemberController(self.sa_session, account_id=self.ma.id)
        self.assertRaises(PermissionDeniedException, mc.upgrade_to_commissioner, self.tm.id)

        mc = PvcMemberController(self.sa_session, account_id=self.operator.id)
        # cant upgrade a commissioner
        self.assertRaises(PublicUserException, mc.upgrade_to_commissioner, self.ctm.id)

        tm = mc.upgrade_to_commissioner(self.tm.id)
        self.assertTrue(tm.is_commissioner)

    def test_simple_disable_as_commissioner(self):
        mc = PvcMemberController(self.sa_session, account_id=self.ma.id)
        self.assertRaises(PermissionDeniedException, mc.disable_commissioner, self.ctm.id)

        mc = PvcMemberController(self.sa_session, account_id=self.operator.id)
        self.assertRaises(PublicUserException, mc.disable_commissioner, self.ctm.id)

        tm = mc.upgrade_to_commissioner(self.tm.id)
        self.assertTrue(tm.is_commissioner)

        mc.disable_commissioner(self.tm.id)

        self.sa_session.expire(self.tm)
        self.assertFalse(tm.is_commissioner)

    def test_subrogation_request_unapproved_subrogator(self):
        tbs = self._create_tbs_account(self.operator)
        service_level = self._create_tbs_service_level(tbs, year=today().year)
        tm = self._member_apply_at_tbs(self.ma, tbs, service_level, is_producer=True)
        controller = PvcSubrogationController(self.sa_session, account_id=self.ma.id)

        self.assertRaises(PublicUserException, controller.request_for_subrogation, tm, 'HENK')

    def test_subrogation_request(self):
        """
        This tests the subrogation initial request by the SUBROGATOR. Ticket #369.
        """

        nc = PvcNotificationController(self.sa_session, account_id=self.operator.id)
        cnt = nc.get_unread_notifications_count()

        controller = PvcSubrogationController(self.sa_session, account_id=self.ma.id)
        subrogation = controller.request_for_subrogation(self.tm, 'COMMISSIONER')

        self.assertEqual(subrogation.status, PvcSubrogation.STATUS_NEW)
        self.assertEqual(subrogation.commissioner, None)
        self.assertEqual(nc.get_unread_notifications_count(), cnt+1)

        # get the new notification and check that it's of type PvcSubrogationNotification
        notif = self.sa_session.query(Notification).order_by(Notification.id.desc()).first()
        self.assertEqual(notif.polymorphic_type, 'PvcSubrogationNotification')

    def test_subrogation_request_by_commissioner(self):
        """
        If a TBS Member is Commissioner only they should not be able to do a request for Subrogation.
        """
        controller = PvcSubrogationController(self.sa_session, account_id=self.ca.id)
        self.assertRaises(PublicUserException, controller.request_for_subrogation, self.ctm, 'COMMISSIONER')

    def test_commissioner_match_operator(self):
        """
        This tests the matching done by the operator #371
        """
        subrogation = self._request_for_subrogation()

        nc = PvcNotificationController(self.sa_session, account_id=self.ma.id)
        cnt = nc.get_unread_notifications_count()

        controller = PvcSubrogationController(self.sa_session, account_id=self.ma.id)
        # Validate that member cannot match
        self.assertRaises(PublicSecurityException, controller.match_commissioner, subrogation.id, self.ctm.id)

        controller = PvcSubrogationController(self.sa_session, account_id=self.operator.id)
        subrogation = controller.match_commissioner(subrogation.id, self.ctm.id)

        self.assertEqual(subrogation.status, PvcSubrogation.STATUS_PENDING)
        self.assertEqual(subrogation.mail_status, PvcSubrogation.MAIL_STATUS_SUBROGATOR_PENDING)
        self.assertEqual(nc.get_unread_notifications_count(), cnt+1)

        # get the new notification and check that it's of type PvcSubrogationNotification
        notif = self.sa_session.query(Notification).order_by(Notification.id.desc()).first()
        self.assertEqual(notif.polymorphic_type, 'PvcSubrogationNotification')

    def test_commissioner_match_tbs(self):
        """
        This tests the matching done by the TBS #371
        """
        subrogation = self._request_for_subrogation()

        wrong_tbs = self._create_tbs_account(self.operator)
        controller = PvcSubrogationController(self.sa_session, account_id=wrong_tbs.id)
        self.assertRaises(PublicUserException, controller.match_commissioner, subrogation.id, self.ctm.id)

        # The correct TBS
        controller = PvcSubrogationController(self.sa_session, account_id=self.ta.id)
        subrogation = controller.match_commissioner(subrogation.id, self.ctm.id)

        self.assertEqual(subrogation.status, PvcSubrogation.STATUS_PENDING)
        self.assertEqual(subrogation.mail_status, PvcSubrogation.MAIL_STATUS_SUBROGATOR_PENDING)

    def test_commissioner_match_different_tbs(self):
        """
        This test makes sure that the TBS for both TBS Members must be the same.
        """
        subrogation = self._request_for_subrogation()

        different_tbs = self._create_tbs_account(self.operator)
        service_level = self._create_tbs_service_level(different_tbs, year=today().year)
        tm = self._member_apply_at_tbs(self.ca, different_tbs, service_level, is_producer=False, is_commissioner=True)
        tm.status = tm.STATUS_APPROVED
        self.sa_session.add(tm)
        self.sa_session.flush()

        controller = PvcSubrogationController(self.sa_session, account_id=self.operator.id)
        self.assertRaises(PublicSecurityException, controller.match_commissioner, subrogation.id, tm.id)

    def test_commissioner_match_subrogator(self):
        # this tests ticket #406
        subrogation = self._request_for_subrogation()

        controller = PvcSubrogationController(self.sa_session, account_id=self.operator.id)
        # Validate that operator cannot match the subrogator.
        self.assertRaises(PublicUserException, controller.match_commissioner, subrogation.id, self.tm.id)


    def test_get_available_commissioners_by_name(self):

        subrogation = self._request_for_subrogation()

        controller = PvcSubrogationController(self.sa_session, account_id=self.ma.id)
        # The member user cannot match Commissioners
        self.assertRaises(PublicSecurityException, controller.get_available_commissioners_by_name, subrogation.id, 'Test Commy')

        controller = PvcSubrogationController(self.sa_session, account_id=self.operator.id)
        result = controller.get_available_commissioners_by_name(subrogation.id, '404 NOT FOUND')
        self.assertEqual(len(result), 0)

        result = controller.get_available_commissioners_by_name(subrogation.id, 'COMM')

        self.assertEqual(len(result), 1)
        self.assertEqual(result[0]['id'], self.ctm.id)

    def test_mandate_upload_by_subrogator(self):

        subrogation = self._request_for_subrogation()
        controller = PvcSubrogationController(self.sa_session, account_id=self.ta.id)
        subrogation = controller.match_commissioner(subrogation.id, self.ctm.id)

        upload_mandate = self._create_uploaded_file('upload_mandate.pdf', 'upload_mandate contents')
        controller = PvcSubrogationController(self.sa_session, account_id=self.ma.id)
        subrogation = controller.save_subrogation(subrogation.id, {'upload_mandate': upload_mandate.file_hash})

        subrogation = controller.get_subrogations(subrogation.id).pop()
        self.assertEqual(subrogation.status, PvcSubrogation.STATUS_SUBROGATOR_APPROVED)
        self.assertEqual(subrogation.mail_status & PvcSubrogation.MAIL_STATUS_SUBROGATOR_PENDING, PvcSubrogation.MAIL_STATUS_SUBROGATOR_PENDING)

    def test_mandate_upload_by_commissioner_before_subrogator(self):
        subrogation = self._request_for_subrogation()
        controller = PvcSubrogationController(self.sa_session, account_id=self.ta.id)
        subrogation = controller.match_commissioner(subrogation.id, self.ctm.id)

        upload_mandate = self._create_uploaded_file('upload_mandate.pdf', 'upload_mandate contents')
        controller = PvcSubrogationController(self.sa_session, account_id=self.ca.id)
        self.assertRaises(PublicUserException, controller.save_subrogation, subrogation.id, {'upload_mandate': upload_mandate.file_hash})

    def test_mandate_upload_by_commissioner(self):
        subrogation = self._request_for_subrogation()
        controller = PvcSubrogationController(self.sa_session, account_id=self.ta.id)
        subrogation = controller.match_commissioner(subrogation.id, self.ctm.id)

        upload_mandate = self._create_uploaded_file('upload_mandate.pdf', 'upload_mandate contents')
        controller = PvcSubrogationController(self.sa_session, account_id=self.ma.id)
        subrogation = controller.save_subrogation(subrogation.id, {'upload_mandate': upload_mandate.file_hash})


        nc = PvcNotificationController(self.sa_session, account_id=self.operator.id)
        cnt = nc.get_unread_notifications_count()

        upload_mandate = self._create_uploaded_file('upload_mandate.pdf', 'upload_mandate contents')
        controller = PvcSubrogationController(self.sa_session, account_id=self.ca.id)
        subrogation = controller.save_subrogation(subrogation.id, {'upload_mandate': upload_mandate.file_hash})

        self.assertEqual(subrogation.status, PvcSubrogation.STATUS_COMMISSIONER_APPROVED)
        self.assertEqual(subrogation.mail_status & PvcSubrogation.MAIL_STATUS_COMMISSIONER_PENDING, PvcSubrogation.MAIL_STATUS_COMMISSIONER_PENDING)

        # operator should have received a new notification
        self.assertEqual(nc.get_unread_notifications_count(), cnt+1)

        # get the new notification and check that it's of type PvcSubrogationNotification
        notif = self.sa_session.query(Notification).order_by(Notification.id.desc()).first()
        self.assertEqual(notif.polymorphic_type, 'PvcSubrogationNotification')

    def test_approval_by_operator(self):
        subrogation = self._request_for_subrogation()
        controller = PvcSubrogationController(self.sa_session, account_id=self.ta.id)
        subrogation = controller.match_commissioner(subrogation.id, self.ctm.id)

        upload_mandate = self._create_uploaded_file('upload_mandate.pdf', 'upload_mandate contents')
        controller = PvcSubrogationController(self.sa_session, account_id=self.ma.id)
        subrogation = controller.save_subrogation(subrogation.id, {'upload_mandate': upload_mandate.file_hash})

        upload_mandate = self._create_uploaded_file('upload_mandate.pdf', 'upload_mandate contents')
        controller = PvcSubrogationController(self.sa_session, account_id=self.ca.id)
        subrogation = controller.save_subrogation(subrogation.id, {'upload_mandate': upload_mandate.file_hash})

        controller = PvcSubrogationController(self.sa_session, account_id=self.ma.id)
        # The member user cannot approve Subrogations
        self.assertRaises(PublicSecurityException, controller.approve_subrogation, subrogation.id)

        snc = PvcNotificationController(self.sa_session, account_id=self.ma.id)
        subrogator_cnt = snc.get_unread_notifications_count()

        cnc = PvcNotificationController(self.sa_session, account_id=self.ca.id)
        commissioner_cnt = cnc.get_unread_notifications_count()

        controller = PvcSubrogationController(self.sa_session, account_id=self.operator.id)
        subrogation = controller.approve_subrogation(subrogation.id)

        self.assertEqual(subrogation.status, PvcSubrogation.STATUS_ACTIVE)
        self.assertEqual(snc.get_unread_notifications_count(), subrogator_cnt+1)
        self.assertEqual(cnc.get_unread_notifications_count(), commissioner_cnt+1)

        # get the new notification and check that it's of type PvcSubrogationNotification
        notif = self.sa_session.query(Notification).order_by(Notification.id.desc()).first()
        self.assertEqual(notif.polymorphic_type, 'PvcSubrogationNotification')

    def test_rejection_by_operator(self):

        subrogation = self._request_for_subrogation()

        controller = PvcSubrogationController(self.sa_session, account_id=self.ma.id)
        # The member user cannot reject Subrogations
        self.assertRaises(PublicSecurityException, controller.reject_subrogation, subrogation.id, 'Reject reason')

        controller = PvcSubrogationController(self.sa_session, account_id=self.operator.id)
        subrogation = controller.reject_subrogation(subrogation.id, 'Cannot find Commy')

        self.assertEqual(subrogation.mail_status & PvcSubrogation.MAIL_STATUS_REJECTION_PENDING, PvcSubrogation.MAIL_STATUS_REJECTION_PENDING)
        self.assertEqual(subrogation.status, PvcSubrogation.STATUS_REJECTED)

    def test_subrogator_mail_task(self):
        self._setup_mails()
        subrogation = self._request_for_subrogation()

        controller = PvcSubrogationController(self.sa_session, account_id=self.ta.id)
        subrogation = controller.match_commissioner(subrogation.id, self.ctm.id)

        self.assertEqual(subrogation.mail_status & PvcSubrogation.MAIL_STATUS_SUBROGATOR_PENDING, PvcSubrogation.MAIL_STATUS_SUBROGATOR_PENDING)

        # Trigger mail task
        SubgrogationMailTask().execute()

        self.sa_session.expire(subrogation)
        self.assertEqual(subrogation.mail_status & PvcSubrogation.MAIL_STATUS_SUBROGATOR_SENT, PvcSubrogation.MAIL_STATUS_SUBROGATOR_SENT)

    def test_commissioner_mail_task(self):
        self._setup_mails()
        subrogation = self._request_for_subrogation()

        controller = PvcSubrogationController(self.sa_session, account_id=self.ta.id)
        subrogation = controller.match_commissioner(subrogation.id, self.ctm.id)

        upload_mandate = self._create_uploaded_file('upload_mandate.pdf', 'upload_mandate contents')
        controller = PvcSubrogationController(self.sa_session, account_id=self.ma.id)
        subrogation = controller.save_subrogation(subrogation.id, {'upload_mandate': upload_mandate.file_hash})

        self.assertEqual(subrogation.mail_status & PvcSubrogation.MAIL_STATUS_COMMISSIONER_PENDING, PvcSubrogation.MAIL_STATUS_COMMISSIONER_PENDING)

        # Trigger mail task
        SubgrogationMailTask().execute()

        self.sa_session.expire(subrogation)
        self.assertEqual(subrogation.mail_status & PvcSubrogation.MAIL_STATUS_COMMISSIONER_SENT, PvcSubrogation.MAIL_STATUS_COMMISSIONER_SENT)

    def test_rejection_mail_task(self):
        self._setup_mails()
        subrogation = self._request_for_subrogation()

        controller = PvcSubrogationController(self.sa_session, account_id=self.operator.id)
        subrogation = controller.reject_subrogation(subrogation.id, 'Cannot find Commy')

        # Trigger mail task
        SubgrogationMailTask().execute()

        self.sa_session.expire(subrogation)
        self.assertEqual(subrogation.mail_status & PvcSubrogation.MAIL_STATUS_REJECTION_SENT, PvcSubrogation.MAIL_STATUS_REJECTION_SENT)

    def test_subrogation_suspension(self):
        subrogation = self._request_for_subrogation()
        controller = PvcSubrogationController(self.sa_session, account_id=self.ta.id)
        subrogation = controller.match_commissioner(subrogation.id, self.ctm.id)

        upload_mandate = self._create_uploaded_file('upload_mandate.pdf', 'upload_mandate contents')
        controller = PvcSubrogationController(self.sa_session, account_id=self.ma.id)
        subrogation = controller.save_subrogation(subrogation.id, {'upload_mandate': upload_mandate.file_hash})

        upload_mandate = self._create_uploaded_file('upload_mandate.pdf', 'upload_mandate contents')
        controller = PvcSubrogationController(self.sa_session, account_id=self.ca.id)
        subrogation = controller.save_subrogation(subrogation.id, {'upload_mandate': upload_mandate.file_hash})

        controller = PvcSubrogationController(self.sa_session, account_id=self.ma.id)
        # The member user cannot suspend Subrogations
        self.assertRaises(PublicSecurityException, controller.suspend_subrogation, subrogation.id)

        controller = PvcSubrogationController(self.sa_session, account_id=self.operator.id)

        self.assertRaises(PublicUserException, controller.suspend_subrogation, subrogation.id)

        subrogation = controller.approve_subrogation(subrogation.id) # approve

        # subrogate some pv modules.
        _modules = self._fill_pv_modules(self.operator)

        pv_controller = PvcPVModuleController(self.sa_session, account_id=self.ma.id)

        pv_controller.save_or_update_module_for_countries(self.tm.member.id, [self.tm.tbs.country.id], _modules[0], subrogation.id)
        pv_controller.save_or_update_module_for_countries(self.tm.member.id, [self.tm.tbs.country.id], _modules[1], subrogation.id)
        pv_controller.save_or_update_module_for_countries(self.tm.member.id, [self.tm.tbs.country.id], _modules[2], subrogation.id)

        subrogation = controller.suspend_subrogation(subrogation.id)

        # assert pv modules as inactive.
        modules = self.sa_session.query(PvcTbsMemberPVModule).filter(PvcTbsMemberPVModule.subrogation_id==subrogation.id).all()
        self.assertEqual(len(modules), 3)

        for module in modules:
            self.assertEqual(module.is_active, PvcTbsMemberPVModule.IS_INACTIVE)

    def test_subrogation_suspension_through_commissioner_disable(self):
        subrogation = self._request_for_subrogation()
        controller = PvcSubrogationController(self.sa_session, account_id=self.ta.id)
        subrogation = controller.match_commissioner(subrogation.id, self.ctm.id)

        upload_mandate = self._create_uploaded_file('upload_mandate.pdf', 'upload_mandate contents')
        controller = PvcSubrogationController(self.sa_session, account_id=self.ma.id)
        subrogation = controller.save_subrogation(subrogation.id, {'upload_mandate': upload_mandate.file_hash})

        upload_mandate = self._create_uploaded_file('upload_mandate.pdf', 'upload_mandate contents')
        controller = PvcSubrogationController(self.sa_session, account_id=self.ca.id)
        subrogation = controller.save_subrogation(subrogation.id, {'upload_mandate': upload_mandate.file_hash})

        controller = PvcSubrogationController(self.sa_session, account_id=self.ma.id)
        # The member user cannot suspend Subrogations
        self.assertRaises(PublicSecurityException, controller.suspend_subrogation, subrogation.id)

        controller = PvcSubrogationController(self.sa_session, account_id=self.operator.id)
        subrogation = controller.approve_subrogation(subrogation.id) # approve

        # subrogate some pv modules.
        _modules = self._fill_pv_modules(self.operator)

        pv_controller = PvcPVModuleController(self.sa_session, account_id=self.ma.id)

        pv_controller.save_or_update_module_for_countries(self.tm.member.id, [self.tm.tbs.country.id], _modules[0], subrogation.id)
        pv_controller.save_or_update_module_for_countries(self.tm.member.id, [self.tm.tbs.country.id], _modules[1], subrogation.id)
        pv_controller.save_or_update_module_for_countries(self.tm.member.id, [self.tm.tbs.country.id], _modules[2], subrogation.id)

        # set CTM to a producer also
        self.ctm.is_producer = True
        self.sa_session.add(self.ctm)
        self.sa_session.flush()

        mc = PvcMemberController(self.sa_session, account_id=self.operator.id)
        mc.disable_commissioner(self.ctm.id)

        # assert pv modules as inactive.
        modules = self.sa_session.query(PvcTbsMemberPVModule).filter(PvcTbsMemberPVModule.subrogation_id==subrogation.id).all()
        self.assertEqual(len(modules), 3)

        for module in modules:
            self.assertEqual(module.is_active, PvcTbsMemberPVModule.IS_INACTIVE)

# below is for running the test from eclipse
if __name__ == '__main__':

    from twisted.scripts.trial import run
    import sys
    sys.argv = ['test_subrogation.py', 'pvcycle.test.unit.test_subrogation.TestCase.test_subrogation_request']
    run()
    sys.exit()












